


























;         ciforth : a generic I86 ISO FORTH by HCC FIG
; $Id: ciarm.gnr,v 6.120 2022/03/31 15:10:56 albert Exp $
; Copyright (2012): Albert van der Horst/Frans van der Markt by GNU Public License
;
;HCC FIG Holland : Hobby Computer Club, Forth Interest Group  Holland
; ;   ciforth $Revision: 6.120 $
;
;
;
;
; If this is a configured assembly file, it should be accompanied with configured
; documentation (texinfo, ps, html.)
; WITHOUT THE DOCUMENTATION: GIVE UP! GET THE REAL THING!
; You have a configured system, if there are NO curly brackets on the next line.
;                          
;
; Configuration of this particular version:
; 32-bits
; running under Linux  ;  with native forth I/O 

; Normally ciforth doesn't observe ISO >IN.
; Contains :
; (there may be no items here.)
;        Security words
;         Loadable words, i.e. all of ISO CORE, more than is needed
;           for a self contained kernel.
; A field in the header to point to source
;
;
; This is a NASM version of ciforth created by ``m4'' from the generic listing.
; It can be assembled using ``nasm'' obtainable via :
; Source: ftp://ftp.us.kernel.org/pub/software/devel/nasm/source/
; URL: http://www.cryogen.com/Nasm/


; This version can be assembled on a Linux system by
;   nasm forth.asm -felf -o forth.o
;   ld -N forth.o -o forth
;And to get the smallest possible executable(optional):
;   strip forth -s -R .data
 


; For assembling on other systems where nasm is available see the
; documentation of nasm.

_COMMENTED(
        A generic version of ISO FORTH for ARM processors
                M4_SUPPLIER

                in cooperation with
                HCC Forth user group
                The Netherlands
                www.forth.hccnet.nl

              based on
              FIG-FORTH
   implemented by:  Charlie Krajewski
                    205 ( BIG ) Blue Rd.
                    Middletown, CT  06457

  The listing has been made possible by the
  prior work of:
               Thomas Newman, Hayward, Ca.

 : other_acknowledgements
         John_Cassidy
         Kim_Harris
         George_Flammer
         Robert_D._Villwock ;

 : for tools
         Richard M. Stallman
         Linus Torvalds

No one who programs with FORTH can afford to be without:
  "Starting Forth  by Leo Brodie" and "Thinking Forth by Leo Brodie".
   Both out of print, but saved on the internet.

This Forth is a descendant in the 300+ (RCS)- generations from fig-Forth.

For nostalgic reasons the following comment has never been removed:
   Although there is much to be said for typing in your own
   listing and getting it running, there is much to be said
   not typing in your own listing.  If you feel that 100+
   pages of plinking is nutty, contact me for availability
   of a disc with source & executable files.  Obtainable at
   a bargain basement price, prepare yourself for bargain
   basement support.

All publications of the FORTH Interest Group are public domain.
They may be further distributed by the inclusion of this
credit notice:
               This publication has been made available by:

               FORTH Interest Group
               P.O. Box 1105
               San Carlos, Ca.  94070
[I feel obliged to keep this last one in (AH). Note that although it is
based on fig-Forth no stone is left unturned.]
)         ;
; ########################################################################################
;                       PREPARATION (no code)
; ########################################################################################

FIGREL  EQU     5       ; FIG RELEASE #
FIGREV  EQU     0       ; FIG REVISION #
USRVER  EQU     0      ; USER VERSION NUMBER, a digit now
;
;      VERY ELEMENTARY .
CW      EQU     4    ; Size of a cell in Forth, not in the bootcode.
ERRORSCREEN EQU     48    ; Screen where the error messages start.
;
;      MEMORY LAYOUT.
; Normally this is specified at the m4 configuration level.
; For a configured system these values can be changed at this single place. 
NBUF    EQU     16   ; No. of buffers, or screens 
KBBUF   EQU     1024      ; Data bytes per disk buffer
US      EQU     0x40*CW  ; User variable space, eats into next data stack, if any.
EM      EQU     (0x4000000)-0x74     ; Where the memory ends w.r.t. ORIG. 1) 
EMP     EQU     (EM-1)/0x1000+1 ; Number of pages.
TAS     EQU     0x10000    ;Size of unitialised area per task.
STACKSIZE   EQU TAS/4  ;For both stacks.  but for now
TIBSIZE     EQU TAS/4  ;TIB size. 
;


;
;      ASCII CHARACTER EQUIVALENTS
;
ABL     EQU     ' '     ; SPACE
ACR     EQU     0x0D     ; CR
AMS     EQU     '-'          ; MINUS SIGN 
ASO     EQU     '['          ; SQUARE BRACKET OPEN 
ASC     EQU     ']'          ; SQUARE BRACKET CLOSE 
ADOT    EQU     '.'          ; PERIOD
ALF     EQU     0x0A     ; LINE FEED, USED INTERNALLY AS LINE ENDER 
AFF     EQU     0x0C     ; FORM FEED
BELL    EQU     0x07     ; ^G
BSIN    EQU     0x08     ; INPUT DELETE CHARACTER
BSOUT   EQU     0x08     ; OUTPUT BACKSPACE ( ^H )
;
;      HEADER RELATED EQUATES
;      dea is the dictionary entry address 
B_DUMMY   EQU     0x01     ; dea is dummy, from namespace link
B_INVIS   EQU     0x02     ; dea is invisible, "smudged".
B_IMMED   EQU     0x04     ; dea is a immediate.
B_DENOT   EQU     0x08     ; dea is a denotation.
C_HOFFSET EQU     0       ; Offsets of code field in cells, w.r.t. dea
D_HOFFSET EQU     1       ; Same for data field
F_HOFFSET EQU     2       ; Same for flag field
L_HOFFSET EQU     3       ; Same for link field
N_HOFFSET EQU     4       ; Same for name field
S_HOFFSET EQU     5       ; Same for source field
PH_OFFSET EQU     5+1   ; Past header field: Start of data area. 
BD_OFFSET EQU     5+1+1 ; Start of BODY for CREATEd word.
HEADSIZE  EQU     (CW*(PH_OFFSET))  ;In cells, only to clean up source.
;
 ;  


BPS     EQU     512             ;Bytes/sector, common to all of MSDOS
SPB     EQU     KBBUF/BPS       ;Sectors per buffer 
;







; Bring the names in line with what Forth want immediately. 
; ------------------------------------------------------------      
;   Start of constants stolen from C.                                 
; ------------------------------------------------------------      

SEEK_SET	EQU	0x0
SEEK_CUR	EQU	0x1
TCGETS	EQU	0x5401
TCSETS	EQU	0x5402
ECHO	EQU	0x8
EAGAIN	EQU	0xb
EINTR	EQU	0x4
EPIPE	EQU	0x20
VMIN	EQU	0x6
VTIME	EQU	0x5
ICANON	EQU	0x2
O_RDWR	EQU	0x2
O_RDONLY	EQU	0x0
O_WRONLY	EQU	0x1
O_CREAT	EQU	0x40
O_NONBLOCK	EQU	0x800
MAP_SHARED	EQU	0x1
MAP_PRIVATE	EQU	0x2
P_PID	EQU	0x1
WEXITED	EQU	0x4
SIZE_TERMIO	EQU	0x3c

; Numbers of system calls. See "Linux kernel Internals" Appendix A. 
; By M.Beck, H. Boehme e.a. Addison Wesley.                         
; The system calls themselves are extensively documented in chapter 
; 2 of the man pages, e.g. "man 2 exit"
exit	EQU	0x1
open	EQU	0x5
close	EQU	0x6
creat	EQU	0x8
unlink	EQU	0xa
chdir	EQU	0xc
read	EQU	0x3
_newselect	EQU	0x8e
write	EQU	0x4
ioctl	EQU	0x36
lseek	EQU	0x13
execve	EQU	0xb
fork	EQU	0x2
pipe	EQU	0x2a
mmap2	EQU	0xc0
waitid	EQU	0x118
; ------------------------------------------------------------      
;   End of constants stolen from C.                                 
; ------------------------------------------------------------      

select EQU _newselect
mmap   EQU mmap2


RAWIO           EQU     (ECHO | ICANON)

; 


_main:



; allocatable, executable 
; Must be normal .text or otherwise linking with c-lib fails. 
; .text is normally not writable but linker -N option takes care of that. 
        _TEXT_
        ;
        GLOBAL  _start       ; Entry point.
        GLOBAL  start        ; Entry point.
_ORIG:
_start:      ; Entry point.
; 









NOBOOT:         ; Skip till here if not booting.


ENDBOOT:










 

; ########################################################################################
;                       FORTH startup CODE + restarts + debug 
; ########################################################################################
;


; 4.15.3.2
start:
;
COLD_ENTRY:

        ADR   R4, USINI
        MOV     R0, SP              ;Remember ARGS.
        STR     R0, [R4, #CW*(31)]
;


; Initialisation of the Forth engine.
        ADR     R4, USINI
        LDR     R0, [R4, #CW*(2)]  ;Parameter stack
        MOV     R13, R0           ;Parameter stack
        LDR     R12, [R4, #CW*(3)]  ;Return stack
        ADR     R11, COLD + CW*(PH_OFFSET)    ;Interpreter pointer
      LDR     R0, [R11],#+(CW*(1))
         LDR     R1,[R0]
         BX      R1  



; 

;
;
; ########################################################################################
;                       FORTH ITSELF
; ########################################################################################
;
_COMMENTED(
   FORTH REGISTERS
   The names under FORTH are used in the generic source.

   FORTH   ARM      FORTH PRESERVATION RULES
   -----   ----     ----- ------------ -----
dnl Note how the following is to pass through m4
           R0-R3   Used within and cluttered by Forth words.
                    across FORTH words.
                    The first of those is {WOR} (probably)

   {WOR}   WOR      Working register.  When entering a word
                    via its code field the DEA is passed in {WOR}.

            R0      Scratch register.  Used to pass data from
                    FORTH words, see label APUSH or macro {_APUSH}

            R1      General register.  Used when two data items are passed
                    from FORTH words, see label DPUSH or macro {_DPUSH}

   {HIP}   HIP      High level Interpreter Pointer.  Must be preserved

   {RPO}   RPO      Return stack pointer.  Must be preserved across
                    FORTH words.

   {SPO}   SPO      Parameter stack pointer.  Must be preserved
                    across FORTH words.

           PC       Machine Process Counter, sometimes used
                    for relative addressing

This is the ARM asignment, not valid for 64 bit.

   R13     SP      ARM stack pointer.
                   In thumb this can be used with pop and push.

   R14     RL      Machine Link Register for calls.
                    Can be used within Forth words, with care.

  R15     PC        Machine Process Counter (Program counter)
                    Cannot be used within Forth words.



            The remaining registers are free to use.
            R0-R3 are the normal Forth work registers.
            Within assembler words WOR can be used too,
            as it has served its purpose.

            Here are some  useful code sequences.
            For next:
                    _NEXT
            For push single and next:
                    _PUSH
            For push double and next:
                    _2PUSH



----------------------------------------------------------
)         ;
_COMMENTED(
---------------------------------------------

   COMMENT CONVENTIONS
   ------- -----------

   =       IS EQUAL TO
   <-      ASSIGNMENT

  NAME        =  Address of name
  (NAME)      =  Contents of name
  DEA         =  DICTIONARY ENTRY ADDRESS
                 fields have fixed offsets from DEA
  CFA         =  CODE FIELD ADDRESS : a pointer to executable code
  DFA         =  DATA FIELD ADDRESS : a pointer to
                        data/high level code/ DOES> pointer
  FFA         =  FLAG FIELD ADDRESS: contains flags
  LFA         =  LINK FIELD ADDRESS: a pointer to the next DEA
  NFA         =  NAME FIELD ADDRESS: a pointer to a name buffer
  _SOURCEFIELD_({SFA =  SOURCE FIELD ADDRESS: a pointer into source code})
  PHA         =  POST HEADER ADDRESS: After all fields

  S1          =  Parameter stack - 1st cell
  S2          =  Parameter stack - 2nd cell
  RP1          =  Return stack    - 1st cell
  R2          =  Return stack    - 2nd cell

  LSB         =  Least significant bit
  MSB         =  Most  significant bit
  LB          =  Low byte
  HB          =  High byte
  LC          =  Low  cell
  HC          =  High cell

------------------------------------------------------------
) ; 
        ;

;
; 
; There may be no jumps to NEXT at all 
; The label NEXT1 is rarely relevant (for _OLDDEBUG_) 
DPUSH:  STR     R1, [R13,#-(CW*(1))]!      ; Fall through.
APUSH:  STR     R0, [R13,#-(CW*(1))]!
NEXT:
;
        LDR     R0, [R11],#+(CW*(1))                  ; R0 <- (HIP) 
NEXT1:
        LDR     R1, [R0]
        BX      R1     ; TO `CFA'
;
;       Dictionary starts here.


;  ************
;  *   ONLY   *
;  ************
;
        ALIGN    4
N_ONLY:
        DD      4
        DB      "ONLY"
        ALIGN    4
ONLY:
        DD    DODOE
        DD    ONLY+HEADSIZE
        DD    0x0
        DD    0
        DD    N_ONLY
        DD    0

        DD      DOVOC
        DD      FORTH     ; NEXT NAMESPACE 
ONLYBODY:

        ALIGN    4

        DD    0x0
        DD    0
        DD    B_DUMMY
        DD    STARTOFONLY
        DD    (ZERO+(CW*(D_HOFFSET)))
        DD    0

;

;  *******************
;  *   ENVIRONMENT   *
;  *******************
;
        ALIGN    4
N_ENV:
        DD      11
        DB      "ENVIRONMENT"
        ALIGN    4
ENV:
        DD    DODOE
        DD    ENV+HEADSIZE
        DD    0x0
        DD    ONLY
        DD    N_ENV
        DD    0

        DD      DOVOC
        DD      ONLY       ; NEXT NAMESPACE 

        ALIGN    4

        DD    0x0
        DD    0
        DD    B_DUMMY
        DD    STARTOFENVIRONMENT
        DD    (ZERO+(CW*(D_HOFFSET)))
        DD    0

;

;  ************
;  *   COLD   *
;  ************
;
        ALIGN    4
N_COLD:
        DD      4
        DB      "COLD"
        ALIGN    4
COLD:
        DD    DOCOL
        DD    COLD+HEADSIZE
        DD    0x0
        DD    ENV
        DD    N_COLD
        DD    0

; Fill user area for single task.
        DD      LIT, USINI      ; Fails if SP/RP are not usable!
        DD      LIT, USINI+(CW*(1)), FETCH
        DD      LIT, US
        DD      LCMOVE
        DD      INIT
        DD      ONE            ; Sign on wanted.


        DD      ZERO, LIT, TCGETS, TERMIO
        DD      LIT, ioctl, XOS, ZLESS
        DD      ZBRAN
        DD      COLD1-$-CW
        ; Not a terminal. Cannot set the terminal, get rid of "OK".
        DD      DROP, ZERO      ; Sign on suppressed.
        DD      LIT,TASK  ; Suppress OK for pipes.
        DD      LIT,OK
        DD      LIT, (CW*(3)), LCMOVE
COLD1:
 ; 

        DD      SKIP
         DD      8
SB0: DB      "/dev/mem"
       ALIGN    4
        DD      LIT, SB0
        DD      LIT, 8
        DD      LIT, 0x00181002
        DD      OFILE, LOR, DEVMEM, STOR
;
       DD      OPTIONS
        DD      ZBRAN
        DD      COLD5-$-CW

        DD      SIGNON    ; Suppressed for scripting! Or any options.
COLD5:
        DD      ABORT
        DD      BYE     ; In case of turnkey programs.
        DD      SEMIS   ; Unnecessary, but helpful for decompilation.
;

;  ************
;  *   USER   *
;  ************
;
        ALIGN    4
N_USER:
        DD      4
        DB      "USER"
        ALIGN    4
USER:
        DD    DOCOL
        DD    USER+HEADSIZE
        DD    0x0
        DD    COLD
        DD    N_USER
        DD    0

        DD      LCONST
        DD      PSCOD
DOUSE:  LDR     R3,[R0, # + (CW*((D_HOFFSET)))] ;PFA  
        ADR     R0, USINI
        LDR     R0,[R0]
        ADD     R0,R0,R3      ;ADDR OF VARIABLE
        STR     R0, [R13,#-(CW*(1))]!
        LDR     R0, [R11],#+(CW*(1))
         LDR     R1,[R0]
         BX      R1  

;

;  ***************
;  *   +ORIGIN   *
;  ***************
;
        ALIGN    4
N_PORIG:
        DD      7
        DB      "+ORIGIN"
        ALIGN    4
PORIG:
        DD    DOCOL
        DD    PORIG+HEADSIZE
        DD    0x0
        DD    USER
        DD    N_PORIG
        DD    0

        DD      LIT
        DD      USINI
        DD      PLUS
        DD      SEMIS
;
;      Initialisation block for user variables through DOC-LINK
;       <<<<< must be in same order as user variables >>>>>
; To be fig compatible, there should be two vectors here  
; for the cold and warm start.
;
; ------------------------ Forth entry code, hooks for restart -------------
; FIXME. You want to jump here from -2 CELLS +USINI / -4 CELLS +USINI 
        ADR     R4, COLD
        MOV     R0, R4
        LDR     R1, [R0]
        BX      R1          ;Hope stacks are still okay.
        ALIGN    4
; Make it possible to revector WARM. You can even make it low level. 
        ADR      R4, WARM
        MOV      R0,R4
        LDR     R1, [R0]          ;Hope stacks are still okay.
        BX      R1
        ALIGN    4
USINI:  DD      STRUSA
        DD      STRUSA  ; INIT (U0) user area of the main task 1
        DD      INITS0  ; INIT (S0)         2
        DD      INITR0  - (CW*(1))
                        ; INIT (R0)         3
        DD      STRTIB  ; INIT (TIB)        4
        DD      BSIN    ; RUBOUT: get rid of latest char 5
        DD      0       ; AVAILABLE         6
        DD      1       ; INIT (WARNING)     7
        DD      INITDP  ;      INIT (FENCE)  8
DPA:    DD      INITDP  ;      INIT (DP)     9
        DD      ENV ;       INIT (VOC-LINK) 10
        DD      39+16        ; INIT (MAX-USER) 
;
        DD      0, 0            ; WHERE             12 13 
        DD      0, 0            ;REMAINDER   14 15 

        RESB    US-($ - USINI)        ; All user can be initialised.
;
;      <<<<< end of data used by cold start >>>>>
;


;  **************
;  *   VMA-IO   *
;  **************
;
        ALIGN    4
N_VMAIO:
        DD      6
        DB      "VMA-IO"
        ALIGN    4
VMAIO:
        DD    DOVAR
        DD    VMAIO+HEADSIZE
        DD    0x0
        DD    PORIG
        DD    N_VMAIO
        DD    0

        DD 0xB2000000
;

;  *******************
;  *   PERIPHERALS   *
;  *******************
;
        ALIGN    4
N_PERIPHERALS:
        DD      11
        DB      "PERIPHERALS"
        ALIGN    4
PERIPHERALS:
        DD    DOVAR
        DD    PERIPHERALS+HEADSIZE
        DD    0x0
        DD    VMAIO
        DD    N_PERIPHERALS
        DD    0

        DD 0x20000000
;

;  ***************
;  *   DEV-MEM   *
;  ***************
;
        ALIGN    4
N_DEVMEM:
        DD      7
        DB      "DEV-MEM"
        ALIGN    4
DEVMEM:
        DD    DOVAR
        DD    DEVMEM+HEADSIZE
        DD    0x0
        DD    PERIPHERALS
        DD    N_DEVMEM
        DD    0

        DD 0
;

;  ***************
;  *   MMAP-IO   *
;  ***************
;
        ALIGN    4
N_MMAPIO:
        DD      7
        DB      "MMAP-IO"
        ALIGN    4
MMAPIO:
        DD    DOCOL
        DD    MMAPIO+HEADSIZE
        DD    0x0
        DD    DEVMEM
        DD    N_MMAPIO
        DD    0

; The libc mmap definition, not necessarily documenting the system call! 
; mmap( void *addr, size_t length, int prot, int flags, int fd,
; off_t offset );     see man mmap

      DD     DEVMEM, FETCH, LDUP, ZERO, GREAT
      DD      ZBRAN
        DD      MMAP2-$-CW
      DD     TOR
      DD     VMAIO, FETCH
      DD     LIT, 0x010000000 ; length of mapped area
      DD     LIT, 3                  ; protection-bits
      DD     LIT, MAP_SHARED         ; flags
      DD     FROMR
      DD     PERIPHERALS, FETCH
 DD     LIT, 12, RSHIFT     ; Number of 4K pages  
      DD     ZERO                ; dummy 
      DD     LIT, mmap, XOS7
MMAP2:
      DD     VMAIO, STOR
      DD      SEMIS   ;Unnecessary, but helpful for decompilation.
;
;

;

;  *************
;  *   ABORT   *
;  *************
;
        ALIGN    4
N_ABORT:
        DD      5
        DB      "ABORT"
        ALIGN    4
ABORT:
        DD    DOCOL
        DD    ABORT+HEADSIZE
        DD    0x0
        DD    MMAPIO
        DD    N_ABORT
        DD    0

        DD      SZERO, FETCH, SPSTO
        DD      ZERO, HANDLER, STOR
        DD      QUIT
        DD      SEMIS   ;Unnecessary, but helpful for decompilation.
;
;

;  ************
;  *   WARM   *
;  ************
;
        ALIGN    4
N_WARM:
        DD      4
        DB      "WARM"
        ALIGN    4
WARM:
        DD    DOCOL
        DD    WARM+HEADSIZE
        DD    0x0
        DD    ABORT
        DD    N_WARM
        DD    0

        DD      INIT
        DD      SIGNON
        DD      ABORT
        DD      SEMIS   ;Unnecessary, but helpful for decompilation.
;

;  ************
;  *   NOOP   *
;  ************
;
        ALIGN    4
N_NOOP:
        DD      4
        DB      "NOOP"
        ALIGN    4
NOOP:
        DD    NOOP+HEADSIZE
        DD    NOOP+HEADSIZE
        DD    0x0
        DD    WARM
        DD    N_NOOP
        DD    0

       LDR     R0, [R11],#+(CW*(1))
         LDR     R1,[R0]
         BX      R1  
;
; 
;

;  ***********
;  *   LIT   *
;  ***********
;
        ALIGN    4
N_LIT:
        DD      3
        DB      "LIT"
        ALIGN    4
LIT:
        DD    LIT+HEADSIZE
        DD    LIT+HEADSIZE
        DD    0x0
        DD    NOOP
        DD    N_LIT
        DD    0

        LDR     R0, [R11],#+(CW*(1))                  ; R0 <- (HIP) 
        STR     R0, [R13,#-(CW*(1))]!
        LDR     R0, [R11],#+(CW*(1))
         LDR     R1,[R0]
         BX      R1            ; TO TOP OF STACK
;

;todo EXECUTE
;  ***************
;  *   EXECUTE   *
;  ***************
;
        ALIGN    4
N_EXEC:
        DD      7
        DB      "EXECUTE"
        ALIGN    4
EXEC:
        DD    EXEC+HEADSIZE
        DD    EXEC+HEADSIZE
        DD    0x0
        DD    LIT
        DD    N_EXEC
        DD    0

        LDR     R0, [R13],#+(CW*(1))      ; GET XT
        LDR    R1, [R0]
        BX     R1  ;(IP) <- (PFA)

;

;  ***************
;  *   RECURSE   *
;  ***************
;
        ALIGN    4
N_RECURSE:
        DD      7
        DB      "RECURSE"
        ALIGN    4
RECURSE:
        DD    DOCOL
        DD    RECURSE+HEADSIZE
        DD    B_IMMED
        DD    EXEC
        DD    N_RECURSE
        DD    0

        DD      LATEST, COMMA
        DD      SEMIS
;

;  **************
;  *   BRANCH   *
;  **************
;
        ALIGN    4
N_BRAN:
        DD      6
        DB      "BRANCH"
        ALIGN    4
BRAN:
        DD    (SKIP+HEADSIZE)
        DD    BRAN+HEADSIZE
        DD    0x0
        DD    RECURSE
        DD    N_BRAN
        DD    0

;

;  ************
;  *   SKIP   *
;  ************
;
        ALIGN    4
N_SKIP:
        DD      4
        DB      "SKIP"
        ALIGN    4
SKIP:
        DD    SKIP+HEADSIZE
        DD    SKIP+HEADSIZE
        DD    0x0
        DD    BRAN
        DD    N_SKIP
        DD    0

BRAN1:
        LDR     R0, [R11],#+(CW*(1))                  ; R0 <- (HIP) 

        SUB     R0,R0, #1
        
         ORR     R0,R0, #(CW-1) 
        ADD     R0,R0, #1
;
        ADD     R11,R11,R0
        LDR     R0, [R11],#+(CW*(1))
         LDR     R1,[R0]
         BX      R1  
;

;  ***************
;  *   0BRANCH   *
;  ***************
;
        ALIGN    4
N_ZBRAN:
        DD      7
        DB      "0BRANCH"
        ALIGN    4
ZBRAN:
        DD    ZBRAN+HEADSIZE
        DD    ZBRAN+HEADSIZE
        DD    0x0
        DD    SKIP
        DD    N_ZBRAN
        DD    0

        LDR     R0, [R13],#+(CW*(1))      ; GET STACK VALUE
        ANDS     R0,R0,R0   ; ZERO?
        BEQ      BRAN1   ; YES, BRANCH
        ADD      R11,R11,# + (CW*(1))
        LDR     R0, [R11],#+(CW*(1))
         LDR     R1,[R0]
         BX      R1  
;

;  ***************
;  *   (+LOOP)   *
;  ***************
;
        ALIGN    4
N_XPLOOP:
        DD      7
        DB      "(+LOOP)"
        ALIGN    4
XPLOOP:
        DD    XPLOOP+HEADSIZE
        DD    XPLOOP+HEADSIZE
        DD    0x0
        DD    ZBRAN
        DD    N_XPLOOP
        DD    0

        LDR     R3, [R13],#+(CW*(1))      ; GET LOOP VALUE
        LDR     R2,[R12]  ; INDEX = INDEX + INCR
        ADD     R2,R2,R3
        STR     R2,[R12]  ; GET NEW INDEX
        LDR     R4, [R12, #+(CW*(1))]
        SUB     R2,R2,R4  ; COMPARE WITH LIMIT
        EOR     R2,R2,R3  ; GENERATE SIGN
        ANDS    R2,R2,R2  ; TEST SIGN
        EOR     R0,R0,R0  ; DEFAULT
        BMI     XPO1     ; KEEP LOOPING
        SUB     R0,R0, #1      ;TRUE
XPO1: STR     R0, [R13,#-(CW*(1))]!
        LDR     R0, [R11],#+(CW*(1))
         LDR     R1,[R0]
         BX      R1  
;

;  ************
;  *   (DO)   *
;  ************
;
        ALIGN    4
N_XDO:
        DD      4
        DB      "(DO)"
        ALIGN    4
XDO:
        DD    XDO+HEADSIZE
        DD    XDO+HEADSIZE
        DD    0x0
        DD    XPLOOP
        DD    N_XDO
        DD    0

        LDR     R2, [R11],#+(CW*(1))                  ; R0 <- (HIP) 
        ADD     R2,R2,R11  ;Make absolute
        LDR     R1, [R13],#+(CW*(1))           ; INITIAL INDEX VALUE
        LDR     R3, [R13],#+(CW*(1))           ; LIMIT VALUE
        
         MOV     R0, R12
         MOV     R12, R13
         MOV     R13, R0
   ; GET RETURN STACK, USES REGISTER R0 ! 
        STR     R2, [R13,#-(CW*(1))]!      ; Target location.
        STR     R3, [R13,#-(CW*(1))]!
        STR     R1, [R13,#-(CW*(1))]!
        
         MOV     R0, R12
         MOV     R12, R13
         MOV     R13, R0
   ; GET PARAMETER STACK, USES REGISTER R0 ! 
        LDR     R0, [R11],#+(CW*(1))
         LDR     R1,[R0]
         BX      R1  
;

;  *************
;  *   (?DO)   *
;  *************
;
        ALIGN    4
N_XQDO:
        DD      5
        DB      "(?DO)"
        ALIGN    4
XQDO:
        DD    XQDO+HEADSIZE
        DD    XQDO+HEADSIZE
        DD    0x0
        DD    XDO
        DD    N_XQDO
        DD    0

        LDR     R2, [R11],#+(CW*(1))                  ; R0 <- (HIP) 
        ADD     R2,R2,R11  ;Make absolute
        LDR     R1, [R13],#+(CW*(1))      ; INITIAL INDEX VALUE
        LDR     R3, [R13],#+(CW*(1))      ; LIMIT VALUE
        CMP     R1,R3
        BGE     QXDO1
        
         MOV     R0, R12
         MOV     R12, R13
         MOV     R13, R0
   ; GET RETURN STACK, USES REGISTER R0 ! 
        STR     R2, [R13,#-(CW*(1))]!      ; Target location.
        STR     R3, [R13,#-(CW*(1))]!
        STR     R1, [R13,#-(CW*(1))]!
        
         MOV     R0, R12
         MOV     R12, R13
         MOV     R13, R0
   ; GET PARAMETER STACK, USES REGISTER R0 ! 
        LDR     R0, [R11],#+(CW*(1))
         LDR     R1,[R0]
         BX      R1  
QXDO1:  MOV     R11,R2
        LDR     R0, [R11],#+(CW*(1))
         LDR     R1,[R0]
         BX      R1  
;

;  *********
;  *   I   *
;  *********
;
        ALIGN    4
N_IDO:
        DD      1
        DB      "I"
        ALIGN    4
IDO:
        DD    IDO+HEADSIZE
        DD    IDO+HEADSIZE
        DD    0x0
        DD    XQDO
        DD    N_IDO
        DD    0

        LDR     R0,[R12] ; GET INDEX VALUE
        STR     R0, [R13,#-(CW*(1))]!
        LDR     R0, [R11],#+(CW*(1))
         LDR     R1,[R0]
         BX      R1            ; TO PARAMETER STACK
;

;  *********
;  *   J   *
;  *********
;
        ALIGN    4
N_JDO:
        DD      1
        DB      "J"
        ALIGN    4
JDO:
        DD    JDO+HEADSIZE
        DD    JDO+HEADSIZE
        DD    0x0
        DD    IDO
        DD    N_JDO
        DD    0

        LDR     R0,[R12,#+(CW*(3))] ; GET INDEX VALUE
        STR     R0, [R13,#-(CW*(1))]!
        LDR     R0, [R11],#+(CW*(1))
         LDR     R1,[R0]
         BX      R1            ; TO PARAMETER STACK
;

;  **************
;  *   UNLOOP   *
;  **************
;
        ALIGN    4
N_UNLOOP:
        DD      6
        DB      "UNLOOP"
        ALIGN    4
UNLOOP:
        DD    UNLOOP+HEADSIZE
        DD    UNLOOP+HEADSIZE
        DD    0x0
        DD    JDO
        DD    N_UNLOOP
        DD    0
      
        ADD R12,R12,#(CW*(3))
        LDR     R0, [R11],#+(CW*(1))
         LDR     R1,[R0]
         BX      R1  
;

;  *************
;  *   DIGIT   *
;  *************
;
        ALIGN    4
N_DIGIT:
        DD      5
        DB      "DIGIT"
        ALIGN    4
DIGIT:
        DD    DIGIT+HEADSIZE
        DD    DIGIT+HEADSIZE
        DD    0x0
        DD    UNLOOP
        DD    N_DIGIT
        DD    0

        LDR     R1, [R13],#+(CW*(1))      ;NUMBER BASE
        LDR     R0, [R13],#+(CW*(1))      ;ASCII DIGIT
        AND     R0,R0,#0xFF
        AND     R1,R1,#0xFF
        SUBS    R0,R0,#'0'
        BLT     DIGI2   ;NUMBER ERROR
        CMP     R0,#9
        BLE     DIGI1   ;NUMBER = 0 THRU 9
        SUB     R0,R0,#7
        CMP     R0,#10   ;NUMBER 'A' THRU 'Z'?
        BLT     DIGI2   ;NO
DIGI1:  CMP     R0,R1   ; COMPARE NUMBER TO BASE
        BGE     DIGI2   ;NUMBER ERROR
        MOV     R1,R0   ;NEW BINARY NUMBER
        
        MOV    R0,#-1  ;TRUE FLAG  
        STR     R1, [R13,#-(CW*(1))]!
        STR     R0, [R13,#-(CW*(1))]!
        LDR     R0, [R11],#+(CW*(1))
         LDR     R1,[R0]
         BX      R1            ;ADD TO STACK
;   NUMBER ERROR
DIGI2:  MOV    R0, #0 ;FALSE FLAG
        STR     R1, [R13,#-(CW*(1))]!
        STR     R0, [R13,#-(CW*(1))]!
        LDR     R0, [R11],#+(CW*(1))
         LDR     R1,[R0]
         BX      R1  
;

;  **************
;  *   ~MATCH   *
;  **************
;
        ALIGN    4
N_NMATCH:
        DD      6
        DB      "~MATCH"
        ALIGN    4
NMATCH:
        DD    DOCOL
        DD    NMATCH+HEADSIZE
        DD    0x0
        DD    DIGIT
        DD    N_NMATCH
        DD    0

;
        DD      TOR                           ; Remember dea
        DD      OVER
        DD      RR, TNFA, FETCH, SFET
        DD      CORA
        DD      FROMR, SWAP                   ; Leave flag on top of dea.
        DD      SEMIS                            ;
;

;  **************
;  *   ?BLANK   *
;  **************
;
        ALIGN    4
N_QBL:
        DD      6
        DB      "?BLANK"
        ALIGN    4
QBL:
        DD    DOCOL
        DD    QBL+HEADSIZE
        DD    0x0
        DD    NMATCH
        DD    N_QBL
        DD    0

        DD      LBL, ONEP, LESS
        DD      SEMIS
;

;  ************
;  *   PP@@   *
;  ************
;
        ALIGN    4
N_PPFET:
        DD      4
        DB      "PP@@"
        ALIGN    4
PPFET:
        DD    DOCOL
        DD    PPFET+HEADSIZE
        DD    0x0
        DD    QBL
        DD    N_PPFET
        DD    0

        DD      SRC, CELLP, TFET
        DD      OVER, EQUAL
        DD      ZBRAN
        DD      PPFET1-$-CW
        DD      ZERO
        DD      BRAN
        DD      PPFET2-$-CW
PPFET1:
        DD      LDUP
        
        DD      CFET
        DD      ONE, PP, PSTOR
PPFET2:
        DD      SEMIS
;

;  ************
;  *   NAME   *
;  ************
;
        ALIGN    4
N_LNAME:
        DD      4
        DB      "NAME"
        ALIGN    4
LNAME:
        DD    DOCOL
        DD    LNAME+HEADSIZE
        DD    0x0
        DD    PPFET
        DD    N_LNAME
        DD    0

        DD      X
PWORD0: DD      DROP
        DD      PPFET, QBL
        DD      OVER, SRC, CELLP, FETCH, LSUB ; At end?
        DD      LAND, ZEQU
        DD      ZBRAN
        DD      PWORD0-$-CW

        DD      X
PWORD1: DD      DROP
        DD      PPFET, QBL
        DD      ZBRAN
        DD      PWORD1-$-CW

        DD      OVER, LSUB
        
        DD      SEMIS
;

;  *************
;  *   PARSE   *
;  *************
;
        ALIGN    4
N_PARSE:
        DD      5
        DB      "PARSE"
        ALIGN    4
PARSE:
        DD    DOCOL
        DD    PARSE+HEADSIZE
        DD    0x0
        DD    LNAME
        DD    N_PARSE
        DD    0

        DD      SRC, CELLP, TFET
        DD      OVER, LSUB
        
        DD      ROT, SSLAS, TSWAP
        DD      ZEQU
        DD      ZBRAN
        DD      PARSE8-$-CW
        DD      DROP, SRC, CELLP, FETCH
PARSE8: 
        DD      PP, STOR
        DD SEMIS
;

;  ***********
;  *   SRC   *
;  ***********
;
        ALIGN    4
N_SRC:
        DD      3
        DB      "SRC"
        ALIGN    4
SRC:
        DD    DOUSE
        DD    (CW*(27))
        DD    0x0
        DD    PARSE
        DD    N_SRC
        DD    0
      ; And 28 and 29.
;

;  **************
;  *   SOURCE   *
;  **************
;
        ALIGN    4
N_SOURCE:
        DD      6
        DB      "SOURCE"
        ALIGN    4
SOURCE:
        DD    DOCOL
        DD    SOURCE+HEADSIZE
        DD    0x0
        DD    SRC
        DD    N_SOURCE
        DD    0

        DD      SRC, TFET
        DD      SWAP, OVER, LSUB
        DD      SEMIS
;

;

;  **********
;  *   CR   *
;  **********
;
        ALIGN    4
N_CR:
        DD      2
        DB      "CR"
        ALIGN    4
CR:
        DD    DOCOL
        DD    CR+HEADSIZE
        DD    0x0
        DD    SOURCE
        DD    N_CR
        DD    0

        DD      LIT,ALF
        DD      EMIT
        DD      ZERO, LOUT, STOR
        DD      SEMIS
;

;  *************
;  *   CMOVE   *
;  *************
;
        ALIGN    4
N_LCMOVE:
        DD      5
        DB      "CMOVE"
        ALIGN    4
LCMOVE:
        DD    LCMOVE+HEADSIZE
        DD    LCMOVE+HEADSIZE
        DD    0x0
        DD    CR
        DD    N_LCMOVE
        DD    0

        MOV     R8,R11   ;save 
        LDR     R2, [R13],#+(CW*(1))      ;count
        LDR     R1, [R13],#+(CW*(1))      ;dest
        LDR     R3, [R13],#+(CW*(1))      ;source
        CMP R2, #0
        BEQ MOVRP2
MOVREP: LDRB    R0, [R3],#+1
        STRB    R0, [R1],#+1
        SUBS    R2,R2, #1
        BNE     MOVREP
MOVRP2: MOV     R11,R8   ;get back 
        LDR     R0, [R11],#+(CW*(1))
         LDR     R1,[R0]
         BX      R1  
;

;  ************
;  *   MOVE   *
;  ************
;
        ALIGN    4
N_LMOVE:
        DD      4
        DB      "MOVE"
        ALIGN    4
LMOVE:
        DD    LMOVE+HEADSIZE
        DD    LMOVE+HEADSIZE
        DD    0x0
        DD    LCMOVE
        DD    N_LMOVE
        DD    0

        LDR     R2, [R13],#+(CW*(1))      ;count,  R2
        LDR     R1, [R13],#+(CW*(1))      ;dest,   R1
        LDR     R3, [R13],#+(CW*(1))      ;source, R3
        ANDS    R3,R3,R3   ; test R3,R1,R2 for 0 
        BEQ     MOVE3
        ANDS    R1,R1,R1
        BEQ     MOVE3
        ANDS    R2,R2,R2
        BEQ     MOVE3
        CMP     R3,R1
        BHI     MOVE2
        ADD     R1,R1,R2
        ADD     R3,R3,R2
        SUB     R1,R1, #1
        SUB     R3,R3, #1
MOVE1:
        LDRB    R0, [R3],#-1
        STRB    R0, [R1],#-1
        SUBS    R2,R2, #1
        BNE     MOVE1
        BAL     MOVE3
MOVE2:
        LDRB    R0, [R3],#+1
        STRB    R0, [R1],#+1
        SUBS    R2,R2, #1
        BNE     MOVE2
MOVE3:
        LDR     R0, [R11],#+(CW*(1))
         LDR     R1,[R0]
         BX      R1  
;

;  ***********
;  *   UM*   *
;  ***********
;
        ALIGN    4
N_USTAR:
        DD      3
        DB      "UM*"
        ALIGN    4
USTAR:
        DD    USTAR+HEADSIZE
        DD    USTAR+HEADSIZE
        DD    0x0
        DD    LMOVE
        DD    N_USTAR
        DD    0

        LDR     R2, [R13],#+(CW*(1))
        LDR     R3, [R13],#+(CW*(1))
 ; end thumb  )
        UMULL   R1,R0,R3,R2 ;UNSIGNED  (R1,R0) = (R3*R2)  
        STR     R1, [R13,#-(CW*(1))]!
        STR     R0, [R13,#-(CW*(1))]!
        LDR     R0, [R11],#+(CW*(1))
         LDR     R1,[R0]
         BX      R1            ;STORE DOUBLE CELL
;

;  **************
;  *   UM/MOD   *
;  **************
;
        ALIGN    4
N_USLAS:
        DD      6
        DB      "UM/MOD"
        ALIGN    4
USLAS:
        DD    USLAS+HEADSIZE
        DD    USLAS+HEADSIZE
        DD    0x0
        DD    USTAR
        DD    N_USLAS
        DD    0

        LDR     R2, [R13],#+(CW*(1))      ;DIVISOR
        LDR     R1, [R13],#+(CW*(1))     ;MSW OF DIVIDEND
        LDR     R0, [R13],#+(CW*(1))      ;LSW OF DIVIDEND
        BL UMDIV
        STR     R1, [R13,#-(CW*(1))]!    ;remainder
        STR     R4, [R13,#-(CW*(1))]!     ;quotient
        LDR     R0, [R11],#+(CW*(1))
         LDR     R1,[R0]
         BX      R1  

_COMMENTED(
_BITS_32{(  division of unsigned-64/unsigned-32})
_BITS_64{(  division of unsigned-128/unsigned-64})
  input:
  div  = R2  = divisor
  modh = R1 = MSW of dividend
  modl = R0  = LSW of dividend
  output:
  quot = R0  = quotient
  rem = R1 = remainder

  Internally R3 is used as a mask, with 1 bit set.
  mask = R3

) 
UMDIV:  STR     LR, [R13,#-(CW*(1))]!
        MOV R3, #1
        LSL R3,R3, #(CW*8-1)  ; init mask with highest bit set
        MOV R4, #0              ; init quot
        CMP R1, R2             ; test modh - div
        BLO UMDIV1              ; modh < div
        ; overflow condition (divide by zero) - show max numbers
        ASR  R4, R3, #(CW*8-1) ; all bits set
        MOV  R1, R4

; 
        BAL  UMDIV3             ; return

UMDIV1:
        ADDS R0,R0, R0         ;   double precision shift (modh, modl)
        ADCS R1,R1, R1      ;   ADD with carry and set flags again !
        BCS  UMDIV4
        CMP  R2, R1            ; test div - modh
        BHI  UMDIV2             ; div >  modh ?
UMDIV4:
        ADD  R4,R4, R3         ; add single pecision mask
        SUB  R1,R1, R2       ; subtract single precision div
UMDIV2:
        LSR R3,R3, #1          ; shift mask one bit to the right
        ANDS  R3,R3,R3
        BNE  UMDIV1
UMDIV3:
        ; now R0 and R1 contain quotient and remainder
                        LDR     R2, [R13],#+(CW*(1))
        BX R2
; 

;

;  ***********
;  *   AND   *
;  ***********
;
        ALIGN    4
N_LAND:
        DD      3
        DB      "AND"
        ALIGN    4
LAND:
        DD    LAND+HEADSIZE
        DD    LAND+HEADSIZE
        DD    0x0
        DD    USLAS
        DD    N_LAND
        DD    0

        LDR     R0, [R13],#+(CW*(1))
        LDR     R3, [R13],#+(CW*(1))
        AND     R0,R0,R3
        STR     R0, [R13,#-(CW*(1))]!
        LDR     R0, [R11],#+(CW*(1))
         LDR     R1,[R0]
         BX      R1  
;

;  **********
;  *   OR   *
;  **********
;
        ALIGN    4
N_LOR:
        DD      2
        DB      "OR"
        ALIGN    4
LOR:
        DD    LOR+HEADSIZE
        DD    LOR+HEADSIZE
        DD    0x0
        DD    LAND
        DD    N_LOR
        DD    0

        LDR     R0, [R13],#+(CW*(1))      ; (S1) <- (S1) OR (S2)
        LDR     R3, [R13],#+(CW*(1))
        ORR      R0,R0,R3
        STR     R0, [R13,#-(CW*(1))]!
        LDR     R0, [R11],#+(CW*(1))
         LDR     R1,[R0]
         BX      R1  
;

;  ***********
;  *   XOR   *
;  ***********
;
        ALIGN    4
N_LXOR:
        DD      3
        DB      "XOR"
        ALIGN    4
LXOR:
        DD    LXOR+HEADSIZE
        DD    LXOR+HEADSIZE
        DD    0x0
        DD    LOR
        DD    N_LXOR
        DD    0

        LDR     R0, [R13],#+(CW*(1))      ; (S1) <- (S1) XOR (S2)
        LDR     R3, [R13],#+(CW*(1))
        EOR     R0,R0,R3
        STR     R0, [R13,#-(CW*(1))]!
        LDR     R0, [R11],#+(CW*(1))
         LDR     R1,[R0]
         BX      R1  
;

;  **************
;  *   INVERT   *
;  **************
;
        ALIGN    4
N_INVERT:
        DD      6
        DB      "INVERT"
        ALIGN    4
INVERT:
        DD    INVERT+HEADSIZE
        DD    INVERT+HEADSIZE
        DD    0x0
        DD    LXOR
        DD    N_INVERT
        DD    0

        LDR     R0, [R13],#+(CW*(1))      ; (S1) <- (S1) XOR (S2)
        MOV R1,#0
        SUB R1,R1,#1
        EOR R0,R0,R1
        STR     R0, [R13,#-(CW*(1))]!
        LDR     R0, [R11],#+(CW*(1))
         LDR     R1,[R0]
         BX      R1  
;

;  ************
;  *   DSP@   *
;  ************
;
        ALIGN    4
N_SPFET:
        DD      4
        DB      "DSP@"
        ALIGN    4
SPFET:
        DD    SPFET+HEADSIZE
        DD    SPFET+HEADSIZE
        DD    0x0
        DD    INVERT
        DD    N_SPFET
        DD    0

        MOV     R0,R13   ; (S1) <- (SP)
        STR     R0, [R13,#-(CW*(1))]!
        LDR     R0, [R11],#+(CW*(1))
         LDR     R1,[R0]
         BX      R1  
;

;  ************
;  *   DSP!   *
;  ************
;
        ALIGN    4
N_SPSTO:
        DD      4
        DB      "DSP!"
        ALIGN    4
SPSTO:
        DD    SPSTO+HEADSIZE
        DD    SPSTO+HEADSIZE
        DD    0x0
        DD    SPFET
        DD    N_SPSTO
        DD    0

        LDR     R0, [R13],#+(CW*(1))
        MOV     R13,R0        ;RESET PARAM STACK POINTER
        LDR     R0, [R11],#+(CW*(1))
         LDR     R1,[R0]
         BX      R1  
;

;  *************
;  *   DEPTH   *
;  *************
;
        ALIGN    4
N_DEPTH:
        DD      5
        DB      "DEPTH"
        ALIGN    4
DEPTH:
        DD    DOCOL
        DD    DEPTH+HEADSIZE
        DD    0x0
        DD    SPSTO
        DD    N_DEPTH
        DD    0

        DD      SZERO, FETCH
        DD      SPFET
        DD      LSUB
        DD      LIT, CW, SLASH
        DD      ONEM
        DD      SEMIS
;

;  ***********
;  *   CLS   *
;  ***********
;
        ALIGN    4
N_CLS:
        DD      3
        DB      "CLS"
        ALIGN    4
CLS:
        DD    DOCOL
        DD    CLS+HEADSIZE
        DD    0x0
        DD    DEPTH
        DD    N_CLS
        DD    0

        DD      SZERO, FETCH
        DD      SPSTO
        DD      SEMIS
;

;  ************
;  *   RSP@   *
;  ************
;
        ALIGN    4
N_RPFET:
        DD      4
        DB      "RSP@"
        ALIGN    4
RPFET:
        DD    RPFET+HEADSIZE
        DD    RPFET+HEADSIZE
        DD    0x0
        DD    CLS
        DD    N_RPFET
        DD    0
      ;(S1) <- (RP)
        STR     R12, [R13,#-(CW*(1))]!
        LDR     R0, [R11],#+(CW*(1))
         LDR     R1,[R0]
         BX      R1  
;

;  ************
;  *   RSP!   *
;  ************
;
        ALIGN    4
N_RPSTO:
        DD      4
        DB      "RSP!"
        ALIGN    4
RPSTO:
        DD    RPSTO+HEADSIZE
        DD    RPSTO+HEADSIZE
        DD    0x0
        DD    RPFET
        DD    N_RPSTO
        DD    0

        LDR     R12, [R13],#+(CW*(1))
        LDR     R0, [R11],#+(CW*(1))
         LDR     R1,[R0]
         BX      R1  
;

;  ************
;  *   EXIT   *
;  ************
;
        ALIGN    4
N_EXIT:
        DD      4
        DB      "EXIT"
        ALIGN    4
EXIT:
        DD    EXIT+HEADSIZE
        DD    EXIT+HEADSIZE
        DD    0x0
        DD    RPSTO
        DD    N_EXIT
        DD    0

        LDR     R11, [R12],#+(CW*(1))  ;(IP) <- (RP1)
        LDR     R0, [R11],#+(CW*(1))
         LDR     R1,[R0]
         BX      R1  
;

;  **********
;  *   CO   *
;  **********
;
        ALIGN    4
N_CO:
        DD      2
        DB      "CO"
        ALIGN    4
CO:
        DD    CO+HEADSIZE
        DD    CO+HEADSIZE
        DD    0x0
        DD    EXIT
        DD    N_CO
        DD    0

        LDR     R1,[R12]
        STR     R11, [R12]
        MOV     R11, R1
        LDR     R0, [R11],#+(CW*(1))
         LDR     R1,[R0]
         BX      R1  
;

;  ***********
;  *   (;)   *
;  ***********
;
        ALIGN    4
N_SEMIS:
        DD      3
        DB      "(;)"
        ALIGN    4
SEMIS:
        DD    (EXIT+HEADSIZE)
        DD    SEMIS+HEADSIZE
        DD    0x0
        DD    CO
        DD    N_SEMIS
        DD    0

;

;  *************
;  *   LEAVE   *
;  *************
;
        ALIGN    4
N_LLEAV:
        DD      5
        DB      "LEAVE"
        ALIGN    4
LLEAV:
        DD    DOCOL
        DD    LLEAV+HEADSIZE
        DD    0x0
        DD    SEMIS
        DD    N_LLEAV
        DD    0
  ;LIMIT <- INDEX
        DD      UNLOOP
        DD      SEMIS
;

;  **********
;  *   >R   *
;  **********
;
        ALIGN    4
N_TOR:
        DD      2
        DB      ">R"
        ALIGN    4
TOR:
        DD    TOR+HEADSIZE
        DD    TOR+HEADSIZE
        DD    0x0
        DD    LLEAV
        DD    N_TOR
        DD    0
        ; (RP1) <- (S1)
        LDR     R3, [R13],#+(CW*(1))      ;GET STACK PARAMETER
        SUB     R12, R12, #(CW*(1))  ;MOVE RETURN STACK DOWN
        STR     R3, [R12]            ;ADD TO RETURN STACK
        LDR     R0, [R11],#+(CW*(1))
         LDR     R1,[R0]
         BX      R1  
;

;  **********
;  *   R>   *
;  **********
;
        ALIGN    4
N_FROMR:
        DD      2
        DB      "R>"
        ALIGN    4
FROMR:
        DD    FROMR+HEADSIZE
        DD    FROMR+HEADSIZE
        DD    0x0
        DD    TOR
        DD    N_FROMR
        DD    0
      ;(S1) <- (RP1)
        LDR     R0,[R12] ; GET RETURN STACK VALUE
        ADD     R12,R12,# (CW*(1))
        STR     R0, [R13,#-(CW*(1))]!
        LDR     R0, [R11],#+(CW*(1))
         LDR     R1,[R0]
         BX      R1  
;

;  *************
;  *   RDROP   *
;  *************
;
        ALIGN    4
N_RDROP:
        DD      5
        DB      "RDROP"
        ALIGN    4
RDROP:
        DD    RDROP+HEADSIZE
        DD    RDROP+HEADSIZE
        DD    0x0
        DD    FROMR
        DD    N_RDROP
        DD    0
      ;(S1) <- (RP1)
        ADD R12,R12,#(CW*(1))
        LDR     R0, [R11],#+(CW*(1))
         LDR     R1,[R0]
         BX      R1  
;

;  **********
;  *   R@   *
;  **********
;
        ALIGN    4
N_RR:
        DD      2
        DB      "R@"
        ALIGN    4
RR:
        DD    (IDO+HEADSIZE)
        DD    RR+HEADSIZE
        DD    0x0
        DD    RDROP
        DD    N_RR
        DD    0

;

;  **********
;  *   0=   *
;  **********
;
        ALIGN    4
N_ZEQU:
        DD      2
        DB      "0="
        ALIGN    4
ZEQU:
        DD    ZEQU+HEADSIZE
        DD    ZEQU+HEADSIZE
        DD    0x0
        DD    RR
        DD    N_ZEQU
        DD    0

        LDR     R2, [R13],#+(CW*(1))
        MOV    R0, #0   ;FALSE
        CMP     R2, #0
        BNE ZEQ1
        SUB     R0,R0, #1          ;
ZEQ1:   STR     R0, [R13,#-(CW*(1))]!
        LDR     R0, [R11],#+(CW*(1))
         LDR     R1,[R0]
         BX      R1  
;

;  **********
;  *   0<   *
;  **********
;
        ALIGN    4
N_ZLESS:
        DD      2
        DB      "0<"
        ALIGN    4
ZLESS:
        DD    ZLESS+HEADSIZE
        DD    ZLESS+HEADSIZE
        DD    0x0
        DD    ZEQU
        DD    N_ZLESS
        DD    0

        LDR     R1, [R13],#+(CW*(1))
        MOV    R0,#0    ;FALSE
        ANDS    R1,R1,R1   ;SET FLAGS
        BPL     ZLESS1
        SUB     R0,R0, #1      ;TRUE
ZLESS1: STR     R0, [R13,#-(CW*(1))]!
        LDR     R0, [R11],#+(CW*(1))
         LDR     R1,[R0]
         BX      R1  
;

;  *********
;  *   +   *
;  *********
;
        ALIGN    4
N_PLUS:
        DD      1
        DB      "+"
        ALIGN    4
PLUS:
        DD    PLUS+HEADSIZE
        DD    PLUS+HEADSIZE
        DD    0x0
        DD    ZLESS
        DD    N_PLUS
        DD    0

        LDR     R0, [R13],#+(CW*(1))      ;(S1) <- (S1) + (S2)
        LDR     R3, [R13],#+(CW*(1))
        ADD     R0,R0,R3
        STR     R0, [R13,#-(CW*(1))]!
        LDR     R0, [R11],#+(CW*(1))
         LDR     R1,[R0]
         BX      R1  
;

;  **********
;  *   D+   *
;  **********
;
        ALIGN    4
N_DPLUS:
        DD      2
        DB      "D+"
        ALIGN    4
DPLUS:
        DD    DPLUS+HEADSIZE
        DD    DPLUS+HEADSIZE
        DD    0x0
        DD    PLUS
        DD    N_DPLUS
        DD    0

        LDR     R0, [R13],#+(CW*(1))      ; YHC
        LDR     R1, [R13],#+(CW*(1))      ; YLC
        LDR     R3, [R13],#+(CW*(1))      ; XHC
        LDR     R2, [R13],#+(CW*(1))      ; XLC
        ADDS    R1,R1,R2
        ADCS    R0,R0,R3   ; SLC
        STR     R1, [R13,#-(CW*(1))]!
        STR     R0, [R13,#-(CW*(1))]!
        LDR     R0, [R11],#+(CW*(1))
         LDR     R1,[R0]
         BX      R1  
;

;  **************
;  *   NEGATE   *
;  **************
;
        ALIGN    4
N_NEGATE:
        DD      6
        DB      "NEGATE"
        ALIGN    4
NEGATE:
        DD    NEGATE+HEADSIZE
        DD    NEGATE+HEADSIZE
        DD    0x0
        DD    DPLUS
        DD    N_NEGATE
        DD    0

        LDR     R0, [R13],#+(CW*(1))
        MVN R0, R0  ; bitwise NOT 
        ADD R0,R0,#1
        STR     R0, [R13,#-(CW*(1))]!
        LDR     R0, [R11],#+(CW*(1))
         LDR     R1,[R0]
         BX      R1  
;

;  ***************
;  *   DNEGATE   *
;  ***************
;
        ALIGN    4
N_DNEGA:
        DD      7
        DB      "DNEGATE"
        ALIGN    4
DNEGA:
        DD    DNEGA+HEADSIZE
        DD    DNEGA+HEADSIZE
        DD    0x0
        DD    NEGATE
        DD    N_DNEGA
        DD    0

        LDR     R0, [R13],#+(CW*(1))   ; MSW
        LDR     R1, [R13],#+(CW*(1))   ; LSW
        MVN R0, R0  ; invert bits 
        MVN R1, R1
        
        ADDS R1, R1, #1  ; make 2's complement 
        ADC R0, R0, #0  ; add possible carry    
        STR     R1, [R13,#-(CW*(1))]!
        STR     R0, [R13,#-(CW*(1))]!
        LDR     R0, [R11],#+(CW*(1))
         LDR     R1,[R0]
         BX      R1  
;
        ;
;

;  ************
;  *   OVER   *
;  ************
;
        ALIGN    4
N_OVER:
        DD      4
        DB      "OVER"
        ALIGN    4
OVER:
        DD    OVER+HEADSIZE
        DD    OVER+HEADSIZE
        DD    0x0
        DD    DNEGA
        DD    N_OVER
        DD    0

        LDR     R1, [R13],#+(CW*(1))
        LDR     R0, [R13],#+(CW*(1))
        STR     R0, [R13,#-(CW*(1))]!
        STR     R1, [R13,#-(CW*(1))]!
        STR     R0, [R13,#-(CW*(1))]!
        LDR     R0, [R11],#+(CW*(1))
         LDR     R1,[R0]
         BX      R1  
;

;  ************
;  *   DROP   *
;  ************
;
        ALIGN    4
N_DROP:
        DD      4
        DB      "DROP"
        ALIGN    4
DROP:
        DD    DROP+HEADSIZE
        DD    DROP+HEADSIZE
        DD    0x0
        DD    OVER
        DD    N_DROP
        DD    0

        LDR     R0, [R13],#+(CW*(1))
        LDR     R0, [R11],#+(CW*(1))
         LDR     R1,[R0]
         BX      R1  
;

;  ***********
;  *   NIP   *
;  ***********
;
        ALIGN    4
N_NIP:
        DD      3
        DB      "NIP"
        ALIGN    4
NIP:
        DD    NIP+HEADSIZE
        DD    NIP+HEADSIZE
        DD    0x0
        DD    DROP
        DD    N_NIP
        DD    0

        LDR     R0, [R13],#+(CW*(1))
        LDR     R3, [R13],#+(CW*(1))
        STR     R0, [R13,#-(CW*(1))]!
        LDR     R0, [R11],#+(CW*(1))
         LDR     R1,[R0]
         BX      R1  
;

;  *************
;  *   2DROP   *
;  *************
;
        ALIGN    4
N_TDROP:
        DD      5
        DB      "2DROP"
        ALIGN    4
TDROP:
        DD    TDROP+HEADSIZE
        DD    TDROP+HEADSIZE
        DD    0x0
        DD    NIP
        DD    N_TDROP
        DD    0

        LDR     R0, [R13],#+(CW*(1))
        LDR     R0, [R13],#+(CW*(1))
        LDR     R0, [R11],#+(CW*(1))
         LDR     R1,[R0]
         BX      R1  
;

;  ************
;  *   SWAP   *
;  ************
;
        ALIGN    4
N_SWAP:
        DD      4
        DB      "SWAP"
        ALIGN    4
SWAP:
        DD    SWAP+HEADSIZE
        DD    SWAP+HEADSIZE
        DD    0x0
        DD    TDROP
        DD    N_SWAP
        DD    0

        LDR     R1, [R13],#+(CW*(1))
        LDR     R0, [R13],#+(CW*(1))
        STR     R1, [R13,#-(CW*(1))]!
        STR     R0, [R13,#-(CW*(1))]!
        LDR     R0, [R11],#+(CW*(1))
         LDR     R1,[R0]
         BX      R1  
;

;  ***********
;  *   DUP   *
;  ***********
;
        ALIGN    4
N_LDUP:
        DD      3
        DB      "DUP"
        ALIGN    4
LDUP:
        DD    LDUP+HEADSIZE
        DD    LDUP+HEADSIZE
        DD    0x0
        DD    SWAP
        DD    N_LDUP
        DD    0

        LDR     R0, [R13],#+(CW*(1))
        STR     R0, [R13,#-(CW*(1))]!
        STR     R0, [R13,#-(CW*(1))]!
        LDR     R0, [R11],#+(CW*(1))
         LDR     R1,[R0]
         BX      R1  
;

;  ************
;  *   2DUP   *
;  ************
;
        ALIGN    4
N_TDUP:
        DD      4
        DB      "2DUP"
        ALIGN    4
TDUP:
        DD    TDUP+HEADSIZE
        DD    TDUP+HEADSIZE
        DD    0x0
        DD    LDUP
        DD    N_TDUP
        DD    0

        LDR     R0, [R13],#+(CW*(1))
        LDR     R1, [R13],#+(CW*(1))
        STR     R1, [R13,#-(CW*(1))]!
        STR     R0, [R13,#-(CW*(1))]!
        STR     R1, [R13,#-(CW*(1))]!
        STR     R0, [R13,#-(CW*(1))]!
        LDR     R0, [R11],#+(CW*(1))
         LDR     R1,[R0]
         BX      R1  
;

;  *************
;  *   2SWAP   *
;  *************
;
        ALIGN    4
N_TSWAP:
        DD      5
        DB      "2SWAP"
        ALIGN    4
TSWAP:
        DD    TSWAP+HEADSIZE
        DD    TSWAP+HEADSIZE
        DD    0x0
        DD    TDUP
        DD    N_TSWAP
        DD    0

        LDR     R3, [R13],#+(CW*(1))
        LDR     R2, [R13],#+(CW*(1))
        LDR     R0, [R13],#+(CW*(1))
        LDR     R1, [R13],#+(CW*(1))
        STR     R2, [R13,#-(CW*(1))]!
        STR     R3, [R13,#-(CW*(1))]!
        STR     R1, [R13,#-(CW*(1))]!
        STR     R0, [R13,#-(CW*(1))]!
        LDR     R0, [R11],#+(CW*(1))
         LDR     R1,[R0]
         BX      R1  
;

;  *************
;  *   2OVER   *
;  *************
;
        ALIGN    4
N_TOVER:
        DD      5
        DB      "2OVER"
        ALIGN    4
TOVER:
        DD    TOVER+HEADSIZE
        DD    TOVER+HEADSIZE
        DD    0x0
        DD    TSWAP
        DD    N_TOVER
        DD    0

        LDR     R3, [R13],#+(CW*(1))
        LDR     R2, [R13],#+(CW*(1))
        LDR     R0, [R13],#+(CW*(1))
        LDR     R1, [R13],#+(CW*(1))
        STR     R1, [R13,#-(CW*(1))]!
        STR     R0, [R13,#-(CW*(1))]!
        STR     R2, [R13,#-(CW*(1))]!
        STR     R3, [R13,#-(CW*(1))]!
        STR     R1, [R13,#-(CW*(1))]!
        STR     R0, [R13,#-(CW*(1))]!
        LDR     R0, [R11],#+(CW*(1))
         LDR     R1,[R0]
         BX      R1  
;

;  **********
;  *   +!   *
;  **********
;
        ALIGN    4
N_PSTOR:
        DD      2
        DB      "+!"
        ALIGN    4
PSTOR:
        DD    PSTOR+HEADSIZE
        DD    PSTOR+HEADSIZE
        DD    0x0
        DD    TOVER
        DD    N_PSTOR
        DD    0

        LDR     R3, [R13],#+(CW*(1))      ;ADDRESS
        LDR     R0, [R13],#+(CW*(1))      ;INCREMENT
        LDR     R2, [R3]
        ADD     R2,R2, R0
        STR     R2, [R3]
        LDR     R0, [R11],#+(CW*(1))
         LDR     R1,[R0]
         BX      R1  
;

;  **************
;  *   TOGGLE   *
;  **************
;
        ALIGN    4
N_TOGGL:
        DD      6
        DB      "TOGGLE"
        ALIGN    4
TOGGL:
        DD    TOGGL+HEADSIZE
        DD    TOGGL+HEADSIZE
        DD    0x0
        DD    PSTOR
        DD    N_TOGGL
        DD    0

        LDR     R0, [R13],#+(CW*(1))      ;BIT PATTERN
        LDR     R3, [R13],#+(CW*(1))      ;ADDR
        LDR     R1, [R3]
        EOR     R1,R1, R0
        STR     R1, [R3]
        LDR     R0, [R11],#+(CW*(1))
         LDR     R1,[R0]
         BX      R1  
;

;  *********
;  *   @   *
;  *********
;
        ALIGN    4
N_FETCH:
        DD      1
        DB      "@"
        ALIGN    4
FETCH:
        DD    FETCH+HEADSIZE
        DD    FETCH+HEADSIZE
        DD    0x0
        DD    TOGGL
        DD    N_FETCH
        DD    0

        LDR     R3, [R13],#+(CW*(1))
        LDR     R0,[R3]
        STR     R0, [R13,#-(CW*(1))]!
        LDR     R0, [R11],#+(CW*(1))
         LDR     R1,[R0]
         BX      R1  
;

;  **********
;  *   P@   *
;  **********
;
        ALIGN    4
N_PFETCH:
        DD      2
        DB      "P@"
        ALIGN    4
PFETCH:
        DD    PFETCH+HEADSIZE
        DD    PFETCH+HEADSIZE
        DD    0x0
        DD    FETCH
        DD    N_PFETCH
        DD    0

        LDR     R3, [R13],#+(CW*(1))
        LDR     R0,[R3]
        STR     R0, [R13,#-(CW*(1))]!
        LDR     R0, [R11],#+(CW*(1))
         LDR     R1,[R0]
         BX      R1  
;
;


;  ***********
;  *   BIC   *
;  ***********
;
        ALIGN    4
N_BITCLR:
        DD      3
        DB      "BIC"
        ALIGN    4
BITCLR:
        DD    BITCLR+HEADSIZE
        DD    BITCLR+HEADSIZE
        DD    0x0
        DD    PFETCH
        DD    N_BITCLR
        DD    0

        LDR     R3, [R13],#+(CW*(1))
        LDR     R0, [R13],#+(CW*(1))
        LDR     R2,[R3]
        BIC     R2,R0
        STR     R2,[R3]
        LDR     R0, [R11],#+(CW*(1))
         LDR     R1,[R0]
         BX      R1  
;

;  ***********
;  *   BIS   *
;  ***********
;
        ALIGN    4
N_BITSET:
        DD      3
        DB      "BIS"
        ALIGN    4
BITSET:
        DD    BITSET+HEADSIZE
        DD    BITSET+HEADSIZE
        DD    0x0
        DD    BITCLR
        DD    N_BITSET
        DD    0

        LDR     R3, [R13],#+(CW*(1))
        LDR     R0, [R13],#+(CW*(1))
        LDR     R2,[R3]
        ORR     R2,R0
        STR     R2,[R3]
        LDR     R0, [R11],#+(CW*(1))
         LDR     R1,[R0]
         BX      R1  
;
;
;

;  **********
;  *   C@   *
;  **********
;
        ALIGN    4
N_CFET:
        DD      2
        DB      "C@"
        ALIGN    4
CFET:
        DD    CFET+HEADSIZE
        DD    CFET+HEADSIZE
        DD    0x0
        DD    BITSET
        DD    N_CFET
        DD    0

        LDR     R3, [R13],#+(CW*(1))
        LDRB    R0,[R3]
        
         AND     R0,R0, #0xFF
        STR     R0, [R13,#-(CW*(1))]!
        LDR     R0, [R11],#+(CW*(1))
         LDR     R1,[R0]
         BX      R1  
;

;  **********
;  *   2@   *
;  **********
;
        ALIGN    4
N_TFET:
        DD      2
        DB      "2@"
        ALIGN    4
TFET:
        DD    TFET+HEADSIZE
        DD    TFET+HEADSIZE
        DD    0x0
        DD    CFET
        DD    N_TFET
        DD    0

        LDR     R3, [R13],#+(CW*(1))      ;ADDR
        LDR     R0,[R3]                      ;MSW
        LDR     R1,[R3,# + (CW*(1))]        ;LSW
        STR     R1, [R13,#-(CW*(1))]!
        STR     R0, [R13,#-(CW*(1))]!
        LDR     R0, [R11],#+(CW*(1))
         LDR     R1,[R0]
         BX      R1  
;

;  *********
;  *   !   *
;  *********
;
        ALIGN    4
N_STOR:
        DD      1
        DB      "!"
        ALIGN    4
STOR:
        DD    STOR+HEADSIZE
        DD    STOR+HEADSIZE
        DD    0x0
        DD    TFET
        DD    N_STOR
        DD    0

        LDR     R3, [R13],#+(CW*(1))      ;ADDR
        LDR     R0, [R13],#+(CW*(1))      ;DATA
        STR     R0, [R3]
        LDR     R0, [R11],#+(CW*(1))
         LDR     R1,[R0]
         BX      R1  
;

;  **********
;  *   P!   *
;  **********
;
        ALIGN    4
N_PRTSTR:
        DD      2
        DB      "P!"
        ALIGN    4
PRTSTR:
        DD    PRTSTR+HEADSIZE
        DD    PRTSTR+HEADSIZE
        DD    0x0
        DD    STOR
        DD    N_PRTSTR
        DD    0

        LDR     R3, [R13],#+(CW*(1))      ;ADDR
        LDR     R0, [R13],#+(CW*(1))      ;DATA
        STR     R0, [R3]
        LDR     R0, [R11],#+(CW*(1))
         LDR     R1,[R0]
         BX      R1  
;
;

;  **********
;  *   C!   *
;  **********
;
        ALIGN    4
N_CSTOR:
        DD      2
        DB      "C!"
        ALIGN    4
CSTOR:
        DD    CSTOR+HEADSIZE
        DD    CSTOR+HEADSIZE
        DD    0x0
        DD    PRTSTR
        DD    N_CSTOR
        DD    0

        LDR     R3, [R13],#+(CW*(1))      ;ADDR
        LDR     R0, [R13],#+(CW*(1))      ;DATA
        STRB         R0, [R3]  ;Store byte
        LDR     R0, [R11],#+(CW*(1))
         LDR     R1,[R0]
         BX      R1  
;

;  **********
;  *   2!   *
;  **********
;
        ALIGN    4
N_TSTOR:
        DD      2
        DB      "2!"
        ALIGN    4
TSTOR:
        DD    TSTOR+HEADSIZE
        DD    TSTOR+HEADSIZE
        DD    0x0
        DD    CSTOR
        DD    N_TSTOR
        DD    0

        LDR     R3, [R13],#+(CW*(1))      ;ADDR
        LDR     R0, [R13],#+(CW*(1))      ;MSW
        STR     R0, [R3]
        LDR     R0, [R13],#+(CW*(1))      ;LSW
        STR     R0, [R3, # +(CW*(1))]
        LDR     R0, [R11],#+(CW*(1))
         LDR     R1,[R0]
         BX      R1  
;

;  **************
;  *   WITHIN   *
;  **************
;
        ALIGN    4
N_WITHIN:
        DD      6
        DB      "WITHIN"
        ALIGN    4
WITHIN:
        DD    DOCOL
        DD    WITHIN+HEADSIZE
        DD    0x0
        DD    TSTOR
        DD    N_WITHIN
        DD    0

        DD      OVER, LSUB, TOR
        DD      LSUB, FROMR
        DD      ULESS
        DD      SEMIS
;
;

;  *********
;  *   :   *
;  *********
;
        ALIGN    4
N_COLON:
        DD      1
        DB      ":"
        ALIGN    4
COLON:
        DD    DOCOL
        DD    COLON+HEADSIZE
        DD    0x0
        DD    WITHIN
        DD    N_COLON
        DD    0

        DD      SCSP
        DD      LNAME
        DD      PCREAT
        DD      LATEST, HIDDEN
        DD      RBRAC
        DD      PSCOD
DOCOL:  STR     R11, [R12,#-(CW*(1))]!
        LDR     R11,[R0, #+(CW*(D_HOFFSET))]  ;(IP) <- (PFA)

; 

        LDR     R0, [R11],#+(CW*(1))
         LDR     R1,[R0]
         BX      R1  


;

;  *********
;  *   ;   *
;  *********
;
        ALIGN    4
N_SEMI:
        DD      1
        DB      ";"
        ALIGN    4
SEMI:
        DD    DOCOL
        DD    SEMI+HEADSIZE
        DD    B_IMMED
        DD    COLON
        DD    N_SEMI
        DD    0

        DD      QCSP
        DD      LIT, SEMIS, COMMA
        DD      LATEST, HIDDEN
        DD      LBRAC
        DD      SEMIS
;

;  ****************
;  *   CONSTANT   *
;  ****************
;
        ALIGN    4
N_LCONST:
        DD      8
        DB      "CONSTANT"
        ALIGN    4
LCONST:
        DD    DOCOL
        DD    LCONST+HEADSIZE
        DD    0x0
        DD    SEMI
        DD    N_LCONST
        DD    0

        DD      LNAME
        DD      PCREAT
        DD      LATEST, TDFA, STOR
        DD      PSCOD
DOCON:  LDR     R0,[R0,#+(CW*((D_HOFFSET)))] ;GET DATA FROM PFA
        STR     R0, [R13,#-(CW*(1))]!
        LDR     R0, [R11],#+(CW*(1))
         LDR     R1,[R0]
         BX      R1  
;

;  ****************
;  *   VARIABLE   *
;  ****************
;
        ALIGN    4
N_VAR:
        DD      8
        DB      "VARIABLE"
        ALIGN    4
VAR:
        DD    DOCOL
        DD    VAR+HEADSIZE
        DD    0x0
        DD    LCONST
        DD    N_VAR
        DD    0

        DD      LDATA
        DD      ZERO, COMMA
        DD      SEMIS
;

;  ************
;  *   DATA   *
;  ************
;
        ALIGN    4
N_LDATA:
        DD      4
        DB      "DATA"
        ALIGN    4
LDATA:
        DD    DOCOL
        DD    LDATA+HEADSIZE
        DD    0x0
        DD    VAR
        DD    N_LDATA
        DD    0

        DD      LNAME
        DD      PCREAT
        DD      PSCOD
DOVAR:  LDR     R0,[R0,#+(CW*((D_HOFFSET)))]  ;(R0) <- PFA
        STR     R0, [R13,#-(CW*(1))]!
        LDR     R0, [R11],#+(CW*(1))
         LDR     R1,[R0]
         BX      R1  
;

;  *********
;  *   _   *
;  *********
;
        ALIGN    4
N_X:
        DD      1
        DB      "_"
        ALIGN    4
X:
        DD    X+HEADSIZE
        DD    X+HEADSIZE
        DD    0x0
        DD    LDATA
        DD    N_X
        DD    0

        STR     R0, [R13,#-(CW*(1))]!
        LDR     R0, [R11],#+(CW*(1))
         LDR     R1,[R0]
         BX      R1   ;Whatever happens to be in R0, i.e. the dea of ``_''.
;

;  *********
;  *   0   *
;  *********
;
        ALIGN    4
N_ZERO:
        DD      1
        DB      "0"
        ALIGN    4
ZERO:
        DD    DOCON
        DD    0
        DD    0x0
        DD    X
        DD    N_ZERO
        DD    0

; The data field of 0 is used for an empty string.
;

;  *********
;  *   1   *
;  *********
;
        ALIGN    4
N_ONE:
        DD      1
        DB      "1"
        ALIGN    4
ONE:
        DD    DOCON
        DD    1
        DD    0x0
        DD    ZERO
        DD    N_ONE
        DD    0

;

;  *********
;  *   2   *
;  *********
;
        ALIGN    4
N_TWO:
        DD      1
        DB      "2"
        ALIGN    4
TWO:
        DD    DOCON
        DD    2
        DD    0x0
        DD    ONE
        DD    N_TWO
        DD    0

;

;  **********
;  *   BL   *
;  **********
;
        ALIGN    4
N_LBL:
        DD      2
        DB      "BL"
        ALIGN    4
LBL:
        DD    DOCON
        DD    ABL
        DD    0x0
        DD    TWO
        DD    N_LBL
        DD    0

;

;
;  **********
;  *   $@   *
;  **********
;
        ALIGN    4
N_SFET:
        DD      2
        DB      "$@"
        ALIGN    4
SFET:
        DD    SFET+HEADSIZE
        DD    SFET+HEADSIZE
        DD    0x0
        DD    LBL
        DD    N_SFET
        DD    0

        LDR     R3, [R13],#+(CW*(1))
        LDR     R0, [R3],#+(CW*(1))
        MOV   R1,R3
        STR     R1, [R13,#-(CW*(1))]!
        STR     R0, [R13,#-(CW*(1))]!
        LDR     R0, [R11],#+(CW*(1))
         LDR     R1,[R0]
         BX      R1  
;

;  **********
;  *   $!   *
;  **********
;
        ALIGN    4
N_SSTOR:
        DD      2
        DB      "$!"
        ALIGN    4
SSTOR:
        DD    DOCOL
        DD    SSTOR+HEADSIZE
        DD    0x0
        DD    SFET
        DD    N_SSTOR
        DD    0

        DD TDUP, STOR, CELLP, SWAP, LMOVE
        DD SEMIS
;

; 

;  ***********
;  *   $+!   *
;  ***********
;
        ALIGN    4
N_SADD:
        DD      3
        DB      "$+!"
        ALIGN    4
SADD:
        DD    DOCOL
        DD    SADD+HEADSIZE
        DD    0x0
        DD    SSTOR
        DD    N_SADD
        DD    0

        DD   LDUP, FETCH, TOR ; Remember old count.
        DD   TDUP, PSTOR
        DD   CELLP, FROMR, PLUS, SWAP, LMOVE
        DD SEMIS
;

;  ***********
;  *   $C+   *
;  ***********
;
        ALIGN    4
N_CHAPP:
        DD      3
        DB      "$C+"
        ALIGN    4
CHAPP:
        DD    DOCOL
        DD    CHAPP+HEADSIZE
        DD    0x0
        DD    SADD
        DD    N_CHAPP
        DD    0

        DD   LDUP, TOR
        DD   LDUP, FETCH, PLUS, CELLP, CSTOR
        DD   ONE, FROMR, PSTOR
        DD SEMIS
;

;  **********
;  *   $,   *
;  **********
;
        ALIGN    4
N_SCOMMA:
        DD      2
        DB      "$,"
        ALIGN    4
SCOMMA:
        DD    DOCOL
        DD    SCOMMA+HEADSIZE
        DD    0x0
        DD    CHAPP
        DD    N_SCOMMA
        DD    0

        DD HERE, TOR
        DD LDUP, CELLP, ALLOT
        DD RR, SSTOR, FROMR
 DD LALIGN
        DD SEMIS
;

;  ***********
;  *   C/L   *
;  ***********
;
        ALIGN    4
N_CSLL:
        DD      3
        DB      "C/L"
        ALIGN    4
CSLL:
        DD    DOCON
        DD    64
        DD    0x0
        DD    SCOMMA
        DD    N_CSLL
        DD    0

;


;  **************
;  *   _FIRST   *
;  **************
;
        ALIGN    4
N__FIRST:
        DD      6
        DB      "_FIRST"
        ALIGN    4
_FIRST:
        DD    DOCON
        DD    BUF1
        DD    0x0
        DD    CSLL
        DD    N__FIRST
        DD    0

 ;  
;
; 
;

;  **************
;  *   _LIMIT   *
;  **************
;
        ALIGN    4
N__LIMIT:
        DD      6
        DB      "_LIMIT"
        ALIGN    4
_LIMIT:
        DD    DOCON
        DD    BUF1+(KBBUF+2*CW)*NBUF
        DD    0x0
        DD    _FIRST
        DD    N__LIMIT
        DD    0

; THE END  OF THE MEMORY 

;  **********
;  *   EM   *
;  **********
;
        ALIGN    4
N_LEM:
        DD      2
        DB      "EM"
        ALIGN    4
LEM:
        DD    DOCON
        DD    ACTUAL_EM
        DD    0x0
        DD    _LIMIT
        DD    N_LEM
        DD    0

;

;  **********
;  *   BM   *
;  **********
;
        ALIGN    4
N_LBM:
        DD      2
        DB      "BM"
        ALIGN    4
LBM:
        DD    DOCON
        DD    _ORIG
        DD    0x0
        DD    LEM
        DD    N_LBM
        DD    0

;

;  *************
;  *   B/BUF   *
;  *************
;
        ALIGN    4
N_BBUF:
        DD      5
        DB      "B/BUF"
        ALIGN    4
BBUF:
        DD    DOCON
        DD    KBBUF
        DD    0x0
        DD    LBM
        DD    N_BBUF
        DD    0

;
; All user variables are initialised 
; with the values from USINI.
; The implementation relies on the initialisation of 
; those with numbers (1..11), so change in concord with USINI.

;  **********
;  *   U0   *
;  **********
;
        ALIGN    4
N_UZERO:
        DD      2
        DB      "U0"
        ALIGN    4
UZERO:
        DD    DOUSE
        DD    (CW*(1))
        DD    0x0
        DD    BBUF
        DD    N_UZERO
        DD    0

;

;  **********
;  *   S0   *
;  **********
;
        ALIGN    4
N_SZERO:
        DD      2
        DB      "S0"
        ALIGN    4
SZERO:
        DD    DOUSE
        DD    (CW*(2))
        DD    0x0
        DD    UZERO
        DD    N_SZERO
        DD    0

;

;  **********
;  *   R0   *
;  **********
;
        ALIGN    4
N_RZERO:
        DD      2
        DB      "R0"
        ALIGN    4
RZERO:
        DD    DOUSE
        DD    (CW*(3))
        DD    0x0
        DD    SZERO
        DD    N_RZERO
        DD    0

;

;  ***********
;  *   TIB   *
;  ***********
;
        ALIGN    4
N_TIB:
        DD      3
        DB      "TIB"
        ALIGN    4
TIB:
        DD    DOUSE
        DD    (CW*(4))
        DD    0x0
        DD    RZERO
        DD    N_TIB
        DD    0

;

;  **************
;  *   RUBOUT   *
;  **************
;
        ALIGN    4
N_RUBOUT:
        DD      6
        DB      "RUBOUT"
        ALIGN    4
RUBOUT:
        DD    DOUSE
        DD    (CW*(5))
        DD    0x0
        DD    TIB
        DD    N_RUBOUT
        DD    0

;

;  ***************
;  *   WARNING   *
;  ***************
;
        ALIGN    4
N_LWARN:
        DD      7
        DB      "WARNING"
        ALIGN    4
LWARN:
        DD    DOUSE
        DD    (CW*(7))
        DD    0x0
        DD    RUBOUT
        DD    N_LWARN
        DD    0

;

;  *************
;  *   FENCE   *
;  *************
;
        ALIGN    4
N_FENCE:
        DD      5
        DB      "FENCE"
        ALIGN    4
FENCE:
        DD    DOUSE
        DD    (CW*(8))
        DD    0x0
        DD    LWARN
        DD    N_FENCE
        DD    0

;

;  **********
;  *   DP   *
;  **********
;
        ALIGN    4
N_LDP:
        DD      2
        DB      "DP"
        ALIGN    4
LDP:
        DD    DOUSE
        DD    (CW*(9))
        DD    0x0
        DD    FENCE
        DD    N_LDP
        DD    0

;

;  ****************
;  *   VOC-LINK   *
;  ****************
;
        ALIGN    4
N_VOCL:
        DD      8
        DB      "VOC-LINK"
        ALIGN    4
VOCL:
        DD    DOUSE
        DD    (CW*(10))
        DD    0x0
        DD    LDP
        DD    N_VOCL
        DD    0

;

;  **************
;  *   OFFSET   *
;  **************
;
        ALIGN    4
N_LOFFSET:
        DD      6
        DB      "OFFSET"
        ALIGN    4
LOFFSET:
        DD    DOUSE
        DD    (CW*(37))
        DD    0x0
        DD    VOCL
        DD    N_LOFFSET
        DD    0

;

;  ****************
;  *   MAX-USER   *
;  ****************
;
        ALIGN    4
N_MAXUSER:
        DD      8
        DB      "MAX-USER"
        ALIGN    4
MAXUSER:
        DD    DOUSE
        DD    (CW*(11))
        DD    0x0
        DD    LOFFSET
        DD    N_MAXUSER
        DD    0

;
; End of user variables with fixed place.
;
;

;  *************
;  *   WHERE   *
;  *************
;
        ALIGN    4
N_LWHERE:
        DD      5
        DB      "WHERE"
        ALIGN    4
LWHERE:
        DD    DOUSE
        DD    (CW*(12))
        DD    0x0
        DD    MAXUSER
        DD    N_LWHERE
        DD    0
    ;  Occupies two CELLS! 
;

;  ***********
;  *   SCR   *
;  ***********
;
        ALIGN    4
N_SCR:
        DD      3
        DB      "SCR"
        ALIGN    4
SCR:
        DD    DOUSE
        DD    (CW*(33))
        DD    0x0
        DD    LWHERE
        DD    N_SCR
        DD    0

;

;  *************
;  *   STATE   *
;  *************
;
        ALIGN    4
N_STATE:
        DD      5
        DB      "STATE"
        ALIGN    4
STATE:
        DD    DOUSE
        DD    (CW*(18))
        DD    0x0
        DD    SCR
        DD    N_STATE
        DD    0

;

;  ************
;  *   BASE   *
;  ************
;
        ALIGN    4
N_BASE:
        DD      4
        DB      "BASE"
        ALIGN    4
BASE:
        DD    DOUSE
        DD    (CW*(19))
        DD    0x0
        DD    STATE
        DD    N_BASE
        DD    0

;

;  ***********
;  *   DPL   *
;  ***********
;
        ALIGN    4
N_DPL:
        DD      3
        DB      "DPL"
        ALIGN    4
DPL:
        DD    DOUSE
        DD    (CW*(20))
        DD    0x0
        DD    BASE
        DD    N_DPL
        DD    0

;

;  ***********
;  *   ECL   *
;  ***********
;
        ALIGN    4
N_ECL:
        DD      3
        DB      "ECL"
        ALIGN    4
ECL:
        DD    DOUSE
        DD    (CW*(38))
        DD    0x0
        DD    DPL
        DD    N_ECL
        DD    0

;

;  ***********
;  *   FLD   *
;  ***********
;
        ALIGN    4
N_LFLD:
        DD      3
        DB      "FLD"
        ALIGN    4
LFLD:
        DD    DOUSE
        DD    (CW*(21))
        DD    0x0
        DD    ECL
        DD    N_LFLD
        DD    0

;


;  ***********
;  *   CSP   *
;  ***********
;
        ALIGN    4
N_LCSP:
        DD      3
        DB      "CSP"
        ALIGN    4
LCSP:
        DD    DOUSE
        DD    (CW*(22))
        DD    0x0
        DD    LFLD
        DD    N_LCSP
        DD    0

;
;

;  **********
;  *   R#   *
;  **********
;
        ALIGN    4
N_RNUM:
        DD      2
        DB      "R#"
        ALIGN    4
RNUM:
        DD    DOUSE
        DD    (CW*(23))
        DD    0x0
        DD    LCSP
        DD    N_RNUM
        DD    0

;

;  ***********
;  *   HLD   *
;  ***********
;
        ALIGN    4
N_HLD:
        DD      3
        DB      "HLD"
        ALIGN    4
HLD:
        DD    DOUSE
        DD    (CW*(24))
        DD    0x0
        DD    RNUM
        DD    N_HLD
        DD    0

;

;  ***********
;  *   OUT   *
;  ***********
;
        ALIGN    4
N_LOUT:
        DD      3
        DB      "OUT"
        ALIGN    4
LOUT:
        DD    DOUSE
        DD    (CW*(25))
        DD    0x0
        DD    HLD
        DD    N_LOUT
        DD    0

;

;  *************
;  *   (BLK)   *
;  *************
;
        ALIGN    4
N_PBLK:
        DD      5
        DB      "(BLK)"
        ALIGN    4
PBLK:
        DD    DOUSE
        DD    (CW*(26))
        DD    0x0
        DD    LOUT
        DD    N_PBLK
        DD    0

;

;  **********
;  *   PP   *
;  **********
;
        ALIGN    4
N_PP:
        DD      2
        DB      "PP"
        ALIGN    4
PP:
        DD    DOUSE
        DD    (CW*(29))
        DD    0x0
        DD    PBLK
        DD    N_PP
        DD    0

;

;  *************
;  *   (>IN)   *
;  *************
;
        ALIGN    4
N_PIIN:
        DD      5
        DB      "(>IN)"
        ALIGN    4
PIIN:
        DD    DOUSE
        DD    (CW*(30))
        DD    0x0
        DD    PP
        DD    N_PIIN
        DD    0

;

;  ************
;  *   ARGS   *
;  ************
;
        ALIGN    4
N_ARGS:
        DD      4
        DB      "ARGS"
        ALIGN    4
ARGS:
        DD    DOUSE
        DD    (CW*(31))
        DD    0x0
        DD    PIIN
        DD    N_ARGS
        DD    0

;

;  ***************
;  *   HANDLER   *
;  ***************
;
        ALIGN    4
N_HANDLER:
        DD      7
        DB      "HANDLER"
        ALIGN    4
HANDLER:
        DD    DOUSE
        DD    (CW*(32))
        DD    0x0
        DD    ARGS
        DD    N_HANDLER
        DD    0

;

;  ***************
;  *   CURRENT   *
;  ***************
;
        ALIGN    4
N_CURR:
        DD      7
        DB      "CURRENT"
        ALIGN    4
CURR:
        DD    DOUSE
        DD    (CW*(34))
        DD    0x0
        DD    HANDLER
        DD    N_CURR
        DD    0

;

;  *****************
;  *   REMAINDER   *
;  *****************
;
        ALIGN    4
N_REMAIND:
        DD      9
        DB      "REMAINDER"
        ALIGN    4
REMAIND:
        DD    DOUSE
        DD    (CW*(14))
        DD    0x0
        DD    CURR
        DD    N_REMAIND
        DD    0

;      IMPORTANT
; REQUIRES ONE MORE CELL!
;

;  ***************
;  *   CONTEXT   *
;  ***************
;
        ALIGN    4
N_CONTEXT:
        DD      7
        DB      "CONTEXT"
        ALIGN    4
CONTEXT:
        DD    DOUSE
        DD    (CW*(39))
        DD    0x0
        DD    REMAIND
        DD    N_CONTEXT
        DD    0
 ; Up to  39+16
;      IMPORTANT
;     16 USER SPACE CELLS MUST BE KEPT FREE
;     IN ADDITION TO THE ONE FOR CONTEXT
;
;========== END USER VARIABLES =============
;

;  **********
;  *   1+   *
;  **********
;
        ALIGN    4
N_ONEP:
        DD      2
        DB      "1+"
        ALIGN    4
ONEP:
        DD    DOCOL
        DD    ONEP+HEADSIZE
        DD    0x0
        DD    CONTEXT
        DD    N_ONEP
        DD    0

        DD      ONE
        DD      PLUS
        DD      SEMIS
;

;  *************
;  *   CELL+   *
;  *************
;
        ALIGN    4
N_CELLP:
        DD      5
        DB      "CELL+"
        ALIGN    4
CELLP:
        DD    DOCOL
        DD    CELLP+HEADSIZE
        DD    0x0
        DD    ONEP
        DD    N_CELLP
        DD    0

        DD      LIT, CW
        DD      PLUS
        DD      SEMIS
;

;
;  *************
;  *   CELLS   *
;  *************
;
        ALIGN    4
N_LCELLS:
        DD      5
        DB      "CELLS"
        ALIGN    4
LCELLS:
        DD    DOCOL
        DD    LCELLS+HEADSIZE
        DD    0x0
        DD    CELLP
        DD    N_LCELLS
        DD    0

        DD       TWO 
        DD      LSHIFT
        DD      SEMIS
;

;  *************
;  *   CHAR+   *
;  *************
;
        ALIGN    4
N_CHARP:
        DD      5
        DB      "CHAR+"
        ALIGN    4
CHARP:
        DD    DOCOL
        DD    (ONEP+HEADSIZE)
        DD    0x0
        DD    LCELLS
        DD    N_CHARP
        DD    0

;

;  *************
;  *   CHARS   *
;  *************
;
        ALIGN    4
N_CHARS:
        DD      5
        DB      "CHARS"
        ALIGN    4
CHARS:
        DD    DOCOL 
        DD    CHARS+HEADSIZE
        DD    0x0
        DD    CHARP
        DD    N_CHARS
        DD    0

        DD      SEMIS
;

;  *************
;  *   ALIGN   *
;  *************
;
        ALIGN    4
N_LALIGN:
        DD      5
        DB      "ALIGN"
        ALIGN    4
LALIGN:
        DD    DOCOL 
        DD    LALIGN+HEADSIZE
        DD    0x0
        DD    CHARS
        DD    N_LALIGN
        DD    0


        DD      LDP, FETCH
        DD      ALIGNED
        DD      LDP, STOR
;
        DD      SEMIS
;

;  ***************
;  *   ALIGNED   *
;  ***************
;
        ALIGN    4
N_ALIGNED :
        DD      7
        DB      "ALIGNED"
        ALIGN    4
ALIGNED :
        DD    ALIGNED +HEADSIZE
        DD    ALIGNED +HEADSIZE
        DD    0x0
        DD    LALIGN
        DD    N_ALIGNED 
        DD    0


        LDR     R0, [R13],#+(CW*(1))
        SUB     R0,R0, #1
        
        ORR     R0, R0, #(CW-1) 
        ADD     R0,R0, #1
        STR     R0, [R13,#-(CW*(1))]!
;
        LDR     R0, [R11],#+(CW*(1))
         LDR     R1,[R0]
         BX      R1  
;

;  ************
;  *   HERE   *
;  ************
;
        ALIGN    4
N_HERE:
        DD      4
        DB      "HERE"
        ALIGN    4
HERE:
        DD    DOCOL
        DD    HERE+HEADSIZE
        DD    0x0
        DD    ALIGNED 
        DD    N_HERE
        DD    0

        DD      LDP
        DD      FETCH
        DD      SEMIS
;

;  *************
;  *   ALLOT   *
;  *************
;
        ALIGN    4
N_ALLOT:
        DD      5
        DB      "ALLOT"
        ALIGN    4
ALLOT:
        DD    DOCOL
        DD    ALLOT+HEADSIZE
        DD    0x0
        DD    HERE
        DD    N_ALLOT
        DD    0

        DD      LDP
        DD      PSTOR
        DD      SEMIS
;

;  *********
;  *   ,   *
;  *********
;
        ALIGN    4
N_COMMA:
        DD      1
        DB      ","
        ALIGN    4
COMMA:
        DD    DOCOL
        DD    COMMA+HEADSIZE
        DD    0x0
        DD    ALLOT
        DD    N_COMMA
        DD    0

        DD      HERE
        DD      LIT
        DD      CW
        DD      ALLOT
        DD      STOR
        DD      SEMIS
;

;  **********
;  *   C,   *
;  **********
;
        ALIGN    4
N_CCOMM:
        DD      2
        DB      "C,"
        ALIGN    4
CCOMM:
        DD    DOCOL
        DD    CCOMM+HEADSIZE
        DD    0x0
        DD    COMMA
        DD    N_CCOMM
        DD    0

        DD      HERE
        DD      ONE
        DD      ALLOT
        DD      CSTOR
        DD      SEMIS
;

;  **********
;  *   2,   *
;  **********
;
        ALIGN    4
N_TCOMM:
        DD      2
        DB      "2,"
        ALIGN    4
TCOMM:
        DD    DOCOL
        DD    TCOMM+HEADSIZE
        DD    0x0
        DD    CCOMM
        DD    N_TCOMM
        DD    0

        DD      HERE
        DD      LIT
        DD      CW*2
        DD      ALLOT
        DD      TSTOR
        DD      SEMIS
;

;  *********
;  *   -   *
;  *********
;
        ALIGN    4
N_LSUB:
        DD      1
        DB      "-"
        ALIGN    4
LSUB:
        DD    LSUB+HEADSIZE
        DD    LSUB+HEADSIZE
        DD    0x0
        DD    TCOMM
        DD    N_LSUB
        DD    0

        LDR     R1, [R13],#+(CW*(1))      ;S1
        LDR     R0, [R13],#+(CW*(1))
        SUB     R0,R0,R1
        STR     R0, [R13,#-(CW*(1))]!
        LDR     R0, [R11],#+(CW*(1))
         LDR     R1,[R0]
         BX      R1     ;S1 = S2 - S1
;

;  *********
;  *   =   *
;  *********
;
        ALIGN    4
N_EQUAL:
        DD      1
        DB      "="
        ALIGN    4
EQUAL:
        DD    DOCOL
        DD    EQUAL+HEADSIZE
        DD    0x0
        DD    LSUB
        DD    N_EQUAL
        DD    0

        DD      LSUB
        DD      ZEQU
        DD      SEMIS
;

;  *********
;  *   <   *
;  *********
;
        ALIGN    4
N_LESS:
        DD      1
        DB      "<"
        ALIGN    4
LESS:
        DD    LESS+HEADSIZE
        DD    LESS+HEADSIZE
        DD    0x0
        DD    EQUAL
        DD    N_LESS
        DD    0

        LDR     R1, [R13],#+(CW*(1))      ;S1
        LDR     R3, [R13],#+(CW*(1))      ;S2
        MOV    R0, #0   ;0 default RESULT
        CMP     R3,R1
        BGE     LES1
        SUB     R0,R0, #1
LES1:   STR     R0, [R13,#-(CW*(1))]!
        LDR     R0, [R11],#+(CW*(1))
         LDR     R1,[R0]
         BX      R1  
;

;  **********
;  *   U<   *
;  **********
;
        ALIGN    4
N_ULESS:
        DD      2
        DB      "U<"
        ALIGN    4
ULESS:
        DD    ULESS+HEADSIZE
        DD    ULESS+HEADSIZE
        DD    0x0
        DD    LESS
        DD    N_ULESS
        DD    0

        LDR     R0, [R13],#+(CW*(1))
        LDR     R1, [R13],#+(CW*(1))
        SUBS    R1,R1,R0
        SBC     R0,R0,R0
        STR     R0, [R13,#-(CW*(1))]!
        LDR     R0, [R11],#+(CW*(1))
         LDR     R1,[R0]
         BX      R1  
;

;  *********
;  *   >   *
;  *********
;
        ALIGN    4
N_GREAT:
        DD      1
        DB      ">"
        ALIGN    4
GREAT:
        DD    DOCOL
        DD    GREAT+HEADSIZE
        DD    0x0
        DD    ULESS
        DD    N_GREAT
        DD    0

        DD      SWAP
        DD      LESS
        DD      SEMIS
;

;  **********
;  *   <>   *
;  **********
;
        ALIGN    4
N_UNEQ:
        DD      2
        DB      "<>"
        ALIGN    4
UNEQ:
        DD    DOCOL
        DD    UNEQ+HEADSIZE
        DD    0x0
        DD    GREAT
        DD    N_UNEQ
        DD    0

        DD      LSUB
        DD      ZEQU
        DD      ZEQU
        DD      SEMIS
;

;  ***********
;  *   ROT   *
;  ***********
;
        ALIGN    4
N_ROT:
        DD      3
        DB      "ROT"
        ALIGN    4
ROT:
        DD    ROT+HEADSIZE
        DD    ROT+HEADSIZE
        DD    0x0
        DD    UNEQ
        DD    N_ROT
        DD    0

        LDR     R1, [R13],#+(CW*(1))      ;S1
        LDR     R3, [R13],#+(CW*(1))      ;S2
        LDR     R0, [R13],#+(CW*(1))      ;S3
        STR     R3, [R13,#-(CW*(1))]!
        STR     R1, [R13,#-(CW*(1))]!
        STR     R0, [R13,#-(CW*(1))]!
        LDR     R0, [R11],#+(CW*(1))
         LDR     R1,[R0]
         BX      R1  
;

;  *************
;  *   SPACE   *
;  *************
;
        ALIGN    4
N_SPACE:
        DD      5
        DB      "SPACE"
        ALIGN    4
SPACE:
        DD    DOCOL
        DD    SPACE+HEADSIZE
        DD    0x0
        DD    ROT
        DD    N_SPACE
        DD    0

        DD      LBL
        DD      EMIT
        DD      SEMIS
;

;  ************
;  *   ?DUP   *
;  ************
;
        ALIGN    4
N_QDUP:
        DD      4
        DB      "?DUP"
        ALIGN    4
QDUP:
        DD    DOCOL
        DD    QDUP+HEADSIZE
        DD    0x0
        DD    SPACE
        DD    N_QDUP
        DD    0

        DD      LDUP
        DD      ZBRAN
        DD      QDUP1-$-CW ; IF
        DD      LDUP    ;THEN
QDUP1:  DD      SEMIS
;

;  **************
;  *   LATEST   *
;  **************
;
        ALIGN    4
N_LATEST:
        DD      6
        DB      "LATEST"
        ALIGN    4
LATEST:
        DD    DOCOL
        DD    LATEST+HEADSIZE
        DD    0x0
        DD    QDUP
        DD    N_LATEST
        DD    0

        DD      CURR
        DD      FETCH
        DD      TLFA
        DD      FETCH
        DD      SEMIS
;

;  ************
;  *   >CFA   *
;  ************
;
        ALIGN    4
N_TCFA:
        DD      4
        DB      ">CFA"
        ALIGN    4
TCFA:
        DD    DOCOL
        DD    TCFA+HEADSIZE
        DD    0x0
        DD    LATEST
        DD    N_TCFA
        DD    0

        DD      LIT, (CW*(C_HOFFSET))
        DD      PLUS
        DD      SEMIS
;

;  ************
;  *   >DFA   *
;  ************
;
        ALIGN    4
N_TDFA:
        DD      4
        DB      ">DFA"
        ALIGN    4
TDFA:
        DD    DOCOL
        DD    TDFA+HEADSIZE
        DD    0x0
        DD    TCFA
        DD    N_TDFA
        DD    0

        DD      LIT, (CW*(D_HOFFSET))
        DD      PLUS
        DD      SEMIS
;

;  ************
;  *   >FFA   *
;  ************
;
        ALIGN    4
N_TFFA:
        DD      4
        DB      ">FFA"
        ALIGN    4
TFFA:
        DD    DOCOL
        DD    TFFA+HEADSIZE
        DD    0x0
        DD    TDFA
        DD    N_TFFA
        DD    0

        DD      LIT, (CW*(F_HOFFSET))
        DD      PLUS
        DD      SEMIS
;

;  ************
;  *   >LFA   *
;  ************
;
        ALIGN    4
N_TLFA:
        DD      4
        DB      ">LFA"
        ALIGN    4
TLFA:
        DD    DOCOL
        DD    TLFA+HEADSIZE
        DD    0x0
        DD    TFFA
        DD    N_TLFA
        DD    0

        DD      LIT, (CW*(L_HOFFSET))
        DD      PLUS
        DD      SEMIS
;

;  ************
;  *   >NFA   *
;  ************
;
        ALIGN    4
N_TNFA:
        DD      4
        DB      ">NFA"
        ALIGN    4
TNFA:
        DD    DOCOL
        DD    TNFA+HEADSIZE
        DD    0x0
        DD    TLFA
        DD    N_TNFA
        DD    0

        DD      LIT,(CW*(N_HOFFSET))
        DD      PLUS
        DD      SEMIS
;


;
;  ************
;  *   >SFA   *
;  ************
;
        ALIGN    4
N_TSFA:
        DD      4
        DB      ">SFA"
        ALIGN    4
TSFA:
        DD    DOCOL
        DD    TSFA+HEADSIZE
        DD    0x0
        DD    TNFA
        DD    N_TSFA
        DD    0

        DD      LIT,(CW*(S_HOFFSET))
        DD      PLUS
        DD      SEMIS
;
;
;

;  ************
;  *   >PHA   *
;  ************
;
        ALIGN    4
N_TPHA:
        DD      4
        DB      ">PHA"
        ALIGN    4
TPHA:
        DD    DOCOL
        DD    TPHA+HEADSIZE
        DD    0x0
        DD    TSFA
        DD    N_TPHA
        DD    0

        DD      LIT,(CW*(PH_OFFSET))
        DD      PLUS
        DD      SEMIS
;

;  *************
;  *   >BODY   *
;  *************
;
        ALIGN    4
N_TOBODY:
        DD      5
        DB      ">BODY"
        ALIGN    4
TOBODY:
        DD    DOCOL
        DD    TOBODY+HEADSIZE
        DD    0x0
        DD    TPHA
        DD    N_TOBODY
        DD    0

        DD      TDFA, FETCH
        DD      CELLP           ; Skip DOES> pointer.
        DD      SEMIS
;

;  *************
;  *   BODY>   *
;  *************
;
        ALIGN    4
N_BODYF:
        DD      5
        DB      "BODY>"
        ALIGN    4
BODYF:
        DD    DOCOL
        DD    BODYF+HEADSIZE
        DD    0x0
        DD    TOBODY
        DD    N_BODYF
        DD    0

        DD      LIT,(CW*(BD_OFFSET))
        DD      LSUB
        DD      SEMIS
;

;  ************
;  *   >WID   *
;  ************
;
        ALIGN    4
N_TWID:
        DD      4
        DB      ">WID"
        ALIGN    4
TWID:
        DD    DOCOL
        DD    TWID+HEADSIZE
        DD    0x0
        DD    BODYF
        DD    N_TWID
        DD    0

        DD      TOBODY
        DD      CELLP ; Skip vfa link.
        DD      SEMIS
;

;  ************
;  *   >VFA   *
;  ************
;
        ALIGN    4
N_TVFA:
        DD      4
        DB      ">VFA"
        ALIGN    4
TVFA:
        DD    DOCOL
        DD    TVFA+HEADSIZE
        DD    0x0
        DD    TWID
        DD    N_TVFA
        DD    0

        DD      TOBODY
        DD      SEMIS
;


;  ************
;  *   !CSP   *
;  ************
;
        ALIGN    4
N_SCSP:
        DD      4
        DB      "!CSP"
        ALIGN    4
SCSP:
        DD    DOCOL
        DD    SCSP+HEADSIZE
        DD    0x0
        DD    TVFA
        DD    N_SCSP
        DD    0

        DD      SPFET
        DD      LCSP
        DD      STOR
        DD      SEMIS
;
;

;  **************
;  *   ?ERROR   *
;  **************
;
        ALIGN    4
N_QERR:
        DD      6
        DB      "?ERROR"
        ALIGN    4
QERR:
        DD    DOCOL
        DD    QERR+HEADSIZE
        DD    0x0
        DD    SCSP
        DD    N_QERR
        DD    0

        DD      SWAP
        DD      ZBRAN
        DD      QERR1-$-CW ;IF
        DD      PP, FETCH
        DD      SRC, FETCH
        DD      LWHERE, TSTOR
        DD      THROW
        DD      BRAN
        DD      QERR2-$-CW  ;ELSE
QERR1:  DD      DROP    ;THEN
QERR2:  DD      SEMIS
;

;  **************
;  *   ?ERRUR   *
;  **************
;
        ALIGN    4
N_QERRUR:
        DD      6
        DB      "?ERRUR"
        ALIGN    4
QERRUR:
        DD    DOCOL
        DD    QERRUR+HEADSIZE
        DD    0x0
        DD    QERR
        DD    N_QERRUR
        DD    0

        DD      ZERO, MIN, LDUP, QERR
        DD      SEMIS
;


;  **************
;  *   ?DELIM   *
;  **************
;
        ALIGN    4
N_QDELIM:
        DD      6
        DB      "?DELIM"
        ALIGN    4
QDELIM:
        DD    DOCOL
        DD    QDELIM+HEADSIZE
        DD    0x0
        DD    QERRUR
        DD    N_QDELIM
        DD    0

        DD      PPFET
        DD      QBL
        DD      ZEQU
        DD      LIT, 10, QERR
        DD      DROP
        DD      SEMIS
;

;  ************
;  *   ?CSP   *
;  ************
;
        ALIGN    4
N_QCSP:
        DD      4
        DB      "?CSP"
        ALIGN    4
QCSP:
        DD    DOCOL
        DD    QCSP+HEADSIZE
        DD    0x0
        DD    QDELIM
        DD    N_QCSP
        DD    0

        DD      SPFET
        DD      LCSP
        DD      FETCH
        DD      LSUB
        DD      LIT, 20, QERR
        DD      SEMIS
;

;  *************
;  *   ?COMP   *
;  *************
;
        ALIGN    4
N_QCOMP:
        DD      5
        DB      "?COMP"
        ALIGN    4
QCOMP:
        DD    DOCOL
        DD    QCOMP+HEADSIZE
        DD    0x0
        DD    QCSP
        DD    N_QCOMP
        DD    0

        DD      STATE
        DD      FETCH
        DD      ZEQU
        DD      LIT, 17, QERR
        DD      SEMIS
;

; 

;  **************
;  *   ?PAIRS   *
;  **************
;
        ALIGN    4
N_QPAIR:
        DD      6
        DB      "?PAIRS"
        ALIGN    4
QPAIR:
        DD    DOCOL
        DD    QPAIR+HEADSIZE
        DD    0x0
        DD    QCOMP
        DD    N_QPAIR
        DD    0

        DD      LSUB
        DD      LIT, 19, QERR
        DD      SEMIS
;

; 
;
;
;

;  *********
;  *   [   *
;  *********
;
        ALIGN    4
N_LBRAC:
        DD      1
        DB      "["
        ALIGN    4
LBRAC:
        DD    DOCOL
        DD    LBRAC+HEADSIZE
        DD    B_IMMED
        DD    QPAIR
        DD    N_LBRAC
        DD    0

        DD      ZERO
        DD      STATE
        DD      STOR
        DD      SEMIS
;

;  *********
;  *   ]   *
;  *********
;
        ALIGN    4
N_RBRAC:
        DD      1
        DB      "]"
        ALIGN    4
RBRAC:
        DD    DOCOL
        DD    RBRAC+HEADSIZE
        DD    0x0
        DD    LBRAC
        DD    N_RBRAC
        DD    0

        DD      ONE
        DD      STATE
        DD      STOR
        DD      SEMIS
;

;  **************
;  *   HIDDEN   *
;  **************
;
        ALIGN    4
N_HIDDEN:
        DD      6
        DB      "HIDDEN"
        ALIGN    4
HIDDEN:
        DD    DOCOL
        DD    HIDDEN+HEADSIZE
        DD    0x0
        DD    RBRAC
        DD    N_HIDDEN
        DD    0

        DD      TFFA
        DD      LIT,B_INVIS
        DD      TOGGL
        DD      SEMIS
;

;  ***********
;  *   HEX   *
;  ***********
;
        ALIGN    4
N_HEX:
        DD      3
        DB      "HEX"
        ALIGN    4
HEX:
        DD    DOCOL
        DD    HEX+HEADSIZE
        DD    0x0
        DD    HIDDEN
        DD    N_HEX
        DD    0

        DD      LIT,16
        DD      BASE
        DD      STOR
        DD      SEMIS
;

;  ***************
;  *   DECIMAL   *
;  ***************
;
        ALIGN    4
N_DECA:
        DD      7
        DB      "DECIMAL"
        ALIGN    4
DECA:
        DD    DOCOL
        DD    DECA+HEADSIZE
        DD    0x0
        DD    HEX
        DD    N_DECA
        DD    0

        DD      LIT,10
        DD      BASE
        DD      STOR
        DD      SEMIS
;

;  ***************
;  *   (;CODE)   *
;  ***************
;
        ALIGN    4
N_PSCOD:
        DD      7
        DB      "(;CODE)"
        ALIGN    4
PSCOD:
        DD    DOCOL
        DD    PSCOD+HEADSIZE
        DD    0x0
        DD    DECA
        DD    N_PSCOD
        DD    0

        DD      FROMR
        DD      LATEST
        DD      TCFA
        DD      STOR
        DD      SEMIS
;

;

;  **************
;  *   CREATE   *
;  **************
;
        ALIGN    4
N_LCREATE:
        DD      6
        DB      "CREATE"
        ALIGN    4
LCREATE:
        DD    DOCOL
        DD    LCREATE+HEADSIZE
        DD    0x0
        DD    PSCOD
        DD    N_LCREATE
        DD    0

        DD      LNAME
        DD      PCREAT
        DD      LIT, HLNOOP, COMMA
        DD      PSCOD
DODOE:  STR     R11, [R12,#-(CW*(1))]!
        LDR     R11,[R0,#+(CW*((D_HOFFSET)))] ;NEW IP 
        MOV     R0, R11
        ADD     R0,R0,#(CW*(1))
        LDR     R11,[R11]
        STR     R0, [R13,#-(CW*(1))]!
        LDR     R0, [R11],#+(CW*(1))
         LDR     R1,[R0]
         BX      R1  
HLNOOP: DD      SEMIS
;

;  *************
;  *   DOES>   *
;  *************
;
        ALIGN    4
N_DOES:
        DD      5
        DB      "DOES>"
        ALIGN    4
DOES:
        DD    DOCOL
        DD    DOES+HEADSIZE
        DD    0x0
        DD    LCREATE
        DD    N_DOES
        DD    0

        DD      FROMR
        DD      LATEST
        DD      TDFA
        DD      FETCH
        DD      STOR
        DD      SEMIS
;

;  *************
;  *   COUNT   *
;  *************
;
        ALIGN    4
N_COUNT:
        DD      5
        DB      "COUNT"
        ALIGN    4
COUNT:
        DD    DOCOL
        DD    COUNT+HEADSIZE
        DD    0x0
        DD    DOES
        DD    N_COUNT
        DD    0

        DD      LDUP
        DD      ONEP
        DD      SWAP
        DD      CFET
        DD      SEMIS
;

;  *****************
;  *   -TRAILING   *
;  *****************
;
        ALIGN    4
N_DTRAI:
        DD      9
        DB      "-TRAILING"
        ALIGN    4
DTRAI:
        DD    DOCOL
        DD    DTRAI+HEADSIZE
        DD    0x0
        DD    COUNT
        DD    N_DTRAI
        DD    0

        DD      LDUP
        DD      ZERO
        DD     XQDO
        DD      DTRA4-$-CW
DTRA1:  DD      OVER
        DD      OVER
        DD      PLUS
        DD      ONE
        DD      LSUB
        DD      CFET
        DD      QBL
        DD      ZEQU
        DD      ZBRAN
        DD      DTRA2-$-CW ;IF
        DD      LLEAV
DTRA2:  DD      ONE
        DD      LSUB    ; THEN
        DD     ONE, XPLOOP, ZBRAN
        DD      DTRA1-$-CW
	DD      UNLOOP    ; LOOP
DTRA4:
        DD      SEMIS
;

;  **********
;  *   S"   *
;  **********
;
        ALIGN    4
N_SQUOT:
        DD      2
        DB      'S"'
        ALIGN    4
SQUOT:
        DD    DOCOL
        DD    SQUOT+HEADSIZE
        DD    B_IMMED
        DD    DTRAI
        DD    N_SQUOT
        DD    0

        DD      DENQ
        DD      SEMIS
;

;  **********
;  *   ."   *
;  **********
;
        ALIGN    4
N_DOTQ:
        DD      2
        DB      '."'
        ALIGN    4
DOTQ:
        DD    DOCOL
        DD    DOTQ+HEADSIZE
        DD    B_IMMED
        DD    SQUOT
        DD    N_DOTQ
        DD    0

        DD      DENQ
        DD      STATE
        DD      FETCH
        DD      ZBRAN
        DD      DOTQ1-$-CW ; IF
        DD      LIT, LTYPE, COMMA
        DD      BRAN
        DD      DOTQ2-$-CW
DOTQ1:
        DD      LTYPE
DOTQ2:
        DD      SEMIS   ; THEN
;

;  **********
;  *   .(   *
;  **********
;
        ALIGN    4
N_DOTP:
        DD      2
        DB      ".("
        ALIGN    4
DOTP:
        DD    DOCOL
        DD    DOTP+HEADSIZE
        DD    B_IMMED
        DD    DOTQ
        DD    N_DOTP
        DD    0

        DD      LIT, ')'
        DD      PARSE
        DD      LTYPE
        DD      SEMIS
;

;  ***************
;  *   SET-SRC   *
;  ***************
;
        ALIGN    4
N_SETSRC:
        DD      7
        DB      "SET-SRC"
        ALIGN    4
SETSRC:
        DD    DOCOL
        DD    SETSRC+HEADSIZE
        DD    0x0
        DD    DOTP
        DD    N_SETSRC
        DD    0

        DD      OVER, PLUS
        DD      OVER, SRC, TSTOR
        DD      PP, STOR ;  IN

;       DC      DOTS
        DD      SEMIS
;

;  ****************
;  *   EVALUATE   *
;  ****************
;
        ALIGN    4
N_EVALUATE:
        DD      8
        DB      "EVALUATE"
        ALIGN    4
EVALUATE:
        DD    DOCOL
        DD    EVALUATE+HEADSIZE
        DD    0x0
        DD    SETSRC
        DD    N_EVALUATE
        DD    0

        DD      SAVE
        DD      SETSRC
        DD      LIT, INTER, CATCH
        DD      RESTO
        DD      THROW
        DD      SEMIS
;

;  ************
;  *   FILL   *
;  ************
;
        ALIGN    4
N_FILL:
        DD      4
        DB      "FILL"
        ALIGN    4
FILL:
        DD    FILL+HEADSIZE
        DD    FILL+HEADSIZE
        DD    0x0
        DD    EVALUATE
        DD    N_FILL
        DD    0

        LDR     R0, [R13],#+(CW*(1))      ; FILL CHAR
        LDR     R2, [R13],#+(CW*(1))      ; FILL COUNT
        LDR     R1, [R13],#+(CW*(1))      ; BEGIN ADDR
        ANDS R2,R2, R2
        BEQ FILL2    ; COUNT=0? 
FILL1:  STRB    R0, [R1],#+1
        SUBS  R2,R2,#1
        BNE FILL1
FILL2:  LDR     R0, [R11],#+(CW*(1))
         LDR     R1,[R0]
         BX      R1  
;

;  ************
;  *   CORA   *
;  ************
;
        ALIGN    4
N_CORA:
        DD      4
        DB      "CORA"
        ALIGN    4
CORA:
        DD    CORA+HEADSIZE
        DD    CORA+HEADSIZE
        DD    0x0
        DD    FILL
        DD    N_CORA
        DD    0

        
        MOV    R0,#0   ; Result, assume all equal
        LDR     R3, [R13],#+(CW*(1))      ; count
        LDR     R4, [R13],#+(CW*(1))      ; addr2
        LDR     R5, [R13],#+(CW*(1))      ; addr1
        MOV     R1,#0
        MOV     R2,#0
CORA1:
        SUBS     R3,R3,#1
        BMI      CORA3        ; Stop looping if R3 < 0  and return 0 
        LDRB    R1, [R5],#+1          ; load byte at R5 in W1, auto-increment R5 
        LDRB    R2, [R4],#+1          ; load byte at R4 in W2, auto-increment R4 
        CMP      R1,R2
        BEQ      CORA1        ; chars equal, keep looping 
        MOV     R0, #1
        BHI      CORA3        ; R1/W1 is lexicographic later, return 1 
        
        MOV    R0,#-1  ;TRUE FLAG  
CORA3:
        
        STR     R0, [R13,#-(CW*(1))]!
        LDR     R0, [R11],#+(CW*(1))
         LDR     R1,[R0]
         BX      R1  
;

;  **********
;  *   $^   *
;  **********
;
        ALIGN    4
N_SINDEX:
        DD      2
        DB      "$^"
        ALIGN    4
SINDEX:
        DD    SINDEX+HEADSIZE
        DD    SINDEX+HEADSIZE
        DD    0x0
        DD    CORA
        DD    N_SINDEX
        DD    0

;     search address of given byte starting from R4 
        LDR     R0, [R13],#+(CW*(1))      ; char
        LDR     R2, [R13],#+(CW*(1))      ; count
        LDR     R4, [R13],#+(CW*(1))      ; addr
        BICS    R1,R1,R1    ;Clear R1 and zero flag
SINDEX1:
        ANDS    R2,R2,R2
        BEQ     SINDEX3
        LDRB    R1, [R4],#+1         ; load next byte from R4 and auto-increment R4
        CMP     R0,R1       ; Compare BYTE and set flags
        BEQ     SINDEX2
        SUB     R2,R2,#1
        BAL     SINDEX1
SINDEX3:
        MOV    R4,#0       ; Not found: 0 
        ADD     R4,R4, #1
SINDEX2:
        SUB     R4,R4, #1
        STR     R4, [R13,#-(CW*(1))]!            ; Return address, or 0 
        LDR     R0, [R11],#+(CW*(1))
         LDR     R1,[R0]
         BX      R1  
;

;  **********
;  *   $/   *
;  **********
;
        ALIGN    4
N_SSLAS:
        DD      2
        DB      "$/"
        ALIGN    4
SSLAS:
        DD    SSLAS+HEADSIZE
        DD    SSLAS+HEADSIZE
        DD    0x0
        DD    SINDEX
        DD    N_SSLAS
        DD    0

        
        LDR     R0, [R13],#+(CW*(1))          ; char
        LDR     R2, [R13],#+(CW*(1))          ; count
        MOV     R5,R2
        LDR     R4, [R13],#+(CW*(1))          ; addr
        MOV     R1,R4   ; Copy
        ANDS    R4,R4,R4    ; test R4 for nullpointer 
        BEQ     SSLAS3   ; Nil pointer 
SSLAS1:
        LDRB    R3, [R4],#+1         ; load byte at R4 in W3, then auto-increment R4
        CMP     R0, R3
        BEQ     SSLAS2
        SUBS    R2,R2, #1
        BNE     SSLAS1

; Not present.
        STR     R2, [R13,#-(CW*(1))]!    ; Nil pointer.
        BAL     SSLAS4
SSLAS2:
        SUB     R5,R5,R2
        SUB     R2,R2,#1 ;Delimiter is not part of first string.
SSLAS3:
        STR     R4, [R13,#-(CW*(1))]!
SSLAS4:
        STR     R2, [R13,#-(CW*(1))]!   ;Remaining length
        STR     R1, [R13,#-(CW*(1))]!   ;Start of first string.
        STR     R5, [R13,#-(CW*(1))]!   ;Skipped length.
        
        LDR     R0, [R11],#+(CW*(1))
         LDR     R1,[R0]
         BX      R1  
;
;  **********
;  *   $\   *
;  **********
;
        ALIGN    4
N_BSLAS:
        DD      2
        DB      "$\"
        ALIGN    4
BSLAS:
        DD    BSLAS+HEADSIZE
        DD    BSLAS+HEADSIZE
        DD    0x0
        DD    SSLAS
        DD    N_BSLAS
        DD    0
   ;_dummy
        LDR     R0, [R11],#+(CW*(1))
         LDR     R1,[R0]
         BX      R1  

;  *************
;  *   ERASE   *
;  *************
;
        ALIGN    4
N_LERASE:
        DD      5
        DB      "ERASE"
        ALIGN    4
LERASE:
        DD    DOCOL
        DD    LERASE+HEADSIZE
        DD    0x0
        DD    BSLAS
        DD    N_LERASE
        DD    0

        DD      ZERO
        DD      FILL
        DD      SEMIS
;

;  *************
;  *   BLANK   *
;  *************
;
        ALIGN    4
N_BLANK:
        DD      5
        DB      "BLANK"
        ALIGN    4
BLANK:
        DD    DOCOL
        DD    BLANK+HEADSIZE
        DD    0x0
        DD    LERASE
        DD    N_BLANK
        DD    0

        DD      LBL
        DD      FILL
        DD      SEMIS
;

;  ************
;  *   HOLD   *
;  ************
;
        ALIGN    4
N_HOLD:
        DD      4
        DB      "HOLD"
        ALIGN    4
HOLD:
        DD    DOCOL
        DD    HOLD+HEADSIZE
        DD    0x0
        DD    BLANK
        DD    N_HOLD
        DD    0

        DD      LIT,-1
        DD      HLD
        DD      PSTOR
        DD      HLD
        DD      FETCH
        DD      CSTOR
        DD      SEMIS
;

;  ***********
;  *   PAD   *
;  ***********
;
        ALIGN    4
N_PAD:
        DD      3
        DB      "PAD"
        ALIGN    4
PAD:
        DD    DOCOL
        DD    PAD+HEADSIZE
        DD    0x0
        DD    HOLD
        DD    N_PAD
        DD    0

        DD      HERE
; Allow for a one line name, a double binary number and some hold char's
        DD      LIT,84+128+64
        DD      PLUS
        DD      SEMIS
;

; 
;

;  ************
;  *   CHAR   *
;  ************
;
        ALIGN    4
N_LCHAR:
        DD      4
        DB      "CHAR"
        ALIGN    4
LCHAR:
        DD    DOCOL
        DD    LCHAR+HEADSIZE
        DD    0x0
        DD    PAD
        DD    N_LCHAR
        DD    0

        DD      LNAME, DROP, CFET
        DD      SEMIS
;

;  **************
;  *   [CHAR]   *
;  **************
;
        ALIGN    4
N_BCHAR:
        DD      6
        DB      "[CHAR]"
        ALIGN    4
BCHAR:
        DD    DOCOL
        DD    BCHAR+HEADSIZE
        DD    B_IMMED
        DD    LCHAR
        DD    N_BCHAR
        DD    0

        DD      LCHAR, LITER
        DD      SEMIS
;

;  ****************
;  *   (NUMBER)   *
;  ****************
;
        ALIGN    4
N_PNUMB:
        DD      8
        DB      "(NUMBER)"
        ALIGN    4
PNUMB:
        DD    DOCOL
        DD    PNUMB+HEADSIZE
        DD    0x0
        DD    BCHAR
        DD    N_PNUMB
        DD    0

        DD      ZERO, ZERO
        DD      ZERO, DPL, STOR
NPNUM1:  DD      PPFET   ; BEGIN
        DD      LDUP, LIT, ADOT, EQUAL
        DD      ZBRAN
        DD      NPNUM2-$-CW ; case 1 
        DD      DROP, DPL, STOR
        DD      BRAN
        DD      NPNUM1-$-CW ; continue 
NPNUM2:
        DD      LDUP, LIT, ',', EQUAL
        DD      ZBRAN
        DD      NPNUM3-$-CW ; case 2 
        DD      TDROP
        DD      BRAN
        DD      NPNUM1-$-CW ; continue 
NPNUM3:
        DD      LDUP, QBL, ZEQU
        DD      ZBRAN
        DD      NPNUM5-$-CW  ; while 
        DD      NIP
        DD      BASE, FETCH, DIGIT
        DD      ZEQU
        DD      LIT, 10, QERR

        DD      SWAP
        DD      BASE
        DD      FETCH
        DD      USTAR
        DD      DROP
        DD      ROT
        DD      BASE
        DD      FETCH
        DD      USTAR
        DD      DPLUS
        DD      BRAN
        DD      NPNUM1-$-CW ; repeat 
NPNUM5:
        DD      DROP, PP, STOR
        DD      SEMIS
;

;  **************
;  *   NUMBER   *
;  **************
;
        ALIGN    4
N_NUMB:
        DD      6
        DB      "NUMBER"
        ALIGN    4
NUMB:
        DD    DOCOL
        DD    NUMB+HEADSIZE
        DD    0x0
        DD    PNUMB
        DD    N_NUMB
        DD    0

LNUMB:
        DD      LIT, -1, PP, PSTOR
        DD      PNUMB, SDLITE
        DD      SEMIS
;

;  ***************
;  *   >NUMBER   *
;  ***************
;
        ALIGN    4
N_TONUMB:
        DD      7
        DB      ">NUMBER"
        ALIGN    4
TONUMB:
        DD    DOCOL
        DD    TONUMB+HEADSIZE
        DD    0x0
        DD    NUMB
        DD    N_TONUMB
        DD    0

        DD      TDUP, PLUS, TOR     ; End available on return stack.
        DD      ZERO
        DD     XQDO
        DD      TONUM9-$-CW
TONUM1:
        DD      LDUP, CFET, BASE, FETCH, DIGIT
        DD      ZEQU
        DD      ZBRAN
        DD      TONUM4-$-CW ; IF
        DD      DROP
        DD      LLEAV
TONUM4:
        DD      SWAP, TOR ; Address out of the way.
        DD      SWAP
        DD      BASE
        DD      FETCH
        DD      USTAR
        DD      DROP
        DD      ROT
        DD      BASE
        DD      FETCH
        DD      USTAR
        DD      DPLUS
        DD      FROMR, ONEP     ; Address back.
        DD     ONE, XPLOOP, ZBRAN
        DD      TONUM1-$-CW
	DD      UNLOOP
TONUM9:
        DD      FROMR
        DD      OVER, LSUB
        DD      SEMIS
;

;  *************
;  *   FOUND   *
;  *************
;
        ALIGN    4
N_FOUND:
        DD      5
        DB      "FOUND"
        ALIGN    4
FOUND:
        DD    DOCOL
        DD    FOUND+HEADSIZE
        DD    0x0
        DD    TONUMB
        DD    N_FOUND
        DD    0

        DD      CONTEXT, TOR
FOUND1: DD      RR, FETCH
;        DC      DOTS
        DD      PFIND, LDUP, ZEQU
        DD      ZBRAN
        DD      FOUND3-$-CW
        DD      DROP
        DD      RR, FETCH, LIT, ONLYBODY, LSUB ;Was this ONLY?
        DD      ZBRAN
        DD      FOUND2-$-CW
        DD      FROMR, CELLP, TOR
        DD      BRAN
        DD      FOUND1-$-CW
FOUND2: DD      ZERO
FOUND3: DD      RDROP
        DD      NIP,NIP
        DD      SEMIS
;

;  ***************
;  *   PRESENT   *
;  ***************
;
        ALIGN    4
N_PRESENT:
        DD      7
        DB      "PRESENT"
        ALIGN    4
PRESENT:
        DD    DOCOL
        DD    PRESENT+HEADSIZE
        DD    0x0
        DD    FOUND
        DD    N_PRESENT
        DD    0

        DD      LDUP, TOR
        DD      FOUND
        DD      LDUP
        DD      ZBRAN
        DD      PRES1-$-CW
        DD      LDUP
        DD      TNFA, FETCH, FETCH ;  Get precise length.
        DD      RR, EQUAL
        DD      LAND
PRES1:
        DD      RDROP
        DD      SEMIS
;


; 
;

;

;  **************
;  *   (FIND)   *
;  **************
;
        ALIGN    4
N_PFIND:
        DD      6
        DB      "(FIND)"
        ALIGN    4
PFIND:
        DD    DOCOL
        DD    PFIND+HEADSIZE
        DD    0x0
        DD    PRESENT
        DD    N_PFIND
        DD    0

; The idea is to have a fast slim loop.
; The optimisation tries to skip, using the extra links.
; At all labels the stack is (sc, dea).
; PFIND0 : nothing known yet about dea
; PFCONTINUE : dea is disapproved, go on 
; PFIND1 : dea is a match, inasfar name and prefix
; PFEXIT : dea is the match, or null.

PFIND0:                       ;Start of fast loop.
         DD      NMATCH
         DD      ZEQU
        DD      ZBRAN
        DD      PFCONTINUE-$-CW
; dea's name and sc have equal length
        DD      TDUP, TNFA, FETCH, FETCH
        DD      EQUAL 
; dea is a prefix 
        DD      OVER, TFFA, FETCH
        DD      LIT, B_DENOT, LAND
        DD      LOR             ; One of those ,ist apply.
        DD      ZBRAN
        DD      PFCONTINUE-$-CW
; Discard still, if invisible ("smudged") or dummy.
        DD      LDUP, TFFA, FETCH
        DD      LIT, B_INVIS | B_DUMMY, LAND
        DD      ZEQU
        DD      ZBRAN
        DD      PFCONTINUE-$-CW
        DD      BRAN
        DD      PFEXIT-$-CW		; Match! 
PFCONTINUE:
        DD	TLFA, FETCH
        DD      LDUP, ZEQU
        DD      ZBRAN
        DD      PFIND0-$-CW
PFEXIT:   
        DD      SEMIS
;

;  *************
;  *   ERROR   *
;  *************
;
        ALIGN    4
N_LERROR:
        DD      5
        DB      "ERROR"
        ALIGN    4
LERROR:
        DD    DOCOL
        DD    LERROR+HEADSIZE
        DD    0x0
        DD    PFIND
        DD    N_LERROR
        DD    0

; Make sure the whole word is shown in case of denotations. 
        DD      LWHERE, TFET
        DD      OVER
        DD      ZBRAN
        DD      ERR2-$-CW      ; Virgin: print empty string.
        DD      OVER, LIT, 20, LSUB
        DD      MAX
        DD      SWAP
        DD      ONEM         ;Backup for if we are at next word.
ERR1:   DD      LDUP, CFET, QBL, ZEQU  ;Show whole word!
        DD      ZBRAN
        DD      ERR2 -$-CW
        DD      ONEP
        DD      BRAN
        DD      ERR1 -$-CW
ERR2:
        DD      OVER, LSUB
        DD      ETYPE
        DD      SKIP
         DD      19
SB1: DB      " ? ciforth ERROR # "
       ALIGN    4
        DD      LIT, SB1
        DD      LIT, 19
        DD      ETYPE
        DD      BASE, FETCH
        DD      DECA
        DD      OVER
        DD      STOD, ZERO, PDDOTR      ;This is about (.) 
        DD      ETYPE
        DD      BASE, STOR
        DD      LIT, -256, MAX
        DD      LIT, 63, MIN
        DD      LIT, MESS, CATCH, DROP  ;Prevent error loops 
        DD      SEMIS
;

;  *************
;  *   CATCH   *
;  *************
;
        ALIGN    4
N_CATCH:
        DD      5
        DB      "CATCH"
        ALIGN    4
CATCH:
        DD    DOCOL
        DD    CATCH+HEADSIZE
        DD    0x0
        DD    LERROR
        DD    N_CATCH
        DD    0

        DD      SPFET, CELLP, TOR
        DD      HANDLER, FETCH, TOR
        DD      RPFET, HANDLER, STOR
        DD      EXEC
        DD      FROMR, HANDLER, STOR
        DD      RDROP, ZERO
        DD      SEMIS
;

;  *************
;  *   THROW   *
;  *************
;
        ALIGN    4
N_THROW:
        DD      5
        DB      "THROW"
        ALIGN    4
THROW:
        DD    DOCOL
        DD    THROW+HEADSIZE
        DD    0x0
        DD    CATCH
        DD    N_THROW
        DD    0

        DD      LDUP
        DD      ZBRAN
        DD      THROW1-$-CW
        DD      HANDLER, FETCH, ZEQU
        DD      ZBRAN
        DD      THROW2-$-CW
        DD      LERROR
        DD      CLS
        DD      QUIT
THROW2:
        DD      HANDLER, FETCH, RPSTO
        DD      FROMR, HANDLER, STOR
        DD      FROMR, SWAP, TOR
        DD      SPSTO
        DD      FROMR
        DD      X
THROW1:
        DD      DROP
        DD      SEMIS
;

; 

; 
;

;  ***********
;  *   ID.   *
;  ***********
;
        ALIGN    4
N_IDDOT:
        DD      3
        DB      "ID."
        ALIGN    4
IDDOT:
        DD    DOCOL
        DD    IDDOT+HEADSIZE
        DD    0x0
        DD    THROW
        DD    N_IDDOT
        DD    0

        DD      LDUP, TFFA, FETCH
        DD      LIT, B_DUMMY, LAND
        DD      ZBRAN
        DD      IDDOT1-$-CW
        DD      DROP
        DD      BRAN
        DD      IDDOT2-$-CW
IDDOT1:
        DD      TNFA
        DD      FETCH
        DD      SFET
        DD      LTYPE
        DD      SPACE
        DD      SPACE
        DD      SPACE
IDDOT2:
        DD      SEMIS
;

;  **************
;  *   HEADER   *
;  **************
;
        ALIGN    4
N_LHEAD:
        DD      6
        DB      "HEADER"
        ALIGN    4
LHEAD:
        DD    DOCOL
        DD    LHEAD+HEADSIZE
        DD    0x0
        DD    IDDOT
        DD    N_LHEAD
        DD    0

        DD      LDUP
        DD      ZEQU
        DD      LIT, 5, QERR
        DD      TDUP
        DD      PRESENT
        DD      LDUP
        DD      ZBRAN
        DD      CREA1-$-CW ;IF
        DD      TNFA, FETCH, SFET
        DD      ETYPE
        DD      LIT,4
        DD      MESS
        DD      X       ;THEN
CREA1:  DD      DROP
 DD      LALIGN
        DD      SCOMMA
 DD      LALIGN
        DD      HERE,TOR

        DD      RR, TPHA, COMMA         ;Code field.
        DD      RR, TPHA, COMMA         ;Data field.
        DD      ZERO, COMMA ;Flag field.
        DD      ZERO, COMMA ;Link field.
        DD      COMMA   ;Name field.


        DD      BLK, FETCH, LDUP, ZEQU
        DD      ZBRAN
        DD      CREA2-$-CW
        DD      DROP, PP, FETCH
CREA2:  DD      COMMA  ;Source field.

                DD      FROMR
        DD      SEMIS
;

;  ************
;  *   LINK   *
;  ************
;
        ALIGN    4
N_LLINK:
        DD      4
        DB      "LINK"
        ALIGN    4
LLINK:
        DD    DOCOL
        DD    LLINK+HEADSIZE
        DD    0x0
        DD    LHEAD
        DD    N_LLINK
        DD    0

        DD      TLFA
        DD      TDUP
        DD      FETCH
        DD      SWAP, TLFA, STOR
        DD      STOR
        DD      SEMIS
;

;  ****************
;  *   (CREATE)   *
;  ****************
;
        ALIGN    4
N_PCREAT:
        DD      8
        DB      "(CREATE)"
        ALIGN    4
PCREAT:
        DD    DOCOL
        DD    PCREAT+HEADSIZE
        DD    0x0
        DD    LLINK
        DD    N_PCREAT
        DD    0

        DD      LHEAD
        DD      CURR, FETCH
        DD      LLINK
        DD      SEMIS
;

;  *****************
;  *   [COMPILE]   *
;  *****************
;
        ALIGN    4
N_BCOMP:
        DD      9
        DB      "[COMPILE]"
        ALIGN    4
BCOMP:
        DD    DOCOL
        DD    BCOMP+HEADSIZE
        DD    B_IMMED
        DD    PCREAT
        DD    N_BCOMP
        DD    0

        DD      LNAME
        DD      PRESENT
        DD      LDUP
        DD      ZEQU
        DD      LIT, 16, QERR
        DD      TCFA
        DD      COMMA
        DD      SEMIS
;

;  ****************
;  *   POSTPONE   *
;  ****************
;
        ALIGN    4
N_POSTP:
        DD      8
        DB      "POSTPONE"
        ALIGN    4
POSTP:
        DD    DOCOL
        DD    POSTP+HEADSIZE
        DD    B_IMMED
        DD    BCOMP
        DD    N_POSTP
        DD    0

        DD      LNAME
        DD      PRESENT
        DD      LDUP
        DD      ZEQU
        DD      LIT, 15, QERR
        DD      LDUP, TFFA, FETCH
        DD      LIT, B_IMMED, LAND, ZEQU
        DD      ZBRAN
        DD      POSTP1-$-CW
         DD      LIT, LIT, COMMA
         DD      COMMA
         DD      LIT, COMMA
POSTP1:
         DD      COMMA
        DD      SEMIS
;

;  ***************
;  *   LITERAL   *
;  ***************
;
        ALIGN    4
N_LITER:
        DD      7
        DB      "LITERAL"
        ALIGN    4
LITER:
        DD    DOCOL
        DD    LITER+HEADSIZE
        DD    B_IMMED
        DD    POSTP
        DD    N_LITER
        DD    0

        DD      STATE
        DD      FETCH
        DD      ZBRAN
        DD      LITE1-$-CW ;IF
        DD      LIT, LIT, COMMA
        DD      COMMA   ;THEN
LITE1:  DD      SEMIS
;

;  ****************
;  *   DLITERAL   *
;  ****************
;
        ALIGN    4
N_DLITE:
        DD      8
        DB      "DLITERAL"
        ALIGN    4
DLITE:
        DD    DOCOL
        DD    DLITE+HEADSIZE
        DD    B_IMMED
        DD    LITER
        DD    N_DLITE
        DD    0

        DD      STATE
        DD      FETCH
        DD      ZBRAN
        DD      DLIT1-$-CW ; IF
        DD      SWAP
        DD      LITER
        DD      LITER   ; THEN
DLIT1:  DD      SEMIS
;
;

;  *****************
;  *   SDLITERAL   *
;  *****************
;
        ALIGN    4
N_SDLITE:
        DD      9
        DB      "SDLITERAL"
        ALIGN    4
SDLITE:
        DD    DOCOL
        DD    SDLITE+HEADSIZE
        DD    B_IMMED
        DD    DLITE
        DD    N_SDLITE
        DD    0

        DD      DPL
        DD      FETCH
        DD      ZBRAN
        DD      SDLIT1-$-CW ; IF
        DD      DLITE
        DD      BRAN
        DD      SDLIT2-$-CW ; IF
SDLIT1:
        DD      DROP, LITER
SDLIT2:
        DD      SEMIS
;


;  **************
;  *   ?STACK   *
;  **************
;
        ALIGN    4
N_QSTAC:
        DD      6
        DB      "?STACK"
        ALIGN    4
QSTAC:
        DD    DOCOL
        DD    QSTAC+HEADSIZE
        DD    0x0
        DD    SDLITE
        DD    N_QSTAC
        DD    0

        DD      SPFET
        DD      SZERO
        DD      FETCH
        DD      SWAP
        DD      ULESS
        DD      ONE, QERR
        DD      SPFET
        DD      HERE
        DD      LIT,0x80
        DD      PLUS
        DD      ULESS
        DD      LIT, 7, QERR
        DD      SEMIS
        ;
;
;

;  *****************
;  *   INTERPRET   *
;  *****************
;
        ALIGN    4
N_INTER:
        DD      9
        DB      "INTERPRET"
        ALIGN    4
INTER:
        DD    DOCOL
        DD    INTER+HEADSIZE
        DD    0x0
        DD    QSTAC
        DD    N_INTER
        DD    0

INTE1:
        DD      LNAME
        DD      LDUP      ; Zero length.
        DD      ZBRAN
        DD      INTE8-$-CW ;WHILE
;       DC      DOTS
;       DC      TDUP, LTYPE
        DD      OVER, TOR       ; Save old parse pointer.
        DD      FOUND
        DD      LDUP, ZEQU
        DD      LIT, 12, QERR
        DD      LDUP, TFFA, FETCH
        DD      LDUP, LIT, B_DENOT, LAND ;Retain copy of flags.
        DD      ZBRAN
        DD      INTE3B-$-CW ;IF
        DD      OVER, TNFA, FETCH, FETCH
        DD      RR, PLUS, PP, STOR  ;Skip over prefix.
INTE3B:                  ;THEN 
        DD      RDROP           ; Drop old parse pointer.
        DD      LIT, B_IMMED, LAND
        DD      STATE, FETCH, ZEQU, LOR
        DD      ZBRAN
        DD      INTE3-$-CW ;IF
        DD      EXEC
        DD      BRAN
        DD      INTE4-$-CW ;IF
INTE3:
        DD      COMMA
                        ;THEN
INTE4:
        DD      QSTAC
        DD      BRAN
        DD      INTE1-$-CW  ;AGAIN
INTE8:  DD      DROP, DROP
        DD      SEMIS
;

;  *****************
;  *   IMMEDIATE   *
;  *****************
;
        ALIGN    4
N_IMMED:
        DD      9
        DB      "IMMEDIATE"
        ALIGN    4
IMMED:
        DD    DOCOL
        DD    IMMED+HEADSIZE
        DD    0x0
        DD    INTER
        DD    N_IMMED
        DD    0

        DD      LATEST
        DD      TFFA
        DD      LIT, B_IMMED
        DD      TOGGL
        DD      SEMIS
;

;  **************
;  *   PREFIX   *
;  **************
;
        ALIGN    4
N_PREFX:
        DD      6
        DB      "PREFIX"
        ALIGN    4
PREFX:
        DD    DOCOL
        DD    PREFX+HEADSIZE
        DD    0x0
        DD    IMMED
        DD    N_PREFX
        DD    0

        DD      LATEST
        DD      TFFA
        DD      LIT, B_DENOT
        DD      TOGGL
        DD      SEMIS
;

;  *****************
;  *   NAMESPACE   *
;  *****************
;
        ALIGN    4
N_VOCAB:
        DD      9
        DB      "NAMESPACE"
        ALIGN    4
VOCAB:
        DD    DOCOL
        DD    VOCAB+HEADSIZE
        DD    0x0
        DD    PREFX
        DD    N_VOCAB
        DD    0

        DD      LCREATE
        DD      LATEST   ; Link this DEA into VOC-LINK chain.
        DD      VOCL
        DD      FETCH
        DD      COMMA
        DD      VOCL
        DD      STOR
        DD      WORDLIST, DROP 
        DD      DOES
DOVOC:
        DD      ALSO
        DD      CELLP   ; Make it a WID. 
        DD      CONTEXT
        DD      STOR
        DD      SEMIS
        ;
;
;   The link to task is a cold start value only.
;   It is updated each time a definition is
;   appended to the 'FORTH' namespace.
;

;  ****************
;  *   WORDLIST   *
;  ****************
;
        ALIGN    4
N_WORDLIST:
        DD      8
        DB      "WORDLIST"
        ALIGN    4
WORDLIST:
        DD    DOCOL
        DD    WORDLIST+HEADSIZE
        DD    0x0
        DD    VOCAB
        DD    N_WORDLIST
        DD    0

        DD      HERE
        DD      ZERO, COMMA   ; Dummy code field
        DD      ZERO, COMMA   ; Dummy data field
        DD      LIT, B_DUMMY, COMMA ; Dummy flag field
        DD      ZERO, COMMA ;Link field: empty chain
        DD      LIT, (ZERO+(CW*(D_HOFFSET))), COMMA ;Empty string for name.
        DD      ZERO, COMMA
                DD      SEMIS
        ;
;

;

;  *******************
;  *   DEFINITIONS   *
;  *******************
;
        ALIGN    4
N_DEFIN:
        DD      11
        DB      "DEFINITIONS"
        ALIGN    4
DEFIN:
        DD    DOCOL
        DD    DEFIN+HEADSIZE
        DD    0x0
        DD    WORDLIST
        DD    N_DEFIN
        DD    0

        DD      CONTEXT
        DD      FETCH
        DD      CURR
        DD      STOR
        DD      SEMIS
;

;  ************
;  *   ALSO   *
;  ************
;
        ALIGN    4
N_ALSO:
        DD      4
        DB      "ALSO"
        ALIGN    4
ALSO:
        DD    DOCOL
        DD    ALSO+HEADSIZE
        DD    0x0
        DD    DEFIN
        DD    N_ALSO
        DD    0

        DD      CONTEXT, LDUP, CELLP
        DD      LIT, (CW*(16-1))
        DD      LMOVE
        DD      LIT, ONLYBODY  ;End sentinel for array of word lists.
        DD      CONTEXT, LIT, (CW*(16)), PLUS
        DD      STOR ;Trim sets of wordset.
        DD      SEMIS
;

;  ****************
;  *   PREVIOUS   *
;  ****************
;
        ALIGN    4
N_PREVI:
        DD      8
        DB      "PREVIOUS"
        ALIGN    4
PREVI:
        DD    DOCOL
        DD    PREVI+HEADSIZE
        DD    0x0
        DD    ALSO
        DD    N_PREVI
        DD    0

        DD      CONTEXT, LDUP, CELLP, SWAP
        DD      LIT, (CW*(16))
        DD      LMOVE
        DD      SEMIS
;

;  *********
;  *   (   *
;  *********
;
        ALIGN    4
N_PAREN:
        DD      1
        DB      "("
        ALIGN    4
PAREN:
        DD    DOCOL
        DD    PAREN+HEADSIZE
        DD    B_IMMED
        DD    PREVI
        DD    N_PAREN
        DD    0

        DD      LIT,')'
        DD      PARSE
        DD      TDROP
        DD      SEMIS
;
;

;  *********
;  *   \   *
;  *********
;
        ALIGN    4
N_BACKS:
        DD      1
        DB      "\"
        ALIGN    4
BACKS:
        DD    DOCOL
        DD    BACKS+HEADSIZE
        DD    B_IMMED
        DD    PAREN
        DD    N_BACKS
        DD    0

; Backup one character, just in case we are at the end of a line.
        DD      LIT, -1, PP, PSTOR
        DD      LIT,ALF
        DD      PARSE
        DD      TDROP
        DD      SEMIS
;

;  ************
;  *   QUIT   *
;  ************
;
        ALIGN    4
N_QUIT:
        DD      4
        DB      "QUIT"
        ALIGN    4
QUIT:
        DD    DOCOL
        DD    QUIT+HEADSIZE
        DD    0x0
        DD    BACKS
        DD    N_QUIT
        DD    0

        DD      LBRAC
QUIT1:                  ;BEGIN
        DD      RZERO
        DD      FETCH
        DD      RPSTO

; debugging 
;        DC      REFTIB  
; end debugging 

        DD      LIT
        DD      PACCEP
        DD      CATCH
        DD      LDUP, LIT, -EPIPE, EQUAL
        DD      ZBRAN
        DD      ENDIF7-$-CW
        DD      BYE     ;End of input, no error!
ENDIF7:
        DD      QERRUR
        DD      ONEP
; debugging        DC      TDUP, LTYPE, CR 
        DD      SETSRC
        DD      INTER
        DD      OK
        DD      BRAN
        DD      QUIT1-$-CW  ;AGAIN
        DD      SEMIS   ;Unnecessary, but helpful for decompilation.
;

;  **********
;  *   OK   *
;  **********
;
        ALIGN    4
N_OK:
        DD      2
        DB      "OK"
        ALIGN    4
OK:
        DD    DOCOL
        DD    OK+HEADSIZE
        DD    0x0
        DD    QUIT
        DD    N_OK
        DD    0

        DD      STATE
        DD      FETCH
        DD      ZEQU
        DD      ZBRAN
        DD      OK2-$-CW ;IF
        DD      SKIP
         DD      3
SB2: DB      " OK"
       ALIGN    4
        DD      LIT, SB2
        DD      LIT, 3
        DD      LTYPE
        DD      CR
OK2:
        DD      SEMIS
;


;  ***************
;  *   OPTIONS   *
;  ***************
;
        ALIGN    4
N_OPTIONS:
        DD      7
        DB      "OPTIONS"
        ALIGN    4
OPTIONS:
        DD    DOCOL
        DD    OPTIONS+HEADSIZE
        DD    0x0
        DD    OK
        DD    N_OPTIONS
        DD    0

;       Execute option.
        DD      ARGS, FETCH, CELLP, CELLP, FETCH, LDUP
        DD      ZBRAN
        DD      OPT4-$-CW ; No options  




;Supposedly on the stack is now a pointer to the option.
        DD      COUNT

        DD      LIT, '-', UNEQ
        DD      ZBRAN
        DD      OPT3-$-CW
        DD      LIT, 3, LDUP, LERROR
        DD      XCODE, STOR, BYE
OPT3:
        DD      CFET
        DD      LIT, 0x1F, LAND
        DD      LLOAD       ;Supposedly stack-neutral.
        DD      ZERO, SWAP ; Sign on suppressed.
OPT4:
        DD      DROP
        DD      SEMIS   ;Unnecessary, but helpful for decompilation.
;


;  *************
;  *   _INIT   *
;  *************
;
        ALIGN    4
N_INIT:
        DD      5
        DB      "_INIT"
        ALIGN    4
INIT:
        DD    DOCOL
        DD    INIT+HEADSIZE
        DD    0x0
        DD    OPTIONS
        DD    N_INIT
        DD    0

        DD      SZERO, FETCH, SPSTO
        DD      ZERO, HANDLER, STOR
        DD      DECA
        DD      ONLY, FORTH, DEFIN

        DD      LIT, 0, BLINI  ;Default, open library file read-only!;

        DD      SEMIS
;


;  ***********
;  *   S>D   *
;  ***********
;
        ALIGN    4
N_STOD:
        DD      3
        DB      "S>D"
        ALIGN    4
STOD:
        DD    STOD+HEADSIZE
        DD    STOD+HEADSIZE
        DD    0x0
        DD    INIT
        DD    N_STOD
        DD    0

        LDR     R1, [R13],#+(CW*(1))      ;S1
        MOV    R0,#0
        ANDS    R1,R1,R1
        BPL     STOD1   ;POS
        SUB     R0,R0, #1      ;NEG
STOD1:
        STR     R1, [R13,#-(CW*(1))]!
        STR     R0, [R13,#-(CW*(1))]!
        LDR     R0, [R11],#+(CW*(1))
         LDR     R1,[R0]
         BX      R1  
;

;  ***********
;  *   ABS   *
;  ***********
;
        ALIGN    4
N_LABS:
        DD      3
        DB      "ABS"
        ALIGN    4
LABS:
        DD    DOCOL
        DD    LABS+HEADSIZE
        DD    0x0
        DD    STOD
        DD    N_LABS
        DD    0

        DD      LDUP
        DD      ZLESS
        DD      ZBRAN
        DD      PM1-$-CW   ;IF
        DD      NEGATE   ;THEN
PM1:
        DD      SEMIS
;

;  ************
;  *   DABS   *
;  ************
;
        ALIGN    4
N_DABS:
        DD      4
        DB      "DABS"
        ALIGN    4
DABS:
        DD    DOCOL
        DD    DABS+HEADSIZE
        DD    0x0
        DD    LABS
        DD    N_DABS
        DD    0

        DD      LDUP
        DD      ZLESS
        DD      ZBRAN
        DD      DPM1-$-CW  ;IF
        DD      DNEGA   ;THEN
DPM1:
        DD      SEMIS
;

;  ***********
;  *   MIN   *
;  ***********
;
        ALIGN    4
N_MIN:
        DD      3
        DB      "MIN"
        ALIGN    4
MIN:
        DD    DOCOL
        DD    MIN+HEADSIZE
        DD    0x0
        DD    DABS
        DD    N_MIN
        DD    0

        DD      TDUP
        DD      GREAT
        DD      ZBRAN
        DD      MIN1-$-CW  ;IF
        DD      SWAP    ;THEN
MIN1:   DD      DROP
        DD      SEMIS
;

;  ***********
;  *   MAX   *
;  ***********
;
        ALIGN    4
N_MAX:
        DD      3
        DB      "MAX"
        ALIGN    4
MAX:
        DD    DOCOL
        DD    MAX+HEADSIZE
        DD    0x0
        DD    MIN
        DD    N_MAX
        DD    0

        DD      TDUP
        DD      LESS
        DD      ZBRAN
        DD      MAX1-$-CW  ;IF
        DD      SWAP    ;THEN
MAX1:   DD      DROP
        DD      SEMIS
;

;  **************
;  *   LSHIFT   *
;  **************
;
        ALIGN    4
N_LSHIFT:
        DD      6
        DB      "LSHIFT"
        ALIGN    4
LSHIFT:
        DD    LSHIFT+HEADSIZE
        DD    LSHIFT+HEADSIZE
        DD    0x0
        DD    MAX
        DD    N_LSHIFT
        DD    0

        LDR     R2, [R13],#+(CW*(1))
        LDR     R0, [R13],#+(CW*(1))
         LSL R0,R0, R2
        STR     R0, [R13,#-(CW*(1))]!
        LDR     R0, [R11],#+(CW*(1))
         LDR     R1,[R0]
         BX      R1  
;

;  **************
;  *   RSHIFT   *
;  **************
;
        ALIGN    4
N_RSHIFT:
        DD      6
        DB      "RSHIFT"
        ALIGN    4
RSHIFT:
        DD    RSHIFT+HEADSIZE
        DD    RSHIFT+HEADSIZE
        DD    0x0
        DD    LSHIFT
        DD    N_RSHIFT
        DD    0

        LDR     R2, [R13],#+(CW*(1))
        LDR     R0, [R13],#+(CW*(1))
        LSR R0,R0, R2
         STR     R0, [R13,#-(CW*(1))]!
        LDR     R0, [R11],#+(CW*(1))
         LDR     R1,[R0]
         BX      R1  
;

;  ***************
;  *   ARSHIFT   *
;  ***************
;
        ALIGN    4
N_ARSHIFT:
        DD      7
        DB      "ARSHIFT"
        ALIGN    4
ARSHIFT:
        DD    ARSHIFT+HEADSIZE
        DD    ARSHIFT+HEADSIZE
        DD    0x0
        DD    RSHIFT
        DD    N_ARSHIFT
        DD    0

        LDR     R2, [R13],#+(CW*(1))
        LDR     R0, [R13],#+(CW*(1))
        ASR R0,R0, R2
        STR     R0, [R13,#-(CW*(1))]!
        LDR     R0, [R11],#+(CW*(1))
         LDR     R1,[R0]
         BX      R1  
;

;  **********
;  *   M*   *
;  **********
;
        ALIGN    4
N_MSTAR:
        DD      2
        DB      "M*"
        ALIGN    4
MSTAR:
        DD    MSTAR+HEADSIZE
        DD    MSTAR+HEADSIZE
        DD    0x0
        DD    ARSHIFT
        DD    N_MSTAR
        DD    0

        LDR     R3, [R13],#+(CW*(1))
        LDR     R2, [R13],#+(CW*(1))
 ; end thumb  )
        SMULL    R1,R0,R2,R3          ;Signed *, 64 bits result  
        STR     R1, [R13,#-(CW*(1))]!
        STR     R0, [R13,#-(CW*(1))]!
        LDR     R0, [R11],#+(CW*(1))
         LDR     R1,[R0]
         BX      R1            ;STORE DOUBLE CELL
;

; High level implementation from FIG-forth 
;  **************
;  *   SM/REM   *
;  **************
;
        ALIGN    4
N_MSLAS:
        DD      6
        DB      "SM/REM"
        ALIGN    4
MSLAS:
        DD    DOCOL
        DD    MSLAS+HEADSIZE
        DD    0x0
        DD    MSTAR
        DD    N_MSLAS
        DD    0

        DD      OVER, TOR, TOR, DABS  ;   OVER >R >R DABS 
        DD      RR, LABS, USLAS       ;   R ABS UM/MOD 
        DD      FROMR, RR, LXOR       ;   R> R XOR 
        DD      ZLESS                 ;   <0 IF NEGATE THEN 
        DD      ZBRAN
        DD      MSLAS1-$-CW
        DD      NEGATE
MSLAS1:
        DD      SWAP, FROMR           ;   SWAP R> 
        DD      ZLESS                 ;   <0 IF NEGATE THEN 
        DD      ZBRAN
        DD      MSLAS2-$-CW
        DD      NEGATE
MSLAS2:
        DD      SWAP                  ;   SWAP ; 
        DD      SEMIS
;
_COMMENTED({
CODE_HEADER({SM/REM},{MSLAS})
        POP(R8)      _C{DIVISOR}
        POP(R10)     _C{MSW OF DIVIDEND}
        POP(R0)      _C{LSW OF DIVIDEND}
        BL           IDIV _C{ not yet 64 bit/32 bit signed}
        _2PUSH_R10          _C{STORE QUOT/REM}
_C


_C{ Divide routine 32/32 signed from Henry Thomas (http://me.henri.net/fp-div.html)}
_C{ Entry  R0: numerator (lo) must be signed positive}
_C{        R8: denominator (den aka divisor) must be non-zero and signed negative}
IDIV:

        lo .req R0
        hi .req R10
        den .req R8
        MOVI  R5, #0   _C{ R5 is sign-memory of den}
        CMP den, #0
        BMI IDIV1
        MVN den,den  _C{ den  <-- two-complement of den}
        ADD den, den, #1
        MOVI  R5, #-1
IDIV1:  MOVI  R6, #0   _C{ R6 is sign-memory of lo}
        CMP lo,#0
        BPL IDIV2
        MVN lo,lo    _C{ lo <-- two-complement of lo}
        ADD lo,lo,#1
        MOVI  R6,#-1

_C{the original routine:}
_C{with positive numerator and negative denominator}

IDIV2:  MOVI  hi, #0 _C{ hi = 0}
        ADDS lo, lo, lo
        .rept 32 _C{ repeat 32 times}
          ADCS hi, den, hi, lsl #1
          SUBCC hi, hi, den
          ADCS lo,lo
        .endr
_C{end of original routine}

        _C{ remainder has sign of numerator}
        CMP R6, #0
        BEQ IDIV3
        MVN hi, hi  _C{ hi  <-- two-complement of remainder}
        ADD hi, hi, #1

IDIV3:
        _C{ quotient has both signs of numerator and denominator}
        CMP R5,R6
        BNE IDIV4
        MVN lo,lo    _C{ lo <-- two-complement of lo}
        ADD lo,lo,#1
IDIV4:
        MOV PC, LR _C{ return}
_C{Exit   R0:  quotient (lo)}
_C{       R10: remainder (hi)}
}) 


;  **********
;  *   2/   *
;  **********
;
        ALIGN    4
N_TWOSL:
        DD      2
        DB      "2/"
        ALIGN    4
TWOSL:
        DD    DOCOL
        DD    TWOSL+HEADSIZE
        DD    0x0
        DD    MSLAS
        DD    N_TWOSL
        DD    0

        DD      STOD, TWO, FMSLAS
        DD      NIP
        DD      SEMIS
;

;  **********
;  *   2*   *
;  **********
;
        ALIGN    4
N_TWOST:
        DD      2
        DB      "2*"
        ALIGN    4
TWOST:
        DD    DOCOL
        DD    TWOST+HEADSIZE
        DD    0x0
        DD    TWOSL
        DD    N_TWOST
        DD    0

        DD      TWO, STAR
        DD      SEMIS
;

;  **********
;  *   1-   *
;  **********
;
        ALIGN    4
N_ONEM:
        DD      2
        DB      "1-"
        ALIGN    4
ONEM:
        DD    DOCOL
        DD    ONEM+HEADSIZE
        DD    0x0
        DD    TWOST
        DD    N_ONEM
        DD    0

        DD      ONE, LSUB
        DD      SEMIS
;

;  **************
;  *   FM/MOD   *
;  **************
;
        ALIGN    4
N_FMSLAS:
        DD      6
        DB      "FM/MOD"
        ALIGN    4
FMSLAS:
        DD    DOCOL
        DD    FMSLAS+HEADSIZE
        DD    0x0
        DD    ONEM
        DD    N_FMSLAS
        DD    0

        DD      LDUP, TOR
        DD      TDUP, LXOR, TOR
        DD      MSLAS
        DD      FROMR, ZLESS
        DD      ZBRAN
        DD      FMMOD1-$-CW
        DD      OVER
        DD      ZBRAN
        DD      FMMOD1-$-CW
        DD      ONE, LSUB
        DD      SWAP, FROMR, PLUS, SWAP
        DD      BRAN
        DD      FMMOD2-$-CW
FMMOD1:
        DD      RDROP
FMMOD2:
        DD      SEMIS
;

;  *********
;  *   *   *
;  *********
;
        ALIGN    4
N_STAR:
        DD      1
        DB      "*"
        ALIGN    4
STAR:
        DD    DOCOL
        DD    STAR+HEADSIZE
        DD    0x0
        DD    FMSLAS
        DD    N_STAR
        DD    0

        DD      MSTAR
        DD      DROP
        DD      SEMIS
;

;  ************
;  *   /MOD   *
;  ************
;
        ALIGN    4
N_SLMOD:
        DD      4
        DB      "/MOD"
        ALIGN    4
SLMOD:
        DD    DOCOL
        DD    SLMOD+HEADSIZE
        DD    0x0
        DD    STAR
        DD    N_SLMOD
        DD    0

        DD      TOR
        DD      STOD
        DD      FROMR
        DD      MSLAS
        DD      SEMIS
;

;  *********
;  *   /   *
;  *********
;
        ALIGN    4
N_SLASH:
        DD      1
        DB      "/"
        ALIGN    4
SLASH:
        DD    DOCOL
        DD    SLASH+HEADSIZE
        DD    0x0
        DD    SLMOD
        DD    N_SLASH
        DD    0

        DD      SLMOD
        DD      NIP
        DD      SEMIS
;

;  ***********
;  *   MOD   *
;  ***********
;
        ALIGN    4
N_LMOD:
        DD      3
        DB      "MOD"
        ALIGN    4
LMOD:
        DD    DOCOL
        DD    LMOD+HEADSIZE
        DD    0x0
        DD    SLASH
        DD    N_LMOD
        DD    0

        DD      SLMOD
        DD      DROP
        DD      SEMIS
;

;  *************
;  *   */MOD   *
;  *************
;
        ALIGN    4
N_SSMOD:
        DD      5
        DB      "*/MOD"
        ALIGN    4
SSMOD:
        DD    DOCOL
        DD    SSMOD+HEADSIZE
        DD    0x0
        DD    LMOD
        DD    N_SSMOD
        DD    0

        DD      TOR
        DD      MSTAR
        DD      FROMR
        DD      MSLAS
        DD      SEMIS
;

;  **********
;  *   */   *
;  **********
;
        ALIGN    4
N_SSLA:
        DD      2
        DB      "*/"
        ALIGN    4
SSLA:
        DD    DOCOL
        DD    SSLA+HEADSIZE
        DD    0x0
        DD    SSMOD
        DD    N_SSLA
        DD    0

        DD      SSMOD
        DD      NIP
        DD      SEMIS
;

;  *************
;  *   M/MOD   *
;  *************
;
        ALIGN    4
N_MSMOD1:
        DD      5
        DB      "M/MOD"
        ALIGN    4
MSMOD1:
        DD    DOCOL
        DD    MSMOD1+HEADSIZE
        DD    0x0
        DD    SSLA
        DD    N_MSMOD1
        DD    0

;  ***************
;  *   UDM/MOD   *
;  ***************
;
        ALIGN    4
N_MSMOD:
        DD      7
        DB      "UDM/MOD"
        ALIGN    4
MSMOD:
        DD    DOCOL
        DD    MSMOD+HEADSIZE
        DD    0x0
        DD    MSMOD1
        DD    N_MSMOD
        DD    0

        DD      TOR
        DD      ZERO
        DD      RR
        DD      USLAS
        DD      FROMR
        DD      SWAP
        DD      TOR
        DD      USLAS
        DD      FROMR
        DD      SEMIS
;

;  **************
;  *   (LINE)   *
;  **************
;
        ALIGN    4
N_PLINE:
        DD      6
        DB      "(LINE)"
        ALIGN    4
PLINE:
        DD    DOCOL
        DD    PLINE+HEADSIZE
        DD    0x0
        DD    MSMOD
        DD    N_PLINE
        DD    0

        DD      TOR
        DD      LIT,64
        DD      MSTAR
        DD      BBUF
        DD      FMSLAS
        DD      FROMR ; This blocks, so is screens.
        DD      PLUS
        DD      BLOCK
        DD      PLUS
        DD      LIT,63
        DD      SEMIS
;

;  **************
;  *   ERRSCR   *
;  **************
;
        ALIGN    4
N_ERRSCR:
        DD      6
        DB      "ERRSCR"
        ALIGN    4
ERRSCR:
        DD    DOVAR
        DD    ERRSCR+HEADSIZE
        DD    0x0
        DD    PLINE
        DD    N_ERRSCR
        DD    0

        DD ERRORSCREEN
;

;  ***************
;  *   MESSAGE   *
;  ***************
;
        ALIGN    4
N_MESS:
        DD      7
        DB      "MESSAGE"
        ALIGN    4
MESS:
        DD    DOCOL
        DD    MESS+HEADSIZE
        DD    0x0
        DD    ERRSCR
        DD    N_MESS
        DD    0

        DD      LWARN
        DD      FETCH
        DD      ZBRAN
        DD      MESS1-$-CW ;IF
        DD      ERRSCR, FETCH
        DD      PLINE, ONEP     ; Also print the '\n' !
        DD      ETYPE
        DD      X
MESS1:                  ;THEN
        DD      DROP
        DD      SEMIS
;
;
;

;
;
_COMMENTED(
 Dit blok  PC@,PC!,PW@,PW! eerst maar even uitgeschakeld voor ARM !
 Dit moet waarschijnlijk helemaal anders als voor Intel
) 
;  ***********
;  *   PC@   *
;  ***********
;
        ALIGN    4
N_PCFET:
        DD      3
        DB      "PC@"
        ALIGN    4
PCFET:
        DD    PCFET+HEADSIZE
        DD    PCFET+HEADSIZE
        DD    0x0
        DD    MESS
        DD    N_PCFET
        DD    0

; FETCH CHARACTER (BYTE) FROM PORT
        LDR     R1, [R13],#+(CW*(1))      ; PORT ADDR
        MOV     R0,#0
;todo        IN      AL,_DX16   ; BYTE INPUT
        STR     R0, [R13,#-(CW*(1))]!
        LDR     R0, [R11],#+(CW*(1))
         LDR     R1,[R0]
         BX      R1  
;

;  ***********
;  *   PC!   *
;  ***********
;
        ALIGN    4
N_PCSTO:
        DD      3
        DB      "PC!"
        ALIGN    4
PCSTO:
        DD    PCSTO+HEADSIZE
        DD    PCSTO+HEADSIZE
        DD    0x0
        DD    PCFET
        DD    N_PCSTO
        DD    0

        LDR     R1, [R13],#+(CW*(1))      ;PORT ADDR
        LDR     R0, [R13],#+(CW*(1))      ;DATA
;todo        OUT     _DX16,AL   ; BYTE OUTPUT
        LDR     R0, [R11],#+(CW*(1))
         LDR     R1,[R0]
         BX      R1  
;

;  ***********
;  *   PW@   *
;  ***********
;
        ALIGN    4
N_PFET:
        DD      3
        DB      "PW@"
        ALIGN    4
PFET:
        DD    PFET+HEADSIZE
        DD    PFET+HEADSIZE
        DD    0x0
        DD    PCSTO
        DD    N_PFET
        DD    0

        LDR     R1, [R13],#+(CW*(1))      ;PORT ADDR
;todo        IN      _AX16,_DX16  ;WORD INPUT
        STR     R0, [R13,#-(CW*(1))]!
        LDR     R0, [R11],#+(CW*(1))
         LDR     R1,[R0]
         BX      R1  
;

;  ***********
;  *   PW!   *
;  ***********
;
        ALIGN    4
N_PSTO:
        DD      3
        DB      "PW!"
        ALIGN    4
PSTO:
        DD    PSTO+HEADSIZE
        DD    PSTO+HEADSIZE
        DD    0x0
        DD    PFET
        DD    N_PSTO
        DD    0

        LDR     R1, [R13],#+(CW*(1))      ;PORT ADDR
        LDR     R0, [R13],#+(CW*(1))      ;DATA
;todo        OUT     _DX16,_AX16   ;WORD OUTPUT
        LDR     R0, [R11],#+(CW*(1))
         LDR     R1,[R0]
         BX      R1  
;
;
;

;  *************
;  *   _PREV   *
;  *************
;
        ALIGN    4
N__PREV:
        DD      5
        DB      "_PREV"
        ALIGN    4
_PREV:
        DD    DOVAR
        DD    _PREV+HEADSIZE
        DD    0x0
        DD    PSTO
        DD    N__PREV
        DD    0

        DD      BUF1

;
;

;  *************
;  *   #BUFF   *
;  *************
;
        ALIGN    4
N_NOBUF:
        DD      5
        DB      "#BUFF"
        ALIGN    4
NOBUF:
        DD    DOCON
        DD    NBUF
        DD    0x0
        DD    _PREV
        DD    N_NOBUF
        DD    0

;

;  ************
;  *   +BUF   *
;  ************
;
        ALIGN    4
N_PBUF:
        DD      4
        DB      "+BUF"
        ALIGN    4
PBUF:
        DD    DOCOL
        DD    PBUF+HEADSIZE
        DD    0x0
        DD    NOBUF
        DD    N_PBUF
        DD    0

        DD      BBUF, CELLP, CELLP
        DD      PLUS,LDUP
        DD      _LIMIT,EQUAL
        DD      ZBRAN
        DD      PBUF1-$-CW
        DD      DROP,_FIRST
PBUF1:  DD      LDUP, _PREV, FETCH, LSUB
        DD      SEMIS
;

;  **************
;  *   UPDATE   *
;  **************
;
        ALIGN    4
N_UPDAT:
        DD      6
        DB      "UPDATE"
        ALIGN    4
UPDAT:
        DD    DOCOL
        DD    UPDAT+HEADSIZE
        DD    0x0
        DD    PBUF
        DD    N_UPDAT
        DD    0

        DD      _PREV, FETCH
        DD      LDUP, CELLP,CELLP
        DD      SWAP, FETCH
        DD      LOFFSET,  FETCH, PLUS
        DD      BLWRITE
        DD      SEMIS
;

;  *********************
;  *   EMPTY-BUFFERS   *
;  *********************
;
        ALIGN    4
N_MTBUF:
        DD      13
        DB      "EMPTY-BUFFERS"
        ALIGN    4
MTBUF:
        DD    DOCOL
        DD    MTBUF+HEADSIZE
        DD    0x0
        DD    UPDAT
        DD    N_MTBUF
        DD    0

        DD      _FIRST
        DD      _LIMIT,OVER
        DD      LSUB,LERASE
        DD      _FIRST, _PREV, STOR
        DD      SEMIS
        ;
;

;  ****************
;  *   (BUFFER)   *
;  ****************
;
        ALIGN    4
N_BUFFER:
        DD      8
        DB      "(BUFFER)"
        ALIGN    4
BUFFER:
        DD    DOCOL
        DD    BUFFER+HEADSIZE
        DD    0x0
        DD    MTBUF
        DD    N_BUFFER
        DD    0

; Find the buffer, if it is already here.
    DD      _PREV, FETCH
BUFFER1:
    DD          TOR, RR, FETCH, OVER, EQUAL
    DD      ZBRAN
        DD      BUFFER3-$-CW
    DD        DROP, FROMR, EXIT
BUFFER3:
    DD          FROMR
    DD      PBUF, ZEQU
    DD      ZBRAN
        DD      BUFFER1-$-CW
; We are left with the previous buffer.
BUFFER2:
    DD       PBUF, ZEQU, LIT, 48, QERR  ; No free buffers available.
    DD       LDUP, CELLP, FETCH
    DD       LIT, -1, GREAT        ; Unlocked buffer?
    DD      ZBRAN
        DD      BUFFER2-$-CW
    DD       TOR
; Fill in the house keeping.
    DD       RR, STOR
    DD       ZERO, RR, CELLP, STOR

    DD       RR, _PREV, STOR
    DD       FROMR
    DD  SEMIS
;


;  *************
;  *   BLOCK   *
;  *************
;
        ALIGN    4
N_BLOCK:
        DD      5
        DB      "BLOCK"
        ALIGN    4
BLOCK:
        DD    DOCOL
        DD    BLOCK+HEADSIZE
        DD    0x0
        DD    BUFFER
        DD    N_BLOCK
        DD    0


        DD      BUFFER
        DD      LDUP, CELLP, FETCH, ONE, LAND, ZEQU
        DD      ZBRAN
        DD      BLOCK1-$-CW
        DD      LDUP, CELLP, CELLP
        DD      OVER, FETCH
        DD      LOFFSET,  FETCH, PLUS
        DD      BLREAD
        DD      LDUP, CELLP, ONE, TOGGL
BLOCK1:
        DD      LDUP, _PREV, STOR
        DD      CELLP, CELLP
        DD      SEMIS
;

;  *************
;  *   FLUSH   *
;  *************
;
        ALIGN    4
N_FLUSH:
        DD      5
        DB      "FLUSH"
        ALIGN    4
FLUSH:
        DD    DOCOL
        DD    (MTBUF+HEADSIZE)
        DD    0x0
        DD    BLOCK
        DD    N_FLUSH
        DD    0


; Unlock all buffers
        DD      _LIMIT
        DD      _FIRST, CELLP
        DD     XDO
        DD      FLUS2-$-CW
FLUS1:  DD      ZERO, IDO, STOR
        DD      LIT,(KBBUF+2*CW)
        DD      PLOOP
        DD      (FLUS1-$)
FLUS2:
        DD      SEMIS
;

;  ************
;  *   SAVE   *
;  ************
;
        ALIGN    4
N_SAVE:
        DD      4
        DB      "SAVE"
        ALIGN    4
SAVE:
        DD    DOCOL
        DD    SAVE+HEADSIZE
        DD    0x0
        DD    FLUSH
        DD    N_SAVE
        DD    0

        DD      FROMR
        DD      SRC, TFET
        DD      PP, FETCH
        DD      TOR, TOR, TOR
        DD      TOR
        DD SEMIS
;

;  ***************
;  *   RESTORE   *
;  ***************
;
        ALIGN    4
N_RESTO:
        DD      7
        DB      "RESTORE"
        ALIGN    4
RESTO:
        DD    DOCOL
        DD    RESTO+HEADSIZE
        DD    0x0
        DD    SAVE
        DD    N_RESTO
        DD    0

        DD      FROMR
        DD      FROMR, FROMR, FROMR
        DD      PP, STOR
        DD      SRC, TSTOR
        DD      TOR
        DD SEMIS
;

; 
;

; 
;

;  ************
;  *   LOCK   *
;  ************
;
        ALIGN    4
N_LLOCK:
        DD      4
        DB      "LOCK"
        ALIGN    4
LLOCK:
        DD    DOCOL
        DD    LLOCK+HEADSIZE
        DD    0x0
        DD    RESTO
        DD    N_LLOCK
        DD    0

        DD      BUFFER
        DD      CELLP
        DD      LIT, -2, SWAP, PSTOR
        DD      SEMIS
;

;  **************
;  *   UNLOCK   *
;  **************
;
        ALIGN    4
N_LUNLOCK:
        DD      6
        DB      "UNLOCK"
        ALIGN    4
LUNLOCK:
        DD    DOCOL
        DD    LUNLOCK+HEADSIZE
        DD    0x0
        DD    LLOCK
        DD    N_LUNLOCK
        DD    0

        DD      BUFFER
        DD      CELLP
        DD      TWO, SWAP, PSTOR
        DD      SEMIS
;

;  ************
;  *   LOAD   *
;  ************
;
        ALIGN    4
N_LLOAD:
        DD      4
        DB      "LOAD"
        ALIGN    4
LLOAD:
        DD    DOCOL
        DD    LLOAD+HEADSIZE
        DD    0x0
        DD    LUNLOCK
        DD    N_LLOAD
        DD    0

        DD      SAVE
        DD      TOR
        DD      RR, LIT, LLOCK, CATCH  ; Be careful about always unlocking.
        DD      LDUP, ZEQU
        DD      ZBRAN
        DD      LOAD5-$-CW
        DD      DROP
        DD      RR, BLOCK     ; Can't fail anymore. 
        DD      BBUF
        DD      SETSRC
        DD      LIT, INTER, CATCH
        DD      RR, LUNLOCK
LOAD5:
        DD      RDROP
        DD      RESTO
        DD      THROW
        DD      SEMIS
;

;  ************
;  *   THRU   *
;  ************
;
        ALIGN    4
N_THRU:
        DD      4
        DB      "THRU"
        ALIGN    4
THRU:
        DD    DOCOL
        DD    THRU+HEADSIZE
        DD    0x0
        DD    LLOAD
        DD    N_THRU
        DD    0

        DD      ONEP, SWAP
        DD     XDO
        DD      THRU2-$-CW
THRU1:
        DD      IDO, LLOAD
        DD     ONE, XPLOOP, ZBRAN
        DD      THRU1-$-CW
	DD      UNLOOP
THRU2:
        DD      SEMIS
;
;

;  ***********
;  *   BLK   *
;  ***********
;
        ALIGN    4
N_BLK:
        DD      3
        DB      "BLK"
        ALIGN    4
BLK:
        DD    DOCOL
        DD    BLK+HEADSIZE
        DD    0x0
        DD    THRU
        DD    N_BLK
        DD    0

        DD      PP, FETCH
        DD      _FIRST, _LIMIT, WITHIN
        DD      SRC, TFET, LSUB
        DD      LIT, 1024, EQUAL, LAND
        DD      ZBRAN
        DD      BLK1-$-CW
        DD      SRC, FETCH, TWO, LCELLS, LSUB, FETCH
        DD      BRAN
        DD      BLK2-$-CW
BLK1:
        DD      ZERO
BLK2:
        DD      PBLK, STOR
        DD      PBLK
        DD      SEMIS
;

;



;  ***********
;  *   XOS   *
;  ***********
;
        ALIGN    4
N_XOS:
        DD      3
        DB      "XOS"
        ALIGN    4
XOS:
        DD    XOS+HEADSIZE
        DD    XOS+HEADSIZE
        DD    0x0
        DD    BLK
        DD    N_XOS
        DD    0


        
        LDR     R7, [R13],#+(CW*(1))        ; Function number
        LDR     R2, [R13],#+(CW*(1))        ; Third parameter, or dummy
        LDR     R1, [R13],#+(CW*(1))        ; Second parameter, or dummy
        LDR     R0, [R13],#+(CW*(1))        ; First parameter.
        SWI     0        ; Generic call on LINUX 
        
        STR     R0, [R13,#-(CW*(1))]!
        LDR     R0, [R11],#+(CW*(1))
         LDR     R1,[R0]
         BX      R1       ; Positive means okay. Negative means -errno.
;

;
;
 ; 





;  ************
;  *   XOS5   *
;  ************
;
        ALIGN    4
N_XOS5:
        DD      4
        DB      "XOS5"
        ALIGN    4
XOS5:
        DD    XOS5+HEADSIZE
        DD    XOS5+HEADSIZE
        DD    0x0
        DD    XOS
        DD    N_XOS5
        DD    0

        
        ;FIXME maybe R7 needs saving. 
        LDR     R7, [R13],#+(CW*(1))        ; Function number
        LDR     R4, [R13],#+(CW*(1))        ; 5th parameter, or dummy
        LDR     R3, [R13],#+(CW*(1))        ; 4th parameter, or dummy
        LDR     R2, [R13],#+(CW*(1))        ; Third parameter, or dummy
        LDR     R1, [R13],#+(CW*(1))        ; Second parameter, or dummy
        LDR     R0, [R13],#+(CW*(1))        ; First parameter.
        SWI     0      ; Generic call on LINUX 
        
        STR     R0, [R13,#-(CW*(1))]!
        LDR     R0, [R11],#+(CW*(1))
         LDR     R1,[R0]
         BX      R1            ; Positive means okay. Negative means -errno.
;

;



;  ************
;  *   XOS7   *
;  ************
;
        ALIGN    4
N_XOS7:
        DD      4
        DB      "XOS7"
        ALIGN    4
XOS7:
        DD    XOS7+HEADSIZE
        DD    XOS7+HEADSIZE
        DD    0x0
        DD    XOS5
        DD    N_XOS7
        DD    0

        
        LDR     R7, [R13],#+(CW*(1))        ; Function number            
        LDR     R6, [R13],#+(CW*(1))        ; 7th parameter, or dummy
        LDR     R5, [R13],#+(CW*(1))        ; 6th parameter, or dummy
        LDR     R4, [R13],#+(CW*(1))        ; 5th parameter, or dummy
        LDR     R3, [R13],#+(CW*(1))        ; 4th parameter, or dummy
        LDR     R2, [R13],#+(CW*(1))        ; Third parameter, or dummy
        LDR     R1, [R13],#+(CW*(1))        ; Second parameter, or dummy
        LDR     R0, [R13],#+(CW*(1))        ; First parameter.
        SVC     #0           ; Generic call on LINUX      
        
        STR     R0, [R13,#-(CW*(1))]!
        LDR     R0, [R11],#+(CW*(1))
         LDR     R1,[R0]
         BX      R1            ; Positive means okay. Negative means -errno.

;
;

;
 ; 


;  **********
;  *   MS   *
;  **********
;
        ALIGN    4
N_MS:
        DD      2
        DB      "MS"
        ALIGN    4
MS:
        DD    DOCOL
        DD    MS+HEADSIZE
        DD    0x0
        DD    XOS7
        DD    N_MS
        DD    0

; Use select because nanosleep has a 20 MS resolution. 

        DD      LIT, 1000, LIT, 1000000, SSMOD ; (ms -- us s) 
        DD      LIT, TV, TSTOR
        DD      ZERO, ZERO
        DD      ZERO, ZERO
        DD      LIT, TV
        DD      LIT, select, XOS5
; 
; 

        DD      QERRUR
        DD      SEMIS
TV:     RESB    (CW*(2))    ; A time_t and a long, both cells.
;
; 

        ;
;------------------------------------
;       CONSOLE RELEATED I/O         
;------------------------------------







;  *****************
;  *   RW-BUFFER   *
;  *****************
;
        ALIGN    4
N_RWBUF:
        DD      9
        DB      "RW-BUFFER"
        ALIGN    4
RWBUF:
        DD    DOVAR
        DD    RWBUF+HEADSIZE
        DD    0x0
        DD    MS
        DD    N_RWBUF
        DD    0

        RESB    0x1000
; 

;


;  ***********
;  *   ZEN   *
;  ***********
;
        ALIGN    4
N_ZEN:
        DD      3
        DB      "ZEN"
        ALIGN    4
ZEN:
        DD    DOCOL
        DD    ZEN+HEADSIZE
        DD    0x0
        DD    RWBUF
        DD    N_ZEN
        DD    0

        DD      RWBUF, SSTOR
        DD      ZERO, RWBUF, CHAPP
        DD      RWBUF, CELLP

        DD      SEMIS
; 



;


;  *****************
;  *   OPEN-FILE   *
;  *****************
;
        ALIGN    4
N_OFILE:
        DD      9
        DB      "OPEN-FILE"
        ALIGN    4
OFILE:
        DD    DOCOL
        DD    OFILE+HEADSIZE
        DD    0x0
        DD    ZEN
        DD    N_OFILE
        DD    0

      DD      TOR, ZEN, FROMR, X
        DD      LIT, open, XOS
        DD      LDUP, ZERO, MIN
        DD      SEMIS 

;


;


;  ******************
;  *   CLOSE-FILE   *
;  ******************
;
        ALIGN    4
N_CFILE:
        DD      10
        DB      "CLOSE-FILE"
        ALIGN    4
CFILE:
        DD    DOCOL
        DD    CFILE+HEADSIZE
        DD    0x0
        DD    OFILE
        DD    N_CFILE
        DD    0

        DD      X, X
        DD      LIT, close, XOS
        DD      SEMIS
;


;



;  *******************
;  *   CREATE-FILE   *
;  *******************
;
        ALIGN    4
N_CREATEF:
        DD      11
        DB      "CREATE-FILE"
        ALIGN    4
CREATEF:
        DD    DOCOL
        DD    CREATEF+HEADSIZE
        DD    0x0
        DD    CFILE
        DD    N_CREATEF
        DD    0

        DD      TOR, TDUP, DFILE, DROP
        DD      ZEN, FROMR, X
        DD      LIT, creat, XOS
        DD      LDUP, ZERO, MAX, SWAP, ZERO, MIN
        DD      SEMIS 


;


;


;  *******************
;  *   DELETE-FILE   *
;  *******************
;
        ALIGN    4
N_DFILE:
        DD      11
        DB      "DELETE-FILE"
        ALIGN    4
DFILE:
        DD    DOCOL
        DD    DFILE+HEADSIZE
        DD    0x0
        DD    CREATEF
        DD    N_DFILE
        DD    0

        DD      ZEN, X, X
        DD      LIT, unlink, XOS
        DD      SEMIS 

;


;


;  *****************
;  *   READ-FILE   *
;  *****************
;
        ALIGN    4
N_RFILE:
        DD      9
        DB      "READ-FILE"
        ALIGN    4
RFILE:
        DD    DOCOL
        DD    RFILE+HEADSIZE
        DD    0x0
        DD    DFILE
        DD    N_RFILE
        DD    0

        DD      ROT, ROT
        DD      LIT, read, XOS
        DD      LDUP, ZERO, MAX, SWAP, ZERO, MIN
        DD      SEMIS
; 
;

;

;  *********************
;  *   FILE-POSITION   *
;  *********************
;
        ALIGN    4
N_FPFILE:
        DD      13
        DB      "FILE-POSITION"
        ALIGN    4
FPFILE:
        DD    DOCOL
        DD    FPFILE+HEADSIZE
        DD    0x0
        DD    RFILE
        DD    N_FPFILE
        DD    0

        DD      ZERO
        DD      LIT, SEEK_CUR
        DD      LIT, lseek, XOS
        DD      ZERO     ;Double precision return result
        DD      OVER, LDUP
        DD      NEGATE, LIT, 256, ULESS, LAND ; Heuristic 
        DD      SEMIS
 ; 

;


;  ***********************
;  *   REPOSITION-FILE   *
;  ***********************
;
        ALIGN    4
N_PFILE:
        DD      15
        DB      "REPOSITION-FILE"
        ALIGN    4
PFILE:
        DD    DOCOL
        DD    PFILE+HEADSIZE
        DD    0x0
        DD    FPFILE
        DD    N_PFILE
        DD    0

        DD      TOR, DROP, FROMR   ;Convert to single precision.
        DD      SWAP
        DD      LIT, SEEK_SET
        DD      LIT, lseek, XOS
        DD      ZERO, MIN
        DD      SEMIS
 ; 



;


;  ******************
;  *   WRITE-FILE   *
;  ******************
;
        ALIGN    4
N_WFILE:
        DD      10
        DB      "WRITE-FILE"
        ALIGN    4
WFILE:
        DD    DOCOL
        DD    WFILE+HEADSIZE
        DD    0x0
        DD    PFILE
        DD    N_WFILE
        DD    0

        DD      ROT, ROT
        DD      LIT, write, XOS
        DD      ZERO, MIN
        DD      SEMIS
; 



;


;  ****************
;  *   GET-FILE   *
;  ****************
;
        ALIGN    4
N_GETFILE:
        DD      8
        DB      "GET-FILE"
        ALIGN    4
GETFILE:
        DD    DOCOL
        DD    GETFILE+HEADSIZE
        DD    0x0
        DD    WFILE
        DD    N_GETFILE
        DD    0

     DD LALIGN
        DD      TDUP, SCOMMA, DROP
        DD      LIT, 'F'+(0x100*('i'+0x100*('L'+0x100*'e'))), COMMA ;Magic number.
        DD      ZERO, OFILE, THROW, TOR
        DD      HERE, LDUP
        DD      LEM, LIT, 6, SLASH, LDUP, ALLOT
        DD      LIT, 1000, LSUB
        DD      RR, RFILE, THROW
        DD      FROMR, CFILE, THROW
        DD      TDUP, PLUS, LDP, STOR        ; No allocation if it fails.
        DD      SEMIS
;

;  ****************
;  *   PUT-FILE   *
;  ****************
;
        ALIGN    4
N_PUTFILE:
        DD      8
        DB      "PUT-FILE"
        ALIGN    4
PUTFILE:
        DD    DOCOL
        DD    PUTFILE+HEADSIZE
        DD    0x0
        DD    GETFILE
        DD    N_PUTFILE
        DD    0

        DD      LIT, 0755
        DD      CREATEF, THROW
        DD      LDUP, TOR
        DD      WFILE, THROW
        DD      FROMR, CFILE, THROW
        DD      SEMIS
;

;  ****************
;  *   INCLUDED   *
;  ****************
;
        ALIGN    4
N_INCLUDED:
        DD      8
        DB      "INCLUDED"
        ALIGN    4
INCLUDED:
        DD    DOCOL
        DD    INCLUDED+HEADSIZE
        DD    0x0
        DD    PUTFILE
        DD    N_INCLUDED
        DD    0

        DD      HERE, TOR
        DD      LIT, GETFILE, CATCH
        DD      LDUP
        DD      ZBRAN
        DD      INCLUD1-$-CW
        DD      FROMR, LDP, STOR
        DD      THROW
        DD      BRAN
        DD      INCLUD2-$-CW
INCLUD1:
        DD      RDROP, DROP
INCLUD2:
        DD      EVALUATE
        DD      SEMIS
;

;  ***************
;  *   INCLUDE   *
;  ***************
;
        ALIGN    4
N_LINCLUD:
        DD      7
        DB      "INCLUDE"
        ALIGN    4
LINCLUD:
        DD    DOCOL
        DD    LINCLUD+HEADSIZE
        DD    0x0
        DD    INCLUDED
        DD    N_LINCLUD
        DD    0

        DD      LNAME, INCLUDED
        DD      SEMIS
;

;  ******************
;  *   REFILL-TIB   *
;  ******************
;
        ALIGN    4
N_REFTIB:
        DD      10
        DB      "REFILL-TIB"
        ALIGN    4
REFTIB:
        DD    DOCOL
        DD    REFTIB+HEADSIZE
        DD    0x0
        DD    LINCLUD
        DD    N_REFTIB
        DD    0

        DD      REMAIND, FETCH, TOR
        DD      TIB, FETCH, RR, PLUS
        DD      LIT, TIBSIZE, RR, LSUB
        DD      ZERO
        DD      RFILE
        DD      QERRUR
        DD      LDUP, ZEQU, LIT, -EPIPE, LAND ; Presumably end of pipe.
        DD      QERRUR
        DD      TIB, FETCH, SWAP, FROMR, PLUS
        DD      REMAIND, TSTOR
        DD      SEMIS
;
;

;

; All I/O goes through the modern device (Unix-standard)
; device I/O. Code fields are filled in during bootup. 
; Lower case labels starting with "c_.." are c-supplied facilities.

;  **************
;  *   ACCEPT   *
;  **************
;
        ALIGN    4
N_ACCEP:
        DD      6
        DB      "ACCEPT"
        ALIGN    4
ACCEP:
        DD    DOCOL
        DD    ACCEP+HEADSIZE
        DD    0x0
        DD    REFTIB
        DD    N_ACCEP
        DD    0

        DD      PACCEP
        DD      TSWAP, ROT, MIN
        DD      LDUP, TOR, LMOVE, FROMR
        DD      SEMIS
;

;  ****************
;  *   (ACCEPT)   *
;  ****************
;
        ALIGN    4
N_PACCEP:
        DD      8
        DB      "(ACCEPT)"
        ALIGN    4
PACCEP:
        DD    DOCOL
        DD    PACCEP+HEADSIZE
        DD    0x0
        DD    ACCEP
        DD    N_PACCEP
        DD    0

PACCEP2:
        DD      REMAIND
        DD      TFET
        DD      LIT, ALF, SINDEX
        DD      ZEQU
        DD      ZBRAN
        DD      PACCEP1-$-CW
        DD      REMAIND, TFET
        DD      TIB, FETCH
        DD      SWAP, LMOVE
        DD      TIB, FETCH
        DD      REMAIND, CELLP, STOR
        DD      REFTIB
        DD      BRAN
        DD      PACCEP2-$-CW
PACCEP1:
        DD      REMAIND, TFET
        DD      LIT, ALF, SSLAS
        DD      TSWAP, REMAIND, TSTOR
        DD      SEMIS


;  ***********
;  *   KEY   *
;  ***********
;
        ALIGN    4
N_KEY:
        DD      3
        DB      "KEY"
        ALIGN    4
KEY:
        DD    DOCOL
        DD    KEY+HEADSIZE
        DD    0x0
        DD    PACCEP
        DD    N_KEY
        DD    0

        DD      ONE, LIT, RAWIO, STTERM ; Do not wait for CR
        DD      ZERO, SPFET     ; Expect single key on stack
        DD      ZERO            ; Standard input
        DD      SWAP, ONE       ; Buffer and length
        DD      LIT, read, XOS
        DD      LDUP, QERRUR
        DD      ZEQU, LIT, -EPIPE, LAND, QERRUR ; Presumably end of pipe.
        DD      ONE, LIT, RAWIO, STTERM       ; Toggle back
        DD      SEMIS
;

;  ************
;  *   KEY?   *
;  ************
;
        ALIGN    4
N_KEYQ:
        DD      4
        DB      "KEY?"
        ALIGN    4
KEYQ:
        DD    DOCOL
        DD    KEYQ+HEADSIZE
        DD    0x0
        DD    KEY
        DD    N_KEYQ
        DD    0

        DD      ZERO, LIT, RAWIO, STTERM
        DD      ZERO, ZERO     ; Time out of zero. See MS.
        DD      LIT, TV, TSTOR
        DD      ONE ; Waiting for standard in.
        DD      LIT, RFDS, STOR
        DD      ONE     ; # of highest channel to watch.
        DD      LIT, RFDS
        DD      ZERO, ZERO  ; Not output, not events.
        
        DD      LIT, TV, LIT, select, XOS5
        DD      LDUP,  QERRUR
        DD      NEGATE        ; 1 -> flag.
        DD      ZERO, LIT, RAWIO, STTERM
        DD      SEMIS

        ALIGN    4
RFDS:   RESB    (CW*(32))     ;32 channels maximum.

;

;  ************
;  *   TYPE   *
;  ************
;
        ALIGN    4
N_LTYPE:
        DD      4
        DB      "TYPE"
        ALIGN    4
LTYPE:
        DD    DOCOL
        DD    LTYPE+HEADSIZE
        DD    0x0
        DD    KEYQ
        DD    N_LTYPE
        DD    0

        DD      LDUP, LOUT, PSTOR
        DD      ONE, WFILE, THROW
        DD      SEMIS
;

;  *************
;  *   ETYPE   *
;  *************
;
        ALIGN    4
N_ETYPE:
        DD      5
        DB      "ETYPE"
        ALIGN    4
ETYPE:
        DD    DOCOL
        DD    ETYPE+HEADSIZE
        DD    0x0
        DD    LTYPE
        DD    N_ETYPE
        DD    0

        DD      TWO, WFILE, THROW
        DD      SEMIS
;

;  ************
;  *   EMIT   *
;  ************
;
        ALIGN    4
N_EMIT:
        DD      4
        DB      "EMIT"
        ALIGN    4
EMIT:
        DD    DOCOL
        DD    EMIT+HEADSIZE
        DD    0x0
        DD    ETYPE
        DD    N_EMIT
        DD    0

        DD      SPFET, ONE, LTYPE
        DD      DROP
        DD      SEMIS
;

;  *************
;  *   DEMIT   *
;  *************
;
        ALIGN    4
N_DEMIT:
        DD      5
        DB      "DEMIT"
        ALIGN    4
DEMIT:
        DD    DEMIT+HEADSIZE
        DD    DEMIT+HEADSIZE
        DD    0x0
        DD    EMIT
        DD    N_DEMIT
        DD    0

        MOV     R1, R13  ; 4 characters on the stack, SP is their address 
        MOV      R0, #1   ; standard output 
        MOV      R2, #4   ; 4 character 
        MOV      R0, #4   ; function number 
        SWI     0
        ADD     R13,R13,#4  ; adjust stack 
        LDR     R0, [R11],#+(CW*(1))
         LDR     R1,[R0]
         BX      R1  


;  **************
;  *   TERMIO   *
;  **************
;
        ALIGN    4
N_TERMIO:
        DD      6
        DB      "TERMIO"
        ALIGN    4
TERMIO:
        DD    DOVAR
        DD    TERMIO+HEADSIZE
        DD    0x0
        DD    DEMIT
        DD    N_TERMIO
        DD    0

        RESB    SIZE_TERMIO     ; Filled in during cold boot.
;

;  ****************
;  *   SET-TERM   *
;  ****************
;
        ALIGN    4
N_STTERM:
        DD      8
        DB      "SET-TERM"
        ALIGN    4
STTERM:
        DD    DOCOL
        DD    STTERM+HEADSIZE
        DD    0x0
        DD    TERMIO
        DD    N_STTERM
        DD    0

SIZEL   EQU 4     ; Also for 64 bits. 
        DD      LIT, (TERMIO+HEADSIZE)+SIZEL*3
        DD      SWAP, TOGGL
        DD      LIT, (TERMIO+HEADSIZE)+SIZEL*4+7
        DD      CSTOR
        DD      ZERO, LIT, TCSETS, TERMIO
        DD      LIT, ioctl, XOS, QERRUR
        DD      SEMIS
;
; 






;

        ;
;------------------------------------
;       SYSTEM DEPENDANT DISK I/O
;------------------------------------

;  ******************
;  *   DISK-ERROR   *
;  ******************
;
        ALIGN    4
N_DERR:
        DD      10
        DB      "DISK-ERROR"
        ALIGN    4
DERR:
        DD    DOVAR
        DD    DERR+HEADSIZE
        DD    0x0
        DD    STTERM
        DD    N_DERR
        DD    0

        DD      -1
;


;  ******************
;  *   BLOCK-FILE   *
;  ******************
;
        ALIGN    4
N_BLFL:
        DD      10
        DB      "BLOCK-FILE"
        ALIGN    4
BLFL:
        DD    DOVAR
        DD    BLFL+HEADSIZE
        DD    0x0
        DD    DERR
        DD    N_BLFL
        DD    0

        DD      9
        DB      "forth.lab"
        RESB    252-9               ; Allow for some path

;  ********************
;  *   BLOCK-HANDLE   *
;  ********************
;
        ALIGN    4
N_BHAN:
        DD      12
        DB      "BLOCK-HANDLE"
        ALIGN    4
BHAN:
        DD    DOVAR
        DD    BHAN+HEADSIZE
        DD    0x0
        DD    BLFL
        DD    N_BHAN
        DD    0

        DD      -1
;

;  *******************
;  *   ?DISK-ERROR   *
;  *******************
;
        ALIGN    4
N_QDSKER:
        DD      11
        DB      "?DISK-ERROR"
        ALIGN    4
QDSKER:
        DD    DOCOL
        DD    QDSKER+HEADSIZE
        DD    0x0
        DD    BHAN
        DD    N_QDSKER
        DD    0

        DD      LIT, 8, QERR
        DD      SEMIS
;
;


;  ******************
;  *   BLOCK-INIT   *
;  ******************
;
        ALIGN    4
N_BLINI:
        DD      10
        DB      "BLOCK-INIT"
        ALIGN    4
BLINI:
        DD    DOCOL
        DD    BLINI+HEADSIZE
        DD    0x0
        DD    QDSKER
        DD    N_BLINI
        DD    0

        DD      BLFL, SFET, DTRAI
        DD      ROT
        DD      OFILE
        DD      ZEQU, NEGATE  ; 0 if disk problems, 1 if not.
        DD      LWARN, FETCH, MIN ; AND but WARNING is 0/1.
        DD      LWARN, STOR
        DD      BHAN, STOR
        DD      MTBUF
        DD      SEMIS
;

;  ******************
;  *   BLOCK-EXIT   *
;  ******************
;
        ALIGN    4
N_BLEXI:
        DD      10
        DB      "BLOCK-EXIT"
        ALIGN    4
BLEXI:
        DD    DOCOL
        DD    BLEXI+HEADSIZE
        DD    0x0
        DD    BLINI
        DD    N_BLEXI
        DD    0

        DD      FLUSH
        DD      BHAN, FETCH
        DD      CFILE
        DD      ZERO, LWARN, STOR
        DD      LIT, -1, BHAN, STOR    ;Regardless of close errors.
        DD      QDSKER
        DD      SEMIS
;

;  **************
;  *   BLSEEK   *
;  **************
;
        ALIGN    4
N_BLSEEK:
        DD      6
        DB      "BLSEEK"
        ALIGN    4
BLSEEK:
        DD    DOCOL
        DD    BLSEEK+HEADSIZE
        DD    0x0
        DD    BLEXI
        DD    N_BLSEEK
        DD    0

        DD      BBUF
        DD      USTAR
        DD      BHAN, FETCH
        DD      PFILE           ; Disk position.
        DD      QDSKER
        DD      SEMIS
;

;  ******************
;  *   BLOCK-READ   *
;  ******************
;
        ALIGN    4
N_BLREAD:
        DD      10
        DB      "BLOCK-READ"
        ALIGN    4
BLREAD:
        DD    DOCOL
        DD    BLREAD+HEADSIZE
        DD    0x0
        DD    BLSEEK
        DD    N_BLREAD
        DD    0

        DD      BLSEEK ; That's done
        DD      BBUF
        DD      BHAN, FETCH
        DD      RFILE           ; Disk read 
        DD      SWAP, BBUF, UNEQ
        DD      LOR, QDSKER
        DD      SEMIS
;

;  *******************
;  *   BLOCK-WRITE   *
;  *******************
;
        ALIGN    4
N_BLWRITE:
        DD      11
        DB      "BLOCK-WRITE"
        ALIGN    4
BLWRITE:
        DD    DOCOL
        DD    BLWRITE+HEADSIZE
        DD    0x0
        DD    BLREAD
        DD    N_BLWRITE
        DD    0

        DD      BLSEEK ; That's done
        DD      BBUF
        DD      BHAN, FETCH
        DD      WFILE           ; Disk write
        DD      QDSKER
        DD      SEMIS
;
;

;  *************
;  *   SHELL   *
;  *************
;
        ALIGN    4
N_SHELL:
        DD      5
        DB      "SHELL"
        ALIGN    4
SHELL:
        DD    DOVAR
        DD    SHELL+HEADSIZE
        DD    0x0
        DD    BLWRITE
        DD    N_SHELL
        DD    0

                 DD      7
        DB      "/bin/sh"

        RESB    252-9               ; Allow for some path

;


;  **************
;  *   SYSTEM   *
;  **************
;
        ALIGN    4
N_SYSTEM:
        DD      6
        DB      "SYSTEM"
        ALIGN    4
SYSTEM:
        DD    DOCOL
        DD    SYSTEM+HEADSIZE
        DD    0x0
        DD    SHELL
        DD    N_SYSTEM
        DD    0

          DD      LIT, COMBUF, SSTOR, ZERO, LIT, COMBUF, CHAPP
          DD      ZERO, SHELL, CHAPP, LIT, -1, SHELL, PSTOR
         DD      X, X, X, LIT, fork, XOS

          DD      LDUP, QERRUR
          DD      LDUP, ZEQU
          DD      ZBRAN
        DD      SYSTEM1-$-CW
        DD      SHELL, CELLP, LIT, SYSARGS
;  Get environment vector
      DD      ARGS, FETCH, SFET, ONEP, LCELLS, PLUS
        DD      LIT, execve, XOS
        DD      LIT, 49, LERROR, BYE   ; Returned, this is always wrong.
SYSTEM1:
; int waitid(idtype_t idtype, id_t id, siginfo_t * infop , int options ) 

        DD      LDUP   ; pid of child
        DD      LIT, P_PID, SWAP
        DD      LIT, SIGINFOT
        DD      LIT, WEXITED, ZERO
        DD      LIT, waitid, XOS5
        DD      LDUP
        DD      LIT, EINTR, EQUAL
        DD      BRAN
        DD      SYSTEM2-$-CW
        DD      DROP
        DD      ZBRAN
        DD      SYSTEM1-$-CW
SYSTEM2:
        DD      QERRUR
        DD      TDROP
        DD      SEMIS
;
        ALIGN    4
SIGINFOT: RESB    CW*9
SYSARG2: DB     "-c"
         DB      0
        ALIGN    4
COMBUF:  RESB    CW+256  ;One cell for high level string manipulation.
        ALIGN    4
SYSARGS:DD   (SHELL+HEADSIZE)+CW, SYSARG2, COMBUF + CW, 0
;
 ; 



;



;


;

;  *********
;  *   '   *
;  *********
;
        ALIGN    4
N_ITICK:
        DD      1
        DB      "'"
        ALIGN    4
ITICK:
        DD    DOCOL 
        DD    ITICK+HEADSIZE
        DD    0x0
        DD    SYSTEM
        DD    N_ITICK
        DD    0

        DD      LNAME, PRESENT
        DD      LDUP, ZEQU
        DD      LIT, 11, QERR
        DD      SEMIS
;

;  ***********
;  *   [']   *
;  ***********
;
        ALIGN    4
N_BTICK:
        DD      3
        DB      "[']"
        ALIGN    4
BTICK:
        DD    DOCOL
        DD    (TICK+HEADSIZE)
        DD    B_IMMED
        DD    ITICK
        DD    N_BTICK
        DD    0

;

;  ******************
;  *   FORGET-VOC   *
;  ******************
;
        ALIGN    4
N_FORGV:
        DD      10
        DB      "FORGET-VOC"
        ALIGN    4
FORGV:
        DD    DOCOL
        DD    FORGV+HEADSIZE
        DD    0x0
        DD    BTICK
        DD    N_FORGV
        DD    0

        DD      TDUP
        DD      SWAP
        DD      ULESS
        DD      ZBRAN
        DD      FORGV1-$-CW
;  Forget part of contents.
        DD      SWAP
        DD      TOR
        DD      TWID
FORGV0:
        DD      LDUP
        DD      LDUP
FORGV3:
        DD      TLFA,FETCH
        DD      LDUP
        DD      RR
        DD      ULESS
        DD      ZBRAN
        DD      FORGV3-$-CW
        DD      SWAP            ;  Not to be forgotten entry found.
        DD      TLFA
        DD      STOR           ;  Short other entries out.
        DD      TLFA,FETCH
        DD      LDUP
        DD      ZEQU
        DD      ZBRAN
        DD      FORGV0-$-CW        ;  Repeat until end of wordlist.
        DD      DROP
        DD      FROMR
        DD      BRAN
        DD      FORGV2-$-CW
FORGV1:
;       { Vocabulary itself is also forgotten.}
        DD      TVFA
        DD      FETCH     ; Unlink by linking next namespace.
        DD      VOCL
        DD      STOR
        DD      ONLY, FORTH
        DD      DEFIN
FORGV2: DD      SEMIS
;

;  **************
;  *   FORGET   *
;  **************
;
        ALIGN    4
N_FORG:
        DD      6
        DB      "FORGET"
        ALIGN    4
FORG:
        DD    DOCOL
        DD    FORG+HEADSIZE
        DD    0x0
        DD    FORGV
        DD    N_FORG
        DD    0

        DD      TICK
        DD      LDUP
        DD      FENCE
        DD      FETCH
        DD      LESS
        DD      LIT, 21, QERR
        DD      LIT,FORGV
        DD      FORV
        DD      TNFA, FETCH, LDP, STOR
        DD      SEMIS
;

;  *************
;  *   (BACK   *
;  *************
;
        ALIGN    4
N_PBACK:
        DD      5
        DB      "(BACK"
        ALIGN    4
PBACK:
        DD    DOCOL
        DD    PBACK+HEADSIZE
        DD    0x0
        DD    FORG
        DD    N_PBACK
        DD    0

        DD      HERE
        DD      SEMIS
;

;  *************
;  *   BACK)   *
;  *************
;
        ALIGN    4
N_BACKP:
        DD      5
        DB      "BACK)"
        ALIGN    4
BACKP:
        DD    DOCOL
        DD    BACKP+HEADSIZE
        DD    0x0
        DD    PBACK
        DD    N_BACKP
        DD    0

        DD      HERE
        DD      CELLP
        DD      LSUB
        DD      COMMA
        DD      SEMIS
;

;  ****************
;  *   (FORWARD   *
;  ****************
;
        ALIGN    4
N_PFORWARD:
        DD      8
        DB      "(FORWARD"
        ALIGN    4
PFORWARD:
        DD    DOCOL
        DD    PFORWARD+HEADSIZE
        DD    0x0
        DD    BACKP
        DD    N_PFORWARD
        DD    0

        DD      HERE
        DD      X
        DD      COMMA
        DD      SEMIS
;

;  ****************
;  *   FORWARD)   *
;  ****************
;
        ALIGN    4
N_FORWARDP:
        DD      8
        DB      "FORWARD)"
        ALIGN    4
FORWARDP:
        DD    DOCOL
        DD    FORWARDP+HEADSIZE
        DD    0x0
        DD    PFORWARD
        DD    N_FORWARDP
        DD    0

        DD      HERE
        DD      OVER
        DD      CELLP
        DD      LSUB
        DD      SWAP
        DD      STOR
        DD      SEMIS
;

;  *************
;  *   BEGIN   *
;  *************
;
        ALIGN    4
N_BEGIN:
        DD      5
        DB      "BEGIN"
        ALIGN    4
BEGIN:
        DD    DOCOL
        DD    BEGIN+HEADSIZE
        DD    B_IMMED
        DD    FORWARDP
        DD    N_BEGIN
        DD    0

        DD      PBACK
        DD      QCOMP, ONE
        DD      SEMIS
;

;  ************
;  *   THEN   *
;  ************
;
        ALIGN    4
N_THEN:
        DD      4
        DB      "THEN"
        ALIGN    4
THEN:
        DD    DOCOL
        DD    THEN+HEADSIZE
        DD    B_IMMED
        DD    BEGIN
        DD    N_THEN
        DD    0

        DD      QCOMP, TWO, QPAIR
        DD      FORWARDP
        DD      SEMIS
;

;  **********
;  *   DO   *
;  **********
;
        ALIGN    4
N_DO:
        DD      2
        DB      "DO"
        ALIGN    4
DO:
        DD    DOCOL
        DD    DO+HEADSIZE
        DD    B_IMMED
        DD    THEN
        DD    N_DO
        DD    0

         DD      LIT, XDO, COMMA, PFORWARD, PBACK
        DD      QCOMP, LIT,3    ; Magic number
        DD      SEMIS
;

;  ***********
;  *   ?DO   *
;  ***********
;
        ALIGN    4
N_QDO:
        DD      3
        DB      "?DO"
        ALIGN    4
QDO:
        DD    DOCOL
        DD    QDO+HEADSIZE
        DD    B_IMMED
        DD    DO
        DD    N_QDO
        DD    0

         DD      LIT, XQDO, COMMA, PFORWARD, PBACK
        DD      QCOMP, LIT,3    ; Magic number
        DD      SEMIS
;

;  ************
;  *   LOOP   *
;  ************
;
        ALIGN    4
N_LLOOP:
        DD      4
        DB      "LOOP"
        ALIGN    4
LLOOP:
        DD    DOCOL
        DD    LLOOP+HEADSIZE
        DD    B_IMMED
        DD    QDO
        DD    N_LLOOP
        DD    0

        DD      LIT, ONE, COMMA ; POSTPONE 1 
        DD      PLOOP
        DD      SEMIS
;

;  *************
;  *   +LOOP   *
;  *************
;
        ALIGN    4
N_PLOOP:
        DD      5
        DB      "+LOOP"
        ALIGN    4
PLOOP:
        DD    DOCOL
        DD    PLOOP+HEADSIZE
        DD    B_IMMED
        DD    LLOOP
        DD    N_PLOOP
        DD    0

        DD      QCOMP, LIT, 3, QPAIR
        DD      LIT, XPLOOP, COMMA
        DD      LIT, ZBRAN, COMMA, BACKP
        DD      LIT, UNLOOP, COMMA
        DD      FORWARDP ; For DO to push the leave address.
        DD      SEMIS
;

;  *************
;  *   UNTIL   *
;  *************
;
        ALIGN    4
N_UNTIL:
        DD      5
        DB      "UNTIL"
        ALIGN    4
UNTIL:
        DD    DOCOL
        DD    UNTIL+HEADSIZE
        DD    B_IMMED
        DD    PLOOP
        DD    N_UNTIL
        DD    0

        DD      QCOMP, ONE, QPAIR
        DD      LIT, ZBRAN, COMMA, BACKP
        DD      SEMIS
;

;  *************
;  *   AGAIN   *
;  *************
;
        ALIGN    4
N_AGAIN:
        DD      5
        DB      "AGAIN"
        ALIGN    4
AGAIN:
        DD    DOCOL
        DD    AGAIN+HEADSIZE
        DD    B_IMMED
        DD    UNTIL
        DD    N_AGAIN
        DD    0

        DD      QCOMP, ONE, QPAIR
        DD      LIT, BRAN, COMMA, BACKP
        DD      SEMIS
;

;  **************
;  *   REPEAT   *
;  **************
;
        ALIGN    4
N_REPEA:
        DD      6
        DB      "REPEAT"
        ALIGN    4
REPEA:
        DD    DOCOL
        DD    REPEA+HEADSIZE
        DD    B_IMMED
        DD    AGAIN
        DD    N_REPEA
        DD    0

        DD      QCOMP, ONE, QPAIR   ; Matches BEGIN ?
        DD      LIT, BRAN, COMMA, BACKP
        DD      LIT, 4, QPAIR ; Matches WHILE ?
        DD      FORWARDP ; WHILE target. 
        DD      SEMIS
;

;  **********
;  *   IF   *
;  **********
;
        ALIGN    4
N_LIF:
        DD      2
        DB      "IF"
        ALIGN    4
LIF:
        DD    DOCOL
        DD    LIF+HEADSIZE
        DD    B_IMMED
        DD    REPEA
        DD    N_LIF
        DD    0

        DD      LIT, ZBRAN, COMMA, PFORWARD
        DD      QCOMP, TWO     ; Magic number
        DD      SEMIS
;

;  ************
;  *   ELSE   *
;  ************
;
        ALIGN    4
N_LELSE:
        DD      4
        DB      "ELSE"
        ALIGN    4
LELSE:
        DD    DOCOL
        DD    LELSE+HEADSIZE
        DD    B_IMMED
        DD    LIF
        DD    N_LELSE
        DD    0

        DD      QCOMP, TWO, QPAIR
        DD      LIT, BRAN, COMMA, PFORWARD
        DD      SWAP
        DD      FORWARDP
        DD      TWO     ; Magic number
        DD      SEMIS
;

;  *************
;  *   WHILE   *
;  *************
;
        ALIGN    4
N_LWHILE:
        DD      5
        DB      "WHILE"
        ALIGN    4
LWHILE:
        DD    DOCOL
        DD    LWHILE+HEADSIZE
        DD    B_IMMED
        DD    LELSE
        DD    N_LWHILE
        DD    0

        DD      QCOMP, LDUP, ONE, QPAIR
        DD      TOR    ;  Save backward target. 
        DD      TOR
        DD      LIT, ZBRAN, COMMA, PFORWARD
        DD      LIT, 4 ; Magic number
        DD      FROMR
        DD      FROMR
        DD      SEMIS
;

;  **************
;  *   SPACES   *
;  **************
;
        ALIGN    4
N_SPACES:
        DD      6
        DB      "SPACES"
        ALIGN    4
SPACES:
        DD    DOCOL
        DD    SPACES+HEADSIZE
        DD    0x0
        DD    LWHILE
        DD    N_SPACES
        DD    0

        DD      ZERO
        DD      MAX
        DD      ZERO
        DD     XQDO
        DD      SPAX1-$-CW
SPAX2:  DD      SPACE
        DD     ONE, XPLOOP, ZBRAN
        DD      SPAX2-$-CW
	DD      UNLOOP    ;LOOP
SPAX1:
        DD      SEMIS
;

;  **********
;  *   <#   *
;  **********
;
        ALIGN    4
N_BDIGS:
        DD      2
        DB      "<#"
        ALIGN    4
BDIGS:
        DD    DOCOL
        DD    BDIGS+HEADSIZE
        DD    0x0
        DD    SPACES
        DD    N_BDIGS
        DD    0

        DD      PAD
        DD      HLD
        DD      STOR
        DD      SEMIS
;

;  **********
;  *   #>   *
;  **********
;
        ALIGN    4
N_EDIGS:
        DD      2
        DB      "#>"
        ALIGN    4
EDIGS:
        DD    DOCOL
        DD    EDIGS+HEADSIZE
        DD    0x0
        DD    BDIGS
        DD    N_EDIGS
        DD    0

        DD      DROP
        DD      DROP
        DD      HLD
        DD      FETCH
        DD      PAD
        DD      OVER
        DD      LSUB
        DD      SEMIS
;

;  ************
;  *   SIGN   *
;  ************
;
        ALIGN    4
N_SIGN:
        DD      4
        DB      "SIGN"
        ALIGN    4
SIGN:
        DD    DOCOL
        DD    SIGN+HEADSIZE
        DD    0x0
        DD    EDIGS
        DD    N_SIGN
        DD    0

        DD      ZLESS
        DD      ZBRAN
        DD      SIGN1-$-CW ;IF
        DD      LIT, AMS
        DD      HOLD    ;THEN
SIGN1:  DD      SEMIS
;

;  *********
;  *   #   *
;  *********
;
        ALIGN    4
N_DIG:
        DD      1
        DB      "#"
        ALIGN    4
DIG:
        DD    DOCOL
        DD    DIG+HEADSIZE
        DD    0x0
        DD    SIGN
        DD    N_DIG
        DD    0

        DD      BASE
        DD      FETCH
        DD      MSMOD
        DD      ROT
        DD      LIT,9
        DD      OVER
        DD      LESS
        DD      ZBRAN
        DD      DIG1-$-CW  ;IF
        DD      LIT,7
        DD      PLUS    ;THEN
DIG1:   DD      LIT,0x30
        DD      PLUS
        DD      HOLD
        DD      SEMIS
;

;  **********
;  *   #S   *
;  **********
;
        ALIGN    4
N_DIGS:
        DD      2
        DB      "#S"
        ALIGN    4
DIGS:
        DD    DOCOL
        DD    DIGS+HEADSIZE
        DD    0x0
        DD    DIG
        DD    N_DIGS
        DD    0

DIGS1:  DD      DIG     ;BEGIN
        DD      OVER
        DD      OVER
        DD      LOR
        DD      ZEQU
        DD      ZBRAN
        DD      DIGS1-$-CW ;UNTIL
        DD      SEMIS
;
; 

;  *************
;  *   (D.R)   *
;  *************
;
        ALIGN    4
N_PDDOTR:
        DD      5
        DB      "(D.R)"
        ALIGN    4
PDDOTR:
        DD    DOCOL
        DD    PDDOTR+HEADSIZE
        DD    0x0
        DD    DIGS
        DD    N_PDDOTR
        DD    0

        DD      TOR
        DD      SWAP
        DD      OVER
        DD      DABS
        DD      BDIGS
        DD      DIGS
        DD      ROT
        DD      SIGN
        DD      EDIGS
        DD      FROMR
        DD      OVER
        DD      LSUB, ZERO, MAX
        DD      ZERO
        DD     XQDO
        DD      PDDOT1-$-CW
PDDOT2:  DD      LBL, HOLD  ;WARNING: HOLD outside of #>.
        DD     ONE, XPLOOP, ZBRAN
        DD      PDDOT2-$-CW
	DD      UNLOOP
PDDOT1:
        DD      EDIGS  ;Drop string instead of number.
        DD      SEMIS
;

;  ***********
;  *   D.R   *
;  ***********
;
        ALIGN    4
N_DDOTR:
        DD      3
        DB      "D.R"
        ALIGN    4
DDOTR:
        DD    DOCOL
        DD    DDOTR+HEADSIZE
        DD    0x0
        DD    PDDOTR
        DD    N_DDOTR
        DD    0

        DD      PDDOTR
        DD      LTYPE
        DD      SEMIS
;

;  **********
;  *   .R   *
;  **********
;
        ALIGN    4
N_DOTR:
        DD      2
        DB      ".R"
        ALIGN    4
DOTR:
        DD    DOCOL
        DD    DOTR+HEADSIZE
        DD    0x0
        DD    DDOTR
        DD    N_DOTR
        DD    0

        DD      TOR
        DD      STOD
        DD      FROMR
        DD      DDOTR
        DD      SEMIS
;

;  **********
;  *   D.   *
;  **********
;
        ALIGN    4
N_DDOT:
        DD      2
        DB      "D."
        ALIGN    4
DDOT:
        DD    DOCOL
        DD    DDOT+HEADSIZE
        DD    0x0
        DD    DOTR
        DD    N_DDOT
        DD    0

        DD      ZERO
        DD      DDOTR
        DD      SPACE
        DD      SEMIS
;

;  *********
;  *   .   *
;  *********
;
        ALIGN    4
N_DOT:
        DD      1
        DB      "."
        ALIGN    4
DOT:
        DD    DOCOL
        DD    DOT+HEADSIZE
        DD    0x0
        DD    DDOT
        DD    N_DOT
        DD    0

        DD      STOD
        DD      DDOT
        DD      SEMIS
;

;  *********
;  *   ?   *
;  *********
;
        ALIGN    4
N_QUES:
        DD      1
        DB      "?"
        ALIGN    4
QUES:
        DD    DOCOL
        DD    QUES+HEADSIZE
        DD    0x0
        DD    DOT
        DD    N_QUES
        DD    0

        DD      FETCH
        DD      DOT
        DD      SEMIS
;

;  **********
;  *   U.   *
;  **********
;
        ALIGN    4
N_UDOT:
        DD      2
        DB      "U."
        ALIGN    4
UDOT:
        DD    DOCOL
        DD    UDOT+HEADSIZE
        DD    0x0
        DD    QUES
        DD    N_UDOT
        DD    0

        DD      ZERO
        DD      DDOT
        DD      SEMIS
;

;  *****************
;  *   FOR-WORDS   *
;  *****************
;
        ALIGN    4
N_FORW:
        DD      9
        DB      "FOR-WORDS"
        ALIGN    4
FORW:
        DD    DOCOL
        DD    FORW+HEADSIZE
        DD    0x0
        DD    UDOT
        DD    N_FORW
        DD    0

        DD      SWAP
        DD      TOR
        DD      TOR
FORW1:  DD      FROMR
        DD      RR
        DD      OVER
        DD      TLFA
        DD      FETCH
        DD      TOR
        DD      EXEC
        DD      RR
        DD      ZEQU
        DD      ZBRAN
        DD      FORW1-$-CW
        DD      RDROP
        DD      RDROP
        DD      SEMIS
;

;  ****************
;  *   FOR-VOCS   *
;  ****************
;
        ALIGN    4
N_FORV:
        DD      8
        DB      "FOR-VOCS"
        ALIGN    4
FORV:
        DD    DOCOL
        DD    FORV+HEADSIZE
        DD    0x0
        DD    FORW
        DD    N_FORV
        DD    0

        DD      TOR
        DD      VOCL
        DD      FETCH
        DD      TOR
FORV1:  DD      FROMR
        DD      RR
        DD      OVER
        DD      TVFA
        DD      FETCH
        DD      TOR
        DD      EXEC
        DD      RR
        DD      ZEQU
        DD      ZBRAN
        DD      FORV1-$-CW
        DD      RDROP
        DD      RDROP
        DD      SEMIS
;

;  *************
;  *   WORDS   *
;  *************
;
        ALIGN    4
N_WORDS:
        DD      5
        DB      "WORDS"
        ALIGN    4
WORDS:
        DD    DOCOL
        DD    WORDS+HEADSIZE
        DD    0x0
        DD    FORV
        DD    N_WORDS
        DD    0

        DD      CSLL
        DD      LOUT
        DD      STOR
        DD      ZERO            ; End sentinel. 
        DD      LIT, NOOP
        DD      CONTEXT
        DD      FETCH
        DD      FORW
WORDS1:
        DD      LDUP
        DD      ZBRAN
        DD      WORDS2-$-CW
        DD      IDDOT
        DD      BRAN
        DD      WORDS1-$-CW
WORDS2:
        DD DROP
        DD      SEMIS
;



;  *****************
;  *   EXIT-CODE   *
;  *****************
;
        ALIGN    4
N_XCODE:
        DD      9
        DB      "EXIT-CODE"
        ALIGN    4
XCODE:
        DD    DOVAR
        DD    XCODE+HEADSIZE
        DD    0x0
        DD    WORDS
        DD    N_XCODE
        DD    0

        DD      0
;


;  ***********
;  *   BYE   *
;  ***********
;
        ALIGN    4
N_BYE:
        DD      3
        DB      "BYE"
        ALIGN    4
BYE:
        DD    DOCOL
        DD    BYE+HEADSIZE
        DD    0x0
        DD    XCODE
        DD    N_BYE
        DD    0

; Exit to linux, with status as stored. 
        DD      XCODE, FETCH, X, X, LIT, exit, XOS
        DD      SEMIS   ;Unnecessary, but helpful for decompilation.
; 
;

;  ************
;  *   LIST   *
;  ************
;
        ALIGN    4
N_LLIST:
        DD      4
        DB      "LIST"
        ALIGN    4
LLIST:
        DD    DOCOL
        DD    LLIST+HEADSIZE
        DD    0x0
        DD    BYE
        DD    N_LLIST
        DD    0

        DD      BASE, FETCH, TOR
        DD      DECA
        DD      SCR,STOR

        DD      SKIP
         DD      6
SB3: DB      "SCR # "
       ALIGN    4
        DD      LIT, SB3
        DD      LIT, 6
        DD      LTYPE
        DD      SCR, FETCH, DOT
        DD      SCR, FETCH, BLOCK, BBUF

        DD      LIT, 16, ZERO
        DD     XDO
        DD      LLIST9-$-CW
LLIST1: DD      LIT, ALF, SSLAS
        DD      CR, IDO, LIT, 2, DOTR
        DD      SPACE, LTYPE
        DD     ONE, XPLOOP, ZBRAN
        DD      LLIST1-$-CW
	DD      UNLOOP
LLIST9:
        DD      TDROP
        DD      FROMR, BASE, STOR
        DD      SEMIS
;

;  *************
;  *   INDEX   *
;  *************
;
        ALIGN    4
N_INDEX:
        DD      5
        DB      "INDEX"
        ALIGN    4
INDEX:
        DD    DOCOL
        DD    INDEX+HEADSIZE
        DD    0x0
        DD    LLIST
        DD    N_INDEX
        DD    0

        DD      LIT,AFF
        DD      EMIT,CR
        DD      ONEP,SWAP
        DD     XDO
        DD      INDE9-$-CW
INDE1:  DD      CR,IDO
        DD      LIT,3
        DD      DOTR,SPACE
        DD      ZERO,IDO
        DD      PLINE, LTYPE
        DD     ONE, XPLOOP, ZBRAN
        DD      INDE1-$-CW
	DD      UNLOOP
INDE9:
        DD      SEMIS
;

;  **********
;  *   .S   *
;  **********
;
        ALIGN    4
N_DOTS:
        DD      2
        DB      ".S"
        ALIGN    4
DOTS:
        DD    DOCOL
        DD    DOTS+HEADSIZE
        DD    0x0
        DD    INDEX
        DD    N_DOTS
        DD    0

        DD      CR
        DD      LIT, 'S', EMIT
        DD      LIT, ASO, EMIT
        DD      SPACE
        DD      SPFET, SZERO, FETCH
DOC2:   DD      OVER, OVER,  LSUB, ZLESS
        DD      ZBRAN
        DD      DOC1-$-CW
        DD      ZERO, CELLP, LSUB, LDUP, FETCH
        DD      DOT
        DD      BRAN
        DD      DOC2-$-CW
DOC1:    DD DROP, DROP
        DD      LIT, ASC, EMIT
        DD SEMIS
;

;  ********************
;  *   ENVIRONMENT?   *
;  ********************
;
        ALIGN    4
N_ENVQ:
        DD      12
        DB      "ENVIRONMENT?"
        ALIGN    4
ENVQ:
        DD    DOCOL
        DD    ENVQ+HEADSIZE
        DD    0x0
        DD    DOTS
        DD    N_ENVQ
        DD    0

        DD      LIT, ENV, TWID, PFIND
        DD      TOR, TDROP, FROMR
        DD      LDUP
        DD      ZBRAN
        DD      ENVQ1-$-CW
        DD      EXEC
        DD      LIT, -1
ENVQ1:
        DD      SEMIS
;

; 
;
; This word is not even fig!

;  ***************
;  *   .SIGNON   *
;  ***************
;
        ALIGN    4
N_SIGNON:
        DD      7
        DB      ".SIGNON"
        ALIGN    4
SIGNON:
        DD    DOCOL
        DD    SIGNON+HEADSIZE
        DD    0x0
        DD    ENVQ
        DD    N_SIGNON
        DD    0

; PRINT CPU TYPE (8088)
        DD      CR
        DD      BASE,FETCH
        DD      LIT,36, BASE,STOR
        DD      LCPU, DDOT
        DD      BASE,STOR
;
        DD      LENAME, LTYPE, SPACE
        DD      LVERSION, LTYPE, SPACE
        DD      CR
        DD      SEMIS
;

;
;**** LAST DICTIONARY WORD OF FORTH ****

;  ************
;  *   TASK   *
;  ************
;
        ALIGN    4
N_TASK:
        DD      4
        DB      "TASK"
        ALIGN    4
TASK:
        DD    DOCOL
        DD    TASK+HEADSIZE
        DD    0x0
        DD    SIGNON
        DD    N_TASK
        DD    0

        DD      SEMIS
;
; End of ONLY namespace. Namespaces all end in a link to 0.

;  *********
;  *   '   *
;  *********
;
        ALIGN    4
N_TICK:
        DD      1
        DB      "'"
        ALIGN    4
TICK:
        DD    DOCOL
        DD    TICK+HEADSIZE
        DD    B_IMMED + B_DENOT
        DD    0
        DD    N_TICK
        DD    0

        DD      ITICK
        DD      LITER
        DD      SEMIS
;

;  *********
;  *   &   *
;  *********
;
        ALIGN    4
N_DCHAR:
        DD      1
        DB      "&"
        ALIGN    4
DCHAR:
        DD    DOCOL
        DD    DCHAR+HEADSIZE
        DD    B_IMMED + B_DENOT
        DD    TICK
        DD    N_DCHAR
        DD    0

        DD      PPFET
        DD      NIP
        DD      LDUP, QBL
        DD      LIT, 10, QERR
        DD      LITER
        DD      QDELIM
        DD      SEMIS

;

;  *********
;  *   ^   *
;  *********
;
        ALIGN    4
N_DCTL:
        DD      1
        DB      "^"
        ALIGN    4
DCTL:
        DD    DOCOL
        DD    DCTL+HEADSIZE
        DD    B_IMMED + B_DENOT
        DD    DCHAR
        DD    N_DCTL
        DD    0

        DD      PPFET
        DD      NIP
        DD      LDUP, QBL
        DD      LIT, 10, QERR
        DD      LIT, '@', LSUB
        DD      LITER
        DD      QDELIM
        DD      SEMIS
;

;  *********
;  *   0   *
;  *********
;
        ALIGN    4
N_DEN0:
        DD      1
        DB      "0"
        ALIGN    4
DEN0:
        DD    DOCOL
        DD    LNUMB
        DD    B_IMMED + B_DENOT
        DD    DCTL
        DD    N_DEN0
        DD    0

;  *********
;  *   1   *
;  *********
;
        ALIGN    4
N_DEN1:
        DD      1
        DB      "1"
        ALIGN    4
DEN1:
        DD    DOCOL
        DD    LNUMB
        DD    B_IMMED + B_DENOT
        DD    DEN0
        DD    N_DEN1
        DD    0

;  *********
;  *   2   *
;  *********
;
        ALIGN    4
N_DEN2:
        DD      1
        DB      "2"
        ALIGN    4
DEN2:
        DD    DOCOL
        DD    LNUMB
        DD    B_IMMED + B_DENOT
        DD    DEN1
        DD    N_DEN2
        DD    0

;  *********
;  *   3   *
;  *********
;
        ALIGN    4
N_DEN3:
        DD      1
        DB      "3"
        ALIGN    4
DEN3:
        DD    DOCOL
        DD    LNUMB
        DD    B_IMMED + B_DENOT
        DD    DEN2
        DD    N_DEN3
        DD    0

;  *********
;  *   4   *
;  *********
;
        ALIGN    4
N_DEN4:
        DD      1
        DB      "4"
        ALIGN    4
DEN4:
        DD    DOCOL
        DD    LNUMB
        DD    B_IMMED + B_DENOT
        DD    DEN3
        DD    N_DEN4
        DD    0

;  *********
;  *   5   *
;  *********
;
        ALIGN    4
N_DEN5:
        DD      1
        DB      "5"
        ALIGN    4
DEN5:
        DD    DOCOL
        DD    LNUMB
        DD    B_IMMED + B_DENOT
        DD    DEN4
        DD    N_DEN5
        DD    0

;  *********
;  *   6   *
;  *********
;
        ALIGN    4
N_DEN6:
        DD      1
        DB      "6"
        ALIGN    4
DEN6:
        DD    DOCOL
        DD    LNUMB
        DD    B_IMMED + B_DENOT
        DD    DEN5
        DD    N_DEN6
        DD    0


;  *********
;  *   7   *
;  *********
;
        ALIGN    4
N_DEN7:
        DD      1
        DB      "7"
        ALIGN    4
DEN7:
        DD    DOCOL
        DD    LNUMB
        DD    B_IMMED + B_DENOT
        DD    DEN6
        DD    N_DEN7
        DD    0

;  *********
;  *   8   *
;  *********
;
        ALIGN    4
N_DEN8:
        DD      1
        DB      "8"
        ALIGN    4
DEN8:
        DD    DOCOL
        DD    LNUMB
        DD    B_IMMED + B_DENOT
        DD    DEN7
        DD    N_DEN8
        DD    0

;  *********
;  *   9   *
;  *********
;
        ALIGN    4
N_DEN9:
        DD      1
        DB      "9"
        ALIGN    4
DEN9:
        DD    DOCOL
        DD    LNUMB
        DD    B_IMMED + B_DENOT
        DD    DEN8
        DD    N_DEN9
        DD    0

;  *********
;  *   A   *
;  *********
;
        ALIGN    4
N_DENA:
        DD      1
        DB      "A"
        ALIGN    4
DENA:
        DD    DOCOL
        DD    LNUMB
        DD    B_IMMED + B_DENOT
        DD    DEN9
        DD    N_DENA
        DD    0

;  *********
;  *   B   *
;  *********
;
        ALIGN    4
N_DENB:
        DD      1
        DB      "B"
        ALIGN    4
DENB:
        DD    DOCOL
        DD    LNUMB
        DD    B_IMMED + B_DENOT
        DD    DENA
        DD    N_DENB
        DD    0


;  *********
;  *   C   *
;  *********
;
        ALIGN    4
N_DENC:
        DD      1
        DB      "C"
        ALIGN    4
DENC:
        DD    DOCOL
        DD    LNUMB
        DD    B_IMMED + B_DENOT
        DD    DENB
        DD    N_DENC
        DD    0

;  *********
;  *   D   *
;  *********
;
        ALIGN    4
N_DEND:
        DD      1
        DB      "D"
        ALIGN    4
DEND:
        DD    DOCOL
        DD    LNUMB
        DD    B_IMMED + B_DENOT
        DD    DENC
        DD    N_DEND
        DD    0

;  *********
;  *   E   *
;  *********
;
        ALIGN    4
N_DENE:
        DD      1
        DB      "E"
        ALIGN    4
DENE:
        DD    DOCOL
        DD    LNUMB
        DD    B_IMMED + B_DENOT
        DD    DEND
        DD    N_DENE
        DD    0

;  *********
;  *   F   *
;  *********
;
        ALIGN    4
N_DENF:
        DD      1
        DB      "F"
        ALIGN    4
DENF:
        DD    DOCOL
        DD    LNUMB
        DD    B_IMMED + B_DENOT
        DD    DENE
        DD    N_DENF
        DD    0

;

;  *********
;  *   -   *
;  *********
;
        ALIGN    4
N_DENM:
        DD      1
        DB      "-"
        ALIGN    4
DENM:
        DD    DOCOL
        DD    DENM+HEADSIZE
        DD    B_IMMED + B_DENOT
        DD    DENF
        DD    N_DENM
        DD    0

        DD      PNUMB, DNEGA, SDLITE
        DD      SEMIS
;

;  *********
;  *   +   *
;  *********
;
        ALIGN    4
N_DENP:
        DD      1
        DB      "+"
        ALIGN    4
DENP:
        DD    DOCOL
        DD    DENP+HEADSIZE
        DD    B_IMMED + B_DENOT
        DD    DENM
        DD    N_DENP
        DD    0

        DD      PNUMB, SDLITE
        DD      SEMIS
;

;  *********
;  *   "   *
;  *********
;
        ALIGN    4
N_DENQ:
        DD      1
        DB      '"'
        ALIGN    4
DENQ:
        DD    DOCOL
        DD    DENQ+HEADSIZE
        DD    B_IMMED + B_DENOT
        DD    DENP
        DD    N_DENQ
        DD    0

        DD      LIT, SKIP, COMMA        ;  'SKIP , HERE >R 0 ,
        DD      HERE, TOR, ZERO, COMMA
DENQ1:
        DD      LIT, '"', PARSE         ;           BEGIN &" PARSE
        DD      PPFET, LDUP, LIT, '"', EQUAL ;           PP@@ DUP &" =
        DD      ZBRAN
        DD      DENQ2-$-CW                 ;           WHILE
        DD      TDROP, ONEP             ;           2DROP 1+ R@ $+!
        DD      LDUP, ALLOT, RR, SADD
        DD      BRAN
        DD      DENQ1-$-CW                  ;           REPEAT
DENQ2:
        DD      QBL, ZEQU
        DD      LIT, 10, QERR           ;           ?BLANK 0= 10 ?ERROR
        DD      DROP                    ;                DROP 
        DD      LDUP, ALLOT, RR, SADD     ;         DUP ALLOT R@ $+!
    DD LALIGN
        DD      FROMR, SFET, DLITE      ;           R> $@ POSTPONE DLITERAL ;
        DD      SEMIS
;

; The FORTH word is the only non-denotation in the ONLY namespace.
;  *************
;  *   FORTH   *
;  *************
;
        ALIGN    4
N_FORTH:
        DD      5
        DB      "FORTH"
        ALIGN    4
FORTH:
        DD    DODOE
        DD    FORTH+HEADSIZE
        DD    0x0
        DD    DENQ
        DD    N_FORTH
        DD    0

        DD      DOVOC
        DD      0       ; END OF NAMESPACE LIST

        ALIGN    4

        DD    0x0
        DD    0
        DD    B_DUMMY
        DD    TASK
        DD    (ZERO+(CW*(D_HOFFSET)))
        DD    0

;
STARTOFONLY    EQU FORTH
;
;

;  ************
;  *   CORE   *
;  ************
;
        ALIGN    4
N_CORE:
        DD      4
        DB      "CORE"
        ALIGN    4
CORE:
        DD    DOCOL
        DD    CORE+HEADSIZE
        DD    0x0
        DD    0
        DD    N_CORE
        DD    0

        DD      ZERO    ; Not (fully) present.
        DD      SEMIS
;

;  ***********
;  *   CPU   *
;  ***********
;
        ALIGN    4
N_LCPU:
        DD      3
        DB      "CPU"
        ALIGN    4
LCPU:
        DD    DOCOL
        DD    LCPU+HEADSIZE
        DD    0x0
        DD    CORE
        DD    N_LCPU
        DD    0

       DD      LIT, 0x0113F683, ZERO      ; 'ARMV7'
             ; 'ARMV8'
        DD      SEMIS
;

;  ***************
;  *   VERSION   *
;  ***************
;
        ALIGN    4
N_LVERSION:
        DD      7
        DB      "VERSION"
        ALIGN    4
LVERSION:
        DD    DOCOL
        DD    LVERSION+HEADSIZE
        DD    0x0
        DD    LCPU
        DD    N_LVERSION
        DD    0

_COMMENTED(
_C{       If this is there it is an official release}
        DC      SKIP
        DC      22
IBMPC:  DSS     "IBM-PC ciforth"
        DB      FIGREL+0x40,ADOT,FIGREV+0x30,ADOT,USRVER+0x30
        DC      LIT, IBMPC, LIT, 22
) ;       If M4_VERSION exists and contains a . it is an official release
        DD      SKIP
         DD      5
SB4: DB      "5.4.1"
       ALIGN    4
        DD      LIT, SB4
        DD      LIT, 5
        DD      SEMIS
;

;  ************
;  *   NAME   *
;  ************
;
        ALIGN    4
N_LENAME:
        DD      4
        DB      "NAME"
        ALIGN    4
LENAME:
        DD    DOCOL
        DD    LENAME+HEADSIZE
        DD    0x0
        DD    LVERSION
        DD    N_LENAME
        DD    0

        DD      SKIP
         DD      7
SB5: DB      "ciforth"
       ALIGN    4
        DD      LIT, SB5
        DD      LIT, 7
        DD      SEMIS
;

;  ****************
;  *   SUPPLIER   *
;  ****************
;
        ALIGN    4
N_SUPPLIER:
        DD      8
        DB      "SUPPLIER"
        ALIGN    4
SUPPLIER:
        DD    DOCOL
        DD    SUPPLIER+HEADSIZE
        DD    0x0
        DD    LENAME
        DD    N_SUPPLIER
        DD    0

        DD      SKIP
         DD      40
SB6: DB      "Albert van der Horst/Frans van der Markt"
       ALIGN    4
        DD      LIT, SB6
        DD      LIT, 40
        DD      SEMIS
;
STARTOFENVIRONMENT    EQU   SUPPLIER
;
;
 ;  

_COMMENTED(

The remaining memory ( up to 'EM' ) is
used for:

        1. EXTENSION DICTIONARY
        2. PARAMETER STACK
        3. TERMINAL INPUT BUFFER
        4. RETURN STACK
        5. USER VARIABLE AREA
        6. DISK BUFFERS (UNLESS REQUIRED <1 MBYTE)


) 

;       This is the proper way to do it.
;       No memory addresses should be arrived at through equates.
;       However now we must teach the linker to keep the
;       two sections together.

FORTHSIZE       EQU     $ - _main
; We must use bss, because of the c-library.
; The dictionary is not a normal bss, because it is executable.
; The linker -N option takes care of that. 
       _BSS_

INITDP:                 ;  It may be that it is not consecutive with TASK.
                        ;  Because of bad linking, resulting in a unusable Forth.
        BUFFERSIZE      EQU  (KBBUF+2*CW)*NBUF

; For more task the TAS area is duplicated; the remainder is the same.
        RESB    EM - FORTHSIZE - TAS - BUFFERSIZE 
; INITR1:                         Return stack for next task
;          RESB    US              ;  For next task, if present. 
; The following framework may be repeated for more tasks.
; It is floating, i.e. it may be shifted up and down, in order
; to have INITR0 at a STACKSIZE boundary. 
        RESB    STACKSIZE - US    ; Data stack, only limited if more tasks present.
INITS0:                         ; Grows down.
STRTIB: RESB    TIBSIZE     ; Terminal input buffer.
        RESB    STACKSIZE       ; Return stack, grows down.
INITR0:                         ; Grows down
STRUSA: RESB    US              ; User area
BUF1:   RESB    BUFFERSIZE      ; FIRST DISK BUFFER
ACTUAL_EM:  ; Differs from EM if code is relocated.

        
; 
;


        ;
_COMMENTED(

  MISC. NOTES AND SCATTERED THOUGHTS

- Remember that all the FORTH words in this version are
  upper case letters.  Use <CAPS LOCK> when in FORTH.



- Subscribe to FORTH Dimensions.  It is a valuable source
  of system and application ideas.  Talking with fellow
  FORTH programmers is sure to stir up some exciting ideas.
  Consider joining a FIG chapter.  See the back of FORTH
  Dimensions for more info.

) 
; Define the entry point, not valid for auto booting.
        
        ; _ORIG


























;





