






















@         ciforth : a generic I86 ISO FORTH by HCC FIG
@ $Id: ciarm.gnr,v 6.120 2022/03/31 15:10:56 albert Exp $
@ Copyright (2012): Albert van der Horst/Frans van der Markt by GNU Public License
@
@HCC FIG Holland : Hobby Computer Club, Forth Interest Group  Holland
@ #   ciforth $Revision: 6.120 $
@
@
@
@
@ If this is a configured assembly file, it should be accompanied with configured
@ documentation (texinfo, ps, html.)
@ WITHOUT THE DOCUMENTATION: GIVE UP! GET THE REAL THING!
@ You have a configured system, if there are NO curly brackets on the next line.
@                          
@
@ Configuration of this particular version:
@ 32-bits
@ running under Linux  @  with native forth I/O 

@ Normally ciforth doesn't observe ISO >IN.
@ Contains :
@ (there may be no items here.)
@        Security words
@         Loadable words, i.e. all of ISO CORE, more than is needed
@           for a self contained kernel.
@ A field in the header to point to source
@
#
# Gnu as version of ciforth created by ``m4'' from generic listing.
# This source can be assembled using versions better than 2.13.
# Provided gcc is installed, 
# this file can be assembled and linked in the following way:
#       as lina.s
#       ld -s -N a.out -o lina -lc
# The -s (strip) may not be necessary.


 /* 
        A generic version of ISO FORTH for ARM processors
                Albert van der Horst/Frans van der Markt

                in cooperation with
                HCC Forth user group
                The Netherlands
                www.forth.hccnet.nl

              based on
              FIG-FORTH
   implemented by:  Charlie Krajewski
                    205 ( BIG ) Blue Rd.
                    Middletown, CT  06457

  The listing has been made possible by the
  prior work of:
               Thomas Newman, Hayward, Ca.

 : other_acknowledgements
         John_Cassidy
         Kim_Harris
         George_Flammer
         Robert_D._Villwock ;

 : for tools
         Richard M. Stallman
         Linus Torvalds

No one who programs with FORTH can afford to be without:
  "Starting Forth  by Leo Brodie" and "Thinking Forth by Leo Brodie".
   Both out of print, but saved on the internet.

This Forth is a descendant in the 300+ (RCS)- generations from fig-Forth.

For nostalgic reasons the following comment has never been removed:
   Although there is much to be said for typing in your own
   listing and getting it running, there is much to be said
   not typing in your own listing.  If you feel that 100+
   pages of plinking is nutty, contact me for availability
   of a disc with source & executable files.  Obtainable at
   a bargain basement price, prepare yourself for bargain
   basement support.

All publications of the FORTH Interest Group are public domain.
They may be further distributed by the inclusion of this
credit notice:
               This publication has been made available by:

               FORTH Interest Group
               P.O. Box 1105
               San Carlos, Ca.  94070
[I feel obliged to keep this last one in (AH). Note that although it is
based on fig-Forth no stone is left unturned.]
 */          #
@ ########################################################################################
@                       PREPARATION (no code)
@ ########################################################################################

FIGREL  =     5       @ FIG RELEASE #
FIGREV  =     0       @ FIG REVISION #
USRVER  =     0      @ USER VERSION NUMBER, a digit now
@
@      VERY ELEMENTARY .
CW      =     4    @ Size of a cell in Forth, not in the bootcode.
ERRORSCREEN =     48    @ Screen where the error messages start.
@
@      MEMORY LAYOUT.
@ Normally this is specified at the m4 configuration level.
@ For a configured system these values can be changed at this single place. 
NBUF    =     16   @ No. of buffers, or screens 
KBBUF   =     1024      @ Data bytes per disk buffer
US      =     0x40*CW  @ User variable space, eats into next data stack, if any.
EM      =     (0x4000000)-0x74     @ Where the memory ends w.r.t. ORIG. 1) 
EMP     =     (EM-1)/0x1000+1 @ Number of pages.
TAS     =     0x10000    @Size of unitialised area per task.
STACKSIZE   = TAS/4  @For both stacks.  but for now
TIBSIZE     = TAS/4  @TIB size. 
@


@
@      ASCII CHARACTER EQUIVALENTS
@
ABL     =     ' '     @ SPACE
ACR     =     0x0D     @ CR
AMS     =     '-'          @ MINUS SIGN 
ASO     =     '['          @ SQUARE BRACKET OPEN 
ASC     =     ']'          @ SQUARE BRACKET CLOSE 
ADOT    =     '.'          @ PERIOD
ALF     =     0x0A     @ LINE FEED, USED INTERNALLY AS LINE ENDER 
AFF     =     0x0C     @ FORM FEED
BELL    =     0x07     @ ^G
BSIN    =     0x08     @ INPUT DELETE CHARACTER
BSOUT   =     0x08     @ OUTPUT BACKSPACE ( ^H )
@
@      HEADER RELATED EQUATES
@      dea is the dictionary entry address 
B_DUMMY   =     0x01     @ dea is dummy, from namespace link
B_INVIS   =     0x02     @ dea is invisible, "smudged".
B_IMMED   =     0x04     @ dea is a immediate.
B_DENOT   =     0x08     @ dea is a denotation.
C_HOFFSET =     0       @ Offsets of code field in cells, w.r.t. dea
D_HOFFSET =     1       @ Same for data field
F_HOFFSET =     2       @ Same for flag field
L_HOFFSET =     3       @ Same for link field
N_HOFFSET =     4       @ Same for name field
S_HOFFSET =     5       @ Same for source field
PH_OFFSET =     5+1   @ Past header field: Start of data area. 
BD_OFFSET =     5+1+1 @ Start of BODY for CREATEd word.
HEADSIZE  =     (CW*(PH_OFFSET))  @In cells, only to clean up source.
@
 @  


BPS     =     512             @Bytes/sector, common to all of MSDOS
SPB     =     KBBUF/BPS       @Sectors per buffer 
@







@ Bring the names in line with what Forth want immediately. 
@ ------------------------------------------------------------      
@   Start of constants stolen from C.                                 
@ ------------------------------------------------------------      

SEEK_SET	=	0x0
SEEK_CUR	=	0x1
TCGETS	=	0x5401
TCSETS	=	0x5402
ECHO	=	0x8
EAGAIN	=	0xb
EINTR	=	0x4
EPIPE	=	0x20
VMIN	=	0x6
VTIME	=	0x5
ICANON	=	0x2
O_RDWR	=	0x2
O_RDONLY	=	0x0
O_WRONLY	=	0x1
O_CREAT	=	0x40
O_NONBLOCK	=	0x800
MAP_SHARED	=	0x1
MAP_PRIVATE	=	0x2
P_PID	=	0x1
WEXITED	=	0x4
SIZE_TERMIO	=	0x3c

@ Numbers of system calls. See "Linux kernel Internals" Appendix A. 
@ By M.Beck, H. Boehme e.a. Addison Wesley.                         
@ The system calls themselves are extensively documented in chapter 
@ 2 of the man pages, e.g. "man 2 exit"
exit	=	0x1
open	=	0x5
close	=	0x6
creat	=	0x8
unlink	=	0xa
chdir	=	0xc
read	=	0x3
_newselect	=	0x8e
write	=	0x4
ioctl	=	0x36
lseek	=	0x13
execve	=	0xb
fork	=	0x2
pipe	=	0x2a
mmap2	=	0xc0
waitid	=	0x118
@ ------------------------------------------------------------      
@   End of constants stolen from C.                                 
@ ------------------------------------------------------------      

select = _newselect
mmap   = mmap2


RAWIO           =     (ECHO | ICANON)

@ 


_main:



@ allocatable, executable 
@ Must be normal .text or otherwise linking with c-lib fails. 
@ .text is normally not writable but linker -N option takes care of that. 
               .arm
        .text

        #
        .global  _start       @ Entry point.
        .global  start        @ Entry point.
_ORIG:
_start:      @ Entry point.
@ 









NOBOOT:         @ Skip till here if not booting.


ENDBOOT:










 

@ ########################################################################################
@                       FORTH startup CODE + restarts + debug 
@ ########################################################################################
@


@ 4.15.3.2
start:
@
COLD_ENTRY:

        ADR   R4, USINI
        MOV     R0, SP              @Remember ARGS.
        STR     R0, [R4, #CW*(31)]
@


@ Initialisation of the Forth engine.
        ADR     R4, USINI
        LDR     R0, [R4, #CW*(2)]  @Parameter stack
        MOV     R13, R0           @Parameter stack
        LDR     R12, [R4, #CW*(3)]  @Return stack
        ADR     R11, COLD + CW*(PH_OFFSET)    @Interpreter pointer
      LDR     R0, [R11],#+(CW*(1))
         LDR     R1,[R0]
         BX      R1  



@ 

@
@
@ ########################################################################################
@                       FORTH ITSELF
@ ########################################################################################
@
 /* 
   FORTH REGISTERS
   The names under FORTH are used in the generic source.

   FORTH   ARM      FORTH PRESERVATION RULES
   -----   ----     ----- ------------ -----
           R0-R3   Used within and cluttered by Forth words.
                    across FORTH words.
                    The first of those is WOR (probably)

   WOR   R0      Working register.  When entering a word
                    via its code field the DEA is passed in WOR.

            R0      Scratch register.  Used to pass data from
                    FORTH words, see label APUSH or macro _APUSH

            R1      General register.  Used when two data items are passed
                    from FORTH words, see label DPUSH or macro _DPUSH

   HIP   R11      High level Interpreter Pointer.  Must be preserved

   RPO   R12      Return stack pointer.  Must be preserved across
                    FORTH words.

   SPO   R13      Parameter stack pointer.  Must be preserved
                    across FORTH words.

           PC       Machine Process Counter, sometimes used
                    for relative addressing

This is the ARM asignment, not valid for 64 bit.

   R13     SP      ARM stack pointer.
                   In thumb this can be used with pop and push.

   R14     RL      Machine Link Register for calls.
                    Can be used within Forth words, with care.

  R15     PC        Machine Process Counter (Program counter)
                    Cannot be used within Forth words.



            The remaining registers are free to use.
            R0-R3 are the normal Forth work registers.
            Within assembler words R0 can be used too,
            as it has served its purpose.

            Here are some  useful code sequences.
            For next:
                    LDR     R0, [R11],#+(CW*(1))
         LDR     R1,[R0]
         BX      R1  
            For push single and next:
                    STR     R0, [R13,#-(CW*(1))]!
        LDR     R0, [R11],#+(CW*(1))
         LDR     R1,[R0]
         BX      R1  
            For push double and next:
                    STR     R1, [R13,#-(CW*(1))]!
        STR     R0, [R13,#-(CW*(1))]!
        LDR     R0, [R11],#+(CW*(1))
         LDR     R1,[R0]
         BX      R1  



----------------------------------------------------------
 */          #
 /* 
---------------------------------------------

   COMMENT CONVENTIONS
   ------- -----------

   =       IS EQUAL TO
   <-      ASSIGNMENT

  NAME        =  Address of name
  (NAME)      =  Contents of name
  DEA         =  DICTIONARY ENTRY ADDRESS
                 fields have fixed offsets from DEA
  CFA         =  CODE FIELD ADDRESS : a pointer to executable code
  DFA         =  DATA FIELD ADDRESS : a pointer to
                        data/high level code/ DOES> pointer
  FFA         =  FLAG FIELD ADDRESS: contains flags
  LFA         =  LINK FIELD ADDRESS: a pointer to the next DEA
  NFA         =  NAME FIELD ADDRESS: a pointer to a name buffer
  SFA =  SOURCE FIELD ADDRESS: a pointer into source code
  PHA         =  POST HEADER ADDRESS: After all fields

  S1          =  Parameter stack - 1st cell
  S2          =  Parameter stack - 2nd cell
  RP1          =  Return stack    - 1st cell
  R2          =  Return stack    - 2nd cell

  LSB         =  Least significant bit
  MSB         =  Most  significant bit
  LB          =  Low byte
  HB          =  High byte
  LC          =  Low  cell
  HC          =  High cell

------------------------------------------------------------
 */  @ 
        #

@
@ 
@ There may be no jumps to NEXT at all 
@ The label NEXT1 is rarely relevant (for _OLDDEBUG_) 
DPUSH:  STR     R1, [R13,#-(CW*(1))]!      @ Fall through.
APUSH:  STR     R0, [R13,#-(CW*(1))]!
NEXT:
@
        LDR     R0, [R11],#+(CW*(1))                  @ R0 <- (HIP) 
NEXT1:
        LDR     R1, [R0]
        BX      R1     @ TO `CFA'
@
@       Dictionary starts here.


@  ************
@  *   ONLY   *
@  ************
@
        .align
N_ONLY:
        .word      4
        .ascii      "ONLY"
        .align
ONLY:
        .word    DODOE
        .word    ONLY+HEADSIZE
        .word    0x0
        .word    0
        .word    N_ONLY
        .word    0

        .word      DOVOC
        .word      FORTH     @ NEXT NAMESPACE 
ONLYBODY:

        .align

        .word    0x0
        .word    0
        .word    B_DUMMY
        .word    STARTOFONLY
        .word    (ZERO+(CW*(D_HOFFSET)))
        .word    0

@

@  *******************
@  *   ENVIRONMENT   *
@  *******************
@
        .align
N_ENV:
        .word      11
        .ascii      "ENVIRONMENT"
        .align
ENV:
        .word    DODOE
        .word    ENV+HEADSIZE
        .word    0x0
        .word    ONLY
        .word    N_ENV
        .word    0

        .word      DOVOC
        .word      ONLY       @ NEXT NAMESPACE 

        .align

        .word    0x0
        .word    0
        .word    B_DUMMY
        .word    STARTOFENVIRONMENT
        .word    (ZERO+(CW*(D_HOFFSET)))
        .word    0

@

@  ************
@  *   COLD   *
@  ************
@
        .align
N_COLD:
        .word      4
        .ascii      "COLD"
        .align
COLD:
        .word    DOCOL
        .word    COLD+HEADSIZE
        .word    0x0
        .word    ENV
        .word    N_COLD
        .word    0

@ Fill user area for single task.
        .word      LIT, USINI      @ Fails if SP/RP are not usable!
        .word      LIT, USINI+(CW*(1)), FETCH
        .word      LIT, US
        .word      LCMOVE
        .word      INIT
        .word      ONE            @ Sign on wanted.


        .word      ZERO, LIT, TCGETS, TERMIO
        .word      LIT, ioctl, XOS, ZLESS
        .word      ZBRAN
        .word      COLD1-.-CW
        @ Not a terminal. Cannot set the terminal, get rid of "OK".
        .word      DROP, ZERO      @ Sign on suppressed.
        .word      LIT,TASK  @ Suppress OK for pipes.
        .word      LIT,OK
        .word      LIT, (CW*(3)), LCMOVE
COLD1:
 @ 

        .word      SKIP
         .word      8
SB0: .ascii      "/dev/mem"
       .align
        .word      LIT, SB0
        .word      LIT, 8
        .word      LIT, 0x00181002
        .word      OFILE, LOR, DEVMEM, STOR
@
       .word      OPTIONS
        .word      ZBRAN
        .word      COLD5-.-CW

        .word      SIGNON    @ Suppressed for scripting! Or any options.
COLD5:
        .word      ABORT
        .word      BYE     @ In case of turnkey programs.
        .word      SEMIS   @ Unnecessary, but helpful for decompilation.
@

@  ************
@  *   USER   *
@  ************
@
        .align
N_USER:
        .word      4
        .ascii      "USER"
        .align
USER:
        .word    DOCOL
        .word    USER+HEADSIZE
        .word    0x0
        .word    COLD
        .word    N_USER
        .word    0

        .word      LCONST
        .word      PSCOD
DOUSE:  LDR     R3,[R0, # + (CW*((D_HOFFSET)))] @PFA  
        ADR     R0, USINI
        LDR     R0,[R0]
        ADD     R0,R0,R3      @ADDR OF VARIABLE
        STR     R0, [R13,#-(CW*(1))]!
        LDR     R0, [R11],#+(CW*(1))
         LDR     R1,[R0]
         BX      R1  

@

@  ***************
@  *   +ORIGIN   *
@  ***************
@
        .align
N_PORIG:
        .word      7
        .ascii      "+ORIGIN"
        .align
PORIG:
        .word    DOCOL
        .word    PORIG+HEADSIZE
        .word    0x0
        .word    USER
        .word    N_PORIG
        .word    0

        .word      LIT
        .word      USINI
        .word      PLUS
        .word      SEMIS
@
@      Initialisation block for user variables through DOC-LINK
@       <<<<< must be in same order as user variables >>>>>
@ To be fig compatible, there should be two vectors here  
@ for the cold and warm start.
@
@ ------------------------ Forth entry code, hooks for restart -------------
@ FIXME. You want to jump here from -2 CELLS +USINI / -4 CELLS +USINI 
        ADR     R4, COLD
        MOV     R0, R4
        LDR     R1, [R0]
        BX      R1          @Hope stacks are still okay.
        .align
@ Make it possible to revector WARM. You can even make it low level. 
        ADR      R4, WARM
        MOV      R0,R4
        LDR     R1, [R0]          @Hope stacks are still okay.
        BX      R1
        .align
USINI:  .word      STRUSA
        .word      STRUSA  @ INIT (U0) user area of the main task 1
        .word      INITS0  @ INIT (S0)         2
        .word      INITR0  - (CW*(1))
                        @ INIT (R0)         3
        .word      STRTIB  @ INIT (TIB)        4
        .word      BSIN    @ RUBOUT: get rid of latest char 5
        .word      0       @ AVAILABLE         6
        .word      1       @ INIT (WARNING)     7
        .word      INITDP  @      INIT (FENCE)  8
DPA:    .word      INITDP  @      INIT (DP)     9
        .word      ENV @       INIT (VOC-LINK) 10
        .word      39+16        @ INIT (MAX-USER) 
@
        .word      0, 0            @ WHERE             12 13 
        .word      0, 0            @REMAINDER   14 15 

        .space    US-(. - USINI)        @ All user can be initialised.
@
@      <<<<< end of data used by cold start >>>>>
@


@  **************
@  *   VMA-IO   *
@  **************
@
        .align
N_VMAIO:
        .word      6
        .ascii      "VMA-IO"
        .align
VMAIO:
        .word    DOVAR
        .word    VMAIO+HEADSIZE
        .word    0x0
        .word    PORIG
        .word    N_VMAIO
        .word    0

        .word 0xB2000000
@

@  *******************
@  *   PERIPHERALS   *
@  *******************
@
        .align
N_PERIPHERALS:
        .word      11
        .ascii      "PERIPHERALS"
        .align
PERIPHERALS:
        .word    DOVAR
        .word    PERIPHERALS+HEADSIZE
        .word    0x0
        .word    VMAIO
        .word    N_PERIPHERALS
        .word    0

        .word 0x20000000
@

@  ***************
@  *   DEV-MEM   *
@  ***************
@
        .align
N_DEVMEM:
        .word      7
        .ascii      "DEV-MEM"
        .align
DEVMEM:
        .word    DOVAR
        .word    DEVMEM+HEADSIZE
        .word    0x0
        .word    PERIPHERALS
        .word    N_DEVMEM
        .word    0

        .word 0
@

@  ***************
@  *   MMAP-IO   *
@  ***************
@
        .align
N_MMAPIO:
        .word      7
        .ascii      "MMAP-IO"
        .align
MMAPIO:
        .word    DOCOL
        .word    MMAPIO+HEADSIZE
        .word    0x0
        .word    DEVMEM
        .word    N_MMAPIO
        .word    0

@ The libc mmap definition, not necessarily documenting the system call! 
@ mmap( void *addr, size_t length, int prot, int flags, int fd,
@ off_t offset );     see man mmap

      .word     DEVMEM, FETCH, LDUP, ZERO, GREAT
      .word      ZBRAN
        .word      MMAP2-.-CW
      .word     TOR
      .word     VMAIO, FETCH
      .word     LIT, 0x010000000 @ length of mapped area
      .word     LIT, 3                  @ protection-bits
      .word     LIT, MAP_SHARED         @ flags
      .word     FROMR
      .word     PERIPHERALS, FETCH
 .word     LIT, 12, RSHIFT     @ Number of 4K pages  
      .word     ZERO                @ dummy 
      .word     LIT, mmap, XOS7
MMAP2:
      .word     VMAIO, STOR
      .word      SEMIS   @Unnecessary, but helpful for decompilation.
@
@

@

@  *************
@  *   ABORT   *
@  *************
@
        .align
N_ABORT:
        .word      5
        .ascii      "ABORT"
        .align
ABORT:
        .word    DOCOL
        .word    ABORT+HEADSIZE
        .word    0x0
        .word    MMAPIO
        .word    N_ABORT
        .word    0

        .word      SZERO, FETCH, SPSTO
        .word      ZERO, HANDLER, STOR
        .word      QUIT
        .word      SEMIS   @Unnecessary, but helpful for decompilation.
@
@

@  ************
@  *   WARM   *
@  ************
@
        .align
N_WARM:
        .word      4
        .ascii      "WARM"
        .align
WARM:
        .word    DOCOL
        .word    WARM+HEADSIZE
        .word    0x0
        .word    ABORT
        .word    N_WARM
        .word    0

        .word      INIT
        .word      SIGNON
        .word      ABORT
        .word      SEMIS   @Unnecessary, but helpful for decompilation.
@

@  ************
@  *   NOOP   *
@  ************
@
        .align
N_NOOP:
        .word      4
        .ascii      "NOOP"
        .align
NOOP:
        .word    NOOP+HEADSIZE
        .word    NOOP+HEADSIZE
        .word    0x0
        .word    WARM
        .word    N_NOOP
        .word    0

       LDR     R0, [R11],#+(CW*(1))
         LDR     R1,[R0]
         BX      R1  
@
@ 
@

@  ***********
@  *   LIT   *
@  ***********
@
        .align
N_LIT:
        .word      3
        .ascii      "LIT"
        .align
LIT:
        .word    LIT+HEADSIZE
        .word    LIT+HEADSIZE
        .word    0x0
        .word    NOOP
        .word    N_LIT
        .word    0

        LDR     R0, [R11],#+(CW*(1))                  @ R0 <- (HIP) 
        STR     R0, [R13,#-(CW*(1))]!
        LDR     R0, [R11],#+(CW*(1))
         LDR     R1,[R0]
         BX      R1            @ TO TOP OF STACK
@

@todo EXECUTE
@  ***************
@  *   EXECUTE   *
@  ***************
@
        .align
N_EXEC:
        .word      7
        .ascii      "EXECUTE"
        .align
EXEC:
        .word    EXEC+HEADSIZE
        .word    EXEC+HEADSIZE
        .word    0x0
        .word    LIT
        .word    N_EXEC
        .word    0

        LDR     R0, [R13],#+(CW*(1))      @ GET XT
        LDR    R1, [R0]
        BX     R1  @(IP) <- (PFA)

@

@  ***************
@  *   RECURSE   *
@  ***************
@
        .align
N_RECURSE:
        .word      7
        .ascii      "RECURSE"
        .align
RECURSE:
        .word    DOCOL
        .word    RECURSE+HEADSIZE
        .word    B_IMMED
        .word    EXEC
        .word    N_RECURSE
        .word    0

        .word      LATEST, COMMA
        .word      SEMIS
@

@  **************
@  *   BRANCH   *
@  **************
@
        .align
N_BRAN:
        .word      6
        .ascii      "BRANCH"
        .align
BRAN:
        .word    (SKIP+HEADSIZE)
        .word    BRAN+HEADSIZE
        .word    0x0
        .word    RECURSE
        .word    N_BRAN
        .word    0

@

@  ************
@  *   SKIP   *
@  ************
@
        .align
N_SKIP:
        .word      4
        .ascii      "SKIP"
        .align
SKIP:
        .word    SKIP+HEADSIZE
        .word    SKIP+HEADSIZE
        .word    0x0
        .word    BRAN
        .word    N_SKIP
        .word    0

BRAN1:
        LDR     R0, [R11],#+(CW*(1))                  @ R0 <- (HIP) 

        SUB     R0,R0, #1
        
         ORR     R0,R0, #(CW-1) 
        ADD     R0,R0, #1
@
        ADD     R11,R11,R0
        LDR     R0, [R11],#+(CW*(1))
         LDR     R1,[R0]
         BX      R1  
@

@  ***************
@  *   0BRANCH   *
@  ***************
@
        .align
N_ZBRAN:
        .word      7
        .ascii      "0BRANCH"
        .align
ZBRAN:
        .word    ZBRAN+HEADSIZE
        .word    ZBRAN+HEADSIZE
        .word    0x0
        .word    SKIP
        .word    N_ZBRAN
        .word    0

        LDR     R0, [R13],#+(CW*(1))      @ GET STACK VALUE
        ANDS     R0,R0,R0   @ ZERO?
        BEQ      BRAN1   @ YES, BRANCH
        ADD      R11,R11,# + (CW*(1))
        LDR     R0, [R11],#+(CW*(1))
         LDR     R1,[R0]
         BX      R1  
@

@  ***************
@  *   (+LOOP)   *
@  ***************
@
        .align
N_XPLOOP:
        .word      7
        .ascii      "(+LOOP)"
        .align
XPLOOP:
        .word    XPLOOP+HEADSIZE
        .word    XPLOOP+HEADSIZE
        .word    0x0
        .word    ZBRAN
        .word    N_XPLOOP
        .word    0

        LDR     R3, [R13],#+(CW*(1))      @ GET LOOP VALUE
        LDR     R2,[R12]  @ INDEX = INDEX + INCR
        ADD     R2,R2,R3
        STR     R2,[R12]  @ GET NEW INDEX
        LDR     R4, [R12, #+(CW*(1))]
        SUB     R2,R2,R4  @ COMPARE WITH LIMIT
        EOR     R2,R2,R3  @ GENERATE SIGN
        ANDS    R2,R2,R2  @ TEST SIGN
        EOR     R0,R0,R0  @ DEFAULT
        BMI     XPO1     @ KEEP LOOPING
        SUB     R0,R0, #1      @TRUE
XPO1: STR     R0, [R13,#-(CW*(1))]!
        LDR     R0, [R11],#+(CW*(1))
         LDR     R1,[R0]
         BX      R1  
@

@  ************
@  *   (DO)   *
@  ************
@
        .align
N_XDO:
        .word      4
        .ascii      "(DO)"
        .align
XDO:
        .word    XDO+HEADSIZE
        .word    XDO+HEADSIZE
        .word    0x0
        .word    XPLOOP
        .word    N_XDO
        .word    0

        LDR     R2, [R11],#+(CW*(1))                  @ R0 <- (HIP) 
        ADD     R2,R2,R11  @Make absolute
        LDR     R1, [R13],#+(CW*(1))           @ INITIAL INDEX VALUE
        LDR     R3, [R13],#+(CW*(1))           @ LIMIT VALUE
        
         MOV     R0, R12
         MOV     R12, R13
         MOV     R13, R0
   @ GET RETURN STACK, USES REGISTER R0 ! 
        STR     R2, [R13,#-(CW*(1))]!      @ Target location.
        STR     R3, [R13,#-(CW*(1))]!
        STR     R1, [R13,#-(CW*(1))]!
        
         MOV     R0, R12
         MOV     R12, R13
         MOV     R13, R0
   @ GET PARAMETER STACK, USES REGISTER R0 ! 
        LDR     R0, [R11],#+(CW*(1))
         LDR     R1,[R0]
         BX      R1  
@

@  *************
@  *   (?DO)   *
@  *************
@
        .align
N_XQDO:
        .word      5
        .ascii      "(?DO)"
        .align
XQDO:
        .word    XQDO+HEADSIZE
        .word    XQDO+HEADSIZE
        .word    0x0
        .word    XDO
        .word    N_XQDO
        .word    0

        LDR     R2, [R11],#+(CW*(1))                  @ R0 <- (HIP) 
        ADD     R2,R2,R11  @Make absolute
        LDR     R1, [R13],#+(CW*(1))      @ INITIAL INDEX VALUE
        LDR     R3, [R13],#+(CW*(1))      @ LIMIT VALUE
        CMP     R1,R3
        BGE     QXDO1
        
         MOV     R0, R12
         MOV     R12, R13
         MOV     R13, R0
   @ GET RETURN STACK, USES REGISTER R0 ! 
        STR     R2, [R13,#-(CW*(1))]!      @ Target location.
        STR     R3, [R13,#-(CW*(1))]!
        STR     R1, [R13,#-(CW*(1))]!
        
         MOV     R0, R12
         MOV     R12, R13
         MOV     R13, R0
   @ GET PARAMETER STACK, USES REGISTER R0 ! 
        LDR     R0, [R11],#+(CW*(1))
         LDR     R1,[R0]
         BX      R1  
QXDO1:  MOV     R11,R2
        LDR     R0, [R11],#+(CW*(1))
         LDR     R1,[R0]
         BX      R1  
@

@  *********
@  *   I   *
@  *********
@
        .align
N_IDO:
        .word      1
        .ascii      "I"
        .align
IDO:
        .word    IDO+HEADSIZE
        .word    IDO+HEADSIZE
        .word    0x0
        .word    XQDO
        .word    N_IDO
        .word    0

        LDR     R0,[R12] @ GET INDEX VALUE
        STR     R0, [R13,#-(CW*(1))]!
        LDR     R0, [R11],#+(CW*(1))
         LDR     R1,[R0]
         BX      R1            @ TO PARAMETER STACK
@

@  *********
@  *   J   *
@  *********
@
        .align
N_JDO:
        .word      1
        .ascii      "J"
        .align
JDO:
        .word    JDO+HEADSIZE
        .word    JDO+HEADSIZE
        .word    0x0
        .word    IDO
        .word    N_JDO
        .word    0

        LDR     R0,[R12,#+(CW*(3))] @ GET INDEX VALUE
        STR     R0, [R13,#-(CW*(1))]!
        LDR     R0, [R11],#+(CW*(1))
         LDR     R1,[R0]
         BX      R1            @ TO PARAMETER STACK
@

@  **************
@  *   UNLOOP   *
@  **************
@
        .align
N_UNLOOP:
        .word      6
        .ascii      "UNLOOP"
        .align
UNLOOP:
        .word    UNLOOP+HEADSIZE
        .word    UNLOOP+HEADSIZE
        .word    0x0
        .word    JDO
        .word    N_UNLOOP
        .word    0
      
        ADD R12,R12,#(CW*(3))
        LDR     R0, [R11],#+(CW*(1))
         LDR     R1,[R0]
         BX      R1  
@

@  *************
@  *   DIGIT   *
@  *************
@
        .align
N_DIGIT:
        .word      5
        .ascii      "DIGIT"
        .align
DIGIT:
        .word    DIGIT+HEADSIZE
        .word    DIGIT+HEADSIZE
        .word    0x0
        .word    UNLOOP
        .word    N_DIGIT
        .word    0

        LDR     R1, [R13],#+(CW*(1))      @NUMBER BASE
        LDR     R0, [R13],#+(CW*(1))      @ASCII DIGIT
        AND     R0,R0,#0xFF
        AND     R1,R1,#0xFF
        SUBS    R0,R0,#'0'
        BLT     DIGI2   @NUMBER ERROR
        CMP     R0,#9
        BLE     DIGI1   @NUMBER = 0 THRU 9
        SUB     R0,R0,#7
        CMP     R0,#10   @NUMBER 'A' THRU 'Z'?
        BLT     DIGI2   @NO
DIGI1:  CMP     R0,R1   @ COMPARE NUMBER TO BASE
        BGE     DIGI2   @NUMBER ERROR
        MOV     R1,R0   @NEW BINARY NUMBER
        
        MOV    R0,#-1  @TRUE FLAG  
        STR     R1, [R13,#-(CW*(1))]!
        STR     R0, [R13,#-(CW*(1))]!
        LDR     R0, [R11],#+(CW*(1))
         LDR     R1,[R0]
         BX      R1            @ADD TO STACK
@   NUMBER ERROR
DIGI2:  MOV    R0, #0 @FALSE FLAG
        STR     R1, [R13,#-(CW*(1))]!
        STR     R0, [R13,#-(CW*(1))]!
        LDR     R0, [R11],#+(CW*(1))
         LDR     R1,[R0]
         BX      R1  
@

@  **************
@  *   ~MATCH   *
@  **************
@
        .align
N_NMATCH:
        .word      6
        .ascii      "~MATCH"
        .align
NMATCH:
        .word    DOCOL
        .word    NMATCH+HEADSIZE
        .word    0x0
        .word    DIGIT
        .word    N_NMATCH
        .word    0

@
        .word      TOR                           @ Remember dea
        .word      OVER
        .word      RR, TNFA, FETCH, SFET
        .word      CORA
        .word      FROMR, SWAP                   @ Leave flag on top of dea.
        .word      SEMIS                            @
@

@  **************
@  *   ?BLANK   *
@  **************
@
        .align
N_QBL:
        .word      6
        .ascii      "?BLANK"
        .align
QBL:
        .word    DOCOL
        .word    QBL+HEADSIZE
        .word    0x0
        .word    NMATCH
        .word    N_QBL
        .word    0

        .word      LBL, ONEP, LESS
        .word      SEMIS
@

@  ************
@  *   PP@@   *
@  ************
@
        .align
N_PPFET:
        .word      4
        .ascii      "PP@@"
        .align
PPFET:
        .word    DOCOL
        .word    PPFET+HEADSIZE
        .word    0x0
        .word    QBL
        .word    N_PPFET
        .word    0

        .word      SRC, CELLP, TFET
        .word      OVER, EQUAL
        .word      ZBRAN
        .word      PPFET1-.-CW
        .word      ZERO
        .word      BRAN
        .word      PPFET2-.-CW
PPFET1:
        .word      LDUP
        
        .word      CFET
        .word      ONE, PP, PSTOR
PPFET2:
        .word      SEMIS
@

@  ************
@  *   NAME   *
@  ************
@
        .align
N_LNAME:
        .word      4
        .ascii      "NAME"
        .align
LNAME:
        .word    DOCOL
        .word    LNAME+HEADSIZE
        .word    0x0
        .word    PPFET
        .word    N_LNAME
        .word    0

        .word      X
PWORD0: .word      DROP
        .word      PPFET, QBL
        .word      OVER, SRC, CELLP, FETCH, LSUB @ At end?
        .word      LAND, ZEQU
        .word      ZBRAN
        .word      PWORD0-.-CW

        .word      X
PWORD1: .word      DROP
        .word      PPFET, QBL
        .word      ZBRAN
        .word      PWORD1-.-CW

        .word      OVER, LSUB
        
        .word      SEMIS
@

@  *************
@  *   PARSE   *
@  *************
@
        .align
N_PARSE:
        .word      5
        .ascii      "PARSE"
        .align
PARSE:
        .word    DOCOL
        .word    PARSE+HEADSIZE
        .word    0x0
        .word    LNAME
        .word    N_PARSE
        .word    0

        .word      SRC, CELLP, TFET
        .word      OVER, LSUB
        
        .word      ROT, SSLAS, TSWAP
        .word      ZEQU
        .word      ZBRAN
        .word      PARSE8-.-CW
        .word      DROP, SRC, CELLP, FETCH
PARSE8: 
        .word      PP, STOR
        .word SEMIS
@

@  ***********
@  *   SRC   *
@  ***********
@
        .align
N_SRC:
        .word      3
        .ascii      "SRC"
        .align
SRC:
        .word    DOUSE
        .word    (CW*(27))
        .word    0x0
        .word    PARSE
        .word    N_SRC
        .word    0
      @ And 28 and 29.
@

@  **************
@  *   SOURCE   *
@  **************
@
        .align
N_SOURCE:
        .word      6
        .ascii      "SOURCE"
        .align
SOURCE:
        .word    DOCOL
        .word    SOURCE+HEADSIZE
        .word    0x0
        .word    SRC
        .word    N_SOURCE
        .word    0

        .word      SRC, TFET
        .word      SWAP, OVER, LSUB
        .word      SEMIS
@

@

@  **********
@  *   CR   *
@  **********
@
        .align
N_CR:
        .word      2
        .ascii      "CR"
        .align
CR:
        .word    DOCOL
        .word    CR+HEADSIZE
        .word    0x0
        .word    SOURCE
        .word    N_CR
        .word    0

        .word      LIT,ALF
        .word      EMIT
        .word      ZERO, LOUT, STOR
        .word      SEMIS
@

@  *************
@  *   CMOVE   *
@  *************
@
        .align
N_LCMOVE:
        .word      5
        .ascii      "CMOVE"
        .align
LCMOVE:
        .word    LCMOVE+HEADSIZE
        .word    LCMOVE+HEADSIZE
        .word    0x0
        .word    CR
        .word    N_LCMOVE
        .word    0

        MOV     R8,R11   @save 
        LDR     R2, [R13],#+(CW*(1))      @count
        LDR     R1, [R13],#+(CW*(1))      @dest
        LDR     R3, [R13],#+(CW*(1))      @source
        CMP R2, #0
        BEQ MOVRP2
MOVREP: LDRB    R0, [R3],#+1
        STRB    R0, [R1],#+1
        SUBS    R2,R2, #1
        BNE     MOVREP
MOVRP2: MOV     R11,R8   @get back 
        LDR     R0, [R11],#+(CW*(1))
         LDR     R1,[R0]
         BX      R1  
@

@  ************
@  *   MOVE   *
@  ************
@
        .align
N_LMOVE:
        .word      4
        .ascii      "MOVE"
        .align
LMOVE:
        .word    LMOVE+HEADSIZE
        .word    LMOVE+HEADSIZE
        .word    0x0
        .word    LCMOVE
        .word    N_LMOVE
        .word    0

        LDR     R2, [R13],#+(CW*(1))      @count,  R2
        LDR     R1, [R13],#+(CW*(1))      @dest,   R1
        LDR     R3, [R13],#+(CW*(1))      @source, R3
        ANDS    R3,R3,R3   @ test R3,R1,R2 for 0 
        BEQ     MOVE3
        ANDS    R1,R1,R1
        BEQ     MOVE3
        ANDS    R2,R2,R2
        BEQ     MOVE3
        CMP     R3,R1
        BHI     MOVE2
        ADD     R1,R1,R2
        ADD     R3,R3,R2
        SUB     R1,R1, #1
        SUB     R3,R3, #1
MOVE1:
        LDRB    R0, [R3],#-1
        STRB    R0, [R1],#-1
        SUBS    R2,R2, #1
        BNE     MOVE1
        BAL     MOVE3
MOVE2:
        LDRB    R0, [R3],#+1
        STRB    R0, [R1],#+1
        SUBS    R2,R2, #1
        BNE     MOVE2
MOVE3:
        LDR     R0, [R11],#+(CW*(1))
         LDR     R1,[R0]
         BX      R1  
@

@  ***********
@  *   UM*   *
@  ***********
@
        .align
N_USTAR:
        .word      3
        .ascii      "UM*"
        .align
USTAR:
        .word    USTAR+HEADSIZE
        .word    USTAR+HEADSIZE
        .word    0x0
        .word    LMOVE
        .word    N_USTAR
        .word    0

        LDR     R2, [R13],#+(CW*(1))
        LDR     R3, [R13],#+(CW*(1))
 @ end thumb  )
        UMULL   R1,R0,R3,R2 @UNSIGNED  (R1,R0) = (R3*R2)  
        STR     R1, [R13,#-(CW*(1))]!
        STR     R0, [R13,#-(CW*(1))]!
        LDR     R0, [R11],#+(CW*(1))
         LDR     R1,[R0]
         BX      R1            @STORE DOUBLE CELL
@

@  **************
@  *   UM/MOD   *
@  **************
@
        .align
N_USLAS:
        .word      6
        .ascii      "UM/MOD"
        .align
USLAS:
        .word    USLAS+HEADSIZE
        .word    USLAS+HEADSIZE
        .word    0x0
        .word    USTAR
        .word    N_USLAS
        .word    0

        LDR     R2, [R13],#+(CW*(1))      @DIVISOR
        LDR     R1, [R13],#+(CW*(1))     @MSW OF DIVIDEND
        LDR     R0, [R13],#+(CW*(1))      @LSW OF DIVIDEND
        BL UMDIV
        STR     R1, [R13,#-(CW*(1))]!    @remainder
        STR     R4, [R13,#-(CW*(1))]!     @quotient
        LDR     R0, [R11],#+(CW*(1))
         LDR     R1,[R0]
         BX      R1  

 /* 
_BITS_32(  division of unsigned-64/unsigned-32)
_BITS_64(  division of unsigned-128/unsigned-64)
  input:
  div  = R2  = divisor
  modh = R1 = MSW of dividend
  modl = R0  = LSW of dividend
  output:
  quot = R0  = quotient
  rem = R1 = remainder

  Internally R3 is used as a mask, with 1 bit set.
  mask = R3

 */  
UMDIV:  STR     LR, [R13,#-(CW*(1))]!
        MOV R3, #1
        LSL R3,R3, #(CW*8-1)  @ init mask with highest bit set
        MOV R4, #0              @ init quot
        CMP R1, R2             @ test modh - div
        BLO UMDIV1              @ modh < div
        @ overflow condition (divide by zero) - show max numbers
        ASR  R4, R3, #(CW*8-1) @ all bits set
        MOV  R1, R4

@ 
        BAL  UMDIV3             @ return

UMDIV1:
        ADDS R0,R0, R0         @   double precision shift (modh, modl)
        ADCS R1,R1, R1      @   ADD with carry and set flags again !
        BCS  UMDIV4
        CMP  R2, R1            @ test div - modh
        BHI  UMDIV2             @ div >  modh ?
UMDIV4:
        ADD  R4,R4, R3         @ add single pecision mask
        SUB  R1,R1, R2       @ subtract single precision div
UMDIV2:
        LSR R3,R3, #1          @ shift mask one bit to the right
        ANDS  R3,R3,R3
        BNE  UMDIV1
UMDIV3:
        @ now R0 and R1 contain quotient and remainder
                        LDR     R2, [R13],#+(CW*(1))
        BX R2
@ 

@

@  ***********
@  *   AND   *
@  ***********
@
        .align
N_LAND:
        .word      3
        .ascii      "AND"
        .align
LAND:
        .word    LAND+HEADSIZE
        .word    LAND+HEADSIZE
        .word    0x0
        .word    USLAS
        .word    N_LAND
        .word    0

        LDR     R0, [R13],#+(CW*(1))
        LDR     R3, [R13],#+(CW*(1))
        AND     R0,R0,R3
        STR     R0, [R13,#-(CW*(1))]!
        LDR     R0, [R11],#+(CW*(1))
         LDR     R1,[R0]
         BX      R1  
@

@  **********
@  *   OR   *
@  **********
@
        .align
N_LOR:
        .word      2
        .ascii      "OR"
        .align
LOR:
        .word    LOR+HEADSIZE
        .word    LOR+HEADSIZE
        .word    0x0
        .word    LAND
        .word    N_LOR
        .word    0

        LDR     R0, [R13],#+(CW*(1))      @ (S1) <- (S1) OR (S2)
        LDR     R3, [R13],#+(CW*(1))
        ORR      R0,R0,R3
        STR     R0, [R13,#-(CW*(1))]!
        LDR     R0, [R11],#+(CW*(1))
         LDR     R1,[R0]
         BX      R1  
@

@  ***********
@  *   XOR   *
@  ***********
@
        .align
N_LXOR:
        .word      3
        .ascii      "XOR"
        .align
LXOR:
        .word    LXOR+HEADSIZE
        .word    LXOR+HEADSIZE
        .word    0x0
        .word    LOR
        .word    N_LXOR
        .word    0

        LDR     R0, [R13],#+(CW*(1))      @ (S1) <- (S1) XOR (S2)
        LDR     R3, [R13],#+(CW*(1))
        EOR     R0,R0,R3
        STR     R0, [R13,#-(CW*(1))]!
        LDR     R0, [R11],#+(CW*(1))
         LDR     R1,[R0]
         BX      R1  
@

@  **************
@  *   INVERT   *
@  **************
@
        .align
N_INVERT:
        .word      6
        .ascii      "INVERT"
        .align
INVERT:
        .word    INVERT+HEADSIZE
        .word    INVERT+HEADSIZE
        .word    0x0
        .word    LXOR
        .word    N_INVERT
        .word    0

        LDR     R0, [R13],#+(CW*(1))      @ (S1) <- (S1) XOR (S2)
        MOV R1,#0
        SUB R1,R1,#1
        EOR R0,R0,R1
        STR     R0, [R13,#-(CW*(1))]!
        LDR     R0, [R11],#+(CW*(1))
         LDR     R1,[R0]
         BX      R1  
@

@  ************
@  *   DSP@   *
@  ************
@
        .align
N_SPFET:
        .word      4
        .ascii      "DSP@"
        .align
SPFET:
        .word    SPFET+HEADSIZE
        .word    SPFET+HEADSIZE
        .word    0x0
        .word    INVERT
        .word    N_SPFET
        .word    0

        MOV     R0,R13   @ (S1) <- (SP)
        STR     R0, [R13,#-(CW*(1))]!
        LDR     R0, [R11],#+(CW*(1))
         LDR     R1,[R0]
         BX      R1  
@

@  ************
@  *   DSP!   *
@  ************
@
        .align
N_SPSTO:
        .word      4
        .ascii      "DSP!"
        .align
SPSTO:
        .word    SPSTO+HEADSIZE
        .word    SPSTO+HEADSIZE
        .word    0x0
        .word    SPFET
        .word    N_SPSTO
        .word    0

        LDR     R0, [R13],#+(CW*(1))
        MOV     R13,R0        @RESET PARAM STACK POINTER
        LDR     R0, [R11],#+(CW*(1))
         LDR     R1,[R0]
         BX      R1  
@

@  *************
@  *   DEPTH   *
@  *************
@
        .align
N_DEPTH:
        .word      5
        .ascii      "DEPTH"
        .align
DEPTH:
        .word    DOCOL
        .word    DEPTH+HEADSIZE
        .word    0x0
        .word    SPSTO
        .word    N_DEPTH
        .word    0

        .word      SZERO, FETCH
        .word      SPFET
        .word      LSUB
        .word      LIT, CW, SLASH
        .word      ONEM
        .word      SEMIS
@

@  ***********
@  *   CLS   *
@  ***********
@
        .align
N_CLS:
        .word      3
        .ascii      "CLS"
        .align
CLS:
        .word    DOCOL
        .word    CLS+HEADSIZE
        .word    0x0
        .word    DEPTH
        .word    N_CLS
        .word    0

        .word      SZERO, FETCH
        .word      SPSTO
        .word      SEMIS
@

@  ************
@  *   RSP@   *
@  ************
@
        .align
N_RPFET:
        .word      4
        .ascii      "RSP@"
        .align
RPFET:
        .word    RPFET+HEADSIZE
        .word    RPFET+HEADSIZE
        .word    0x0
        .word    CLS
        .word    N_RPFET
        .word    0
      @(S1) <- (RP)
        STR     R12, [R13,#-(CW*(1))]!
        LDR     R0, [R11],#+(CW*(1))
         LDR     R1,[R0]
         BX      R1  
@

@  ************
@  *   RSP!   *
@  ************
@
        .align
N_RPSTO:
        .word      4
        .ascii      "RSP!"
        .align
RPSTO:
        .word    RPSTO+HEADSIZE
        .word    RPSTO+HEADSIZE
        .word    0x0
        .word    RPFET
        .word    N_RPSTO
        .word    0

        LDR     R12, [R13],#+(CW*(1))
        LDR     R0, [R11],#+(CW*(1))
         LDR     R1,[R0]
         BX      R1  
@

@  ************
@  *   EXIT   *
@  ************
@
        .align
N_EXIT:
        .word      4
        .ascii      "EXIT"
        .align
EXIT:
        .word    EXIT+HEADSIZE
        .word    EXIT+HEADSIZE
        .word    0x0
        .word    RPSTO
        .word    N_EXIT
        .word    0

        LDR     R11, [R12],#+(CW*(1))  @(IP) <- (RP1)
        LDR     R0, [R11],#+(CW*(1))
         LDR     R1,[R0]
         BX      R1  
@

@  **********
@  *   CO   *
@  **********
@
        .align
N_CO:
        .word      2
        .ascii      "CO"
        .align
CO:
        .word    CO+HEADSIZE
        .word    CO+HEADSIZE
        .word    0x0
        .word    EXIT
        .word    N_CO
        .word    0

        LDR     R1,[R12]
        STR     R11, [R12]
        MOV     R11, R1
        LDR     R0, [R11],#+(CW*(1))
         LDR     R1,[R0]
         BX      R1  
@

@  ***********
@  *   (;)   *
@  ***********
@
        .align
N_SEMIS:
        .word      3
        .ascii      "(;)"
        .align
SEMIS:
        .word    (EXIT+HEADSIZE)
        .word    SEMIS+HEADSIZE
        .word    0x0
        .word    CO
        .word    N_SEMIS
        .word    0

@

@  *************
@  *   LEAVE   *
@  *************
@
        .align
N_LLEAV:
        .word      5
        .ascii      "LEAVE"
        .align
LLEAV:
        .word    DOCOL
        .word    LLEAV+HEADSIZE
        .word    0x0
        .word    SEMIS
        .word    N_LLEAV
        .word    0
  @LIMIT <- INDEX
        .word      UNLOOP
        .word      SEMIS
@

@  **********
@  *   >R   *
@  **********
@
        .align
N_TOR:
        .word      2
        .ascii      ">R"
        .align
TOR:
        .word    TOR+HEADSIZE
        .word    TOR+HEADSIZE
        .word    0x0
        .word    LLEAV
        .word    N_TOR
        .word    0
        @ (RP1) <- (S1)
        LDR     R3, [R13],#+(CW*(1))      @GET STACK PARAMETER
        SUB     R12, R12, #(CW*(1))  @MOVE RETURN STACK DOWN
        STR     R3, [R12]            @ADD TO RETURN STACK
        LDR     R0, [R11],#+(CW*(1))
         LDR     R1,[R0]
         BX      R1  
@

@  **********
@  *   R>   *
@  **********
@
        .align
N_FROMR:
        .word      2
        .ascii      "R>"
        .align
FROMR:
        .word    FROMR+HEADSIZE
        .word    FROMR+HEADSIZE
        .word    0x0
        .word    TOR
        .word    N_FROMR
        .word    0
      @(S1) <- (RP1)
        LDR     R0,[R12] @ GET RETURN STACK VALUE
        ADD     R12,R12,# (CW*(1))
        STR     R0, [R13,#-(CW*(1))]!
        LDR     R0, [R11],#+(CW*(1))
         LDR     R1,[R0]
         BX      R1  
@

@  *************
@  *   RDROP   *
@  *************
@
        .align
N_RDROP:
        .word      5
        .ascii      "RDROP"
        .align
RDROP:
        .word    RDROP+HEADSIZE
        .word    RDROP+HEADSIZE
        .word    0x0
        .word    FROMR
        .word    N_RDROP
        .word    0
      @(S1) <- (RP1)
        ADD R12,R12,#(CW*(1))
        LDR     R0, [R11],#+(CW*(1))
         LDR     R1,[R0]
         BX      R1  
@

@  **********
@  *   R@   *
@  **********
@
        .align
N_RR:
        .word      2
        .ascii      "R@"
        .align
RR:
        .word    (IDO+HEADSIZE)
        .word    RR+HEADSIZE
        .word    0x0
        .word    RDROP
        .word    N_RR
        .word    0

@

@  **********
@  *   0=   *
@  **********
@
        .align
N_ZEQU:
        .word      2
        .ascii      "0="
        .align
ZEQU:
        .word    ZEQU+HEADSIZE
        .word    ZEQU+HEADSIZE
        .word    0x0
        .word    RR
        .word    N_ZEQU
        .word    0

        LDR     R2, [R13],#+(CW*(1))
        MOV    R0, #0   @FALSE
        CMP     R2, #0
        BNE ZEQ1
        SUB     R0,R0, #1          @
ZEQ1:   STR     R0, [R13,#-(CW*(1))]!
        LDR     R0, [R11],#+(CW*(1))
         LDR     R1,[R0]
         BX      R1  
@

@  **********
@  *   0<   *
@  **********
@
        .align
N_ZLESS:
        .word      2
        .ascii      "0<"
        .align
ZLESS:
        .word    ZLESS+HEADSIZE
        .word    ZLESS+HEADSIZE
        .word    0x0
        .word    ZEQU
        .word    N_ZLESS
        .word    0

        LDR     R1, [R13],#+(CW*(1))
        MOV    R0,#0    @FALSE
        ANDS    R1,R1,R1   @SET FLAGS
        BPL     ZLESS1
        SUB     R0,R0, #1      @TRUE
ZLESS1: STR     R0, [R13,#-(CW*(1))]!
        LDR     R0, [R11],#+(CW*(1))
         LDR     R1,[R0]
         BX      R1  
@

@  *********
@  *   +   *
@  *********
@
        .align
N_PLUS:
        .word      1
        .ascii      "+"
        .align
PLUS:
        .word    PLUS+HEADSIZE
        .word    PLUS+HEADSIZE
        .word    0x0
        .word    ZLESS
        .word    N_PLUS
        .word    0

        LDR     R0, [R13],#+(CW*(1))      @(S1) <- (S1) + (S2)
        LDR     R3, [R13],#+(CW*(1))
        ADD     R0,R0,R3
        STR     R0, [R13,#-(CW*(1))]!
        LDR     R0, [R11],#+(CW*(1))
         LDR     R1,[R0]
         BX      R1  
@

@  **********
@  *   D+   *
@  **********
@
        .align
N_DPLUS:
        .word      2
        .ascii      "D+"
        .align
DPLUS:
        .word    DPLUS+HEADSIZE
        .word    DPLUS+HEADSIZE
        .word    0x0
        .word    PLUS
        .word    N_DPLUS
        .word    0

        LDR     R0, [R13],#+(CW*(1))      @ YHC
        LDR     R1, [R13],#+(CW*(1))      @ YLC
        LDR     R3, [R13],#+(CW*(1))      @ XHC
        LDR     R2, [R13],#+(CW*(1))      @ XLC
        ADDS    R1,R1,R2
        ADCS    R0,R0,R3   @ SLC
        STR     R1, [R13,#-(CW*(1))]!
        STR     R0, [R13,#-(CW*(1))]!
        LDR     R0, [R11],#+(CW*(1))
         LDR     R1,[R0]
         BX      R1  
@

@  **************
@  *   NEGATE   *
@  **************
@
        .align
N_NEGATE:
        .word      6
        .ascii      "NEGATE"
        .align
NEGATE:
        .word    NEGATE+HEADSIZE
        .word    NEGATE+HEADSIZE
        .word    0x0
        .word    DPLUS
        .word    N_NEGATE
        .word    0

        LDR     R0, [R13],#+(CW*(1))
        MVN R0, R0  @ bitwise NOT 
        ADD R0,R0,#1
        STR     R0, [R13,#-(CW*(1))]!
        LDR     R0, [R11],#+(CW*(1))
         LDR     R1,[R0]
         BX      R1  
@

@  ***************
@  *   DNEGATE   *
@  ***************
@
        .align
N_DNEGA:
        .word      7
        .ascii      "DNEGATE"
        .align
DNEGA:
        .word    DNEGA+HEADSIZE
        .word    DNEGA+HEADSIZE
        .word    0x0
        .word    NEGATE
        .word    N_DNEGA
        .word    0

        LDR     R0, [R13],#+(CW*(1))   @ MSW
        LDR     R1, [R13],#+(CW*(1))   @ LSW
        MVN R0, R0  @ invert bits 
        MVN R1, R1
        
        ADDS R1, R1, #1  @ make 2's complement 
        ADC R0, R0, #0  @ add possible carry    
        STR     R1, [R13,#-(CW*(1))]!
        STR     R0, [R13,#-(CW*(1))]!
        LDR     R0, [R11],#+(CW*(1))
         LDR     R1,[R0]
         BX      R1  
@
        #
@

@  ************
@  *   OVER   *
@  ************
@
        .align
N_OVER:
        .word      4
        .ascii      "OVER"
        .align
OVER:
        .word    OVER+HEADSIZE
        .word    OVER+HEADSIZE
        .word    0x0
        .word    DNEGA
        .word    N_OVER
        .word    0

        LDR     R1, [R13],#+(CW*(1))
        LDR     R0, [R13],#+(CW*(1))
        STR     R0, [R13,#-(CW*(1))]!
        STR     R1, [R13,#-(CW*(1))]!
        STR     R0, [R13,#-(CW*(1))]!
        LDR     R0, [R11],#+(CW*(1))
         LDR     R1,[R0]
         BX      R1  
@

@  ************
@  *   DROP   *
@  ************
@
        .align
N_DROP:
        .word      4
        .ascii      "DROP"
        .align
DROP:
        .word    DROP+HEADSIZE
        .word    DROP+HEADSIZE
        .word    0x0
        .word    OVER
        .word    N_DROP
        .word    0

        LDR     R0, [R13],#+(CW*(1))
        LDR     R0, [R11],#+(CW*(1))
         LDR     R1,[R0]
         BX      R1  
@

@  ***********
@  *   NIP   *
@  ***********
@
        .align
N_NIP:
        .word      3
        .ascii      "NIP"
        .align
NIP:
        .word    NIP+HEADSIZE
        .word    NIP+HEADSIZE
        .word    0x0
        .word    DROP
        .word    N_NIP
        .word    0

        LDR     R0, [R13],#+(CW*(1))
        LDR     R3, [R13],#+(CW*(1))
        STR     R0, [R13,#-(CW*(1))]!
        LDR     R0, [R11],#+(CW*(1))
         LDR     R1,[R0]
         BX      R1  
@

@  *************
@  *   2DROP   *
@  *************
@
        .align
N_TDROP:
        .word      5
        .ascii      "2DROP"
        .align
TDROP:
        .word    TDROP+HEADSIZE
        .word    TDROP+HEADSIZE
        .word    0x0
        .word    NIP
        .word    N_TDROP
        .word    0

        LDR     R0, [R13],#+(CW*(1))
        LDR     R0, [R13],#+(CW*(1))
        LDR     R0, [R11],#+(CW*(1))
         LDR     R1,[R0]
         BX      R1  
@

@  ************
@  *   SWAP   *
@  ************
@
        .align
N_SWAP:
        .word      4
        .ascii      "SWAP"
        .align
SWAP:
        .word    SWAP+HEADSIZE
        .word    SWAP+HEADSIZE
        .word    0x0
        .word    TDROP
        .word    N_SWAP
        .word    0

        LDR     R1, [R13],#+(CW*(1))
        LDR     R0, [R13],#+(CW*(1))
        STR     R1, [R13,#-(CW*(1))]!
        STR     R0, [R13,#-(CW*(1))]!
        LDR     R0, [R11],#+(CW*(1))
         LDR     R1,[R0]
         BX      R1  
@

@  ***********
@  *   DUP   *
@  ***********
@
        .align
N_LDUP:
        .word      3
        .ascii      "DUP"
        .align
LDUP:
        .word    LDUP+HEADSIZE
        .word    LDUP+HEADSIZE
        .word    0x0
        .word    SWAP
        .word    N_LDUP
        .word    0

        LDR     R0, [R13],#+(CW*(1))
        STR     R0, [R13,#-(CW*(1))]!
        STR     R0, [R13,#-(CW*(1))]!
        LDR     R0, [R11],#+(CW*(1))
         LDR     R1,[R0]
         BX      R1  
@

@  ************
@  *   2DUP   *
@  ************
@
        .align
N_TDUP:
        .word      4
        .ascii      "2DUP"
        .align
TDUP:
        .word    TDUP+HEADSIZE
        .word    TDUP+HEADSIZE
        .word    0x0
        .word    LDUP
        .word    N_TDUP
        .word    0

        LDR     R0, [R13],#+(CW*(1))
        LDR     R1, [R13],#+(CW*(1))
        STR     R1, [R13,#-(CW*(1))]!
        STR     R0, [R13,#-(CW*(1))]!
        STR     R1, [R13,#-(CW*(1))]!
        STR     R0, [R13,#-(CW*(1))]!
        LDR     R0, [R11],#+(CW*(1))
         LDR     R1,[R0]
         BX      R1  
@

@  *************
@  *   2SWAP   *
@  *************
@
        .align
N_TSWAP:
        .word      5
        .ascii      "2SWAP"
        .align
TSWAP:
        .word    TSWAP+HEADSIZE
        .word    TSWAP+HEADSIZE
        .word    0x0
        .word    TDUP
        .word    N_TSWAP
        .word    0

        LDR     R3, [R13],#+(CW*(1))
        LDR     R2, [R13],#+(CW*(1))
        LDR     R0, [R13],#+(CW*(1))
        LDR     R1, [R13],#+(CW*(1))
        STR     R2, [R13,#-(CW*(1))]!
        STR     R3, [R13,#-(CW*(1))]!
        STR     R1, [R13,#-(CW*(1))]!
        STR     R0, [R13,#-(CW*(1))]!
        LDR     R0, [R11],#+(CW*(1))
         LDR     R1,[R0]
         BX      R1  
@

@  *************
@  *   2OVER   *
@  *************
@
        .align
N_TOVER:
        .word      5
        .ascii      "2OVER"
        .align
TOVER:
        .word    TOVER+HEADSIZE
        .word    TOVER+HEADSIZE
        .word    0x0
        .word    TSWAP
        .word    N_TOVER
        .word    0

        LDR     R3, [R13],#+(CW*(1))
        LDR     R2, [R13],#+(CW*(1))
        LDR     R0, [R13],#+(CW*(1))
        LDR     R1, [R13],#+(CW*(1))
        STR     R1, [R13,#-(CW*(1))]!
        STR     R0, [R13,#-(CW*(1))]!
        STR     R2, [R13,#-(CW*(1))]!
        STR     R3, [R13,#-(CW*(1))]!
        STR     R1, [R13,#-(CW*(1))]!
        STR     R0, [R13,#-(CW*(1))]!
        LDR     R0, [R11],#+(CW*(1))
         LDR     R1,[R0]
         BX      R1  
@

@  **********
@  *   +!   *
@  **********
@
        .align
N_PSTOR:
        .word      2
        .ascii      "+!"
        .align
PSTOR:
        .word    PSTOR+HEADSIZE
        .word    PSTOR+HEADSIZE
        .word    0x0
        .word    TOVER
        .word    N_PSTOR
        .word    0

        LDR     R3, [R13],#+(CW*(1))      @ADDRESS
        LDR     R0, [R13],#+(CW*(1))      @INCREMENT
        LDR     R2, [R3]
        ADD     R2,R2, R0
        STR     R2, [R3]
        LDR     R0, [R11],#+(CW*(1))
         LDR     R1,[R0]
         BX      R1  
@

@  **************
@  *   TOGGLE   *
@  **************
@
        .align
N_TOGGL:
        .word      6
        .ascii      "TOGGLE"
        .align
TOGGL:
        .word    TOGGL+HEADSIZE
        .word    TOGGL+HEADSIZE
        .word    0x0
        .word    PSTOR
        .word    N_TOGGL
        .word    0

        LDR     R0, [R13],#+(CW*(1))      @BIT PATTERN
        LDR     R3, [R13],#+(CW*(1))      @ADDR
        LDR     R1, [R3]
        EOR     R1,R1, R0
        STR     R1, [R3]
        LDR     R0, [R11],#+(CW*(1))
         LDR     R1,[R0]
         BX      R1  
@

@  *********
@  *   @   *
@  *********
@
        .align
N_FETCH:
        .word      1
        .ascii      "@"
        .align
FETCH:
        .word    FETCH+HEADSIZE
        .word    FETCH+HEADSIZE
        .word    0x0
        .word    TOGGL
        .word    N_FETCH
        .word    0

        LDR     R3, [R13],#+(CW*(1))
        LDR     R0,[R3]
        STR     R0, [R13,#-(CW*(1))]!
        LDR     R0, [R11],#+(CW*(1))
         LDR     R1,[R0]
         BX      R1  
@

@  **********
@  *   P@   *
@  **********
@
        .align
N_PFETCH:
        .word      2
        .ascii      "P@"
        .align
PFETCH:
        .word    PFETCH+HEADSIZE
        .word    PFETCH+HEADSIZE
        .word    0x0
        .word    FETCH
        .word    N_PFETCH
        .word    0

        LDR     R3, [R13],#+(CW*(1))
        LDR     R0,[R3]
        STR     R0, [R13,#-(CW*(1))]!
        LDR     R0, [R11],#+(CW*(1))
         LDR     R1,[R0]
         BX      R1  
@
@


@  ***********
@  *   BIC   *
@  ***********
@
        .align
N_BITCLR:
        .word      3
        .ascii      "BIC"
        .align
BITCLR:
        .word    BITCLR+HEADSIZE
        .word    BITCLR+HEADSIZE
        .word    0x0
        .word    PFETCH
        .word    N_BITCLR
        .word    0

        LDR     R3, [R13],#+(CW*(1))
        LDR     R0, [R13],#+(CW*(1))
        LDR     R2,[R3]
        BIC     R2,R0
        STR     R2,[R3]
        LDR     R0, [R11],#+(CW*(1))
         LDR     R1,[R0]
         BX      R1  
@

@  ***********
@  *   BIS   *
@  ***********
@
        .align
N_BITSET:
        .word      3
        .ascii      "BIS"
        .align
BITSET:
        .word    BITSET+HEADSIZE
        .word    BITSET+HEADSIZE
        .word    0x0
        .word    BITCLR
        .word    N_BITSET
        .word    0

        LDR     R3, [R13],#+(CW*(1))
        LDR     R0, [R13],#+(CW*(1))
        LDR     R2,[R3]
        ORR     R2,R0
        STR     R2,[R3]
        LDR     R0, [R11],#+(CW*(1))
         LDR     R1,[R0]
         BX      R1  
@
@
@

@  **********
@  *   C@   *
@  **********
@
        .align
N_CFET:
        .word      2
        .ascii      "C@"
        .align
CFET:
        .word    CFET+HEADSIZE
        .word    CFET+HEADSIZE
        .word    0x0
        .word    BITSET
        .word    N_CFET
        .word    0

        LDR     R3, [R13],#+(CW*(1))
        LDRB    R0,[R3]
        
         AND     R0,R0, #0xFF
        STR     R0, [R13,#-(CW*(1))]!
        LDR     R0, [R11],#+(CW*(1))
         LDR     R1,[R0]
         BX      R1  
@

@  **********
@  *   2@   *
@  **********
@
        .align
N_TFET:
        .word      2
        .ascii      "2@"
        .align
TFET:
        .word    TFET+HEADSIZE
        .word    TFET+HEADSIZE
        .word    0x0
        .word    CFET
        .word    N_TFET
        .word    0

        LDR     R3, [R13],#+(CW*(1))      @ADDR
        LDR     R0,[R3]                      @MSW
        LDR     R1,[R3,# + (CW*(1))]        @LSW
        STR     R1, [R13,#-(CW*(1))]!
        STR     R0, [R13,#-(CW*(1))]!
        LDR     R0, [R11],#+(CW*(1))
         LDR     R1,[R0]
         BX      R1  
@

@  *********
@  *   !   *
@  *********
@
        .align
N_STOR:
        .word      1
        .ascii      "!"
        .align
STOR:
        .word    STOR+HEADSIZE
        .word    STOR+HEADSIZE
        .word    0x0
        .word    TFET
        .word    N_STOR
        .word    0

        LDR     R3, [R13],#+(CW*(1))      @ADDR
        LDR     R0, [R13],#+(CW*(1))      @DATA
        STR     R0, [R3]
        LDR     R0, [R11],#+(CW*(1))
         LDR     R1,[R0]
         BX      R1  
@

@  **********
@  *   P!   *
@  **********
@
        .align
N_PRTSTR:
        .word      2
        .ascii      "P!"
        .align
PRTSTR:
        .word    PRTSTR+HEADSIZE
        .word    PRTSTR+HEADSIZE
        .word    0x0
        .word    STOR
        .word    N_PRTSTR
        .word    0

        LDR     R3, [R13],#+(CW*(1))      @ADDR
        LDR     R0, [R13],#+(CW*(1))      @DATA
        STR     R0, [R3]
        LDR     R0, [R11],#+(CW*(1))
         LDR     R1,[R0]
         BX      R1  
@
@

@  **********
@  *   C!   *
@  **********
@
        .align
N_CSTOR:
        .word      2
        .ascii      "C!"
        .align
CSTOR:
        .word    CSTOR+HEADSIZE
        .word    CSTOR+HEADSIZE
        .word    0x0
        .word    PRTSTR
        .word    N_CSTOR
        .word    0

        LDR     R3, [R13],#+(CW*(1))      @ADDR
        LDR     R0, [R13],#+(CW*(1))      @DATA
        STRB         R0, [R3]  @Store byte
        LDR     R0, [R11],#+(CW*(1))
         LDR     R1,[R0]
         BX      R1  
@

@  **********
@  *   2!   *
@  **********
@
        .align
N_TSTOR:
        .word      2
        .ascii      "2!"
        .align
TSTOR:
        .word    TSTOR+HEADSIZE
        .word    TSTOR+HEADSIZE
        .word    0x0
        .word    CSTOR
        .word    N_TSTOR
        .word    0

        LDR     R3, [R13],#+(CW*(1))      @ADDR
        LDR     R0, [R13],#+(CW*(1))      @MSW
        STR     R0, [R3]
        LDR     R0, [R13],#+(CW*(1))      @LSW
        STR     R0, [R3, # +(CW*(1))]
        LDR     R0, [R11],#+(CW*(1))
         LDR     R1,[R0]
         BX      R1  
@

@  **************
@  *   WITHIN   *
@  **************
@
        .align
N_WITHIN:
        .word      6
        .ascii      "WITHIN"
        .align
WITHIN:
        .word    DOCOL
        .word    WITHIN+HEADSIZE
        .word    0x0
        .word    TSTOR
        .word    N_WITHIN
        .word    0

        .word      OVER, LSUB, TOR
        .word      LSUB, FROMR
        .word      ULESS
        .word      SEMIS
@
@

@  *********
@  *   :   *
@  *********
@
        .align
N_COLON:
        .word      1
        .ascii      ":"
        .align
COLON:
        .word    DOCOL
        .word    COLON+HEADSIZE
        .word    0x0
        .word    WITHIN
        .word    N_COLON
        .word    0

        .word      SCSP
        .word      LNAME
        .word      PCREAT
        .word      LATEST, HIDDEN
        .word      RBRAC
        .word      PSCOD
DOCOL:  STR     R11, [R12,#-(CW*(1))]!
        LDR     R11,[R0, #+(CW*(D_HOFFSET))]  @(IP) <- (PFA)

@ 

        LDR     R0, [R11],#+(CW*(1))
         LDR     R1,[R0]
         BX      R1  


@

@  *********
@  *   ;   *
@  *********
@
        .align
N_SEMI:
        .word      1
        .ascii      ";"
        .align
SEMI:
        .word    DOCOL
        .word    SEMI+HEADSIZE
        .word    B_IMMED
        .word    COLON
        .word    N_SEMI
        .word    0

        .word      QCSP
        .word      LIT, SEMIS, COMMA
        .word      LATEST, HIDDEN
        .word      LBRAC
        .word      SEMIS
@

@  ****************
@  *   CONSTANT   *
@  ****************
@
        .align
N_LCONST:
        .word      8
        .ascii      "CONSTANT"
        .align
LCONST:
        .word    DOCOL
        .word    LCONST+HEADSIZE
        .word    0x0
        .word    SEMI
        .word    N_LCONST
        .word    0

        .word      LNAME
        .word      PCREAT
        .word      LATEST, TDFA, STOR
        .word      PSCOD
DOCON:  LDR     R0,[R0,#+(CW*((D_HOFFSET)))] @GET DATA FROM PFA
        STR     R0, [R13,#-(CW*(1))]!
        LDR     R0, [R11],#+(CW*(1))
         LDR     R1,[R0]
         BX      R1  
@

@  ****************
@  *   VARIABLE   *
@  ****************
@
        .align
N_VAR:
        .word      8
        .ascii      "VARIABLE"
        .align
VAR:
        .word    DOCOL
        .word    VAR+HEADSIZE
        .word    0x0
        .word    LCONST
        .word    N_VAR
        .word    0

        .word      LDATA
        .word      ZERO, COMMA
        .word      SEMIS
@

@  ************
@  *   DATA   *
@  ************
@
        .align
N_LDATA:
        .word      4
        .ascii      "DATA"
        .align
LDATA:
        .word    DOCOL
        .word    LDATA+HEADSIZE
        .word    0x0
        .word    VAR
        .word    N_LDATA
        .word    0

        .word      LNAME
        .word      PCREAT
        .word      PSCOD
DOVAR:  LDR     R0,[R0,#+(CW*((D_HOFFSET)))]  @(R0) <- PFA
        STR     R0, [R13,#-(CW*(1))]!
        LDR     R0, [R11],#+(CW*(1))
         LDR     R1,[R0]
         BX      R1  
@

@  *********
@  *   _   *
@  *********
@
        .align
N_X:
        .word      1
        .ascii      "_"
        .align
X:
        .word    X+HEADSIZE
        .word    X+HEADSIZE
        .word    0x0
        .word    LDATA
        .word    N_X
        .word    0

        STR     R0, [R13,#-(CW*(1))]!
        LDR     R0, [R11],#+(CW*(1))
         LDR     R1,[R0]
         BX      R1   @Whatever happens to be in R0, i.e. the dea of ``_''.
@

@  *********
@  *   0   *
@  *********
@
        .align
N_ZERO:
        .word      1
        .ascii      "0"
        .align
ZERO:
        .word    DOCON
        .word    0
        .word    0x0
        .word    X
        .word    N_ZERO
        .word    0

@ The data field of 0 is used for an empty string.
@

@  *********
@  *   1   *
@  *********
@
        .align
N_ONE:
        .word      1
        .ascii      "1"
        .align
ONE:
        .word    DOCON
        .word    1
        .word    0x0
        .word    ZERO
        .word    N_ONE
        .word    0

@

@  *********
@  *   2   *
@  *********
@
        .align
N_TWO:
        .word      1
        .ascii      "2"
        .align
TWO:
        .word    DOCON
        .word    2
        .word    0x0
        .word    ONE
        .word    N_TWO
        .word    0

@

@  **********
@  *   BL   *
@  **********
@
        .align
N_LBL:
        .word      2
        .ascii      "BL"
        .align
LBL:
        .word    DOCON
        .word    ABL
        .word    0x0
        .word    TWO
        .word    N_LBL
        .word    0

@

@
@  **********
@  *   $@   *
@  **********
@
        .align
N_SFET:
        .word      2
        .ascii      "\$@"
        .align
SFET:
        .word    SFET+HEADSIZE
        .word    SFET+HEADSIZE
        .word    0x0
        .word    LBL
        .word    N_SFET
        .word    0

        LDR     R3, [R13],#+(CW*(1))
        LDR     R0, [R3],#+(CW*(1))
        MOV   R1,R3
        STR     R1, [R13,#-(CW*(1))]!
        STR     R0, [R13,#-(CW*(1))]!
        LDR     R0, [R11],#+(CW*(1))
         LDR     R1,[R0]
         BX      R1  
@

@  **********
@  *   $!   *
@  **********
@
        .align
N_SSTOR:
        .word      2
        .ascii      "\$!"
        .align
SSTOR:
        .word    DOCOL
        .word    SSTOR+HEADSIZE
        .word    0x0
        .word    SFET
        .word    N_SSTOR
        .word    0

        .word TDUP, STOR, CELLP, SWAP, LMOVE
        .word SEMIS
@

@ 

@  ***********
@  *   $+!   *
@  ***********
@
        .align
N_SADD:
        .word      3
        .ascii      "\$+!"
        .align
SADD:
        .word    DOCOL
        .word    SADD+HEADSIZE
        .word    0x0
        .word    SSTOR
        .word    N_SADD
        .word    0

        .word   LDUP, FETCH, TOR @ Remember old count.
        .word   TDUP, PSTOR
        .word   CELLP, FROMR, PLUS, SWAP, LMOVE
        .word SEMIS
@

@  ***********
@  *   $C+   *
@  ***********
@
        .align
N_CHAPP:
        .word      3
        .ascii      "\$C+"
        .align
CHAPP:
        .word    DOCOL
        .word    CHAPP+HEADSIZE
        .word    0x0
        .word    SADD
        .word    N_CHAPP
        .word    0

        .word   LDUP, TOR
        .word   LDUP, FETCH, PLUS, CELLP, CSTOR
        .word   ONE, FROMR, PSTOR
        .word SEMIS
@

@  **********
@  *   $,   *
@  **********
@
        .align
N_SCOMMA:
        .word      2
        .ascii      "\$,"
        .align
SCOMMA:
        .word    DOCOL
        .word    SCOMMA+HEADSIZE
        .word    0x0
        .word    CHAPP
        .word    N_SCOMMA
        .word    0

        .word HERE, TOR
        .word LDUP, CELLP, ALLOT
        .word RR, SSTOR, FROMR
 .word LALIGN
        .word SEMIS
@

@  ***********
@  *   C/L   *
@  ***********
@
        .align
N_CSLL:
        .word      3
        .ascii      "C/L"
        .align
CSLL:
        .word    DOCON
        .word    64
        .word    0x0
        .word    SCOMMA
        .word    N_CSLL
        .word    0

@


@  **************
@  *   _FIRST   *
@  **************
@
        .align
N__FIRST:
        .word      6
        .ascii      "_FIRST"
        .align
_FIRST:
        .word    DOCON
        .word    BUF1
        .word    0x0
        .word    CSLL
        .word    N__FIRST
        .word    0

 @  
@
@ 
@

@  **************
@  *   _LIMIT   *
@  **************
@
        .align
N__LIMIT:
        .word      6
        .ascii      "_LIMIT"
        .align
_LIMIT:
        .word    DOCON
        .word    BUF1+(KBBUF+2*CW)*NBUF
        .word    0x0
        .word    _FIRST
        .word    N__LIMIT
        .word    0

@ THE END  OF THE MEMORY 

@  **********
@  *   EM   *
@  **********
@
        .align
N_LEM:
        .word      2
        .ascii      "EM"
        .align
LEM:
        .word    DOCON
        .word    ACTUAL_EM
        .word    0x0
        .word    _LIMIT
        .word    N_LEM
        .word    0

@

@  **********
@  *   BM   *
@  **********
@
        .align
N_LBM:
        .word      2
        .ascii      "BM"
        .align
LBM:
        .word    DOCON
        .word    _ORIG
        .word    0x0
        .word    LEM
        .word    N_LBM
        .word    0

@

@  *************
@  *   B/BUF   *
@  *************
@
        .align
N_BBUF:
        .word      5
        .ascii      "B/BUF"
        .align
BBUF:
        .word    DOCON
        .word    KBBUF
        .word    0x0
        .word    LBM
        .word    N_BBUF
        .word    0

@
@ All user variables are initialised 
@ with the values from USINI.
@ The implementation relies on the initialisation of 
@ those with numbers (1..11), so change in concord with USINI.

@  **********
@  *   U0   *
@  **********
@
        .align
N_UZERO:
        .word      2
        .ascii      "U0"
        .align
UZERO:
        .word    DOUSE
        .word    (CW*(1))
        .word    0x0
        .word    BBUF
        .word    N_UZERO
        .word    0

@

@  **********
@  *   S0   *
@  **********
@
        .align
N_SZERO:
        .word      2
        .ascii      "S0"
        .align
SZERO:
        .word    DOUSE
        .word    (CW*(2))
        .word    0x0
        .word    UZERO
        .word    N_SZERO
        .word    0

@

@  **********
@  *   R0   *
@  **********
@
        .align
N_RZERO:
        .word      2
        .ascii      "R0"
        .align
RZERO:
        .word    DOUSE
        .word    (CW*(3))
        .word    0x0
        .word    SZERO
        .word    N_RZERO
        .word    0

@

@  ***********
@  *   TIB   *
@  ***********
@
        .align
N_TIB:
        .word      3
        .ascii      "TIB"
        .align
TIB:
        .word    DOUSE
        .word    (CW*(4))
        .word    0x0
        .word    RZERO
        .word    N_TIB
        .word    0

@

@  **************
@  *   RUBOUT   *
@  **************
@
        .align
N_RUBOUT:
        .word      6
        .ascii      "RUBOUT"
        .align
RUBOUT:
        .word    DOUSE
        .word    (CW*(5))
        .word    0x0
        .word    TIB
        .word    N_RUBOUT
        .word    0

@

@  ***************
@  *   WARNING   *
@  ***************
@
        .align
N_LWARN:
        .word      7
        .ascii      "WARNING"
        .align
LWARN:
        .word    DOUSE
        .word    (CW*(7))
        .word    0x0
        .word    RUBOUT
        .word    N_LWARN
        .word    0

@

@  *************
@  *   FENCE   *
@  *************
@
        .align
N_FENCE:
        .word      5
        .ascii      "FENCE"
        .align
FENCE:
        .word    DOUSE
        .word    (CW*(8))
        .word    0x0
        .word    LWARN
        .word    N_FENCE
        .word    0

@

@  **********
@  *   DP   *
@  **********
@
        .align
N_LDP:
        .word      2
        .ascii      "DP"
        .align
LDP:
        .word    DOUSE
        .word    (CW*(9))
        .word    0x0
        .word    FENCE
        .word    N_LDP
        .word    0

@

@  ****************
@  *   VOC-LINK   *
@  ****************
@
        .align
N_VOCL:
        .word      8
        .ascii      "VOC-LINK"
        .align
VOCL:
        .word    DOUSE
        .word    (CW*(10))
        .word    0x0
        .word    LDP
        .word    N_VOCL
        .word    0

@

@  **************
@  *   OFFSET   *
@  **************
@
        .align
N_LOFFSET:
        .word      6
        .ascii      "OFFSET"
        .align
LOFFSET:
        .word    DOUSE
        .word    (CW*(37))
        .word    0x0
        .word    VOCL
        .word    N_LOFFSET
        .word    0

@

@  ****************
@  *   MAX-USER   *
@  ****************
@
        .align
N_MAXUSER:
        .word      8
        .ascii      "MAX-USER"
        .align
MAXUSER:
        .word    DOUSE
        .word    (CW*(11))
        .word    0x0
        .word    LOFFSET
        .word    N_MAXUSER
        .word    0

@
@ End of user variables with fixed place.
@
@

@  *************
@  *   WHERE   *
@  *************
@
        .align
N_LWHERE:
        .word      5
        .ascii      "WHERE"
        .align
LWHERE:
        .word    DOUSE
        .word    (CW*(12))
        .word    0x0
        .word    MAXUSER
        .word    N_LWHERE
        .word    0
    @  Occupies two CELLS! 
@

@  ***********
@  *   SCR   *
@  ***********
@
        .align
N_SCR:
        .word      3
        .ascii      "SCR"
        .align
SCR:
        .word    DOUSE
        .word    (CW*(33))
        .word    0x0
        .word    LWHERE
        .word    N_SCR
        .word    0

@

@  *************
@  *   STATE   *
@  *************
@
        .align
N_STATE:
        .word      5
        .ascii      "STATE"
        .align
STATE:
        .word    DOUSE
        .word    (CW*(18))
        .word    0x0
        .word    SCR
        .word    N_STATE
        .word    0

@

@  ************
@  *   BASE   *
@  ************
@
        .align
N_BASE:
        .word      4
        .ascii      "BASE"
        .align
BASE:
        .word    DOUSE
        .word    (CW*(19))
        .word    0x0
        .word    STATE
        .word    N_BASE
        .word    0

@

@  ***********
@  *   DPL   *
@  ***********
@
        .align
N_DPL:
        .word      3
        .ascii      "DPL"
        .align
DPL:
        .word    DOUSE
        .word    (CW*(20))
        .word    0x0
        .word    BASE
        .word    N_DPL
        .word    0

@

@  ***********
@  *   ECL   *
@  ***********
@
        .align
N_ECL:
        .word      3
        .ascii      "ECL"
        .align
ECL:
        .word    DOUSE
        .word    (CW*(38))
        .word    0x0
        .word    DPL
        .word    N_ECL
        .word    0

@

@  ***********
@  *   FLD   *
@  ***********
@
        .align
N_LFLD:
        .word      3
        .ascii      "FLD"
        .align
LFLD:
        .word    DOUSE
        .word    (CW*(21))
        .word    0x0
        .word    ECL
        .word    N_LFLD
        .word    0

@


@  ***********
@  *   CSP   *
@  ***********
@
        .align
N_LCSP:
        .word      3
        .ascii      "CSP"
        .align
LCSP:
        .word    DOUSE
        .word    (CW*(22))
        .word    0x0
        .word    LFLD
        .word    N_LCSP
        .word    0

@
@

@  **********
@  *   R#   *
@  **********
@
        .align
N_RNUM:
        .word      2
        .ascii      "R#"
        .align
RNUM:
        .word    DOUSE
        .word    (CW*(23))
        .word    0x0
        .word    LCSP
        .word    N_RNUM
        .word    0

@

@  ***********
@  *   HLD   *
@  ***********
@
        .align
N_HLD:
        .word      3
        .ascii      "HLD"
        .align
HLD:
        .word    DOUSE
        .word    (CW*(24))
        .word    0x0
        .word    RNUM
        .word    N_HLD
        .word    0

@

@  ***********
@  *   OUT   *
@  ***********
@
        .align
N_LOUT:
        .word      3
        .ascii      "OUT"
        .align
LOUT:
        .word    DOUSE
        .word    (CW*(25))
        .word    0x0
        .word    HLD
        .word    N_LOUT
        .word    0

@

@  *************
@  *   (BLK)   *
@  *************
@
        .align
N_PBLK:
        .word      5
        .ascii      "(BLK)"
        .align
PBLK:
        .word    DOUSE
        .word    (CW*(26))
        .word    0x0
        .word    LOUT
        .word    N_PBLK
        .word    0

@

@  **********
@  *   PP   *
@  **********
@
        .align
N_PP:
        .word      2
        .ascii      "PP"
        .align
PP:
        .word    DOUSE
        .word    (CW*(29))
        .word    0x0
        .word    PBLK
        .word    N_PP
        .word    0

@

@  *************
@  *   (>IN)   *
@  *************
@
        .align
N_PIIN:
        .word      5
        .ascii      "(>IN)"
        .align
PIIN:
        .word    DOUSE
        .word    (CW*(30))
        .word    0x0
        .word    PP
        .word    N_PIIN
        .word    0

@

@  ************
@  *   ARGS   *
@  ************
@
        .align
N_ARGS:
        .word      4
        .ascii      "ARGS"
        .align
ARGS:
        .word    DOUSE
        .word    (CW*(31))
        .word    0x0
        .word    PIIN
        .word    N_ARGS
        .word    0

@

@  ***************
@  *   HANDLER   *
@  ***************
@
        .align
N_HANDLER:
        .word      7
        .ascii      "HANDLER"
        .align
HANDLER:
        .word    DOUSE
        .word    (CW*(32))
        .word    0x0
        .word    ARGS
        .word    N_HANDLER
        .word    0

@

@  ***************
@  *   CURRENT   *
@  ***************
@
        .align
N_CURR:
        .word      7
        .ascii      "CURRENT"
        .align
CURR:
        .word    DOUSE
        .word    (CW*(34))
        .word    0x0
        .word    HANDLER
        .word    N_CURR
        .word    0

@

@  *****************
@  *   REMAINDER   *
@  *****************
@
        .align
N_REMAIND:
        .word      9
        .ascii      "REMAINDER"
        .align
REMAIND:
        .word    DOUSE
        .word    (CW*(14))
        .word    0x0
        .word    CURR
        .word    N_REMAIND
        .word    0

@      IMPORTANT
@ REQUIRES ONE MORE CELL!
@

@  ***************
@  *   CONTEXT   *
@  ***************
@
        .align
N_CONTEXT:
        .word      7
        .ascii      "CONTEXT"
        .align
CONTEXT:
        .word    DOUSE
        .word    (CW*(39))
        .word    0x0
        .word    REMAIND
        .word    N_CONTEXT
        .word    0
 @ Up to  39+16
@      IMPORTANT
@     16 USER SPACE CELLS MUST BE KEPT FREE
@     IN ADDITION TO THE ONE FOR CONTEXT
@
@========== END USER VARIABLES =============
@

@  **********
@  *   1+   *
@  **********
@
        .align
N_ONEP:
        .word      2
        .ascii      "1+"
        .align
ONEP:
        .word    DOCOL
        .word    ONEP+HEADSIZE
        .word    0x0
        .word    CONTEXT
        .word    N_ONEP
        .word    0

        .word      ONE
        .word      PLUS
        .word      SEMIS
@

@  *************
@  *   CELL+   *
@  *************
@
        .align
N_CELLP:
        .word      5
        .ascii      "CELL+"
        .align
CELLP:
        .word    DOCOL
        .word    CELLP+HEADSIZE
        .word    0x0
        .word    ONEP
        .word    N_CELLP
        .word    0

        .word      LIT, CW
        .word      PLUS
        .word      SEMIS
@

@
@  *************
@  *   CELLS   *
@  *************
@
        .align
N_LCELLS:
        .word      5
        .ascii      "CELLS"
        .align
LCELLS:
        .word    DOCOL
        .word    LCELLS+HEADSIZE
        .word    0x0
        .word    CELLP
        .word    N_LCELLS
        .word    0

        .word       TWO 
        .word      LSHIFT
        .word      SEMIS
@

@  *************
@  *   CHAR+   *
@  *************
@
        .align
N_CHARP:
        .word      5
        .ascii      "CHAR+"
        .align
CHARP:
        .word    DOCOL
        .word    (ONEP+HEADSIZE)
        .word    0x0
        .word    LCELLS
        .word    N_CHARP
        .word    0

@

@  *************
@  *   CHARS   *
@  *************
@
        .align
N_CHARS:
        .word      5
        .ascii      "CHARS"
        .align
CHARS:
        .word    DOCOL 
        .word    CHARS+HEADSIZE
        .word    0x0
        .word    CHARP
        .word    N_CHARS
        .word    0

        .word      SEMIS
@

@  *************
@  *   ALIGN   *
@  *************
@
        .align
N_LALIGN:
        .word      5
        .ascii      "ALIGN"
        .align
LALIGN:
        .word    DOCOL 
        .word    LALIGN+HEADSIZE
        .word    0x0
        .word    CHARS
        .word    N_LALIGN
        .word    0


        .word      LDP, FETCH
        .word      ALIGNED
        .word      LDP, STOR
@
        .word      SEMIS
@

@  ***************
@  *   ALIGNED   *
@  ***************
@
        .align
N_ALIGNED :
        .word      7
        .ascii      "ALIGNED"
        .align
ALIGNED :
        .word    ALIGNED +HEADSIZE
        .word    ALIGNED +HEADSIZE
        .word    0x0
        .word    LALIGN
        .word    N_ALIGNED 
        .word    0


        LDR     R0, [R13],#+(CW*(1))
        SUB     R0,R0, #1
        
        ORR     R0, R0, #(CW-1) 
        ADD     R0,R0, #1
        STR     R0, [R13,#-(CW*(1))]!
@
        LDR     R0, [R11],#+(CW*(1))
         LDR     R1,[R0]
         BX      R1  
@

@  ************
@  *   HERE   *
@  ************
@
        .align
N_HERE:
        .word      4
        .ascii      "HERE"
        .align
HERE:
        .word    DOCOL
        .word    HERE+HEADSIZE
        .word    0x0
        .word    ALIGNED 
        .word    N_HERE
        .word    0

        .word      LDP
        .word      FETCH
        .word      SEMIS
@

@  *************
@  *   ALLOT   *
@  *************
@
        .align
N_ALLOT:
        .word      5
        .ascii      "ALLOT"
        .align
ALLOT:
        .word    DOCOL
        .word    ALLOT+HEADSIZE
        .word    0x0
        .word    HERE
        .word    N_ALLOT
        .word    0

        .word      LDP
        .word      PSTOR
        .word      SEMIS
@

@  *********
@  *   ,   *
@  *********
@
        .align
N_COMMA:
        .word      1
        .ascii      ","
        .align
COMMA:
        .word    DOCOL
        .word    COMMA+HEADSIZE
        .word    0x0
        .word    ALLOT
        .word    N_COMMA
        .word    0

        .word      HERE
        .word      LIT
        .word      CW
        .word      ALLOT
        .word      STOR
        .word      SEMIS
@

@  **********
@  *   C,   *
@  **********
@
        .align
N_CCOMM:
        .word      2
        .ascii      "C,"
        .align
CCOMM:
        .word    DOCOL
        .word    CCOMM+HEADSIZE
        .word    0x0
        .word    COMMA
        .word    N_CCOMM
        .word    0

        .word      HERE
        .word      ONE
        .word      ALLOT
        .word      CSTOR
        .word      SEMIS
@

@  **********
@  *   2,   *
@  **********
@
        .align
N_TCOMM:
        .word      2
        .ascii      "2,"
        .align
TCOMM:
        .word    DOCOL
        .word    TCOMM+HEADSIZE
        .word    0x0
        .word    CCOMM
        .word    N_TCOMM
        .word    0

        .word      HERE
        .word      LIT
        .word      CW*2
        .word      ALLOT
        .word      TSTOR
        .word      SEMIS
@

@  *********
@  *   -   *
@  *********
@
        .align
N_LSUB:
        .word      1
        .ascii      "-"
        .align
LSUB:
        .word    LSUB+HEADSIZE
        .word    LSUB+HEADSIZE
        .word    0x0
        .word    TCOMM
        .word    N_LSUB
        .word    0

        LDR     R1, [R13],#+(CW*(1))      @S1
        LDR     R0, [R13],#+(CW*(1))
        SUB     R0,R0,R1
        STR     R0, [R13,#-(CW*(1))]!
        LDR     R0, [R11],#+(CW*(1))
         LDR     R1,[R0]
         BX      R1     @S1 = S2 - S1
@

@  *********
@  *   =   *
@  *********
@
        .align
N_EQUAL:
        .word      1
        .ascii      "="
        .align
EQUAL:
        .word    DOCOL
        .word    EQUAL+HEADSIZE
        .word    0x0
        .word    LSUB
        .word    N_EQUAL
        .word    0

        .word      LSUB
        .word      ZEQU
        .word      SEMIS
@

@  *********
@  *   <   *
@  *********
@
        .align
N_LESS:
        .word      1
        .ascii      "<"
        .align
LESS:
        .word    LESS+HEADSIZE
        .word    LESS+HEADSIZE
        .word    0x0
        .word    EQUAL
        .word    N_LESS
        .word    0

        LDR     R1, [R13],#+(CW*(1))      @S1
        LDR     R3, [R13],#+(CW*(1))      @S2
        MOV    R0, #0   @0 default RESULT
        CMP     R3,R1
        BGE     LES1
        SUB     R0,R0, #1
LES1:   STR     R0, [R13,#-(CW*(1))]!
        LDR     R0, [R11],#+(CW*(1))
         LDR     R1,[R0]
         BX      R1  
@

@  **********
@  *   U<   *
@  **********
@
        .align
N_ULESS:
        .word      2
        .ascii      "U<"
        .align
ULESS:
        .word    ULESS+HEADSIZE
        .word    ULESS+HEADSIZE
        .word    0x0
        .word    LESS
        .word    N_ULESS
        .word    0

        LDR     R0, [R13],#+(CW*(1))
        LDR     R1, [R13],#+(CW*(1))
        SUBS    R1,R1,R0
        SBC     R0,R0,R0
        STR     R0, [R13,#-(CW*(1))]!
        LDR     R0, [R11],#+(CW*(1))
         LDR     R1,[R0]
         BX      R1  
@

@  *********
@  *   >   *
@  *********
@
        .align
N_GREAT:
        .word      1
        .ascii      ">"
        .align
GREAT:
        .word    DOCOL
        .word    GREAT+HEADSIZE
        .word    0x0
        .word    ULESS
        .word    N_GREAT
        .word    0

        .word      SWAP
        .word      LESS
        .word      SEMIS
@

@  **********
@  *   <>   *
@  **********
@
        .align
N_UNEQ:
        .word      2
        .ascii      "<>"
        .align
UNEQ:
        .word    DOCOL
        .word    UNEQ+HEADSIZE
        .word    0x0
        .word    GREAT
        .word    N_UNEQ
        .word    0

        .word      LSUB
        .word      ZEQU
        .word      ZEQU
        .word      SEMIS
@

@  ***********
@  *   ROT   *
@  ***********
@
        .align
N_ROT:
        .word      3
        .ascii      "ROT"
        .align
ROT:
        .word    ROT+HEADSIZE
        .word    ROT+HEADSIZE
        .word    0x0
        .word    UNEQ
        .word    N_ROT
        .word    0

        LDR     R1, [R13],#+(CW*(1))      @S1
        LDR     R3, [R13],#+(CW*(1))      @S2
        LDR     R0, [R13],#+(CW*(1))      @S3
        STR     R3, [R13,#-(CW*(1))]!
        STR     R1, [R13,#-(CW*(1))]!
        STR     R0, [R13,#-(CW*(1))]!
        LDR     R0, [R11],#+(CW*(1))
         LDR     R1,[R0]
         BX      R1  
@

@  *************
@  *   SPACE   *
@  *************
@
        .align
N_SPACE:
        .word      5
        .ascii      "SPACE"
        .align
SPACE:
        .word    DOCOL
        .word    SPACE+HEADSIZE
        .word    0x0
        .word    ROT
        .word    N_SPACE
        .word    0

        .word      LBL
        .word      EMIT
        .word      SEMIS
@

@  ************
@  *   ?DUP   *
@  ************
@
        .align
N_QDUP:
        .word      4
        .ascii      "?DUP"
        .align
QDUP:
        .word    DOCOL
        .word    QDUP+HEADSIZE
        .word    0x0
        .word    SPACE
        .word    N_QDUP
        .word    0

        .word      LDUP
        .word      ZBRAN
        .word      QDUP1-.-CW @ IF
        .word      LDUP    @THEN
QDUP1:  .word      SEMIS
@

@  **************
@  *   LATEST   *
@  **************
@
        .align
N_LATEST:
        .word      6
        .ascii      "LATEST"
        .align
LATEST:
        .word    DOCOL
        .word    LATEST+HEADSIZE
        .word    0x0
        .word    QDUP
        .word    N_LATEST
        .word    0

        .word      CURR
        .word      FETCH
        .word      TLFA
        .word      FETCH
        .word      SEMIS
@

@  ************
@  *   >CFA   *
@  ************
@
        .align
N_TCFA:
        .word      4
        .ascii      ">CFA"
        .align
TCFA:
        .word    DOCOL
        .word    TCFA+HEADSIZE
        .word    0x0
        .word    LATEST
        .word    N_TCFA
        .word    0

        .word      LIT, (CW*(C_HOFFSET))
        .word      PLUS
        .word      SEMIS
@

@  ************
@  *   >DFA   *
@  ************
@
        .align
N_TDFA:
        .word      4
        .ascii      ">DFA"
        .align
TDFA:
        .word    DOCOL
        .word    TDFA+HEADSIZE
        .word    0x0
        .word    TCFA
        .word    N_TDFA
        .word    0

        .word      LIT, (CW*(D_HOFFSET))
        .word      PLUS
        .word      SEMIS
@

@  ************
@  *   >FFA   *
@  ************
@
        .align
N_TFFA:
        .word      4
        .ascii      ">FFA"
        .align
TFFA:
        .word    DOCOL
        .word    TFFA+HEADSIZE
        .word    0x0
        .word    TDFA
        .word    N_TFFA
        .word    0

        .word      LIT, (CW*(F_HOFFSET))
        .word      PLUS
        .word      SEMIS
@

@  ************
@  *   >LFA   *
@  ************
@
        .align
N_TLFA:
        .word      4
        .ascii      ">LFA"
        .align
TLFA:
        .word    DOCOL
        .word    TLFA+HEADSIZE
        .word    0x0
        .word    TFFA
        .word    N_TLFA
        .word    0

        .word      LIT, (CW*(L_HOFFSET))
        .word      PLUS
        .word      SEMIS
@

@  ************
@  *   >NFA   *
@  ************
@
        .align
N_TNFA:
        .word      4
        .ascii      ">NFA"
        .align
TNFA:
        .word    DOCOL
        .word    TNFA+HEADSIZE
        .word    0x0
        .word    TLFA
        .word    N_TNFA
        .word    0

        .word      LIT,(CW*(N_HOFFSET))
        .word      PLUS
        .word      SEMIS
@


@
@  ************
@  *   >SFA   *
@  ************
@
        .align
N_TSFA:
        .word      4
        .ascii      ">SFA"
        .align
TSFA:
        .word    DOCOL
        .word    TSFA+HEADSIZE
        .word    0x0
        .word    TNFA
        .word    N_TSFA
        .word    0

        .word      LIT,(CW*(S_HOFFSET))
        .word      PLUS
        .word      SEMIS
@
@
@

@  ************
@  *   >PHA   *
@  ************
@
        .align
N_TPHA:
        .word      4
        .ascii      ">PHA"
        .align
TPHA:
        .word    DOCOL
        .word    TPHA+HEADSIZE
        .word    0x0
        .word    TSFA
        .word    N_TPHA
        .word    0

        .word      LIT,(CW*(PH_OFFSET))
        .word      PLUS
        .word      SEMIS
@

@  *************
@  *   >BODY   *
@  *************
@
        .align
N_TOBODY:
        .word      5
        .ascii      ">BODY"
        .align
TOBODY:
        .word    DOCOL
        .word    TOBODY+HEADSIZE
        .word    0x0
        .word    TPHA
        .word    N_TOBODY
        .word    0

        .word      TDFA, FETCH
        .word      CELLP           @ Skip DOES> pointer.
        .word      SEMIS
@

@  *************
@  *   BODY>   *
@  *************
@
        .align
N_BODYF:
        .word      5
        .ascii      "BODY>"
        .align
BODYF:
        .word    DOCOL
        .word    BODYF+HEADSIZE
        .word    0x0
        .word    TOBODY
        .word    N_BODYF
        .word    0

        .word      LIT,(CW*(BD_OFFSET))
        .word      LSUB
        .word      SEMIS
@

@  ************
@  *   >WID   *
@  ************
@
        .align
N_TWID:
        .word      4
        .ascii      ">WID"
        .align
TWID:
        .word    DOCOL
        .word    TWID+HEADSIZE
        .word    0x0
        .word    BODYF
        .word    N_TWID
        .word    0

        .word      TOBODY
        .word      CELLP @ Skip vfa link.
        .word      SEMIS
@

@  ************
@  *   >VFA   *
@  ************
@
        .align
N_TVFA:
        .word      4
        .ascii      ">VFA"
        .align
TVFA:
        .word    DOCOL
        .word    TVFA+HEADSIZE
        .word    0x0
        .word    TWID
        .word    N_TVFA
        .word    0

        .word      TOBODY
        .word      SEMIS
@


@  ************
@  *   !CSP   *
@  ************
@
        .align
N_SCSP:
        .word      4
        .ascii      "!CSP"
        .align
SCSP:
        .word    DOCOL
        .word    SCSP+HEADSIZE
        .word    0x0
        .word    TVFA
        .word    N_SCSP
        .word    0

        .word      SPFET
        .word      LCSP
        .word      STOR
        .word      SEMIS
@
@

@  **************
@  *   ?ERROR   *
@  **************
@
        .align
N_QERR:
        .word      6
        .ascii      "?ERROR"
        .align
QERR:
        .word    DOCOL
        .word    QERR+HEADSIZE
        .word    0x0
        .word    SCSP
        .word    N_QERR
        .word    0

        .word      SWAP
        .word      ZBRAN
        .word      QERR1-.-CW @IF
        .word      PP, FETCH
        .word      SRC, FETCH
        .word      LWHERE, TSTOR
        .word      THROW
        .word      BRAN
        .word      QERR2-.-CW  @ELSE
QERR1:  .word      DROP    @THEN
QERR2:  .word      SEMIS
@

@  **************
@  *   ?ERRUR   *
@  **************
@
        .align
N_QERRUR:
        .word      6
        .ascii      "?ERRUR"
        .align
QERRUR:
        .word    DOCOL
        .word    QERRUR+HEADSIZE
        .word    0x0
        .word    QERR
        .word    N_QERRUR
        .word    0

        .word      ZERO, MIN, LDUP, QERR
        .word      SEMIS
@


@  **************
@  *   ?DELIM   *
@  **************
@
        .align
N_QDELIM:
        .word      6
        .ascii      "?DELIM"
        .align
QDELIM:
        .word    DOCOL
        .word    QDELIM+HEADSIZE
        .word    0x0
        .word    QERRUR
        .word    N_QDELIM
        .word    0

        .word      PPFET
        .word      QBL
        .word      ZEQU
        .word      LIT, 10, QERR
        .word      DROP
        .word      SEMIS
@

@  ************
@  *   ?CSP   *
@  ************
@
        .align
N_QCSP:
        .word      4
        .ascii      "?CSP"
        .align
QCSP:
        .word    DOCOL
        .word    QCSP+HEADSIZE
        .word    0x0
        .word    QDELIM
        .word    N_QCSP
        .word    0

        .word      SPFET
        .word      LCSP
        .word      FETCH
        .word      LSUB
        .word      LIT, 20, QERR
        .word      SEMIS
@

@  *************
@  *   ?COMP   *
@  *************
@
        .align
N_QCOMP:
        .word      5
        .ascii      "?COMP"
        .align
QCOMP:
        .word    DOCOL
        .word    QCOMP+HEADSIZE
        .word    0x0
        .word    QCSP
        .word    N_QCOMP
        .word    0

        .word      STATE
        .word      FETCH
        .word      ZEQU
        .word      LIT, 17, QERR
        .word      SEMIS
@

@ 

@  **************
@  *   ?PAIRS   *
@  **************
@
        .align
N_QPAIR:
        .word      6
        .ascii      "?PAIRS"
        .align
QPAIR:
        .word    DOCOL
        .word    QPAIR+HEADSIZE
        .word    0x0
        .word    QCOMP
        .word    N_QPAIR
        .word    0

        .word      LSUB
        .word      LIT, 19, QERR
        .word      SEMIS
@

@ 
@
@
@

@  *********
@  *   [   *
@  *********
@
        .align
N_LBRAC:
        .word      1
        .ascii      "["
        .align
LBRAC:
        .word    DOCOL
        .word    LBRAC+HEADSIZE
        .word    B_IMMED
        .word    QPAIR
        .word    N_LBRAC
        .word    0

        .word      ZERO
        .word      STATE
        .word      STOR
        .word      SEMIS
@

@  *********
@  *   ]   *
@  *********
@
        .align
N_RBRAC:
        .word      1
        .ascii      "]"
        .align
RBRAC:
        .word    DOCOL
        .word    RBRAC+HEADSIZE
        .word    0x0
        .word    LBRAC
        .word    N_RBRAC
        .word    0

        .word      ONE
        .word      STATE
        .word      STOR
        .word      SEMIS
@

@  **************
@  *   HIDDEN   *
@  **************
@
        .align
N_HIDDEN:
        .word      6
        .ascii      "HIDDEN"
        .align
HIDDEN:
        .word    DOCOL
        .word    HIDDEN+HEADSIZE
        .word    0x0
        .word    RBRAC
        .word    N_HIDDEN
        .word    0

        .word      TFFA
        .word      LIT,B_INVIS
        .word      TOGGL
        .word      SEMIS
@

@  ***********
@  *   HEX   *
@  ***********
@
        .align
N_HEX:
        .word      3
        .ascii      "HEX"
        .align
HEX:
        .word    DOCOL
        .word    HEX+HEADSIZE
        .word    0x0
        .word    HIDDEN
        .word    N_HEX
        .word    0

        .word      LIT,16
        .word      BASE
        .word      STOR
        .word      SEMIS
@

@  ***************
@  *   DECIMAL   *
@  ***************
@
        .align
N_DECA:
        .word      7
        .ascii      "DECIMAL"
        .align
DECA:
        .word    DOCOL
        .word    DECA+HEADSIZE
        .word    0x0
        .word    HEX
        .word    N_DECA
        .word    0

        .word      LIT,10
        .word      BASE
        .word      STOR
        .word      SEMIS
@

@  ***************
@  *   (;CODE)   *
@  ***************
@
        .align
N_PSCOD:
        .word      7
        .ascii      "(;CODE)"
        .align
PSCOD:
        .word    DOCOL
        .word    PSCOD+HEADSIZE
        .word    0x0
        .word    DECA
        .word    N_PSCOD
        .word    0

        .word      FROMR
        .word      LATEST
        .word      TCFA
        .word      STOR
        .word      SEMIS
@

@

@  **************
@  *   CREATE   *
@  **************
@
        .align
N_LCREATE:
        .word      6
        .ascii      "CREATE"
        .align
LCREATE:
        .word    DOCOL
        .word    LCREATE+HEADSIZE
        .word    0x0
        .word    PSCOD
        .word    N_LCREATE
        .word    0

        .word      LNAME
        .word      PCREAT
        .word      LIT, HLNOOP, COMMA
        .word      PSCOD
DODOE:  STR     R11, [R12,#-(CW*(1))]!
        LDR     R11,[R0,#+(CW*((D_HOFFSET)))] @NEW IP 
        MOV     R0, R11
        ADD     R0,R0,#(CW*(1))
        LDR     R11,[R11]
        STR     R0, [R13,#-(CW*(1))]!
        LDR     R0, [R11],#+(CW*(1))
         LDR     R1,[R0]
         BX      R1  
HLNOOP: .word      SEMIS
@

@  *************
@  *   DOES>   *
@  *************
@
        .align
N_DOES:
        .word      5
        .ascii      "DOES>"
        .align
DOES:
        .word    DOCOL
        .word    DOES+HEADSIZE
        .word    0x0
        .word    LCREATE
        .word    N_DOES
        .word    0

        .word      FROMR
        .word      LATEST
        .word      TDFA
        .word      FETCH
        .word      STOR
        .word      SEMIS
@

@  *************
@  *   COUNT   *
@  *************
@
        .align
N_COUNT:
        .word      5
        .ascii      "COUNT"
        .align
COUNT:
        .word    DOCOL
        .word    COUNT+HEADSIZE
        .word    0x0
        .word    DOES
        .word    N_COUNT
        .word    0

        .word      LDUP
        .word      ONEP
        .word      SWAP
        .word      CFET
        .word      SEMIS
@

@  *****************
@  *   -TRAILING   *
@  *****************
@
        .align
N_DTRAI:
        .word      9
        .ascii      "-TRAILING"
        .align
DTRAI:
        .word    DOCOL
        .word    DTRAI+HEADSIZE
        .word    0x0
        .word    COUNT
        .word    N_DTRAI
        .word    0

        .word      LDUP
        .word      ZERO
        .word     XQDO
        .word      DTRA4-.-CW
DTRA1:  .word      OVER
        .word      OVER
        .word      PLUS
        .word      ONE
        .word      LSUB
        .word      CFET
        .word      QBL
        .word      ZEQU
        .word      ZBRAN
        .word      DTRA2-.-CW @IF
        .word      LLEAV
DTRA2:  .word      ONE
        .word      LSUB    @ THEN
        .word     ONE, XPLOOP, ZBRAN
        .word      DTRA1-.-CW
	.word      UNLOOP    @ LOOP
DTRA4:
        .word      SEMIS
@

@  **********
@  *   S"   *
@  **********
@
        .align
N_SQUOT:
        .word      2
        .ascii      "S\""
        .align
SQUOT:
        .word    DOCOL
        .word    SQUOT+HEADSIZE
        .word    B_IMMED
        .word    DTRAI
        .word    N_SQUOT
        .word    0

        .word      DENQ
        .word      SEMIS
@

@  **********
@  *   ."   *
@  **********
@
        .align
N_DOTQ:
        .word      2
        .ascii      ".\""
        .align
DOTQ:
        .word    DOCOL
        .word    DOTQ+HEADSIZE
        .word    B_IMMED
        .word    SQUOT
        .word    N_DOTQ
        .word    0

        .word      DENQ
        .word      STATE
        .word      FETCH
        .word      ZBRAN
        .word      DOTQ1-.-CW @ IF
        .word      LIT, LTYPE, COMMA
        .word      BRAN
        .word      DOTQ2-.-CW
DOTQ1:
        .word      LTYPE
DOTQ2:
        .word      SEMIS   @ THEN
@

@  **********
@  *   .(   *
@  **********
@
        .align
N_DOTP:
        .word      2
        .ascii      ".("
        .align
DOTP:
        .word    DOCOL
        .word    DOTP+HEADSIZE
        .word    B_IMMED
        .word    DOTQ
        .word    N_DOTP
        .word    0

        .word      LIT, ')'
        .word      PARSE
        .word      LTYPE
        .word      SEMIS
@

@  ***************
@  *   SET-SRC   *
@  ***************
@
        .align
N_SETSRC:
        .word      7
        .ascii      "SET-SRC"
        .align
SETSRC:
        .word    DOCOL
        .word    SETSRC+HEADSIZE
        .word    0x0
        .word    DOTP
        .word    N_SETSRC
        .word    0

        .word      OVER, PLUS
        .word      OVER, SRC, TSTOR
        .word      PP, STOR @  IN

@       DC      DOTS
        .word      SEMIS
@

@  ****************
@  *   EVALUATE   *
@  ****************
@
        .align
N_EVALUATE:
        .word      8
        .ascii      "EVALUATE"
        .align
EVALUATE:
        .word    DOCOL
        .word    EVALUATE+HEADSIZE
        .word    0x0
        .word    SETSRC
        .word    N_EVALUATE
        .word    0

        .word      SAVE
        .word      SETSRC
        .word      LIT, INTER, CATCH
        .word      RESTO
        .word      THROW
        .word      SEMIS
@

@  ************
@  *   FILL   *
@  ************
@
        .align
N_FILL:
        .word      4
        .ascii      "FILL"
        .align
FILL:
        .word    FILL+HEADSIZE
        .word    FILL+HEADSIZE
        .word    0x0
        .word    EVALUATE
        .word    N_FILL
        .word    0

        LDR     R0, [R13],#+(CW*(1))      @ FILL CHAR
        LDR     R2, [R13],#+(CW*(1))      @ FILL COUNT
        LDR     R1, [R13],#+(CW*(1))      @ BEGIN ADDR
        ANDS R2,R2, R2
        BEQ FILL2    @ COUNT=0? 
FILL1:  STRB    R0, [R1],#+1
        SUBS  R2,R2,#1
        BNE FILL1
FILL2:  LDR     R0, [R11],#+(CW*(1))
         LDR     R1,[R0]
         BX      R1  
@

@  ************
@  *   CORA   *
@  ************
@
        .align
N_CORA:
        .word      4
        .ascii      "CORA"
        .align
CORA:
        .word    CORA+HEADSIZE
        .word    CORA+HEADSIZE
        .word    0x0
        .word    FILL
        .word    N_CORA
        .word    0

        
        MOV    R0,#0   @ Result, assume all equal
        LDR     R3, [R13],#+(CW*(1))      @ count
        LDR     R4, [R13],#+(CW*(1))      @ addr2
        LDR     R5, [R13],#+(CW*(1))      @ addr1
        MOV     R1,#0
        MOV     R2,#0
CORA1:
        SUBS     R3,R3,#1
        BMI      CORA3        @ Stop looping if R3 < 0  and return 0 
        LDRB    R1, [R5],#+1          @ load byte at R5 in W1, auto-increment R5 
        LDRB    R2, [R4],#+1          @ load byte at R4 in W2, auto-increment R4 
        CMP      R1,R2
        BEQ      CORA1        @ chars equal, keep looping 
        MOV     R0, #1
        BHI      CORA3        @ R1/W1 is lexicographic later, return 1 
        
        MOV    R0,#-1  @TRUE FLAG  
CORA3:
        
        STR     R0, [R13,#-(CW*(1))]!
        LDR     R0, [R11],#+(CW*(1))
         LDR     R1,[R0]
         BX      R1  
@

@  **********
@  *   $^   *
@  **********
@
        .align
N_SINDEX:
        .word      2
        .ascii      "\$^"
        .align
SINDEX:
        .word    SINDEX+HEADSIZE
        .word    SINDEX+HEADSIZE
        .word    0x0
        .word    CORA
        .word    N_SINDEX
        .word    0

@     search address of given byte starting from R4 
        LDR     R0, [R13],#+(CW*(1))      @ char
        LDR     R2, [R13],#+(CW*(1))      @ count
        LDR     R4, [R13],#+(CW*(1))      @ addr
        BICS    R1,R1,R1    @Clear R1 and zero flag
SINDEX1:
        ANDS    R2,R2,R2
        BEQ     SINDEX3
        LDRB    R1, [R4],#+1         @ load next byte from R4 and auto-increment R4
        CMP     R0,R1       @ Compare BYTE and set flags
        BEQ     SINDEX2
        SUB     R2,R2,#1
        BAL     SINDEX1
SINDEX3:
        MOV    R4,#0       @ Not found: 0 
        ADD     R4,R4, #1
SINDEX2:
        SUB     R4,R4, #1
        STR     R4, [R13,#-(CW*(1))]!            @ Return address, or 0 
        LDR     R0, [R11],#+(CW*(1))
         LDR     R1,[R0]
         BX      R1  
@

@  **********
@  *   $/   *
@  **********
@
        .align
N_SSLAS:
        .word      2
        .ascii      "\$/"
        .align
SSLAS:
        .word    SSLAS+HEADSIZE
        .word    SSLAS+HEADSIZE
        .word    0x0
        .word    SINDEX
        .word    N_SSLAS
        .word    0

        
        LDR     R0, [R13],#+(CW*(1))          @ char
        LDR     R2, [R13],#+(CW*(1))          @ count
        MOV     R5,R2
        LDR     R4, [R13],#+(CW*(1))          @ addr
        MOV     R1,R4   @ Copy
        ANDS    R4,R4,R4    @ test R4 for nullpointer 
        BEQ     SSLAS3   @ Nil pointer 
SSLAS1:
        LDRB    R3, [R4],#+1         @ load byte at R4 in W3, then auto-increment R4
        CMP     R0, R3
        BEQ     SSLAS2
        SUBS    R2,R2, #1
        BNE     SSLAS1

@ Not present.
        STR     R2, [R13,#-(CW*(1))]!    @ Nil pointer.
        BAL     SSLAS4
SSLAS2:
        SUB     R5,R5,R2
        SUB     R2,R2,#1 @Delimiter is not part of first string.
SSLAS3:
        STR     R4, [R13,#-(CW*(1))]!
SSLAS4:
        STR     R2, [R13,#-(CW*(1))]!   @Remaining length
        STR     R1, [R13,#-(CW*(1))]!   @Start of first string.
        STR     R5, [R13,#-(CW*(1))]!   @Skipped length.
        
        LDR     R0, [R11],#+(CW*(1))
         LDR     R1,[R0]
         BX      R1  
@
@  **********
@  *   $\   *
@  **********
@
        .align
N_BSLAS:
        .word      2
        .ascii      "\$\\"
        .align
BSLAS:
        .word    BSLAS+HEADSIZE
        .word    BSLAS+HEADSIZE
        .word    0x0
        .word    SSLAS
        .word    N_BSLAS
        .word    0
   @_dummy
        LDR     R0, [R11],#+(CW*(1))
         LDR     R1,[R0]
         BX      R1  

@  *************
@  *   ERASE   *
@  *************
@
        .align
N_LERASE:
        .word      5
        .ascii      "ERASE"
        .align
LERASE:
        .word    DOCOL
        .word    LERASE+HEADSIZE
        .word    0x0
        .word    BSLAS
        .word    N_LERASE
        .word    0

        .word      ZERO
        .word      FILL
        .word      SEMIS
@

@  *************
@  *   BLANK   *
@  *************
@
        .align
N_BLANK:
        .word      5
        .ascii      "BLANK"
        .align
BLANK:
        .word    DOCOL
        .word    BLANK+HEADSIZE
        .word    0x0
        .word    LERASE
        .word    N_BLANK
        .word    0

        .word      LBL
        .word      FILL
        .word      SEMIS
@

@  ************
@  *   HOLD   *
@  ************
@
        .align
N_HOLD:
        .word      4
        .ascii      "HOLD"
        .align
HOLD:
        .word    DOCOL
        .word    HOLD+HEADSIZE
        .word    0x0
        .word    BLANK
        .word    N_HOLD
        .word    0

        .word      LIT,-1
        .word      HLD
        .word      PSTOR
        .word      HLD
        .word      FETCH
        .word      CSTOR
        .word      SEMIS
@

@  ***********
@  *   PAD   *
@  ***********
@
        .align
N_PAD:
        .word      3
        .ascii      "PAD"
        .align
PAD:
        .word    DOCOL
        .word    PAD+HEADSIZE
        .word    0x0
        .word    HOLD
        .word    N_PAD
        .word    0

        .word      HERE
@ Allow for a one line name, a double binary number and some hold char's
        .word      LIT,84+128+64
        .word      PLUS
        .word      SEMIS
@

@ 
@

@  ************
@  *   CHAR   *
@  ************
@
        .align
N_LCHAR:
        .word      4
        .ascii      "CHAR"
        .align
LCHAR:
        .word    DOCOL
        .word    LCHAR+HEADSIZE
        .word    0x0
        .word    PAD
        .word    N_LCHAR
        .word    0

        .word      LNAME, DROP, CFET
        .word      SEMIS
@

@  **************
@  *   [CHAR]   *
@  **************
@
        .align
N_BCHAR:
        .word      6
        .ascii      "[CHAR]"
        .align
BCHAR:
        .word    DOCOL
        .word    BCHAR+HEADSIZE
        .word    B_IMMED
        .word    LCHAR
        .word    N_BCHAR
        .word    0

        .word      LCHAR, LITER
        .word      SEMIS
@

@  ****************
@  *   (NUMBER)   *
@  ****************
@
        .align
N_PNUMB:
        .word      8
        .ascii      "(NUMBER)"
        .align
PNUMB:
        .word    DOCOL
        .word    PNUMB+HEADSIZE
        .word    0x0
        .word    BCHAR
        .word    N_PNUMB
        .word    0

        .word      ZERO, ZERO
        .word      ZERO, DPL, STOR
NPNUM1:  .word      PPFET   @ BEGIN
        .word      LDUP, LIT, ADOT, EQUAL
        .word      ZBRAN
        .word      NPNUM2-.-CW @ case 1 
        .word      DROP, DPL, STOR
        .word      BRAN
        .word      NPNUM1-.-CW @ continue 
NPNUM2:
        .word      LDUP, LIT, ',', EQUAL
        .word      ZBRAN
        .word      NPNUM3-.-CW @ case 2 
        .word      TDROP
        .word      BRAN
        .word      NPNUM1-.-CW @ continue 
NPNUM3:
        .word      LDUP, QBL, ZEQU
        .word      ZBRAN
        .word      NPNUM5-.-CW  @ while 
        .word      NIP
        .word      BASE, FETCH, DIGIT
        .word      ZEQU
        .word      LIT, 10, QERR

        .word      SWAP
        .word      BASE
        .word      FETCH
        .word      USTAR
        .word      DROP
        .word      ROT
        .word      BASE
        .word      FETCH
        .word      USTAR
        .word      DPLUS
        .word      BRAN
        .word      NPNUM1-.-CW @ repeat 
NPNUM5:
        .word      DROP, PP, STOR
        .word      SEMIS
@

@  **************
@  *   NUMBER   *
@  **************
@
        .align
N_NUMB:
        .word      6
        .ascii      "NUMBER"
        .align
NUMB:
        .word    DOCOL
        .word    NUMB+HEADSIZE
        .word    0x0
        .word    PNUMB
        .word    N_NUMB
        .word    0

LNUMB:
        .word      LIT, -1, PP, PSTOR
        .word      PNUMB, SDLITE
        .word      SEMIS
@

@  ***************
@  *   >NUMBER   *
@  ***************
@
        .align
N_TONUMB:
        .word      7
        .ascii      ">NUMBER"
        .align
TONUMB:
        .word    DOCOL
        .word    TONUMB+HEADSIZE
        .word    0x0
        .word    NUMB
        .word    N_TONUMB
        .word    0

        .word      TDUP, PLUS, TOR     @ End available on return stack.
        .word      ZERO
        .word     XQDO
        .word      TONUM9-.-CW
TONUM1:
        .word      LDUP, CFET, BASE, FETCH, DIGIT
        .word      ZEQU
        .word      ZBRAN
        .word      TONUM4-.-CW @ IF
        .word      DROP
        .word      LLEAV
TONUM4:
        .word      SWAP, TOR @ Address out of the way.
        .word      SWAP
        .word      BASE
        .word      FETCH
        .word      USTAR
        .word      DROP
        .word      ROT
        .word      BASE
        .word      FETCH
        .word      USTAR
        .word      DPLUS
        .word      FROMR, ONEP     @ Address back.
        .word     ONE, XPLOOP, ZBRAN
        .word      TONUM1-.-CW
	.word      UNLOOP
TONUM9:
        .word      FROMR
        .word      OVER, LSUB
        .word      SEMIS
@

@  *************
@  *   FOUND   *
@  *************
@
        .align
N_FOUND:
        .word      5
        .ascii      "FOUND"
        .align
FOUND:
        .word    DOCOL
        .word    FOUND+HEADSIZE
        .word    0x0
        .word    TONUMB
        .word    N_FOUND
        .word    0

        .word      CONTEXT, TOR
FOUND1: .word      RR, FETCH
@        DC      DOTS
        .word      PFIND, LDUP, ZEQU
        .word      ZBRAN
        .word      FOUND3-.-CW
        .word      DROP
        .word      RR, FETCH, LIT, ONLYBODY, LSUB @Was this ONLY?
        .word      ZBRAN
        .word      FOUND2-.-CW
        .word      FROMR, CELLP, TOR
        .word      BRAN
        .word      FOUND1-.-CW
FOUND2: .word      ZERO
FOUND3: .word      RDROP
        .word      NIP,NIP
        .word      SEMIS
@

@  ***************
@  *   PRESENT   *
@  ***************
@
        .align
N_PRESENT:
        .word      7
        .ascii      "PRESENT"
        .align
PRESENT:
        .word    DOCOL
        .word    PRESENT+HEADSIZE
        .word    0x0
        .word    FOUND
        .word    N_PRESENT
        .word    0

        .word      LDUP, TOR
        .word      FOUND
        .word      LDUP
        .word      ZBRAN
        .word      PRES1-.-CW
        .word      LDUP
        .word      TNFA, FETCH, FETCH @  Get precise length.
        .word      RR, EQUAL
        .word      LAND
PRES1:
        .word      RDROP
        .word      SEMIS
@


@ 
@

@

@  **************
@  *   (FIND)   *
@  **************
@
        .align
N_PFIND:
        .word      6
        .ascii      "(FIND)"
        .align
PFIND:
        .word    DOCOL
        .word    PFIND+HEADSIZE
        .word    0x0
        .word    PRESENT
        .word    N_PFIND
        .word    0

@ The idea is to have a fast slim loop.
@ The optimisation tries to skip, using the extra links.
@ At all labels the stack is (sc, dea).
@ PFIND0 : nothing known yet about dea
@ PFCONTINUE : dea is disapproved, go on 
@ PFIND1 : dea is a match, inasfar name and prefix
@ PFEXIT : dea is the match, or null.

PFIND0:                       @Start of fast loop.
         .word      NMATCH
         .word      ZEQU
        .word      ZBRAN
        .word      PFCONTINUE-.-CW
@ dea's name and sc have equal length
        .word      TDUP, TNFA, FETCH, FETCH
        .word      EQUAL 
@ dea is a prefix 
        .word      OVER, TFFA, FETCH
        .word      LIT, B_DENOT, LAND
        .word      LOR             @ One of those ,ist apply.
        .word      ZBRAN
        .word      PFCONTINUE-.-CW
@ Discard still, if invisible ("smudged") or dummy.
        .word      LDUP, TFFA, FETCH
        .word      LIT, B_INVIS | B_DUMMY, LAND
        .word      ZEQU
        .word      ZBRAN
        .word      PFCONTINUE-.-CW
        .word      BRAN
        .word      PFEXIT-.-CW		@ Match! 
PFCONTINUE:
        .word	TLFA, FETCH
        .word      LDUP, ZEQU
        .word      ZBRAN
        .word      PFIND0-.-CW
PFEXIT:   
        .word      SEMIS
@

@  *************
@  *   ERROR   *
@  *************
@
        .align
N_LERROR:
        .word      5
        .ascii      "ERROR"
        .align
LERROR:
        .word    DOCOL
        .word    LERROR+HEADSIZE
        .word    0x0
        .word    PFIND
        .word    N_LERROR
        .word    0

@ Make sure the whole word is shown in case of denotations. 
        .word      LWHERE, TFET
        .word      OVER
        .word      ZBRAN
        .word      ERR2-.-CW      @ Virgin: print empty string.
        .word      OVER, LIT, 20, LSUB
        .word      MAX
        .word      SWAP
        .word      ONEM         @Backup for if we are at next word.
ERR1:   .word      LDUP, CFET, QBL, ZEQU  @Show whole word!
        .word      ZBRAN
        .word      ERR2 -.-CW
        .word      ONEP
        .word      BRAN
        .word      ERR1 -.-CW
ERR2:
        .word      OVER, LSUB
        .word      ETYPE
        .word      SKIP
         .word      19
SB1: .ascii      " ? ciforth ERROR # "
       .align
        .word      LIT, SB1
        .word      LIT, 19
        .word      ETYPE
        .word      BASE, FETCH
        .word      DECA
        .word      OVER
        .word      STOD, ZERO, PDDOTR      @This is about (.) 
        .word      ETYPE
        .word      BASE, STOR
        .word      LIT, -256, MAX
        .word      LIT, 63, MIN
        .word      LIT, MESS, CATCH, DROP  @Prevent error loops 
        .word      SEMIS
@

@  *************
@  *   CATCH   *
@  *************
@
        .align
N_CATCH:
        .word      5
        .ascii      "CATCH"
        .align
CATCH:
        .word    DOCOL
        .word    CATCH+HEADSIZE
        .word    0x0
        .word    LERROR
        .word    N_CATCH
        .word    0

        .word      SPFET, CELLP, TOR
        .word      HANDLER, FETCH, TOR
        .word      RPFET, HANDLER, STOR
        .word      EXEC
        .word      FROMR, HANDLER, STOR
        .word      RDROP, ZERO
        .word      SEMIS
@

@  *************
@  *   THROW   *
@  *************
@
        .align
N_THROW:
        .word      5
        .ascii      "THROW"
        .align
THROW:
        .word    DOCOL
        .word    THROW+HEADSIZE
        .word    0x0
        .word    CATCH
        .word    N_THROW
        .word    0

        .word      LDUP
        .word      ZBRAN
        .word      THROW1-.-CW
        .word      HANDLER, FETCH, ZEQU
        .word      ZBRAN
        .word      THROW2-.-CW
        .word      LERROR
        .word      CLS
        .word      QUIT
THROW2:
        .word      HANDLER, FETCH, RPSTO
        .word      FROMR, HANDLER, STOR
        .word      FROMR, SWAP, TOR
        .word      SPSTO
        .word      FROMR
        .word      X
THROW1:
        .word      DROP
        .word      SEMIS
@

@ 

@ 
@

@  ***********
@  *   ID.   *
@  ***********
@
        .align
N_IDDOT:
        .word      3
        .ascii      "ID."
        .align
IDDOT:
        .word    DOCOL
        .word    IDDOT+HEADSIZE
        .word    0x0
        .word    THROW
        .word    N_IDDOT
        .word    0

        .word      LDUP, TFFA, FETCH
        .word      LIT, B_DUMMY, LAND
        .word      ZBRAN
        .word      IDDOT1-.-CW
        .word      DROP
        .word      BRAN
        .word      IDDOT2-.-CW
IDDOT1:
        .word      TNFA
        .word      FETCH
        .word      SFET
        .word      LTYPE
        .word      SPACE
        .word      SPACE
        .word      SPACE
IDDOT2:
        .word      SEMIS
@

@  **************
@  *   HEADER   *
@  **************
@
        .align
N_LHEAD:
        .word      6
        .ascii      "HEADER"
        .align
LHEAD:
        .word    DOCOL
        .word    LHEAD+HEADSIZE
        .word    0x0
        .word    IDDOT
        .word    N_LHEAD
        .word    0

        .word      LDUP
        .word      ZEQU
        .word      LIT, 5, QERR
        .word      TDUP
        .word      PRESENT
        .word      LDUP
        .word      ZBRAN
        .word      CREA1-.-CW @IF
        .word      TNFA, FETCH, SFET
        .word      ETYPE
        .word      LIT,4
        .word      MESS
        .word      X       @THEN
CREA1:  .word      DROP
 .word      LALIGN
        .word      SCOMMA
 .word      LALIGN
        .word      HERE,TOR

        .word      RR, TPHA, COMMA         @Code field.
        .word      RR, TPHA, COMMA         @Data field.
        .word      ZERO, COMMA @Flag field.
        .word      ZERO, COMMA @Link field.
        .word      COMMA   @Name field.


        .word      BLK, FETCH, LDUP, ZEQU
        .word      ZBRAN
        .word      CREA2-.-CW
        .word      DROP, PP, FETCH
CREA2:  .word      COMMA  @Source field.

                .word      FROMR
        .word      SEMIS
@

@  ************
@  *   LINK   *
@  ************
@
        .align
N_LLINK:
        .word      4
        .ascii      "LINK"
        .align
LLINK:
        .word    DOCOL
        .word    LLINK+HEADSIZE
        .word    0x0
        .word    LHEAD
        .word    N_LLINK
        .word    0

        .word      TLFA
        .word      TDUP
        .word      FETCH
        .word      SWAP, TLFA, STOR
        .word      STOR
        .word      SEMIS
@

@  ****************
@  *   (CREATE)   *
@  ****************
@
        .align
N_PCREAT:
        .word      8
        .ascii      "(CREATE)"
        .align
PCREAT:
        .word    DOCOL
        .word    PCREAT+HEADSIZE
        .word    0x0
        .word    LLINK
        .word    N_PCREAT
        .word    0

        .word      LHEAD
        .word      CURR, FETCH
        .word      LLINK
        .word      SEMIS
@

@  *****************
@  *   [COMPILE]   *
@  *****************
@
        .align
N_BCOMP:
        .word      9
        .ascii      "[COMPILE]"
        .align
BCOMP:
        .word    DOCOL
        .word    BCOMP+HEADSIZE
        .word    B_IMMED
        .word    PCREAT
        .word    N_BCOMP
        .word    0

        .word      LNAME
        .word      PRESENT
        .word      LDUP
        .word      ZEQU
        .word      LIT, 16, QERR
        .word      TCFA
        .word      COMMA
        .word      SEMIS
@

@  ****************
@  *   POSTPONE   *
@  ****************
@
        .align
N_POSTP:
        .word      8
        .ascii      "POSTPONE"
        .align
POSTP:
        .word    DOCOL
        .word    POSTP+HEADSIZE
        .word    B_IMMED
        .word    BCOMP
        .word    N_POSTP
        .word    0

        .word      LNAME
        .word      PRESENT
        .word      LDUP
        .word      ZEQU
        .word      LIT, 15, QERR
        .word      LDUP, TFFA, FETCH
        .word      LIT, B_IMMED, LAND, ZEQU
        .word      ZBRAN
        .word      POSTP1-.-CW
         .word      LIT, LIT, COMMA
         .word      COMMA
         .word      LIT, COMMA
POSTP1:
         .word      COMMA
        .word      SEMIS
@

@  ***************
@  *   LITERAL   *
@  ***************
@
        .align
N_LITER:
        .word      7
        .ascii      "LITERAL"
        .align
LITER:
        .word    DOCOL
        .word    LITER+HEADSIZE
        .word    B_IMMED
        .word    POSTP
        .word    N_LITER
        .word    0

        .word      STATE
        .word      FETCH
        .word      ZBRAN
        .word      LITE1-.-CW @IF
        .word      LIT, LIT, COMMA
        .word      COMMA   @THEN
LITE1:  .word      SEMIS
@

@  ****************
@  *   DLITERAL   *
@  ****************
@
        .align
N_DLITE:
        .word      8
        .ascii      "DLITERAL"
        .align
DLITE:
        .word    DOCOL
        .word    DLITE+HEADSIZE
        .word    B_IMMED
        .word    LITER
        .word    N_DLITE
        .word    0

        .word      STATE
        .word      FETCH
        .word      ZBRAN
        .word      DLIT1-.-CW @ IF
        .word      SWAP
        .word      LITER
        .word      LITER   @ THEN
DLIT1:  .word      SEMIS
@
@

@  *****************
@  *   SDLITERAL   *
@  *****************
@
        .align
N_SDLITE:
        .word      9
        .ascii      "SDLITERAL"
        .align
SDLITE:
        .word    DOCOL
        .word    SDLITE+HEADSIZE
        .word    B_IMMED
        .word    DLITE
        .word    N_SDLITE
        .word    0

        .word      DPL
        .word      FETCH
        .word      ZBRAN
        .word      SDLIT1-.-CW @ IF
        .word      DLITE
        .word      BRAN
        .word      SDLIT2-.-CW @ IF
SDLIT1:
        .word      DROP, LITER
SDLIT2:
        .word      SEMIS
@


@  **************
@  *   ?STACK   *
@  **************
@
        .align
N_QSTAC:
        .word      6
        .ascii      "?STACK"
        .align
QSTAC:
        .word    DOCOL
        .word    QSTAC+HEADSIZE
        .word    0x0
        .word    SDLITE
        .word    N_QSTAC
        .word    0

        .word      SPFET
        .word      SZERO
        .word      FETCH
        .word      SWAP
        .word      ULESS
        .word      ONE, QERR
        .word      SPFET
        .word      HERE
        .word      LIT,0x80
        .word      PLUS
        .word      ULESS
        .word      LIT, 7, QERR
        .word      SEMIS
        #
@
@

@  *****************
@  *   INTERPRET   *
@  *****************
@
        .align
N_INTER:
        .word      9
        .ascii      "INTERPRET"
        .align
INTER:
        .word    DOCOL
        .word    INTER+HEADSIZE
        .word    0x0
        .word    QSTAC
        .word    N_INTER
        .word    0

INTE1:
        .word      LNAME
        .word      LDUP      @ Zero length.
        .word      ZBRAN
        .word      INTE8-.-CW @WHILE
@       DC      DOTS
@       DC      TDUP, LTYPE
        .word      OVER, TOR       @ Save old parse pointer.
        .word      FOUND
        .word      LDUP, ZEQU
        .word      LIT, 12, QERR
        .word      LDUP, TFFA, FETCH
        .word      LDUP, LIT, B_DENOT, LAND @Retain copy of flags.
        .word      ZBRAN
        .word      INTE3B-.-CW @IF
        .word      OVER, TNFA, FETCH, FETCH
        .word      RR, PLUS, PP, STOR  @Skip over prefix.
INTE3B:                  @THEN 
        .word      RDROP           @ Drop old parse pointer.
        .word      LIT, B_IMMED, LAND
        .word      STATE, FETCH, ZEQU, LOR
        .word      ZBRAN
        .word      INTE3-.-CW @IF
        .word      EXEC
        .word      BRAN
        .word      INTE4-.-CW @IF
INTE3:
        .word      COMMA
                        @THEN
INTE4:
        .word      QSTAC
        .word      BRAN
        .word      INTE1-.-CW  @AGAIN
INTE8:  .word      DROP, DROP
        .word      SEMIS
@

@  *****************
@  *   IMMEDIATE   *
@  *****************
@
        .align
N_IMMED:
        .word      9
        .ascii      "IMMEDIATE"
        .align
IMMED:
        .word    DOCOL
        .word    IMMED+HEADSIZE
        .word    0x0
        .word    INTER
        .word    N_IMMED
        .word    0

        .word      LATEST
        .word      TFFA
        .word      LIT, B_IMMED
        .word      TOGGL
        .word      SEMIS
@

@  **************
@  *   PREFIX   *
@  **************
@
        .align
N_PREFX:
        .word      6
        .ascii      "PREFIX"
        .align
PREFX:
        .word    DOCOL
        .word    PREFX+HEADSIZE
        .word    0x0
        .word    IMMED
        .word    N_PREFX
        .word    0

        .word      LATEST
        .word      TFFA
        .word      LIT, B_DENOT
        .word      TOGGL
        .word      SEMIS
@

@  *****************
@  *   NAMESPACE   *
@  *****************
@
        .align
N_VOCAB:
        .word      9
        .ascii      "NAMESPACE"
        .align
VOCAB:
        .word    DOCOL
        .word    VOCAB+HEADSIZE
        .word    0x0
        .word    PREFX
        .word    N_VOCAB
        .word    0

        .word      LCREATE
        .word      LATEST   @ Link this DEA into VOC-LINK chain.
        .word      VOCL
        .word      FETCH
        .word      COMMA
        .word      VOCL
        .word      STOR
        .word      WORDLIST, DROP 
        .word      DOES
DOVOC:
        .word      ALSO
        .word      CELLP   @ Make it a WID. 
        .word      CONTEXT
        .word      STOR
        .word      SEMIS
        #
@
@   The link to task is a cold start value only.
@   It is updated each time a definition is
@   appended to the 'FORTH' namespace.
@

@  ****************
@  *   WORDLIST   *
@  ****************
@
        .align
N_WORDLIST:
        .word      8
        .ascii      "WORDLIST"
        .align
WORDLIST:
        .word    DOCOL
        .word    WORDLIST+HEADSIZE
        .word    0x0
        .word    VOCAB
        .word    N_WORDLIST
        .word    0

        .word      HERE
        .word      ZERO, COMMA   @ Dummy code field
        .word      ZERO, COMMA   @ Dummy data field
        .word      LIT, B_DUMMY, COMMA @ Dummy flag field
        .word      ZERO, COMMA @Link field: empty chain
        .word      LIT, (ZERO+(CW*(D_HOFFSET))), COMMA @Empty string for name.
        .word      ZERO, COMMA
                .word      SEMIS
        #
@

@

@  *******************
@  *   DEFINITIONS   *
@  *******************
@
        .align
N_DEFIN:
        .word      11
        .ascii      "DEFINITIONS"
        .align
DEFIN:
        .word    DOCOL
        .word    DEFIN+HEADSIZE
        .word    0x0
        .word    WORDLIST
        .word    N_DEFIN
        .word    0

        .word      CONTEXT
        .word      FETCH
        .word      CURR
        .word      STOR
        .word      SEMIS
@

@  ************
@  *   ALSO   *
@  ************
@
        .align
N_ALSO:
        .word      4
        .ascii      "ALSO"
        .align
ALSO:
        .word    DOCOL
        .word    ALSO+HEADSIZE
        .word    0x0
        .word    DEFIN
        .word    N_ALSO
        .word    0

        .word      CONTEXT, LDUP, CELLP
        .word      LIT, (CW*(16-1))
        .word      LMOVE
        .word      LIT, ONLYBODY  @End sentinel for array of word lists.
        .word      CONTEXT, LIT, (CW*(16)), PLUS
        .word      STOR @Trim sets of wordset.
        .word      SEMIS
@

@  ****************
@  *   PREVIOUS   *
@  ****************
@
        .align
N_PREVI:
        .word      8
        .ascii      "PREVIOUS"
        .align
PREVI:
        .word    DOCOL
        .word    PREVI+HEADSIZE
        .word    0x0
        .word    ALSO
        .word    N_PREVI
        .word    0

        .word      CONTEXT, LDUP, CELLP, SWAP
        .word      LIT, (CW*(16))
        .word      LMOVE
        .word      SEMIS
@

@  *********
@  *   (   *
@  *********
@
        .align
N_PAREN:
        .word      1
        .ascii      "("
        .align
PAREN:
        .word    DOCOL
        .word    PAREN+HEADSIZE
        .word    B_IMMED
        .word    PREVI
        .word    N_PAREN
        .word    0

        .word      LIT,')'
        .word      PARSE
        .word      TDROP
        .word      SEMIS
@
@

@  *********
@  *   \   *
@  *********
@
        .align
N_BACKS:
        .word      1
        .ascii      "\\"
        .align
BACKS:
        .word    DOCOL
        .word    BACKS+HEADSIZE
        .word    B_IMMED
        .word    PAREN
        .word    N_BACKS
        .word    0

@ Backup one character, just in case we are at the end of a line.
        .word      LIT, -1, PP, PSTOR
        .word      LIT,ALF
        .word      PARSE
        .word      TDROP
        .word      SEMIS
@

@  ************
@  *   QUIT   *
@  ************
@
        .align
N_QUIT:
        .word      4
        .ascii      "QUIT"
        .align
QUIT:
        .word    DOCOL
        .word    QUIT+HEADSIZE
        .word    0x0
        .word    BACKS
        .word    N_QUIT
        .word    0

        .word      LBRAC
QUIT1:                  @BEGIN
        .word      RZERO
        .word      FETCH
        .word      RPSTO

@ debugging 
@        DC      REFTIB  
@ end debugging 

        .word      LIT
        .word      PACCEP
        .word      CATCH
        .word      LDUP, LIT, -EPIPE, EQUAL
        .word      ZBRAN
        .word      ENDIF7-.-CW
        .word      BYE     @End of input, no error!
ENDIF7:
        .word      QERRUR
        .word      ONEP
@ debugging        DC      TDUP, LTYPE, CR 
        .word      SETSRC
        .word      INTER
        .word      OK
        .word      BRAN
        .word      QUIT1-.-CW  @AGAIN
        .word      SEMIS   @Unnecessary, but helpful for decompilation.
@

@  **********
@  *   OK   *
@  **********
@
        .align
N_OK:
        .word      2
        .ascii      "OK"
        .align
OK:
        .word    DOCOL
        .word    OK+HEADSIZE
        .word    0x0
        .word    QUIT
        .word    N_OK
        .word    0

        .word      STATE
        .word      FETCH
        .word      ZEQU
        .word      ZBRAN
        .word      OK2-.-CW @IF
        .word      SKIP
         .word      3
SB2: .ascii      " OK"
       .align
        .word      LIT, SB2
        .word      LIT, 3
        .word      LTYPE
        .word      CR
OK2:
        .word      SEMIS
@


@  ***************
@  *   OPTIONS   *
@  ***************
@
        .align
N_OPTIONS:
        .word      7
        .ascii      "OPTIONS"
        .align
OPTIONS:
        .word    DOCOL
        .word    OPTIONS+HEADSIZE
        .word    0x0
        .word    OK
        .word    N_OPTIONS
        .word    0

@       Execute option.
        .word      ARGS, FETCH, CELLP, CELLP, FETCH, LDUP
        .word      ZBRAN
        .word      OPT4-.-CW @ No options  




@Supposedly on the stack is now a pointer to the option.
        .word      COUNT

        .word      LIT, '-', UNEQ
        .word      ZBRAN
        .word      OPT3-.-CW
        .word      LIT, 3, LDUP, LERROR
        .word      XCODE, STOR, BYE
OPT3:
        .word      CFET
        .word      LIT, 0x1F, LAND
        .word      LLOAD       @Supposedly stack-neutral.
        .word      ZERO, SWAP @ Sign on suppressed.
OPT4:
        .word      DROP
        .word      SEMIS   @Unnecessary, but helpful for decompilation.
@


@  *************
@  *   _INIT   *
@  *************
@
        .align
N_INIT:
        .word      5
        .ascii      "_INIT"
        .align
INIT:
        .word    DOCOL
        .word    INIT+HEADSIZE
        .word    0x0
        .word    OPTIONS
        .word    N_INIT
        .word    0

        .word      SZERO, FETCH, SPSTO
        .word      ZERO, HANDLER, STOR
        .word      DECA
        .word      ONLY, FORTH, DEFIN

        .word      LIT, 0, BLINI  @Default, open library file read-only!@

        .word      SEMIS
@


@  ***********
@  *   S>D   *
@  ***********
@
        .align
N_STOD:
        .word      3
        .ascii      "S>D"
        .align
STOD:
        .word    STOD+HEADSIZE
        .word    STOD+HEADSIZE
        .word    0x0
        .word    INIT
        .word    N_STOD
        .word    0

        LDR     R1, [R13],#+(CW*(1))      @S1
        MOV    R0,#0
        ANDS    R1,R1,R1
        BPL     STOD1   @POS
        SUB     R0,R0, #1      @NEG
STOD1:
        STR     R1, [R13,#-(CW*(1))]!
        STR     R0, [R13,#-(CW*(1))]!
        LDR     R0, [R11],#+(CW*(1))
         LDR     R1,[R0]
         BX      R1  
@

@  ***********
@  *   ABS   *
@  ***********
@
        .align
N_LABS:
        .word      3
        .ascii      "ABS"
        .align
LABS:
        .word    DOCOL
        .word    LABS+HEADSIZE
        .word    0x0
        .word    STOD
        .word    N_LABS
        .word    0

        .word      LDUP
        .word      ZLESS
        .word      ZBRAN
        .word      PM1-.-CW   @IF
        .word      NEGATE   @THEN
PM1:
        .word      SEMIS
@

@  ************
@  *   DABS   *
@  ************
@
        .align
N_DABS:
        .word      4
        .ascii      "DABS"
        .align
DABS:
        .word    DOCOL
        .word    DABS+HEADSIZE
        .word    0x0
        .word    LABS
        .word    N_DABS
        .word    0

        .word      LDUP
        .word      ZLESS
        .word      ZBRAN
        .word      DPM1-.-CW  @IF
        .word      DNEGA   @THEN
DPM1:
        .word      SEMIS
@

@  ***********
@  *   MIN   *
@  ***********
@
        .align
N_MIN:
        .word      3
        .ascii      "MIN"
        .align
MIN:
        .word    DOCOL
        .word    MIN+HEADSIZE
        .word    0x0
        .word    DABS
        .word    N_MIN
        .word    0

        .word      TDUP
        .word      GREAT
        .word      ZBRAN
        .word      MIN1-.-CW  @IF
        .word      SWAP    @THEN
MIN1:   .word      DROP
        .word      SEMIS
@

@  ***********
@  *   MAX   *
@  ***********
@
        .align
N_MAX:
        .word      3
        .ascii      "MAX"
        .align
MAX:
        .word    DOCOL
        .word    MAX+HEADSIZE
        .word    0x0
        .word    MIN
        .word    N_MAX
        .word    0

        .word      TDUP
        .word      LESS
        .word      ZBRAN
        .word      MAX1-.-CW  @IF
        .word      SWAP    @THEN
MAX1:   .word      DROP
        .word      SEMIS
@

@  **************
@  *   LSHIFT   *
@  **************
@
        .align
N_LSHIFT:
        .word      6
        .ascii      "LSHIFT"
        .align
LSHIFT:
        .word    LSHIFT+HEADSIZE
        .word    LSHIFT+HEADSIZE
        .word    0x0
        .word    MAX
        .word    N_LSHIFT
        .word    0

        LDR     R2, [R13],#+(CW*(1))
        LDR     R0, [R13],#+(CW*(1))
         LSL R0,R0, R2
        STR     R0, [R13,#-(CW*(1))]!
        LDR     R0, [R11],#+(CW*(1))
         LDR     R1,[R0]
         BX      R1  
@

@  **************
@  *   RSHIFT   *
@  **************
@
        .align
N_RSHIFT:
        .word      6
        .ascii      "RSHIFT"
        .align
RSHIFT:
        .word    RSHIFT+HEADSIZE
        .word    RSHIFT+HEADSIZE
        .word    0x0
        .word    LSHIFT
        .word    N_RSHIFT
        .word    0

        LDR     R2, [R13],#+(CW*(1))
        LDR     R0, [R13],#+(CW*(1))
        LSR R0,R0, R2
         STR     R0, [R13,#-(CW*(1))]!
        LDR     R0, [R11],#+(CW*(1))
         LDR     R1,[R0]
         BX      R1  
@

@  ***************
@  *   ARSHIFT   *
@  ***************
@
        .align
N_ARSHIFT:
        .word      7
        .ascii      "ARSHIFT"
        .align
ARSHIFT:
        .word    ARSHIFT+HEADSIZE
        .word    ARSHIFT+HEADSIZE
        .word    0x0
        .word    RSHIFT
        .word    N_ARSHIFT
        .word    0

        LDR     R2, [R13],#+(CW*(1))
        LDR     R0, [R13],#+(CW*(1))
        ASR R0,R0, R2
        STR     R0, [R13,#-(CW*(1))]!
        LDR     R0, [R11],#+(CW*(1))
         LDR     R1,[R0]
         BX      R1  
@

@  **********
@  *   M*   *
@  **********
@
        .align
N_MSTAR:
        .word      2
        .ascii      "M*"
        .align
MSTAR:
        .word    MSTAR+HEADSIZE
        .word    MSTAR+HEADSIZE
        .word    0x0
        .word    ARSHIFT
        .word    N_MSTAR
        .word    0

        LDR     R3, [R13],#+(CW*(1))
        LDR     R2, [R13],#+(CW*(1))
 @ end thumb  )
        SMULL    R1,R0,R2,R3          @Signed *, 64 bits result  
        STR     R1, [R13,#-(CW*(1))]!
        STR     R0, [R13,#-(CW*(1))]!
        LDR     R0, [R11],#+(CW*(1))
         LDR     R1,[R0]
         BX      R1            @STORE DOUBLE CELL
@

@ High level implementation from FIG-forth 
@  **************
@  *   SM/REM   *
@  **************
@
        .align
N_MSLAS:
        .word      6
        .ascii      "SM/REM"
        .align
MSLAS:
        .word    DOCOL
        .word    MSLAS+HEADSIZE
        .word    0x0
        .word    MSTAR
        .word    N_MSLAS
        .word    0

        .word      OVER, TOR, TOR, DABS  @   OVER >R >R DABS 
        .word      RR, LABS, USLAS       @   R ABS UM/MOD 
        .word      FROMR, RR, LXOR       @   R> R XOR 
        .word      ZLESS                 @   <0 IF NEGATE THEN 
        .word      ZBRAN
        .word      MSLAS1-.-CW
        .word      NEGATE
MSLAS1:
        .word      SWAP, FROMR           @   SWAP R> 
        .word      ZLESS                 @   <0 IF NEGATE THEN 
        .word      ZBRAN
        .word      MSLAS2-.-CW
        .word      NEGATE
MSLAS2:
        .word      SWAP                  @   SWAP ; 
        .word      SEMIS
@
 /* 
CODE_HEADER({SM/REM},{MSLAS})
        POP(R8)      _C{DIVISOR}
        POP(R10)     _C{MSW OF DIVIDEND}
        POP(R0)      _C{LSW OF DIVIDEND}
        BL           IDIV _C{ not yet 64 bit/32 bit signed}
        _2PUSH_R10          _C{STORE QUOT/REM}
_C


_C{ Divide routine 32/32 signed from Henry Thomas (http://me.henri.net/fp-div.html)}
_C{ Entry  R0: numerator (lo) must be signed positive}
_C{        R8: denominator (den aka divisor) must be non-zero and signed negative}
IDIV:

        lo .req R0
        hi .req R10
        den .req R8
        MOVI  R5, #0   _C{ R5 is sign-memory of den}
        CMP den, #0
        BMI IDIV1
        MVN den,den  _C{ den  <-- two-complement of den}
        ADD den, den, #1
        MOVI  R5, #-1
IDIV1:  MOVI  R6, #0   _C{ R6 is sign-memory of lo}
        CMP lo,#0
        BPL IDIV2
        MVN lo,lo    _C{ lo <-- two-complement of lo}
        ADD lo,lo,#1
        MOVI  R6,#-1

_C{the original routine:}
_C{with positive numerator and negative denominator}

IDIV2:  MOVI  hi, #0 _C{ hi = 0}
        ADDS lo, lo, lo
        .rept 32 _C{ repeat 32 times}
          ADCS hi, den, hi, lsl #1
          SUBCC hi, hi, den
          ADCS lo,lo
        .endr
_C{end of original routine}

        _C{ remainder has sign of numerator}
        CMP R6, #0
        BEQ IDIV3
        MVN hi, hi  _C{ hi  <-- two-complement of remainder}
        ADD hi, hi, #1

IDIV3:
        _C{ quotient has both signs of numerator and denominator}
        CMP R5,R6
        BNE IDIV4
        MVN lo,lo    _C{ lo <-- two-complement of lo}
        ADD lo,lo,#1
IDIV4:
        MOV PC, LR _C{ return}
_C{Exit   R0:  quotient (lo)}
_C{       R10: remainder (hi)}
 */  


@  **********
@  *   2/   *
@  **********
@
        .align
N_TWOSL:
        .word      2
        .ascii      "2/"
        .align
TWOSL:
        .word    DOCOL
        .word    TWOSL+HEADSIZE
        .word    0x0
        .word    MSLAS
        .word    N_TWOSL
        .word    0

        .word      STOD, TWO, FMSLAS
        .word      NIP
        .word      SEMIS
@

@  **********
@  *   2*   *
@  **********
@
        .align
N_TWOST:
        .word      2
        .ascii      "2*"
        .align
TWOST:
        .word    DOCOL
        .word    TWOST+HEADSIZE
        .word    0x0
        .word    TWOSL
        .word    N_TWOST
        .word    0

        .word      TWO, STAR
        .word      SEMIS
@

@  **********
@  *   1-   *
@  **********
@
        .align
N_ONEM:
        .word      2
        .ascii      "1-"
        .align
ONEM:
        .word    DOCOL
        .word    ONEM+HEADSIZE
        .word    0x0
        .word    TWOST
        .word    N_ONEM
        .word    0

        .word      ONE, LSUB
        .word      SEMIS
@

@  **************
@  *   FM/MOD   *
@  **************
@
        .align
N_FMSLAS:
        .word      6
        .ascii      "FM/MOD"
        .align
FMSLAS:
        .word    DOCOL
        .word    FMSLAS+HEADSIZE
        .word    0x0
        .word    ONEM
        .word    N_FMSLAS
        .word    0

        .word      LDUP, TOR
        .word      TDUP, LXOR, TOR
        .word      MSLAS
        .word      FROMR, ZLESS
        .word      ZBRAN
        .word      FMMOD1-.-CW
        .word      OVER
        .word      ZBRAN
        .word      FMMOD1-.-CW
        .word      ONE, LSUB
        .word      SWAP, FROMR, PLUS, SWAP
        .word      BRAN
        .word      FMMOD2-.-CW
FMMOD1:
        .word      RDROP
FMMOD2:
        .word      SEMIS
@

@  *********
@  *   *   *
@  *********
@
        .align
N_STAR:
        .word      1
        .ascii      "*"
        .align
STAR:
        .word    DOCOL
        .word    STAR+HEADSIZE
        .word    0x0
        .word    FMSLAS
        .word    N_STAR
        .word    0

        .word      MSTAR
        .word      DROP
        .word      SEMIS
@

@  ************
@  *   /MOD   *
@  ************
@
        .align
N_SLMOD:
        .word      4
        .ascii      "/MOD"
        .align
SLMOD:
        .word    DOCOL
        .word    SLMOD+HEADSIZE
        .word    0x0
        .word    STAR
        .word    N_SLMOD
        .word    0

        .word      TOR
        .word      STOD
        .word      FROMR
        .word      MSLAS
        .word      SEMIS
@

@  *********
@  *   /   *
@  *********
@
        .align
N_SLASH:
        .word      1
        .ascii      "/"
        .align
SLASH:
        .word    DOCOL
        .word    SLASH+HEADSIZE
        .word    0x0
        .word    SLMOD
        .word    N_SLASH
        .word    0

        .word      SLMOD
        .word      NIP
        .word      SEMIS
@

@  ***********
@  *   MOD   *
@  ***********
@
        .align
N_LMOD:
        .word      3
        .ascii      "MOD"
        .align
LMOD:
        .word    DOCOL
        .word    LMOD+HEADSIZE
        .word    0x0
        .word    SLASH
        .word    N_LMOD
        .word    0

        .word      SLMOD
        .word      DROP
        .word      SEMIS
@

@  *************
@  *   */MOD   *
@  *************
@
        .align
N_SSMOD:
        .word      5
        .ascii      "*/MOD"
        .align
SSMOD:
        .word    DOCOL
        .word    SSMOD+HEADSIZE
        .word    0x0
        .word    LMOD
        .word    N_SSMOD
        .word    0

        .word      TOR
        .word      MSTAR
        .word      FROMR
        .word      MSLAS
        .word      SEMIS
@

@  **********
@  *   */   *
@  **********
@
        .align
N_SSLA:
        .word      2
        .ascii      "*/"
        .align
SSLA:
        .word    DOCOL
        .word    SSLA+HEADSIZE
        .word    0x0
        .word    SSMOD
        .word    N_SSLA
        .word    0

        .word      SSMOD
        .word      NIP
        .word      SEMIS
@

@  *************
@  *   M/MOD   *
@  *************
@
        .align
N_MSMOD1:
        .word      5
        .ascii      "M/MOD"
        .align
MSMOD1:
        .word    DOCOL
        .word    MSMOD1+HEADSIZE
        .word    0x0
        .word    SSLA
        .word    N_MSMOD1
        .word    0

@  ***************
@  *   UDM/MOD   *
@  ***************
@
        .align
N_MSMOD:
        .word      7
        .ascii      "UDM/MOD"
        .align
MSMOD:
        .word    DOCOL
        .word    MSMOD+HEADSIZE
        .word    0x0
        .word    MSMOD1
        .word    N_MSMOD
        .word    0

        .word      TOR
        .word      ZERO
        .word      RR
        .word      USLAS
        .word      FROMR
        .word      SWAP
        .word      TOR
        .word      USLAS
        .word      FROMR
        .word      SEMIS
@

@  **************
@  *   (LINE)   *
@  **************
@
        .align
N_PLINE:
        .word      6
        .ascii      "(LINE)"
        .align
PLINE:
        .word    DOCOL
        .word    PLINE+HEADSIZE
        .word    0x0
        .word    MSMOD
        .word    N_PLINE
        .word    0

        .word      TOR
        .word      LIT,64
        .word      MSTAR
        .word      BBUF
        .word      FMSLAS
        .word      FROMR @ This blocks, so is screens.
        .word      PLUS
        .word      BLOCK
        .word      PLUS
        .word      LIT,63
        .word      SEMIS
@

@  **************
@  *   ERRSCR   *
@  **************
@
        .align
N_ERRSCR:
        .word      6
        .ascii      "ERRSCR"
        .align
ERRSCR:
        .word    DOVAR
        .word    ERRSCR+HEADSIZE
        .word    0x0
        .word    PLINE
        .word    N_ERRSCR
        .word    0

        .word ERRORSCREEN
@

@  ***************
@  *   MESSAGE   *
@  ***************
@
        .align
N_MESS:
        .word      7
        .ascii      "MESSAGE"
        .align
MESS:
        .word    DOCOL
        .word    MESS+HEADSIZE
        .word    0x0
        .word    ERRSCR
        .word    N_MESS
        .word    0

        .word      LWARN
        .word      FETCH
        .word      ZBRAN
        .word      MESS1-.-CW @IF
        .word      ERRSCR, FETCH
        .word      PLINE, ONEP     @ Also print the '\n' !
        .word      ETYPE
        .word      X
MESS1:                  @THEN
        .word      DROP
        .word      SEMIS
@
@
@

@
@
 /* 
 Dit blok  PC@,PC!,PW@,PW! eerst maar even uitgeschakeld voor ARM !
 Dit moet waarschijnlijk helemaal anders als voor Intel
 */  
@  ***********
@  *   PC@   *
@  ***********
@
        .align
N_PCFET:
        .word      3
        .ascii      "PC@"
        .align
PCFET:
        .word    PCFET+HEADSIZE
        .word    PCFET+HEADSIZE
        .word    0x0
        .word    MESS
        .word    N_PCFET
        .word    0

@ FETCH CHARACTER (BYTE) FROM PORT
        LDR     R1, [R13],#+(CW*(1))      @ PORT ADDR
        MOV     R0,#0
@todo        IN      AL,_DX16   @ BYTE INPUT
        STR     R0, [R13,#-(CW*(1))]!
        LDR     R0, [R11],#+(CW*(1))
         LDR     R1,[R0]
         BX      R1  
@

@  ***********
@  *   PC!   *
@  ***********
@
        .align
N_PCSTO:
        .word      3
        .ascii      "PC!"
        .align
PCSTO:
        .word    PCSTO+HEADSIZE
        .word    PCSTO+HEADSIZE
        .word    0x0
        .word    PCFET
        .word    N_PCSTO
        .word    0

        LDR     R1, [R13],#+(CW*(1))      @PORT ADDR
        LDR     R0, [R13],#+(CW*(1))      @DATA
@todo        OUT     _DX16,AL   @ BYTE OUTPUT
        LDR     R0, [R11],#+(CW*(1))
         LDR     R1,[R0]
         BX      R1  
@

@  ***********
@  *   PW@   *
@  ***********
@
        .align
N_PFET:
        .word      3
        .ascii      "PW@"
        .align
PFET:
        .word    PFET+HEADSIZE
        .word    PFET+HEADSIZE
        .word    0x0
        .word    PCSTO
        .word    N_PFET
        .word    0

        LDR     R1, [R13],#+(CW*(1))      @PORT ADDR
@todo        IN      _AX16,_DX16  @WORD INPUT
        STR     R0, [R13,#-(CW*(1))]!
        LDR     R0, [R11],#+(CW*(1))
         LDR     R1,[R0]
         BX      R1  
@

@  ***********
@  *   PW!   *
@  ***********
@
        .align
N_PSTO:
        .word      3
        .ascii      "PW!"
        .align
PSTO:
        .word    PSTO+HEADSIZE
        .word    PSTO+HEADSIZE
        .word    0x0
        .word    PFET
        .word    N_PSTO
        .word    0

        LDR     R1, [R13],#+(CW*(1))      @PORT ADDR
        LDR     R0, [R13],#+(CW*(1))      @DATA
@todo        OUT     _DX16,_AX16   @WORD OUTPUT
        LDR     R0, [R11],#+(CW*(1))
         LDR     R1,[R0]
         BX      R1  
@
@
@

@  *************
@  *   _PREV   *
@  *************
@
        .align
N__PREV:
        .word      5
        .ascii      "_PREV"
        .align
_PREV:
        .word    DOVAR
        .word    _PREV+HEADSIZE
        .word    0x0
        .word    PSTO
        .word    N__PREV
        .word    0

        .word      BUF1

@
@

@  *************
@  *   #BUFF   *
@  *************
@
        .align
N_NOBUF:
        .word      5
        .ascii      "#BUFF"
        .align
NOBUF:
        .word    DOCON
        .word    NBUF
        .word    0x0
        .word    _PREV
        .word    N_NOBUF
        .word    0

@

@  ************
@  *   +BUF   *
@  ************
@
        .align
N_PBUF:
        .word      4
        .ascii      "+BUF"
        .align
PBUF:
        .word    DOCOL
        .word    PBUF+HEADSIZE
        .word    0x0
        .word    NOBUF
        .word    N_PBUF
        .word    0

        .word      BBUF, CELLP, CELLP
        .word      PLUS,LDUP
        .word      _LIMIT,EQUAL
        .word      ZBRAN
        .word      PBUF1-.-CW
        .word      DROP,_FIRST
PBUF1:  .word      LDUP, _PREV, FETCH, LSUB
        .word      SEMIS
@

@  **************
@  *   UPDATE   *
@  **************
@
        .align
N_UPDAT:
        .word      6
        .ascii      "UPDATE"
        .align
UPDAT:
        .word    DOCOL
        .word    UPDAT+HEADSIZE
        .word    0x0
        .word    PBUF
        .word    N_UPDAT
        .word    0

        .word      _PREV, FETCH
        .word      LDUP, CELLP,CELLP
        .word      SWAP, FETCH
        .word      LOFFSET,  FETCH, PLUS
        .word      BLWRITE
        .word      SEMIS
@

@  *********************
@  *   EMPTY-BUFFERS   *
@  *********************
@
        .align
N_MTBUF:
        .word      13
        .ascii      "EMPTY-BUFFERS"
        .align
MTBUF:
        .word    DOCOL
        .word    MTBUF+HEADSIZE
        .word    0x0
        .word    UPDAT
        .word    N_MTBUF
        .word    0

        .word      _FIRST
        .word      _LIMIT,OVER
        .word      LSUB,LERASE
        .word      _FIRST, _PREV, STOR
        .word      SEMIS
        #
@

@  ****************
@  *   (BUFFER)   *
@  ****************
@
        .align
N_BUFFER:
        .word      8
        .ascii      "(BUFFER)"
        .align
BUFFER:
        .word    DOCOL
        .word    BUFFER+HEADSIZE
        .word    0x0
        .word    MTBUF
        .word    N_BUFFER
        .word    0

@ Find the buffer, if it is already here.
    .word      _PREV, FETCH
BUFFER1:
    .word          TOR, RR, FETCH, OVER, EQUAL
    .word      ZBRAN
        .word      BUFFER3-.-CW
    .word        DROP, FROMR, EXIT
BUFFER3:
    .word          FROMR
    .word      PBUF, ZEQU
    .word      ZBRAN
        .word      BUFFER1-.-CW
@ We are left with the previous buffer.
BUFFER2:
    .word       PBUF, ZEQU, LIT, 48, QERR  @ No free buffers available.
    .word       LDUP, CELLP, FETCH
    .word       LIT, -1, GREAT        @ Unlocked buffer?
    .word      ZBRAN
        .word      BUFFER2-.-CW
    .word       TOR
@ Fill in the house keeping.
    .word       RR, STOR
    .word       ZERO, RR, CELLP, STOR

    .word       RR, _PREV, STOR
    .word       FROMR
    .word  SEMIS
@


@  *************
@  *   BLOCK   *
@  *************
@
        .align
N_BLOCK:
        .word      5
        .ascii      "BLOCK"
        .align
BLOCK:
        .word    DOCOL
        .word    BLOCK+HEADSIZE
        .word    0x0
        .word    BUFFER
        .word    N_BLOCK
        .word    0


        .word      BUFFER
        .word      LDUP, CELLP, FETCH, ONE, LAND, ZEQU
        .word      ZBRAN
        .word      BLOCK1-.-CW
        .word      LDUP, CELLP, CELLP
        .word      OVER, FETCH
        .word      LOFFSET,  FETCH, PLUS
        .word      BLREAD
        .word      LDUP, CELLP, ONE, TOGGL
BLOCK1:
        .word      LDUP, _PREV, STOR
        .word      CELLP, CELLP
        .word      SEMIS
@

@  *************
@  *   FLUSH   *
@  *************
@
        .align
N_FLUSH:
        .word      5
        .ascii      "FLUSH"
        .align
FLUSH:
        .word    DOCOL
        .word    (MTBUF+HEADSIZE)
        .word    0x0
        .word    BLOCK
        .word    N_FLUSH
        .word    0


@ Unlock all buffers
        .word      _LIMIT
        .word      _FIRST, CELLP
        .word     XDO
        .word      FLUS2-.-CW
FLUS1:  .word      ZERO, IDO, STOR
        .word      LIT,(KBBUF+2*CW)
        .word      PLOOP
        .word      (FLUS1-.)
FLUS2:
        .word      SEMIS
@

@  ************
@  *   SAVE   *
@  ************
@
        .align
N_SAVE:
        .word      4
        .ascii      "SAVE"
        .align
SAVE:
        .word    DOCOL
        .word    SAVE+HEADSIZE
        .word    0x0
        .word    FLUSH
        .word    N_SAVE
        .word    0

        .word      FROMR
        .word      SRC, TFET
        .word      PP, FETCH
        .word      TOR, TOR, TOR
        .word      TOR
        .word SEMIS
@

@  ***************
@  *   RESTORE   *
@  ***************
@
        .align
N_RESTO:
        .word      7
        .ascii      "RESTORE"
        .align
RESTO:
        .word    DOCOL
        .word    RESTO+HEADSIZE
        .word    0x0
        .word    SAVE
        .word    N_RESTO
        .word    0

        .word      FROMR
        .word      FROMR, FROMR, FROMR
        .word      PP, STOR
        .word      SRC, TSTOR
        .word      TOR
        .word SEMIS
@

@ 
@

@ 
@

@  ************
@  *   LOCK   *
@  ************
@
        .align
N_LLOCK:
        .word      4
        .ascii      "LOCK"
        .align
LLOCK:
        .word    DOCOL
        .word    LLOCK+HEADSIZE
        .word    0x0
        .word    RESTO
        .word    N_LLOCK
        .word    0

        .word      BUFFER
        .word      CELLP
        .word      LIT, -2, SWAP, PSTOR
        .word      SEMIS
@

@  **************
@  *   UNLOCK   *
@  **************
@
        .align
N_LUNLOCK:
        .word      6
        .ascii      "UNLOCK"
        .align
LUNLOCK:
        .word    DOCOL
        .word    LUNLOCK+HEADSIZE
        .word    0x0
        .word    LLOCK
        .word    N_LUNLOCK
        .word    0

        .word      BUFFER
        .word      CELLP
        .word      TWO, SWAP, PSTOR
        .word      SEMIS
@

@  ************
@  *   LOAD   *
@  ************
@
        .align
N_LLOAD:
        .word      4
        .ascii      "LOAD"
        .align
LLOAD:
        .word    DOCOL
        .word    LLOAD+HEADSIZE
        .word    0x0
        .word    LUNLOCK
        .word    N_LLOAD
        .word    0

        .word      SAVE
        .word      TOR
        .word      RR, LIT, LLOCK, CATCH  @ Be careful about always unlocking.
        .word      LDUP, ZEQU
        .word      ZBRAN
        .word      LOAD5-.-CW
        .word      DROP
        .word      RR, BLOCK     @ Can't fail anymore. 
        .word      BBUF
        .word      SETSRC
        .word      LIT, INTER, CATCH
        .word      RR, LUNLOCK
LOAD5:
        .word      RDROP
        .word      RESTO
        .word      THROW
        .word      SEMIS
@

@  ************
@  *   THRU   *
@  ************
@
        .align
N_THRU:
        .word      4
        .ascii      "THRU"
        .align
THRU:
        .word    DOCOL
        .word    THRU+HEADSIZE
        .word    0x0
        .word    LLOAD
        .word    N_THRU
        .word    0

        .word      ONEP, SWAP
        .word     XDO
        .word      THRU2-.-CW
THRU1:
        .word      IDO, LLOAD
        .word     ONE, XPLOOP, ZBRAN
        .word      THRU1-.-CW
	.word      UNLOOP
THRU2:
        .word      SEMIS
@
@

@  ***********
@  *   BLK   *
@  ***********
@
        .align
N_BLK:
        .word      3
        .ascii      "BLK"
        .align
BLK:
        .word    DOCOL
        .word    BLK+HEADSIZE
        .word    0x0
        .word    THRU
        .word    N_BLK
        .word    0

        .word      PP, FETCH
        .word      _FIRST, _LIMIT, WITHIN
        .word      SRC, TFET, LSUB
        .word      LIT, 1024, EQUAL, LAND
        .word      ZBRAN
        .word      BLK1-.-CW
        .word      SRC, FETCH, TWO, LCELLS, LSUB, FETCH
        .word      BRAN
        .word      BLK2-.-CW
BLK1:
        .word      ZERO
BLK2:
        .word      PBLK, STOR
        .word      PBLK
        .word      SEMIS
@

@



@  ***********
@  *   XOS   *
@  ***********
@
        .align
N_XOS:
        .word      3
        .ascii      "XOS"
        .align
XOS:
        .word    XOS+HEADSIZE
        .word    XOS+HEADSIZE
        .word    0x0
        .word    BLK
        .word    N_XOS
        .word    0


        
        LDR     R7, [R13],#+(CW*(1))        @ Function number
        LDR     R2, [R13],#+(CW*(1))        @ Third parameter, or dummy
        LDR     R1, [R13],#+(CW*(1))        @ Second parameter, or dummy
        LDR     R0, [R13],#+(CW*(1))        @ First parameter.
        SWI     0        @ Generic call on LINUX 
        
        STR     R0, [R13,#-(CW*(1))]!
        LDR     R0, [R11],#+(CW*(1))
         LDR     R1,[R0]
         BX      R1       @ Positive means okay. Negative means -errno.
@

@
@
 @ 





@  ************
@  *   XOS5   *
@  ************
@
        .align
N_XOS5:
        .word      4
        .ascii      "XOS5"
        .align
XOS5:
        .word    XOS5+HEADSIZE
        .word    XOS5+HEADSIZE
        .word    0x0
        .word    XOS
        .word    N_XOS5
        .word    0

        
        @FIXME maybe R7 needs saving. 
        LDR     R7, [R13],#+(CW*(1))        @ Function number
        LDR     R4, [R13],#+(CW*(1))        @ 5th parameter, or dummy
        LDR     R3, [R13],#+(CW*(1))        @ 4th parameter, or dummy
        LDR     R2, [R13],#+(CW*(1))        @ Third parameter, or dummy
        LDR     R1, [R13],#+(CW*(1))        @ Second parameter, or dummy
        LDR     R0, [R13],#+(CW*(1))        @ First parameter.
        SWI     0      @ Generic call on LINUX 
        
        STR     R0, [R13,#-(CW*(1))]!
        LDR     R0, [R11],#+(CW*(1))
         LDR     R1,[R0]
         BX      R1            @ Positive means okay. Negative means -errno.
@

@



@  ************
@  *   XOS7   *
@  ************
@
        .align
N_XOS7:
        .word      4
        .ascii      "XOS7"
        .align
XOS7:
        .word    XOS7+HEADSIZE
        .word    XOS7+HEADSIZE
        .word    0x0
        .word    XOS5
        .word    N_XOS7
        .word    0

        
        LDR     R7, [R13],#+(CW*(1))        @ Function number            
        LDR     R6, [R13],#+(CW*(1))        @ 7th parameter, or dummy
        LDR     R5, [R13],#+(CW*(1))        @ 6th parameter, or dummy
        LDR     R4, [R13],#+(CW*(1))        @ 5th parameter, or dummy
        LDR     R3, [R13],#+(CW*(1))        @ 4th parameter, or dummy
        LDR     R2, [R13],#+(CW*(1))        @ Third parameter, or dummy
        LDR     R1, [R13],#+(CW*(1))        @ Second parameter, or dummy
        LDR     R0, [R13],#+(CW*(1))        @ First parameter.
        SVC     #0           @ Generic call on LINUX      
        
        STR     R0, [R13,#-(CW*(1))]!
        LDR     R0, [R11],#+(CW*(1))
         LDR     R1,[R0]
         BX      R1            @ Positive means okay. Negative means -errno.

@
@

@
 @ 


@  **********
@  *   MS   *
@  **********
@
        .align
N_MS:
        .word      2
        .ascii      "MS"
        .align
MS:
        .word    DOCOL
        .word    MS+HEADSIZE
        .word    0x0
        .word    XOS7
        .word    N_MS
        .word    0

@ Use select because nanosleep has a 20 MS resolution. 

        .word      LIT, 1000, LIT, 1000000, SSMOD @ (ms -- us s) 
        .word      LIT, TV, TSTOR
        .word      ZERO, ZERO
        .word      ZERO, ZERO
        .word      LIT, TV
        .word      LIT, select, XOS5
@ 
@ 

        .word      QERRUR
        .word      SEMIS
TV:     .space    (CW*(2))    @ A time_t and a long, both cells.
@
@ 

        #
@------------------------------------
@       CONSOLE RELEATED I/O         
@------------------------------------







@  *****************
@  *   RW-BUFFER   *
@  *****************
@
        .align
N_RWBUF:
        .word      9
        .ascii      "RW-BUFFER"
        .align
RWBUF:
        .word    DOVAR
        .word    RWBUF+HEADSIZE
        .word    0x0
        .word    MS
        .word    N_RWBUF
        .word    0

        .space    0x1000
@ 

@


@  ***********
@  *   ZEN   *
@  ***********
@
        .align
N_ZEN:
        .word      3
        .ascii      "ZEN"
        .align
ZEN:
        .word    DOCOL
        .word    ZEN+HEADSIZE
        .word    0x0
        .word    RWBUF
        .word    N_ZEN
        .word    0

        .word      RWBUF, SSTOR
        .word      ZERO, RWBUF, CHAPP
        .word      RWBUF, CELLP

        .word      SEMIS
@ 



@


@  *****************
@  *   OPEN-FILE   *
@  *****************
@
        .align
N_OFILE:
        .word      9
        .ascii      "OPEN-FILE"
        .align
OFILE:
        .word    DOCOL
        .word    OFILE+HEADSIZE
        .word    0x0
        .word    ZEN
        .word    N_OFILE
        .word    0

      .word      TOR, ZEN, FROMR, X
        .word      LIT, open, XOS
        .word      LDUP, ZERO, MIN
        .word      SEMIS 

@


@


@  ******************
@  *   CLOSE-FILE   *
@  ******************
@
        .align
N_CFILE:
        .word      10
        .ascii      "CLOSE-FILE"
        .align
CFILE:
        .word    DOCOL
        .word    CFILE+HEADSIZE
        .word    0x0
        .word    OFILE
        .word    N_CFILE
        .word    0

        .word      X, X
        .word      LIT, close, XOS
        .word      SEMIS
@


@



@  *******************
@  *   CREATE-FILE   *
@  *******************
@
        .align
N_CREATEF:
        .word      11
        .ascii      "CREATE-FILE"
        .align
CREATEF:
        .word    DOCOL
        .word    CREATEF+HEADSIZE
        .word    0x0
        .word    CFILE
        .word    N_CREATEF
        .word    0

        .word      TOR, TDUP, DFILE, DROP
        .word      ZEN, FROMR, X
        .word      LIT, creat, XOS
        .word      LDUP, ZERO, MAX, SWAP, ZERO, MIN
        .word      SEMIS 


@


@


@  *******************
@  *   DELETE-FILE   *
@  *******************
@
        .align
N_DFILE:
        .word      11
        .ascii      "DELETE-FILE"
        .align
DFILE:
        .word    DOCOL
        .word    DFILE+HEADSIZE
        .word    0x0
        .word    CREATEF
        .word    N_DFILE
        .word    0

        .word      ZEN, X, X
        .word      LIT, unlink, XOS
        .word      SEMIS 

@


@


@  *****************
@  *   READ-FILE   *
@  *****************
@
        .align
N_RFILE:
        .word      9
        .ascii      "READ-FILE"
        .align
RFILE:
        .word    DOCOL
        .word    RFILE+HEADSIZE
        .word    0x0
        .word    DFILE
        .word    N_RFILE
        .word    0

        .word      ROT, ROT
        .word      LIT, read, XOS
        .word      LDUP, ZERO, MAX, SWAP, ZERO, MIN
        .word      SEMIS
@ 
@

@

@  *********************
@  *   FILE-POSITION   *
@  *********************
@
        .align
N_FPFILE:
        .word      13
        .ascii      "FILE-POSITION"
        .align
FPFILE:
        .word    DOCOL
        .word    FPFILE+HEADSIZE
        .word    0x0
        .word    RFILE
        .word    N_FPFILE
        .word    0

        .word      ZERO
        .word      LIT, SEEK_CUR
        .word      LIT, lseek, XOS
        .word      ZERO     @Double precision return result
        .word      OVER, LDUP
        .word      NEGATE, LIT, 256, ULESS, LAND @ Heuristic 
        .word      SEMIS
 @ 

@


@  ***********************
@  *   REPOSITION-FILE   *
@  ***********************
@
        .align
N_PFILE:
        .word      15
        .ascii      "REPOSITION-FILE"
        .align
PFILE:
        .word    DOCOL
        .word    PFILE+HEADSIZE
        .word    0x0
        .word    FPFILE
        .word    N_PFILE
        .word    0

        .word      TOR, DROP, FROMR   @Convert to single precision.
        .word      SWAP
        .word      LIT, SEEK_SET
        .word      LIT, lseek, XOS
        .word      ZERO, MIN
        .word      SEMIS
 @ 



@


@  ******************
@  *   WRITE-FILE   *
@  ******************
@
        .align
N_WFILE:
        .word      10
        .ascii      "WRITE-FILE"
        .align
WFILE:
        .word    DOCOL
        .word    WFILE+HEADSIZE
        .word    0x0
        .word    PFILE
        .word    N_WFILE
        .word    0

        .word      ROT, ROT
        .word      LIT, write, XOS
        .word      ZERO, MIN
        .word      SEMIS
@ 



@


@  ****************
@  *   GET-FILE   *
@  ****************
@
        .align
N_GETFILE:
        .word      8
        .ascii      "GET-FILE"
        .align
GETFILE:
        .word    DOCOL
        .word    GETFILE+HEADSIZE
        .word    0x0
        .word    WFILE
        .word    N_GETFILE
        .word    0

     .word LALIGN
        .word      TDUP, SCOMMA, DROP
        .word      LIT, 'F'+(0x100*('i'+0x100*('L'+0x100*'e'))), COMMA @Magic number.
        .word      ZERO, OFILE, THROW, TOR
        .word      HERE, LDUP
        .word      LEM, LIT, 6, SLASH, LDUP, ALLOT
        .word      LIT, 1000, LSUB
        .word      RR, RFILE, THROW
        .word      FROMR, CFILE, THROW
        .word      TDUP, PLUS, LDP, STOR        @ No allocation if it fails.
        .word      SEMIS
@

@  ****************
@  *   PUT-FILE   *
@  ****************
@
        .align
N_PUTFILE:
        .word      8
        .ascii      "PUT-FILE"
        .align
PUTFILE:
        .word    DOCOL
        .word    PUTFILE+HEADSIZE
        .word    0x0
        .word    GETFILE
        .word    N_PUTFILE
        .word    0

        .word      LIT, 0755
        .word      CREATEF, THROW
        .word      LDUP, TOR
        .word      WFILE, THROW
        .word      FROMR, CFILE, THROW
        .word      SEMIS
@

@  ****************
@  *   INCLUDED   *
@  ****************
@
        .align
N_INCLUDED:
        .word      8
        .ascii      "INCLUDED"
        .align
INCLUDED:
        .word    DOCOL
        .word    INCLUDED+HEADSIZE
        .word    0x0
        .word    PUTFILE
        .word    N_INCLUDED
        .word    0

        .word      HERE, TOR
        .word      LIT, GETFILE, CATCH
        .word      LDUP
        .word      ZBRAN
        .word      INCLUD1-.-CW
        .word      FROMR, LDP, STOR
        .word      THROW
        .word      BRAN
        .word      INCLUD2-.-CW
INCLUD1:
        .word      RDROP, DROP
INCLUD2:
        .word      EVALUATE
        .word      SEMIS
@

@  ***************
@  *   INCLUDE   *
@  ***************
@
        .align
N_LINCLUD:
        .word      7
        .ascii      "INCLUDE"
        .align
LINCLUD:
        .word    DOCOL
        .word    LINCLUD+HEADSIZE
        .word    0x0
        .word    INCLUDED
        .word    N_LINCLUD
        .word    0

        .word      LNAME, INCLUDED
        .word      SEMIS
@

@  ******************
@  *   REFILL-TIB   *
@  ******************
@
        .align
N_REFTIB:
        .word      10
        .ascii      "REFILL-TIB"
        .align
REFTIB:
        .word    DOCOL
        .word    REFTIB+HEADSIZE
        .word    0x0
        .word    LINCLUD
        .word    N_REFTIB
        .word    0

        .word      REMAIND, FETCH, TOR
        .word      TIB, FETCH, RR, PLUS
        .word      LIT, TIBSIZE, RR, LSUB
        .word      ZERO
        .word      RFILE
        .word      QERRUR
        .word      LDUP, ZEQU, LIT, -EPIPE, LAND @ Presumably end of pipe.
        .word      QERRUR
        .word      TIB, FETCH, SWAP, FROMR, PLUS
        .word      REMAIND, TSTOR
        .word      SEMIS
@
@

@

@ All I/O goes through the modern device (Unix-standard)
@ device I/O. Code fields are filled in during bootup. 
@ Lower case labels starting with "c_.." are c-supplied facilities.

@  **************
@  *   ACCEPT   *
@  **************
@
        .align
N_ACCEP:
        .word      6
        .ascii      "ACCEPT"
        .align
ACCEP:
        .word    DOCOL
        .word    ACCEP+HEADSIZE
        .word    0x0
        .word    REFTIB
        .word    N_ACCEP
        .word    0

        .word      PACCEP
        .word      TSWAP, ROT, MIN
        .word      LDUP, TOR, LMOVE, FROMR
        .word      SEMIS
@

@  ****************
@  *   (ACCEPT)   *
@  ****************
@
        .align
N_PACCEP:
        .word      8
        .ascii      "(ACCEPT)"
        .align
PACCEP:
        .word    DOCOL
        .word    PACCEP+HEADSIZE
        .word    0x0
        .word    ACCEP
        .word    N_PACCEP
        .word    0

PACCEP2:
        .word      REMAIND
        .word      TFET
        .word      LIT, ALF, SINDEX
        .word      ZEQU
        .word      ZBRAN
        .word      PACCEP1-.-CW
        .word      REMAIND, TFET
        .word      TIB, FETCH
        .word      SWAP, LMOVE
        .word      TIB, FETCH
        .word      REMAIND, CELLP, STOR
        .word      REFTIB
        .word      BRAN
        .word      PACCEP2-.-CW
PACCEP1:
        .word      REMAIND, TFET
        .word      LIT, ALF, SSLAS
        .word      TSWAP, REMAIND, TSTOR
        .word      SEMIS


@  ***********
@  *   KEY   *
@  ***********
@
        .align
N_KEY:
        .word      3
        .ascii      "KEY"
        .align
KEY:
        .word    DOCOL
        .word    KEY+HEADSIZE
        .word    0x0
        .word    PACCEP
        .word    N_KEY
        .word    0

        .word      ONE, LIT, RAWIO, STTERM @ Do not wait for CR
        .word      ZERO, SPFET     @ Expect single key on stack
        .word      ZERO            @ Standard input
        .word      SWAP, ONE       @ Buffer and length
        .word      LIT, read, XOS
        .word      LDUP, QERRUR
        .word      ZEQU, LIT, -EPIPE, LAND, QERRUR @ Presumably end of pipe.
        .word      ONE, LIT, RAWIO, STTERM       @ Toggle back
        .word      SEMIS
@

@  ************
@  *   KEY?   *
@  ************
@
        .align
N_KEYQ:
        .word      4
        .ascii      "KEY?"
        .align
KEYQ:
        .word    DOCOL
        .word    KEYQ+HEADSIZE
        .word    0x0
        .word    KEY
        .word    N_KEYQ
        .word    0

        .word      ZERO, LIT, RAWIO, STTERM
        .word      ZERO, ZERO     @ Time out of zero. See MS.
        .word      LIT, TV, TSTOR
        .word      ONE @ Waiting for standard in.
        .word      LIT, RFDS, STOR
        .word      ONE     @ # of highest channel to watch.
        .word      LIT, RFDS
        .word      ZERO, ZERO  @ Not output, not events.
        
        .word      LIT, TV, LIT, select, XOS5
        .word      LDUP,  QERRUR
        .word      NEGATE        @ 1 -> flag.
        .word      ZERO, LIT, RAWIO, STTERM
        .word      SEMIS

        .align
RFDS:   .space    (CW*(32))     @32 channels maximum.

@

@  ************
@  *   TYPE   *
@  ************
@
        .align
N_LTYPE:
        .word      4
        .ascii      "TYPE"
        .align
LTYPE:
        .word    DOCOL
        .word    LTYPE+HEADSIZE
        .word    0x0
        .word    KEYQ
        .word    N_LTYPE
        .word    0

        .word      LDUP, LOUT, PSTOR
        .word      ONE, WFILE, THROW
        .word      SEMIS
@

@  *************
@  *   ETYPE   *
@  *************
@
        .align
N_ETYPE:
        .word      5
        .ascii      "ETYPE"
        .align
ETYPE:
        .word    DOCOL
        .word    ETYPE+HEADSIZE
        .word    0x0
        .word    LTYPE
        .word    N_ETYPE
        .word    0

        .word      TWO, WFILE, THROW
        .word      SEMIS
@

@  ************
@  *   EMIT   *
@  ************
@
        .align
N_EMIT:
        .word      4
        .ascii      "EMIT"
        .align
EMIT:
        .word    DOCOL
        .word    EMIT+HEADSIZE
        .word    0x0
        .word    ETYPE
        .word    N_EMIT
        .word    0

        .word      SPFET, ONE, LTYPE
        .word      DROP
        .word      SEMIS
@

@  *************
@  *   DEMIT   *
@  *************
@
        .align
N_DEMIT:
        .word      5
        .ascii      "DEMIT"
        .align
DEMIT:
        .word    DEMIT+HEADSIZE
        .word    DEMIT+HEADSIZE
        .word    0x0
        .word    EMIT
        .word    N_DEMIT
        .word    0

        MOV     R1, R13  @ 4 characters on the stack, SP is their address 
        MOV      R0, #1   @ standard output 
        MOV      R2, #4   @ 4 character 
        MOV      R0, #4   @ function number 
        SWI     0
        ADD     R13,R13,#4  @ adjust stack 
        LDR     R0, [R11],#+(CW*(1))
         LDR     R1,[R0]
         BX      R1  


@  **************
@  *   TERMIO   *
@  **************
@
        .align
N_TERMIO:
        .word      6
        .ascii      "TERMIO"
        .align
TERMIO:
        .word    DOVAR
        .word    TERMIO+HEADSIZE
        .word    0x0
        .word    DEMIT
        .word    N_TERMIO
        .word    0

        .space    SIZE_TERMIO     @ Filled in during cold boot.
@

@  ****************
@  *   SET-TERM   *
@  ****************
@
        .align
N_STTERM:
        .word      8
        .ascii      "SET-TERM"
        .align
STTERM:
        .word    DOCOL
        .word    STTERM+HEADSIZE
        .word    0x0
        .word    TERMIO
        .word    N_STTERM
        .word    0

SIZEL   = 4     @ Also for 64 bits. 
        .word      LIT, (TERMIO+HEADSIZE)+SIZEL*3
        .word      SWAP, TOGGL
        .word      LIT, (TERMIO+HEADSIZE)+SIZEL*4+7
        .word      CSTOR
        .word      ZERO, LIT, TCSETS, TERMIO
        .word      LIT, ioctl, XOS, QERRUR
        .word      SEMIS
@
@ 






@

        #
@------------------------------------
@       SYSTEM DEPENDANT DISK I/O
@------------------------------------

@  ******************
@  *   DISK-ERROR   *
@  ******************
@
        .align
N_DERR:
        .word      10
        .ascii      "DISK-ERROR"
        .align
DERR:
        .word    DOVAR
        .word    DERR+HEADSIZE
        .word    0x0
        .word    STTERM
        .word    N_DERR
        .word    0

        .word      -1
@


@  ******************
@  *   BLOCK-FILE   *
@  ******************
@
        .align
N_BLFL:
        .word      10
        .ascii      "BLOCK-FILE"
        .align
BLFL:
        .word    DOVAR
        .word    BLFL+HEADSIZE
        .word    0x0
        .word    DERR
        .word    N_BLFL
        .word    0

        .word      9
        .ascii      "forth.lab"
        .space    252-9               @ Allow for some path

@  ********************
@  *   BLOCK-HANDLE   *
@  ********************
@
        .align
N_BHAN:
        .word      12
        .ascii      "BLOCK-HANDLE"
        .align
BHAN:
        .word    DOVAR
        .word    BHAN+HEADSIZE
        .word    0x0
        .word    BLFL
        .word    N_BHAN
        .word    0

        .word      -1
@

@  *******************
@  *   ?DISK-ERROR   *
@  *******************
@
        .align
N_QDSKER:
        .word      11
        .ascii      "?DISK-ERROR"
        .align
QDSKER:
        .word    DOCOL
        .word    QDSKER+HEADSIZE
        .word    0x0
        .word    BHAN
        .word    N_QDSKER
        .word    0

        .word      LIT, 8, QERR
        .word      SEMIS
@
@


@  ******************
@  *   BLOCK-INIT   *
@  ******************
@
        .align
N_BLINI:
        .word      10
        .ascii      "BLOCK-INIT"
        .align
BLINI:
        .word    DOCOL
        .word    BLINI+HEADSIZE
        .word    0x0
        .word    QDSKER
        .word    N_BLINI
        .word    0

        .word      BLFL, SFET, DTRAI
        .word      ROT
        .word      OFILE
        .word      ZEQU, NEGATE  @ 0 if disk problems, 1 if not.
        .word      LWARN, FETCH, MIN @ AND but WARNING is 0/1.
        .word      LWARN, STOR
        .word      BHAN, STOR
        .word      MTBUF
        .word      SEMIS
@

@  ******************
@  *   BLOCK-EXIT   *
@  ******************
@
        .align
N_BLEXI:
        .word      10
        .ascii      "BLOCK-EXIT"
        .align
BLEXI:
        .word    DOCOL
        .word    BLEXI+HEADSIZE
        .word    0x0
        .word    BLINI
        .word    N_BLEXI
        .word    0

        .word      FLUSH
        .word      BHAN, FETCH
        .word      CFILE
        .word      ZERO, LWARN, STOR
        .word      LIT, -1, BHAN, STOR    @Regardless of close errors.
        .word      QDSKER
        .word      SEMIS
@

@  **************
@  *   BLSEEK   *
@  **************
@
        .align
N_BLSEEK:
        .word      6
        .ascii      "BLSEEK"
        .align
BLSEEK:
        .word    DOCOL
        .word    BLSEEK+HEADSIZE
        .word    0x0
        .word    BLEXI
        .word    N_BLSEEK
        .word    0

        .word      BBUF
        .word      USTAR
        .word      BHAN, FETCH
        .word      PFILE           @ Disk position.
        .word      QDSKER
        .word      SEMIS
@

@  ******************
@  *   BLOCK-READ   *
@  ******************
@
        .align
N_BLREAD:
        .word      10
        .ascii      "BLOCK-READ"
        .align
BLREAD:
        .word    DOCOL
        .word    BLREAD+HEADSIZE
        .word    0x0
        .word    BLSEEK
        .word    N_BLREAD
        .word    0

        .word      BLSEEK @ That's done
        .word      BBUF
        .word      BHAN, FETCH
        .word      RFILE           @ Disk read 
        .word      SWAP, BBUF, UNEQ
        .word      LOR, QDSKER
        .word      SEMIS
@

@  *******************
@  *   BLOCK-WRITE   *
@  *******************
@
        .align
N_BLWRITE:
        .word      11
        .ascii      "BLOCK-WRITE"
        .align
BLWRITE:
        .word    DOCOL
        .word    BLWRITE+HEADSIZE
        .word    0x0
        .word    BLREAD
        .word    N_BLWRITE
        .word    0

        .word      BLSEEK @ That's done
        .word      BBUF
        .word      BHAN, FETCH
        .word      WFILE           @ Disk write
        .word      QDSKER
        .word      SEMIS
@
@

@  *************
@  *   SHELL   *
@  *************
@
        .align
N_SHELL:
        .word      5
        .ascii      "SHELL"
        .align
SHELL:
        .word    DOVAR
        .word    SHELL+HEADSIZE
        .word    0x0
        .word    BLWRITE
        .word    N_SHELL
        .word    0

                 .word      7
        .ascii      "/bin/sh"

        .space    252-9               @ Allow for some path

@


@  **************
@  *   SYSTEM   *
@  **************
@
        .align
N_SYSTEM:
        .word      6
        .ascii      "SYSTEM"
        .align
SYSTEM:
        .word    DOCOL
        .word    SYSTEM+HEADSIZE
        .word    0x0
        .word    SHELL
        .word    N_SYSTEM
        .word    0

          .word      LIT, COMBUF, SSTOR, ZERO, LIT, COMBUF, CHAPP
          .word      ZERO, SHELL, CHAPP, LIT, -1, SHELL, PSTOR
         .word      X, X, X, LIT, fork, XOS

          .word      LDUP, QERRUR
          .word      LDUP, ZEQU
          .word      ZBRAN
        .word      SYSTEM1-.-CW
        .word      SHELL, CELLP, LIT, SYSARGS
@  Get environment vector
      .word      ARGS, FETCH, SFET, ONEP, LCELLS, PLUS
        .word      LIT, execve, XOS
        .word      LIT, 49, LERROR, BYE   @ Returned, this is always wrong.
SYSTEM1:
@ int waitid(idtype_t idtype, id_t id, siginfo_t * infop , int options ) 

        .word      LDUP   @ pid of child
        .word      LIT, P_PID, SWAP
        .word      LIT, SIGINFOT
        .word      LIT, WEXITED, ZERO
        .word      LIT, waitid, XOS5
        .word      LDUP
        .word      LIT, EINTR, EQUAL
        .word      BRAN
        .word      SYSTEM2-.-CW
        .word      DROP
        .word      ZBRAN
        .word      SYSTEM1-.-CW
SYSTEM2:
        .word      QERRUR
        .word      TDROP
        .word      SEMIS
@
        .align
SIGINFOT: .space    CW*9
SYSARG2: .ascii     "-c"
         .byte      0
        .align
COMBUF:  .space    CW+256  @One cell for high level string manipulation.
        .align
SYSARGS:.word   (SHELL+HEADSIZE)+CW, SYSARG2, COMBUF + CW, 0
@
 @ 



@



@


@

@  *********
@  *   '   *
@  *********
@
        .align
N_ITICK:
        .word      1
        .ascii      "'"
        .align
ITICK:
        .word    DOCOL 
        .word    ITICK+HEADSIZE
        .word    0x0
        .word    SYSTEM
        .word    N_ITICK
        .word    0

        .word      LNAME, PRESENT
        .word      LDUP, ZEQU
        .word      LIT, 11, QERR
        .word      SEMIS
@

@  ***********
@  *   [']   *
@  ***********
@
        .align
N_BTICK:
        .word      3
        .ascii      "[']"
        .align
BTICK:
        .word    DOCOL
        .word    (TICK+HEADSIZE)
        .word    B_IMMED
        .word    ITICK
        .word    N_BTICK
        .word    0

@

@  ******************
@  *   FORGET-VOC   *
@  ******************
@
        .align
N_FORGV:
        .word      10
        .ascii      "FORGET-VOC"
        .align
FORGV:
        .word    DOCOL
        .word    FORGV+HEADSIZE
        .word    0x0
        .word    BTICK
        .word    N_FORGV
        .word    0

        .word      TDUP
        .word      SWAP
        .word      ULESS
        .word      ZBRAN
        .word      FORGV1-.-CW
@  Forget part of contents.
        .word      SWAP
        .word      TOR
        .word      TWID
FORGV0:
        .word      LDUP
        .word      LDUP
FORGV3:
        .word      TLFA,FETCH
        .word      LDUP
        .word      RR
        .word      ULESS
        .word      ZBRAN
        .word      FORGV3-.-CW
        .word      SWAP            @  Not to be forgotten entry found.
        .word      TLFA
        .word      STOR           @  Short other entries out.
        .word      TLFA,FETCH
        .word      LDUP
        .word      ZEQU
        .word      ZBRAN
        .word      FORGV0-.-CW        @  Repeat until end of wordlist.
        .word      DROP
        .word      FROMR
        .word      BRAN
        .word      FORGV2-.-CW
FORGV1:
@       { Vocabulary itself is also forgotten.}
        .word      TVFA
        .word      FETCH     @ Unlink by linking next namespace.
        .word      VOCL
        .word      STOR
        .word      ONLY, FORTH
        .word      DEFIN
FORGV2: .word      SEMIS
@

@  **************
@  *   FORGET   *
@  **************
@
        .align
N_FORG:
        .word      6
        .ascii      "FORGET"
        .align
FORG:
        .word    DOCOL
        .word    FORG+HEADSIZE
        .word    0x0
        .word    FORGV
        .word    N_FORG
        .word    0

        .word      TICK
        .word      LDUP
        .word      FENCE
        .word      FETCH
        .word      LESS
        .word      LIT, 21, QERR
        .word      LIT,FORGV
        .word      FORV
        .word      TNFA, FETCH, LDP, STOR
        .word      SEMIS
@

@  *************
@  *   (BACK   *
@  *************
@
        .align
N_PBACK:
        .word      5
        .ascii      "(BACK"
        .align
PBACK:
        .word    DOCOL
        .word    PBACK+HEADSIZE
        .word    0x0
        .word    FORG
        .word    N_PBACK
        .word    0

        .word      HERE
        .word      SEMIS
@

@  *************
@  *   BACK)   *
@  *************
@
        .align
N_BACKP:
        .word      5
        .ascii      "BACK)"
        .align
BACKP:
        .word    DOCOL
        .word    BACKP+HEADSIZE
        .word    0x0
        .word    PBACK
        .word    N_BACKP
        .word    0

        .word      HERE
        .word      CELLP
        .word      LSUB
        .word      COMMA
        .word      SEMIS
@

@  ****************
@  *   (FORWARD   *
@  ****************
@
        .align
N_PFORWARD:
        .word      8
        .ascii      "(FORWARD"
        .align
PFORWARD:
        .word    DOCOL
        .word    PFORWARD+HEADSIZE
        .word    0x0
        .word    BACKP
        .word    N_PFORWARD
        .word    0

        .word      HERE
        .word      X
        .word      COMMA
        .word      SEMIS
@

@  ****************
@  *   FORWARD)   *
@  ****************
@
        .align
N_FORWARDP:
        .word      8
        .ascii      "FORWARD)"
        .align
FORWARDP:
        .word    DOCOL
        .word    FORWARDP+HEADSIZE
        .word    0x0
        .word    PFORWARD
        .word    N_FORWARDP
        .word    0

        .word      HERE
        .word      OVER
        .word      CELLP
        .word      LSUB
        .word      SWAP
        .word      STOR
        .word      SEMIS
@

@  *************
@  *   BEGIN   *
@  *************
@
        .align
N_BEGIN:
        .word      5
        .ascii      "BEGIN"
        .align
BEGIN:
        .word    DOCOL
        .word    BEGIN+HEADSIZE
        .word    B_IMMED
        .word    FORWARDP
        .word    N_BEGIN
        .word    0

        .word      PBACK
        .word      QCOMP, ONE
        .word      SEMIS
@

@  ************
@  *   THEN   *
@  ************
@
        .align
N_THEN:
        .word      4
        .ascii      "THEN"
        .align
THEN:
        .word    DOCOL
        .word    THEN+HEADSIZE
        .word    B_IMMED
        .word    BEGIN
        .word    N_THEN
        .word    0

        .word      QCOMP, TWO, QPAIR
        .word      FORWARDP
        .word      SEMIS
@

@  **********
@  *   DO   *
@  **********
@
        .align
N_DO:
        .word      2
        .ascii      "DO"
        .align
DO:
        .word    DOCOL
        .word    DO+HEADSIZE
        .word    B_IMMED
        .word    THEN
        .word    N_DO
        .word    0

         .word      LIT, XDO, COMMA, PFORWARD, PBACK
        .word      QCOMP, LIT,3    @ Magic number
        .word      SEMIS
@

@  ***********
@  *   ?DO   *
@  ***********
@
        .align
N_QDO:
        .word      3
        .ascii      "?DO"
        .align
QDO:
        .word    DOCOL
        .word    QDO+HEADSIZE
        .word    B_IMMED
        .word    DO
        .word    N_QDO
        .word    0

         .word      LIT, XQDO, COMMA, PFORWARD, PBACK
        .word      QCOMP, LIT,3    @ Magic number
        .word      SEMIS
@

@  ************
@  *   LOOP   *
@  ************
@
        .align
N_LLOOP:
        .word      4
        .ascii      "LOOP"
        .align
LLOOP:
        .word    DOCOL
        .word    LLOOP+HEADSIZE
        .word    B_IMMED
        .word    QDO
        .word    N_LLOOP
        .word    0

        .word      LIT, ONE, COMMA @ POSTPONE 1 
        .word      PLOOP
        .word      SEMIS
@

@  *************
@  *   +LOOP   *
@  *************
@
        .align
N_PLOOP:
        .word      5
        .ascii      "+LOOP"
        .align
PLOOP:
        .word    DOCOL
        .word    PLOOP+HEADSIZE
        .word    B_IMMED
        .word    LLOOP
        .word    N_PLOOP
        .word    0

        .word      QCOMP, LIT, 3, QPAIR
        .word      LIT, XPLOOP, COMMA
        .word      LIT, ZBRAN, COMMA, BACKP
        .word      LIT, UNLOOP, COMMA
        .word      FORWARDP @ For DO to push the leave address.
        .word      SEMIS
@

@  *************
@  *   UNTIL   *
@  *************
@
        .align
N_UNTIL:
        .word      5
        .ascii      "UNTIL"
        .align
UNTIL:
        .word    DOCOL
        .word    UNTIL+HEADSIZE
        .word    B_IMMED
        .word    PLOOP
        .word    N_UNTIL
        .word    0

        .word      QCOMP, ONE, QPAIR
        .word      LIT, ZBRAN, COMMA, BACKP
        .word      SEMIS
@

@  *************
@  *   AGAIN   *
@  *************
@
        .align
N_AGAIN:
        .word      5
        .ascii      "AGAIN"
        .align
AGAIN:
        .word    DOCOL
        .word    AGAIN+HEADSIZE
        .word    B_IMMED
        .word    UNTIL
        .word    N_AGAIN
        .word    0

        .word      QCOMP, ONE, QPAIR
        .word      LIT, BRAN, COMMA, BACKP
        .word      SEMIS
@

@  **************
@  *   REPEAT   *
@  **************
@
        .align
N_REPEA:
        .word      6
        .ascii      "REPEAT"
        .align
REPEA:
        .word    DOCOL
        .word    REPEA+HEADSIZE
        .word    B_IMMED
        .word    AGAIN
        .word    N_REPEA
        .word    0

        .word      QCOMP, ONE, QPAIR   @ Matches BEGIN ?
        .word      LIT, BRAN, COMMA, BACKP
        .word      LIT, 4, QPAIR @ Matches WHILE ?
        .word      FORWARDP @ WHILE target. 
        .word      SEMIS
@

@  **********
@  *   IF   *
@  **********
@
        .align
N_LIF:
        .word      2
        .ascii      "IF"
        .align
LIF:
        .word    DOCOL
        .word    LIF+HEADSIZE
        .word    B_IMMED
        .word    REPEA
        .word    N_LIF
        .word    0

        .word      LIT, ZBRAN, COMMA, PFORWARD
        .word      QCOMP, TWO     @ Magic number
        .word      SEMIS
@

@  ************
@  *   ELSE   *
@  ************
@
        .align
N_LELSE:
        .word      4
        .ascii      "ELSE"
        .align
LELSE:
        .word    DOCOL
        .word    LELSE+HEADSIZE
        .word    B_IMMED
        .word    LIF
        .word    N_LELSE
        .word    0

        .word      QCOMP, TWO, QPAIR
        .word      LIT, BRAN, COMMA, PFORWARD
        .word      SWAP
        .word      FORWARDP
        .word      TWO     @ Magic number
        .word      SEMIS
@

@  *************
@  *   WHILE   *
@  *************
@
        .align
N_LWHILE:
        .word      5
        .ascii      "WHILE"
        .align
LWHILE:
        .word    DOCOL
        .word    LWHILE+HEADSIZE
        .word    B_IMMED
        .word    LELSE
        .word    N_LWHILE
        .word    0

        .word      QCOMP, LDUP, ONE, QPAIR
        .word      TOR    @  Save backward target. 
        .word      TOR
        .word      LIT, ZBRAN, COMMA, PFORWARD
        .word      LIT, 4 @ Magic number
        .word      FROMR
        .word      FROMR
        .word      SEMIS
@

@  **************
@  *   SPACES   *
@  **************
@
        .align
N_SPACES:
        .word      6
        .ascii      "SPACES"
        .align
SPACES:
        .word    DOCOL
        .word    SPACES+HEADSIZE
        .word    0x0
        .word    LWHILE
        .word    N_SPACES
        .word    0

        .word      ZERO
        .word      MAX
        .word      ZERO
        .word     XQDO
        .word      SPAX1-.-CW
SPAX2:  .word      SPACE
        .word     ONE, XPLOOP, ZBRAN
        .word      SPAX2-.-CW
	.word      UNLOOP    @LOOP
SPAX1:
        .word      SEMIS
@

@  **********
@  *   <#   *
@  **********
@
        .align
N_BDIGS:
        .word      2
        .ascii      "<#"
        .align
BDIGS:
        .word    DOCOL
        .word    BDIGS+HEADSIZE
        .word    0x0
        .word    SPACES
        .word    N_BDIGS
        .word    0

        .word      PAD
        .word      HLD
        .word      STOR
        .word      SEMIS
@

@  **********
@  *   #>   *
@  **********
@
        .align
N_EDIGS:
        .word      2
        .ascii      "#>"
        .align
EDIGS:
        .word    DOCOL
        .word    EDIGS+HEADSIZE
        .word    0x0
        .word    BDIGS
        .word    N_EDIGS
        .word    0

        .word      DROP
        .word      DROP
        .word      HLD
        .word      FETCH
        .word      PAD
        .word      OVER
        .word      LSUB
        .word      SEMIS
@

@  ************
@  *   SIGN   *
@  ************
@
        .align
N_SIGN:
        .word      4
        .ascii      "SIGN"
        .align
SIGN:
        .word    DOCOL
        .word    SIGN+HEADSIZE
        .word    0x0
        .word    EDIGS
        .word    N_SIGN
        .word    0

        .word      ZLESS
        .word      ZBRAN
        .word      SIGN1-.-CW @IF
        .word      LIT, AMS
        .word      HOLD    @THEN
SIGN1:  .word      SEMIS
@

@  *********
@  *   #   *
@  *********
@
        .align
N_DIG:
        .word      1
        .ascii      "#"
        .align
DIG:
        .word    DOCOL
        .word    DIG+HEADSIZE
        .word    0x0
        .word    SIGN
        .word    N_DIG
        .word    0

        .word      BASE
        .word      FETCH
        .word      MSMOD
        .word      ROT
        .word      LIT,9
        .word      OVER
        .word      LESS
        .word      ZBRAN
        .word      DIG1-.-CW  @IF
        .word      LIT,7
        .word      PLUS    @THEN
DIG1:   .word      LIT,0x30
        .word      PLUS
        .word      HOLD
        .word      SEMIS
@

@  **********
@  *   #S   *
@  **********
@
        .align
N_DIGS:
        .word      2
        .ascii      "#S"
        .align
DIGS:
        .word    DOCOL
        .word    DIGS+HEADSIZE
        .word    0x0
        .word    DIG
        .word    N_DIGS
        .word    0

DIGS1:  .word      DIG     @BEGIN
        .word      OVER
        .word      OVER
        .word      LOR
        .word      ZEQU
        .word      ZBRAN
        .word      DIGS1-.-CW @UNTIL
        .word      SEMIS
@
@ 

@  *************
@  *   (D.R)   *
@  *************
@
        .align
N_PDDOTR:
        .word      5
        .ascii      "(D.R)"
        .align
PDDOTR:
        .word    DOCOL
        .word    PDDOTR+HEADSIZE
        .word    0x0
        .word    DIGS
        .word    N_PDDOTR
        .word    0

        .word      TOR
        .word      SWAP
        .word      OVER
        .word      DABS
        .word      BDIGS
        .word      DIGS
        .word      ROT
        .word      SIGN
        .word      EDIGS
        .word      FROMR
        .word      OVER
        .word      LSUB, ZERO, MAX
        .word      ZERO
        .word     XQDO
        .word      PDDOT1-.-CW
PDDOT2:  .word      LBL, HOLD  @WARNING: HOLD outside of #>.
        .word     ONE, XPLOOP, ZBRAN
        .word      PDDOT2-.-CW
	.word      UNLOOP
PDDOT1:
        .word      EDIGS  @Drop string instead of number.
        .word      SEMIS
@

@  ***********
@  *   D.R   *
@  ***********
@
        .align
N_DDOTR:
        .word      3
        .ascii      "D.R"
        .align
DDOTR:
        .word    DOCOL
        .word    DDOTR+HEADSIZE
        .word    0x0
        .word    PDDOTR
        .word    N_DDOTR
        .word    0

        .word      PDDOTR
        .word      LTYPE
        .word      SEMIS
@

@  **********
@  *   .R   *
@  **********
@
        .align
N_DOTR:
        .word      2
        .ascii      ".R"
        .align
DOTR:
        .word    DOCOL
        .word    DOTR+HEADSIZE
        .word    0x0
        .word    DDOTR
        .word    N_DOTR
        .word    0

        .word      TOR
        .word      STOD
        .word      FROMR
        .word      DDOTR
        .word      SEMIS
@

@  **********
@  *   D.   *
@  **********
@
        .align
N_DDOT:
        .word      2
        .ascii      "D."
        .align
DDOT:
        .word    DOCOL
        .word    DDOT+HEADSIZE
        .word    0x0
        .word    DOTR
        .word    N_DDOT
        .word    0

        .word      ZERO
        .word      DDOTR
        .word      SPACE
        .word      SEMIS
@

@  *********
@  *   .   *
@  *********
@
        .align
N_DOT:
        .word      1
        .ascii      "."
        .align
DOT:
        .word    DOCOL
        .word    DOT+HEADSIZE
        .word    0x0
        .word    DDOT
        .word    N_DOT
        .word    0

        .word      STOD
        .word      DDOT
        .word      SEMIS
@

@  *********
@  *   ?   *
@  *********
@
        .align
N_QUES:
        .word      1
        .ascii      "?"
        .align
QUES:
        .word    DOCOL
        .word    QUES+HEADSIZE
        .word    0x0
        .word    DOT
        .word    N_QUES
        .word    0

        .word      FETCH
        .word      DOT
        .word      SEMIS
@

@  **********
@  *   U.   *
@  **********
@
        .align
N_UDOT:
        .word      2
        .ascii      "U."
        .align
UDOT:
        .word    DOCOL
        .word    UDOT+HEADSIZE
        .word    0x0
        .word    QUES
        .word    N_UDOT
        .word    0

        .word      ZERO
        .word      DDOT
        .word      SEMIS
@

@  *****************
@  *   FOR-WORDS   *
@  *****************
@
        .align
N_FORW:
        .word      9
        .ascii      "FOR-WORDS"
        .align
FORW:
        .word    DOCOL
        .word    FORW+HEADSIZE
        .word    0x0
        .word    UDOT
        .word    N_FORW
        .word    0

        .word      SWAP
        .word      TOR
        .word      TOR
FORW1:  .word      FROMR
        .word      RR
        .word      OVER
        .word      TLFA
        .word      FETCH
        .word      TOR
        .word      EXEC
        .word      RR
        .word      ZEQU
        .word      ZBRAN
        .word      FORW1-.-CW
        .word      RDROP
        .word      RDROP
        .word      SEMIS
@

@  ****************
@  *   FOR-VOCS   *
@  ****************
@
        .align
N_FORV:
        .word      8
        .ascii      "FOR-VOCS"
        .align
FORV:
        .word    DOCOL
        .word    FORV+HEADSIZE
        .word    0x0
        .word    FORW
        .word    N_FORV
        .word    0

        .word      TOR
        .word      VOCL
        .word      FETCH
        .word      TOR
FORV1:  .word      FROMR
        .word      RR
        .word      OVER
        .word      TVFA
        .word      FETCH
        .word      TOR
        .word      EXEC
        .word      RR
        .word      ZEQU
        .word      ZBRAN
        .word      FORV1-.-CW
        .word      RDROP
        .word      RDROP
        .word      SEMIS
@

@  *************
@  *   WORDS   *
@  *************
@
        .align
N_WORDS:
        .word      5
        .ascii      "WORDS"
        .align
WORDS:
        .word    DOCOL
        .word    WORDS+HEADSIZE
        .word    0x0
        .word    FORV
        .word    N_WORDS
        .word    0

        .word      CSLL
        .word      LOUT
        .word      STOR
        .word      ZERO            @ End sentinel. 
        .word      LIT, NOOP
        .word      CONTEXT
        .word      FETCH
        .word      FORW
WORDS1:
        .word      LDUP
        .word      ZBRAN
        .word      WORDS2-.-CW
        .word      IDDOT
        .word      BRAN
        .word      WORDS1-.-CW
WORDS2:
        .word DROP
        .word      SEMIS
@



@  *****************
@  *   EXIT-CODE   *
@  *****************
@
        .align
N_XCODE:
        .word      9
        .ascii      "EXIT-CODE"
        .align
XCODE:
        .word    DOVAR
        .word    XCODE+HEADSIZE
        .word    0x0
        .word    WORDS
        .word    N_XCODE
        .word    0

        .word      0
@


@  ***********
@  *   BYE   *
@  ***********
@
        .align
N_BYE:
        .word      3
        .ascii      "BYE"
        .align
BYE:
        .word    DOCOL
        .word    BYE+HEADSIZE
        .word    0x0
        .word    XCODE
        .word    N_BYE
        .word    0

@ Exit to linux, with status as stored. 
        .word      XCODE, FETCH, X, X, LIT, exit, XOS
        .word      SEMIS   @Unnecessary, but helpful for decompilation.
@ 
@

@  ************
@  *   LIST   *
@  ************
@
        .align
N_LLIST:
        .word      4
        .ascii      "LIST"
        .align
LLIST:
        .word    DOCOL
        .word    LLIST+HEADSIZE
        .word    0x0
        .word    BYE
        .word    N_LLIST
        .word    0

        .word      BASE, FETCH, TOR
        .word      DECA
        .word      SCR,STOR

        .word      SKIP
         .word      6
SB3: .ascii      "SCR # "
       .align
        .word      LIT, SB3
        .word      LIT, 6
        .word      LTYPE
        .word      SCR, FETCH, DOT
        .word      SCR, FETCH, BLOCK, BBUF

        .word      LIT, 16, ZERO
        .word     XDO
        .word      LLIST9-.-CW
LLIST1: .word      LIT, ALF, SSLAS
        .word      CR, IDO, LIT, 2, DOTR
        .word      SPACE, LTYPE
        .word     ONE, XPLOOP, ZBRAN
        .word      LLIST1-.-CW
	.word      UNLOOP
LLIST9:
        .word      TDROP
        .word      FROMR, BASE, STOR
        .word      SEMIS
@

@  *************
@  *   INDEX   *
@  *************
@
        .align
N_INDEX:
        .word      5
        .ascii      "INDEX"
        .align
INDEX:
        .word    DOCOL
        .word    INDEX+HEADSIZE
        .word    0x0
        .word    LLIST
        .word    N_INDEX
        .word    0

        .word      LIT,AFF
        .word      EMIT,CR
        .word      ONEP,SWAP
        .word     XDO
        .word      INDE9-.-CW
INDE1:  .word      CR,IDO
        .word      LIT,3
        .word      DOTR,SPACE
        .word      ZERO,IDO
        .word      PLINE, LTYPE
        .word     ONE, XPLOOP, ZBRAN
        .word      INDE1-.-CW
	.word      UNLOOP
INDE9:
        .word      SEMIS
@

@  **********
@  *   .S   *
@  **********
@
        .align
N_DOTS:
        .word      2
        .ascii      ".S"
        .align
DOTS:
        .word    DOCOL
        .word    DOTS+HEADSIZE
        .word    0x0
        .word    INDEX
        .word    N_DOTS
        .word    0

        .word      CR
        .word      LIT, 'S', EMIT
        .word      LIT, ASO, EMIT
        .word      SPACE
        .word      SPFET, SZERO, FETCH
DOC2:   .word      OVER, OVER,  LSUB, ZLESS
        .word      ZBRAN
        .word      DOC1-.-CW
        .word      ZERO, CELLP, LSUB, LDUP, FETCH
        .word      DOT
        .word      BRAN
        .word      DOC2-.-CW
DOC1:    .word DROP, DROP
        .word      LIT, ASC, EMIT
        .word SEMIS
@

@  ********************
@  *   ENVIRONMENT?   *
@  ********************
@
        .align
N_ENVQ:
        .word      12
        .ascii      "ENVIRONMENT?"
        .align
ENVQ:
        .word    DOCOL
        .word    ENVQ+HEADSIZE
        .word    0x0
        .word    DOTS
        .word    N_ENVQ
        .word    0

        .word      LIT, ENV, TWID, PFIND
        .word      TOR, TDROP, FROMR
        .word      LDUP
        .word      ZBRAN
        .word      ENVQ1-.-CW
        .word      EXEC
        .word      LIT, -1
ENVQ1:
        .word      SEMIS
@

@ 
@
@ This word is not even fig!

@  ***************
@  *   .SIGNON   *
@  ***************
@
        .align
N_SIGNON:
        .word      7
        .ascii      ".SIGNON"
        .align
SIGNON:
        .word    DOCOL
        .word    SIGNON+HEADSIZE
        .word    0x0
        .word    ENVQ
        .word    N_SIGNON
        .word    0

@ PRINT CPU TYPE (8088)
        .word      CR
        .word      BASE,FETCH
        .word      LIT,36, BASE,STOR
        .word      LCPU, DDOT
        .word      BASE,STOR
@
        .word      LENAME, LTYPE, SPACE
        .word      LVERSION, LTYPE, SPACE
        .word      CR
        .word      SEMIS
@

@
@**** LAST DICTIONARY WORD OF FORTH ****

@  ************
@  *   TASK   *
@  ************
@
        .align
N_TASK:
        .word      4
        .ascii      "TASK"
        .align
TASK:
        .word    DOCOL
        .word    TASK+HEADSIZE
        .word    0x0
        .word    SIGNON
        .word    N_TASK
        .word    0

        .word      SEMIS
@
@ End of ONLY namespace. Namespaces all end in a link to 0.

@  *********
@  *   '   *
@  *********
@
        .align
N_TICK:
        .word      1
        .ascii      "'"
        .align
TICK:
        .word    DOCOL
        .word    TICK+HEADSIZE
        .word    B_IMMED + B_DENOT
        .word    0
        .word    N_TICK
        .word    0

        .word      ITICK
        .word      LITER
        .word      SEMIS
@

@  *********
@  *   &   *
@  *********
@
        .align
N_DCHAR:
        .word      1
        .ascii      "&"
        .align
DCHAR:
        .word    DOCOL
        .word    DCHAR+HEADSIZE
        .word    B_IMMED + B_DENOT
        .word    TICK
        .word    N_DCHAR
        .word    0

        .word      PPFET
        .word      NIP
        .word      LDUP, QBL
        .word      LIT, 10, QERR
        .word      LITER
        .word      QDELIM
        .word      SEMIS

@

@  *********
@  *   ^   *
@  *********
@
        .align
N_DCTL:
        .word      1
        .ascii      "^"
        .align
DCTL:
        .word    DOCOL
        .word    DCTL+HEADSIZE
        .word    B_IMMED + B_DENOT
        .word    DCHAR
        .word    N_DCTL
        .word    0

        .word      PPFET
        .word      NIP
        .word      LDUP, QBL
        .word      LIT, 10, QERR
        .word      LIT, '@', LSUB
        .word      LITER
        .word      QDELIM
        .word      SEMIS
@

@  *********
@  *   0   *
@  *********
@
        .align
N_DEN0:
        .word      1
        .ascii      "0"
        .align
DEN0:
        .word    DOCOL
        .word    LNUMB
        .word    B_IMMED + B_DENOT
        .word    DCTL
        .word    N_DEN0
        .word    0

@  *********
@  *   1   *
@  *********
@
        .align
N_DEN1:
        .word      1
        .ascii      "1"
        .align
DEN1:
        .word    DOCOL
        .word    LNUMB
        .word    B_IMMED + B_DENOT
        .word    DEN0
        .word    N_DEN1
        .word    0

@  *********
@  *   2   *
@  *********
@
        .align
N_DEN2:
        .word      1
        .ascii      "2"
        .align
DEN2:
        .word    DOCOL
        .word    LNUMB
        .word    B_IMMED + B_DENOT
        .word    DEN1
        .word    N_DEN2
        .word    0

@  *********
@  *   3   *
@  *********
@
        .align
N_DEN3:
        .word      1
        .ascii      "3"
        .align
DEN3:
        .word    DOCOL
        .word    LNUMB
        .word    B_IMMED + B_DENOT
        .word    DEN2
        .word    N_DEN3
        .word    0

@  *********
@  *   4   *
@  *********
@
        .align
N_DEN4:
        .word      1
        .ascii      "4"
        .align
DEN4:
        .word    DOCOL
        .word    LNUMB
        .word    B_IMMED + B_DENOT
        .word    DEN3
        .word    N_DEN4
        .word    0

@  *********
@  *   5   *
@  *********
@
        .align
N_DEN5:
        .word      1
        .ascii      "5"
        .align
DEN5:
        .word    DOCOL
        .word    LNUMB
        .word    B_IMMED + B_DENOT
        .word    DEN4
        .word    N_DEN5
        .word    0

@  *********
@  *   6   *
@  *********
@
        .align
N_DEN6:
        .word      1
        .ascii      "6"
        .align
DEN6:
        .word    DOCOL
        .word    LNUMB
        .word    B_IMMED + B_DENOT
        .word    DEN5
        .word    N_DEN6
        .word    0


@  *********
@  *   7   *
@  *********
@
        .align
N_DEN7:
        .word      1
        .ascii      "7"
        .align
DEN7:
        .word    DOCOL
        .word    LNUMB
        .word    B_IMMED + B_DENOT
        .word    DEN6
        .word    N_DEN7
        .word    0

@  *********
@  *   8   *
@  *********
@
        .align
N_DEN8:
        .word      1
        .ascii      "8"
        .align
DEN8:
        .word    DOCOL
        .word    LNUMB
        .word    B_IMMED + B_DENOT
        .word    DEN7
        .word    N_DEN8
        .word    0

@  *********
@  *   9   *
@  *********
@
        .align
N_DEN9:
        .word      1
        .ascii      "9"
        .align
DEN9:
        .word    DOCOL
        .word    LNUMB
        .word    B_IMMED + B_DENOT
        .word    DEN8
        .word    N_DEN9
        .word    0

@  *********
@  *   A   *
@  *********
@
        .align
N_DENA:
        .word      1
        .ascii      "A"
        .align
DENA:
        .word    DOCOL
        .word    LNUMB
        .word    B_IMMED + B_DENOT
        .word    DEN9
        .word    N_DENA
        .word    0

@  *********
@  *   B   *
@  *********
@
        .align
N_DENB:
        .word      1
        .ascii      "B"
        .align
DENB:
        .word    DOCOL
        .word    LNUMB
        .word    B_IMMED + B_DENOT
        .word    DENA
        .word    N_DENB
        .word    0


@  *********
@  *   C   *
@  *********
@
        .align
N_DENC:
        .word      1
        .ascii      "C"
        .align
DENC:
        .word    DOCOL
        .word    LNUMB
        .word    B_IMMED + B_DENOT
        .word    DENB
        .word    N_DENC
        .word    0

@  *********
@  *   D   *
@  *********
@
        .align
N_DEND:
        .word      1
        .ascii      "D"
        .align
DEND:
        .word    DOCOL
        .word    LNUMB
        .word    B_IMMED + B_DENOT
        .word    DENC
        .word    N_DEND
        .word    0

@  *********
@  *   E   *
@  *********
@
        .align
N_DENE:
        .word      1
        .ascii      "E"
        .align
DENE:
        .word    DOCOL
        .word    LNUMB
        .word    B_IMMED + B_DENOT
        .word    DEND
        .word    N_DENE
        .word    0

@  *********
@  *   F   *
@  *********
@
        .align
N_DENF:
        .word      1
        .ascii      "F"
        .align
DENF:
        .word    DOCOL
        .word    LNUMB
        .word    B_IMMED + B_DENOT
        .word    DENE
        .word    N_DENF
        .word    0

@

@  *********
@  *   -   *
@  *********
@
        .align
N_DENM:
        .word      1
        .ascii      "-"
        .align
DENM:
        .word    DOCOL
        .word    DENM+HEADSIZE
        .word    B_IMMED + B_DENOT
        .word    DENF
        .word    N_DENM
        .word    0

        .word      PNUMB, DNEGA, SDLITE
        .word      SEMIS
@

@  *********
@  *   +   *
@  *********
@
        .align
N_DENP:
        .word      1
        .ascii      "+"
        .align
DENP:
        .word    DOCOL
        .word    DENP+HEADSIZE
        .word    B_IMMED + B_DENOT
        .word    DENM
        .word    N_DENP
        .word    0

        .word      PNUMB, SDLITE
        .word      SEMIS
@

@  *********
@  *   "   *
@  *********
@
        .align
N_DENQ:
        .word      1
        .ascii      "\""
        .align
DENQ:
        .word    DOCOL
        .word    DENQ+HEADSIZE
        .word    B_IMMED + B_DENOT
        .word    DENP
        .word    N_DENQ
        .word    0

        .word      LIT, SKIP, COMMA        @  'SKIP , HERE >R 0 ,
        .word      HERE, TOR, ZERO, COMMA
DENQ1:
        .word      LIT, '"', PARSE         @           BEGIN &" PARSE
        .word      PPFET, LDUP, LIT, '"', EQUAL @           PP@@ DUP &" =
        .word      ZBRAN
        .word      DENQ2-.-CW                 @           WHILE
        .word      TDROP, ONEP             @           2DROP 1+ R@ $+!
        .word      LDUP, ALLOT, RR, SADD
        .word      BRAN
        .word      DENQ1-.-CW                  @           REPEAT
DENQ2:
        .word      QBL, ZEQU
        .word      LIT, 10, QERR           @           ?BLANK 0= 10 ?ERROR
        .word      DROP                    @                DROP 
        .word      LDUP, ALLOT, RR, SADD     @         DUP ALLOT R@ $+!
    .word LALIGN
        .word      FROMR, SFET, DLITE      @           R> $@ POSTPONE DLITERAL ;
        .word      SEMIS
@

@ The FORTH word is the only non-denotation in the ONLY namespace.
@  *************
@  *   FORTH   *
@  *************
@
        .align
N_FORTH:
        .word      5
        .ascii      "FORTH"
        .align
FORTH:
        .word    DODOE
        .word    FORTH+HEADSIZE
        .word    0x0
        .word    DENQ
        .word    N_FORTH
        .word    0

        .word      DOVOC
        .word      0       @ END OF NAMESPACE LIST

        .align

        .word    0x0
        .word    0
        .word    B_DUMMY
        .word    TASK
        .word    (ZERO+(CW*(D_HOFFSET)))
        .word    0

@
STARTOFONLY    = FORTH
@
@

@  ************
@  *   CORE   *
@  ************
@
        .align
N_CORE:
        .word      4
        .ascii      "CORE"
        .align
CORE:
        .word    DOCOL
        .word    CORE+HEADSIZE
        .word    0x0
        .word    0
        .word    N_CORE
        .word    0

        .word      ZERO    @ Not (fully) present.
        .word      SEMIS
@

@  ***********
@  *   CPU   *
@  ***********
@
        .align
N_LCPU:
        .word      3
        .ascii      "CPU"
        .align
LCPU:
        .word    DOCOL
        .word    LCPU+HEADSIZE
        .word    0x0
        .word    CORE
        .word    N_LCPU
        .word    0

       .word      LIT, 0x0113F683, ZERO      @ 'ARMV7'
             @ 'ARMV8'
        .word      SEMIS
@

@  ***************
@  *   VERSION   *
@  ***************
@
        .align
N_LVERSION:
        .word      7
        .ascii      "VERSION"
        .align
LVERSION:
        .word    DOCOL
        .word    LVERSION+HEADSIZE
        .word    0x0
        .word    LCPU
        .word    N_LVERSION
        .word    0

 /* 
@       If this is there it is an official release
        .word      SKIP
        .word      22
IBMPC:  .ascii     "IBM-PC ciforth"
        .byte      FIGREL+0x40,ADOT,FIGREV+0x30,ADOT,USRVER+0x30
        .word      LIT, IBMPC, LIT, 22
 */  @       If M4_VERSION exists and contains a . it is an official release
        .word      SKIP
         .word      5
SB4: .ascii      "5.4.1"
       .align
        .word      LIT, SB4
        .word      LIT, 5
        .word      SEMIS
@

@  ************
@  *   NAME   *
@  ************
@
        .align
N_LENAME:
        .word      4
        .ascii      "NAME"
        .align
LENAME:
        .word    DOCOL
        .word    LENAME+HEADSIZE
        .word    0x0
        .word    LVERSION
        .word    N_LENAME
        .word    0

        .word      SKIP
         .word      7
SB5: .ascii      "ciforth"
       .align
        .word      LIT, SB5
        .word      LIT, 7
        .word      SEMIS
@

@  ****************
@  *   SUPPLIER   *
@  ****************
@
        .align
N_SUPPLIER:
        .word      8
        .ascii      "SUPPLIER"
        .align
SUPPLIER:
        .word    DOCOL
        .word    SUPPLIER+HEADSIZE
        .word    0x0
        .word    LENAME
        .word    N_SUPPLIER
        .word    0

        .word      SKIP
         .word      40
SB6: .ascii      "Albert van der Horst/Frans van der Markt"
       .align
        .word      LIT, SB6
        .word      LIT, 40
        .word      SEMIS
@
STARTOFENVIRONMENT    =   SUPPLIER
@
@
 @  

 /* 

The remaining memory ( up to 'EM' ) is
used for:

        1. EXTENSION DICTIONARY
        2. PARAMETER STACK
        3. TERMINAL INPUT BUFFER
        4. RETURN STACK
        5. USER VARIABLE AREA
        6. DISK BUFFERS (UNLESS REQUIRED <1 MBYTE)


 */  

@       This is the proper way to do it.
@       No memory addresses should be arrived at through equates.
@       However now we must teach the linker to keep the
@       two sections together.

FORTHSIZE       =     . - _main
@ We must use bss, because of the c-library.
@ The dictionary is not a normal bss, because it is executable.
@ The linker -N option takes care of that. 
       .bss @

INITDP:                 @  It may be that it is not consecutive with TASK.
                        @  Because of bad linking, resulting in a unusable Forth.
        BUFFERSIZE      =  (KBBUF+2*CW)*NBUF

@ For more task the TAS area is duplicated; the remainder is the same.
        .space    EM - FORTHSIZE - TAS - BUFFERSIZE 
@ INITR1:                         Return stack for next task
@          .space    US              @  For next task, if present. 
@ The following framework may be repeated for more tasks.
@ It is floating, i.e. it may be shifted up and down, in order
@ to have INITR0 at a STACKSIZE boundary. 
        .space    STACKSIZE - US    @ Data stack, only limited if more tasks present.
INITS0:                         @ Grows down.
STRTIB: .space    TIBSIZE     @ Terminal input buffer.
        .space    STACKSIZE       @ Return stack, grows down.
INITR0:                         @ Grows down
STRUSA: .space    US              @ User area
BUF1:   .space    BUFFERSIZE      @ FIRST DISK BUFFER
ACTUAL_EM:  @ Differs from EM if code is relocated.

        
@ 
@


        #
 /* 

  MISC. NOTES AND SCATTERED THOUGHTS

- Remember that all the FORTH words in this version are
  upper case letters.  Use <CAPS LOCK> when in FORTH.



- Subscribe to FORTH Dimensions.  It is a valuable source
  of system and application ideas.  Talking with fellow
  FORTH programmers is sure to stir up some exciting ideas.
  Consider joining a FIG chapter.  See the back of FORTH
  Dimensions for more info.

 */  
@ Define the entry point, not valid for auto booting.
        
        # _ORIG


























@





